/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-1999 Emmanuel CURIS
   Cration : le 16 fvrier 1999

   Import des coordonnes cristallographiques sauves en ASCII avec Carine v.3

   Mthode brutale et pas trs propre
   Permet de faire au passage :
     - la cration de plusieurs mailles
     - la conversion de coordonnes en coordonnes orthonormales
*/

#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#include "systeme.h"

#include "atomes.h"
#include "cristallo.h"
#include "fichiers.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Interface de paramtrage et d'appel ------ */

/* -------- Passer l'en-tte dfini par Carine -------- */

int MOL_Import_Init_Carine(FILE *fichier, struct infos_lecture *infos)
{
  int c;
  char *commande;
  boolean trouve = FAUX;

  if ( ( NULL == fichier ) || ( NULL == infos ) ) return -666; /* Ne devrait jamais arriver ! */

  /* On contrle qu'il s'agit bien d'un fichier Carine [empirique] */
  commande = lit_commande( fichier );
  if ( NULL == commande ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le chargement du fichier Carine\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
  if ( strcmp( commande, "CaRIne" ) != 0 ) {
    OPTION_A_FAIRE( "Fichier ne commenant pas par CaRINE. Avertir l'utilisateur et poser la question" );
    /* Pour l'instant, on annule */
    return -58;
  }
  free( commande ); commande = NULL;

  while ( FAUX == trouve ) {
    /* On ignore la fin de la ligne */
    c = ignore_ligne( fichier );
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }

    /* On lit la commande suivante */
    commande = lit_commande( fichier );
    if ( NULL == commande ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le chargement du fichier Carine\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    if ( 0 == strcmp( "Atom", commande ) ) {
      trouve = VRAI;
    }
    if ( infos->ligne_maille != 0 ) { /* On lit les paramtres de maille dans le fichier */
      if ( 0 == strcmp( "Cell", commande ) ) {
	free( commande );

	/* On lit la sous-commande [le 2e mot en fait] */
	commande = lit_commande( fichier );
	if ( NULL == commande ) {
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le chargement du fichier Carine\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  return -1;
	}
	if ( 'P' == *commande ) { /* Parameters(...), achve la ligne normalement */
	  c = ignore_ligne( fichier );
	  fscanf(fichier, "a=%lf b=%lf c=%lf", &(infos->maille->a), &(infos->maille->b), &(infos->maille->c) );
	  fscanf(fichier, "%*[^=]=%lf beta=%lf gamma=%lf", &(infos->maille->alpha), &(infos->maille->beta), &(infos->maille->gamma) );

	  OPTION_A_FAIRE( "Analyser les paramtres, pour voir si accord avec groupe d'espace" );
	  printf("%s : a=%g, b=%g, c=%g, a=%g, b=%g, g=%g\n", __FUNCTION__,
		 infos->maille->a, infos->maille->b, infos->maille->c, infos->maille->alpha, infos->maille->beta, infos->maille->gamma);
	}
      }
    }
    if ( infos->ligne_groupe != 0 ) { /* On lit le groupe d'espace dans le fichier */
      if ( 0 == strcmp( "Space", commande ) ) {
	unsigned int nbr_groupe;

	fscanf( fichier, "Group Number=%u", &nbr_groupe );
	OPTION_A_FAIRE( "Reconnaissance du groupe d'espace" );
      }
    }

    /* Plus besoin de la commande */
    free( commande ); commande = NULL;
  }

  /* On ignore la fin de la ligne "Atom..." */
  c = ignore_ligne( fichier );
  if ( EOF == c ) {
    (void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* On est bien, mainenant, au dbut des positions atomiques */
  return 0;
}

