/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système

   Arbres simplistes d'usage courant : demander une valeur (texte, entier, flottant,...)
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <math.h> /* Pour log10 */

#include "txt_utils.h"

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int BI_creer_boutons(int num_arbre, unsigned int haut_boite, unsigned int nbr_boutons, unsigned int premier_bouton,
			    unsigned int bouton_ok, const char * const textes[]);
static int BI_generer_texte(int num_arbre, char **lignes, unsigned int nbr_lignes, unsigned int premier_objet);

static int BI_ControlerBoiteDemande(const BI_Boite_Demande *contenu);

static int BI_CreerChampEditable(unsigned int num_arbre, unsigned int num_objet_1, const struct infos_demande *valeur, int x, int y,
				 unsigned int larg_texte, unsigned int larg_editable);
static int BI_LireChampEditable (unsigned int num_arbre, unsigned int num_objet_1, struct infos_demande *valeur, unsigned int *nbr_objets);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Les briques élémentaires ~-~-~-~-~-~-~-~- */

/* -------- Vérifier que les paramètres passés sont corrects --------
   Renvoie 0 si aucun problème, <0 en cas d'erreur

   **            : on ne doit pas avoir un appel récursif
   contenu       : la boîte -- non-NULL, avec un texte et au plus 3 boutons
   -------- */

static int BI_ControlerBoiteDemande(const BI_Boite_Demande *contenu)
{
  /* Contrôle de la boîte de demande elle-même */
  if ( NULL == contenu ) {
    (void) fprintf(stderr, "%s, %s l.%u : Boîte d'alerte = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
  if ( NULL == contenu->texte ) {
    (void) fprintf(stderr, "%s, %s l.%u : Boîte d'alerte sans texte\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  if ( contenu->nbr_boutons > 3 ) {
    (void) fprintf(stderr, "%s, %s l.%u : 3 boutons au plus, %u demandés\n",
		   __FILE__, __FUNCTION__, __LINE__, contenu->nbr_boutons );
    return -3;
  }

  /* On vérifie pour l'appel récursif */
  if ( VRAI == arbres[ BI_ArbreAlerte ].drapeaux.ouvert ) {
    (void) fprintf(stderr, "%s, %s l.%u : Arbre alerte/demande déjà ouvert, appel récursif interdit\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -4;
  }

  /* Pas de problème détecté */
  return 0;
}

/* -------- Construction d'un champ éditable adapté au type de valeur demandé --------
   Renvoie le nombre d'objets créés [0 en cas de problème !]
 */

static int BI_CreerChampEditable(unsigned int numero_arbre, unsigned int num_objet_1, const struct infos_demande *valeur, int x, int y,
				 unsigned int larg_texte, unsigned int larg_editable)
{
  unsigned int proprietes = ARB_OPTIONS_EDIT_INFINI; /* Quel que soit le type, aucune raison de limiter la longueur ! */
  unsigned int objets_crees = 0, objet_editable;

  /* On ne contrôle pas le numéro de l'arbre : supposé déjà fait auparavant ! */

  /* On définit les propriétés de l'objet */
  if ( 0 == ( valeur->proprietes & BI_EDITABLE_TEXTE ) ) {
    /* Ce n'est pas un texte : ce sont donc des nombres ! */
    proprietes |= ARB_OPTIONS_EDIT_NOMBRE;

    if ( valeur->proprietes & BI_EDITABLE_ENTIER ) {
      proprietes |= ARB_OPTIONS_EDIT_ENTIER;
    }
  }

  /* S'il y a un texte, cela fait forcément un objet créé */
  if ( valeur->texte != NULL ) { 
    objets_crees++;
    objet_editable = num_objet_1 + 1; /* Le texte est d'ailleurs le premier objet créé ! */
  } else {
    objet_editable = num_objet_1;
  }

  /* On réalise l'objet en fonction de son type */
  switch ( valeur->type )
  {
  case BI_CHAMP_DEMANDE_ENTIER_U:
    ARB_CreerEditable( numero_arbre, num_objet_1, x, y, larg_texte, larg_editable, proprietes, valeur->texte );

    OBJ_OptionsEditable( numero_arbre, objet_editable, valeur->proprietes | BI_EDITABLE_ENTIER | BI_EDITABLE_MIN, 0.0, -1.0, valeur->unite );
    OBJ_MettreEntier_U( BI_ArbreAlerte, objet_editable, valeur->valeur_initiale.entier_U );

    objets_crees++;
    break;

  case BI_CHAMP_DEMANDE_ENTIER_S:
    ARB_CreerEditable( numero_arbre, num_objet_1, x, y, larg_texte, larg_editable, proprietes, valeur->texte );

    OBJ_OptionsEditable( numero_arbre, objet_editable, valeur->proprietes | BI_EDITABLE_ENTIER, 0.0, -1.0, valeur->unite );
    OBJ_MettreEntier_S( numero_arbre, objet_editable, valeur->valeur_initiale.entier_S );

    objets_crees++;
    break;

  case BI_CHAMP_DEMANDE_REEL:
    ARB_CreerEditable( numero_arbre, num_objet_1, x, y, larg_texte, larg_editable, proprietes, valeur->texte );

    OBJ_OptionsEditable( numero_arbre, objet_editable, valeur->proprietes, valeur->min, valeur->max, valeur->unite );
    OBJ_MettreNombre( numero_arbre, objet_editable, valeur->valeur_initiale.reel, valeur->erreur );

    objets_crees++;
    break;

  case BI_CHAMP_DEMANDE_COMPLEXE:
    OPTION_A_FAIRE( "Demande d'une valeur complexe" );

    objets_crees += 3;
    break;

  case BI_CHAMP_DEMANDE_CHAINE:
    ARB_CreerEditable( numero_arbre, num_objet_1, x, y, larg_texte, larg_editable, ARB_OPTIONS_EDIT_INFINI, valeur->texte );

    OBJ_CopierTexte( numero_arbre, objet_editable, valeur->valeur_initiale.chaine, 0 );

    objets_crees++;
    break;

  case BI_CHAMP_DEMANDE_LISTE:
    if ( NULL == valeur->texte ) {
      OPTION_A_FAIRE( "Choix dans une liste de valeurs, sans texte" );
    } else {
      ARB_CreerTexte( numero_arbre, num_objet_1    , x, y, larg_texte, valeur->texte );
      ARB_CreerObjet( numero_arbre, num_objet_1 + 1, G_SSMENU, x + larg_texte + 1, y, larg_editable, 1 );
    }

    OBJ_MettreMenu( numero_arbre, objet_editable, valeur->nombre_options, valeur->options, NULL, NULL );

    objets_crees++;
    break;

  case BI_CHAMP_DEMANDE_BINAIRE:
    OPTION_A_FAIRE( "Choix dans une alternative OUI/NON" );

    objets_crees++;
    break;
  }

  /* Combien d'objets a-t-on créé ? */
  return objets_crees;
}

/* -------- Lire le contenu d'un champ de demande de valeur
   Renvoie 0 si tout va bien
           <0 en cas d'erreur grave
	   >0 si une contrainte n'est pas respectée [> max, < min,...]

	   dans valeur, la valeur entrée par l'utilisateur [acceptable ou non !]
           dans nbr_objets, le nombre d'objets constituant le champ de demande de valeur
 */

static int BI_LireChampEditable(unsigned int numero_arbre, unsigned int objet_editable, struct infos_demande *valeur, unsigned int *nbr_objets)
{
  int reponse = 0;
  unsigned int objets = 0;

  /* On ne contrôle ni l'arbre, ni le numéro d'objet : supposé fait avant ; contrôlé après de toute façon... */
  if ( valeur->texte != NULL ) {
    /* S'il y a un texte, il est dans le premier objet ! */
    objet_editable++;
    objets = 1;
  }

  switch ( valeur->type )
  {
  case BI_CHAMP_DEMANDE_ENTIER_U:
    valeur->valeur_donnee.entier_U = OBJ_LireEntier_U( numero_arbre, objet_editable );

    /* Contrôle de validité */
    if ( valeur->proprietes & BI_EDITABLE_MIN ) {
      if ( valeur->valeur_donnee.entier_U < valeur->min ) {
	AVERTISSEMENT( "Valeur hors intervalle : %lu [min = %g]", valeur->valeur_donnee.entier_U, valeur->min );
	reponse = +1;
      }
    }
    if ( valeur->proprietes & BI_EDITABLE_MAX ) {
      if ( valeur->valeur_donnee.entier_U > valeur->max ) {
	AVERTISSEMENT( "Valeur hors intervalle : %lu [max = %g]", valeur->valeur_donnee.entier_U, valeur->max );
	reponse = +2;
      }
    }

    objets++;
    break;

  case BI_CHAMP_DEMANDE_ENTIER_S:
    valeur->valeur_donnee.entier_S = OBJ_LireEntier_S( numero_arbre, objet_editable );

    /* Contrôle de validité */
    if ( valeur->proprietes & BI_EDITABLE_MIN ) {
      if ( valeur->valeur_donnee.entier_S < valeur->min ) {
	AVERTISSEMENT( "Valeur hors intervalle : %ld [min = %g]", valeur->valeur_donnee.entier_S, valeur->min );
	reponse = +1;
      }
    }
    if ( valeur->proprietes & BI_EDITABLE_MAX ) {
      if ( valeur->valeur_donnee.entier_S > valeur->max ) {
	AVERTISSEMENT( "Valeur hors intervalle : %ld [max = %g]", valeur->valeur_donnee.entier_S, valeur->max );
	reponse = +2;
      }
    }

    objets++;
    break;

  case BI_CHAMP_DEMANDE_REEL:
    valeur->valeur_donnee.reel = OBJ_LireNombre( numero_arbre, objet_editable, &( valeur->erreur ) );

    /* Contrôle de validité */
    if ( valeur->proprietes & BI_EDITABLE_MIN ) {
      if ( valeur->valeur_donnee.reel < valeur->min ) {
	AVERTISSEMENT( "Valeur hors intervalle : %g [min = %g]", valeur->valeur_donnee.reel, valeur->min );
	reponse = +1;
      }
    }
    if ( valeur->proprietes & BI_EDITABLE_MAX ) {
      if ( valeur->valeur_donnee.reel > valeur->max ) {
	AVERTISSEMENT( "Valeur hors intervalle : %g [max = %g]", valeur->valeur_donnee.reel, valeur->max );
	reponse = +2;
      }
    }

    objets++;
    break;

  case BI_CHAMP_DEMANDE_COMPLEXE:
    OPTION_A_FAIRE( "Demande d'une valeur complexe" );
    objets += 3;
    break;

  case BI_CHAMP_DEMANDE_CHAINE:
    /* ATTENTION, il faut copier ce texte car il sera libéré avec l'arbre !
       Il faut donc le libérer avec free() après utilisation ! */
    if ( valeur->valeur_donnee.chaine != NULL ) {
      /* Peu probable, mais peut arriver si appels successifs sur le même objet -> on libère l'ancienne ! */
      free( valeur->valeur_donnee.chaine ); valeur->valeur_donnee.chaine = NULL;
    }
    valeur->valeur_donnee.chaine = lit_chaine( numero_arbre, objet_editable, 0, NULL, TXT_COPIER ) ;
    objets++;
    break;

  case BI_CHAMP_DEMANDE_LISTE:
    valeur->valeur_donnee.numero_option = OBJ_OptionMenu( numero_arbre, objet_editable, 0, QUESTION );
    objets++;
    break;

  case BI_CHAMP_DEMANDE_BINAIRE:
    OPTION_A_FAIRE( "Choix dans une alternative OUI/NON" );

    objets++;
    break;
  }

  if ( nbr_objets != NULL ) *nbr_objets = objets;

  return reponse;
}

/* -------- Construction des boutons pour sortir -------- */

static int BI_creer_boutons(int num_arbre, unsigned int haut_boite, unsigned int nbr_boutons, unsigned int premier_bouton,
			    unsigned int bouton_ok, const char * const textes[])
{
  int reponse;
  unsigned int i;
  OBJECT *objets;

  objets = arbres[ num_arbre ].adresse;
  if (NULL == objets) {
    (void) fprintf(stderr, "BI_creer_boutons : table des objets absente [arbre %d]\n", num_arbre);
    return -1;
  }

  for (i = 0 ; i < nbr_boutons ; i++) {
    unsigned int objet;

    objet = premier_bouton + i;
    reponse = ARB_CreerObjet(num_arbre, objet, G_BUTTON, 0, (int) haut_boite - 1, 0, 1);
    if (reponse != 0) {
      (void) fprintf(stderr, "BI_creer_boutons : erreur dans ARB_CreerObjet() [bouton %u]\n", objet);
      return -2;
    }
    objets[ objet ].ob_flags |= EXIT;
    if (i == bouton_ok)
      objets[ objet ].ob_flags |= DEFAULT;
    objets[ objet ].ob_y = haut_boite - 2;
    objets[ objet ].ob_spec.texte = textes[ i ];
  }
  if (nbr_boutons == 1) {
    objets[premier_bouton].ob_x = 11;
    objets[premier_bouton].ob_width = 30;
  } else if (nbr_boutons == 2) {
    objets[premier_bouton].ob_x = 4;
    objets[premier_bouton].ob_width = 20;
    objets[premier_bouton + 1].ob_x = 28;
    objets[premier_bouton + 1].ob_width = 20;
  } else if (nbr_boutons == 3) {
    objets[premier_bouton].ob_x = 4;
    objets[premier_bouton].ob_width = 12;
    objets[premier_bouton + 1].ob_x = 19;
    objets[premier_bouton + 1].ob_width = 14;
    objets[premier_bouton + 2].ob_x = 36;
    objets[premier_bouton + 2].ob_width = 12;
  }

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Construction du texte d'informations -------- */

static int BI_generer_texte(int num_arbre, char **lignes, unsigned int nbr_lignes, unsigned int premier_objet)
{
  int reponse;
  unsigned int i;
  OBJECT *objets;

  objets = arbres[ num_arbre ].adresse;
  if (NULL == objets) {
    (void) fprintf(stderr, "BI_generer_texte : table des objets absente [arbre %d]\n", num_arbre);
    return -1;
  }

  /* Préparation des objets */
  reponse = ARB_CreerNTextes(num_arbre, premier_objet, nbr_lignes, ROOT, 1, 1, 50);
  if (reponse != 0) {
    (void) fprintf(stderr, "BI_generer_texte : erreur dans ARB_CreerNTextes() [arbre %d]\n", num_arbre);
    return -2;
  }

  /* Mise en place des textes */
  for (i = 0 ; i < nbr_lignes ; i++) {
    objets[premier_objet + i].ob_spec.texte = lignes[i];
  }

  /* Tout s'est bien passé */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Les fonctions de demande "complètes" ~-~-~-~-~-~-~-~- */

/* -------- Demander un texte -------- */

char *BI_Demande_Texte(Fenetre *mere, const BI_Boite_Demande *contenu, const char *texte_initial, ...)
{
  char *texte_traite, *resultat, **lignes;
  unsigned int haut_boite, nbr_objets, obj_editable;
  unsigned long nbr_lignes;
  int reponse, retour;
  va_list parametres;
  OBJECT *objets __attribute__((__unused__));

  /* On contrôle que la boîte est convenable & peut être ouverte */
  retour = BI_ControlerBoiteDemande( contenu );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Incohérence de la boîte de demande [code %d ; texte : %s]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, contenu->texte );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* Préparation du texte */
  va_start(parametres, texte_initial);
  texte_traite = TXT_vGenereChaine(contenu->texte, parametres);
  va_end(parametres);
  if ( NULL == texte_traite ) {
    (void) fprintf(stderr, "BI_Demande_Texte : impossible de préparer le texte !\n");
    return NULL;
  }

  lignes = TXT_GenereLignes(texte_traite, 50, '|', &nbr_lignes);
  if (NULL == lignes) {
    (void) fprintf(stderr, "BI_Demande_Texte : impossible de générer les lignes du texte !\n");
    (void) fprintf(stderr, "   Message : %s\n", texte_traite);
    free(texte_traite);
    return NULL;
  }

  nbr_objets = nbr_lignes + 1 + contenu->nbr_boutons + 1; /* Lignes, fond, boutons, champ d'entrée */
  haut_boite = nbr_lignes + 6; /* Une par ligne + 1 avant, 1 pour la demande, 1 après, 1 entre, 1 boutons, 1 en bas */

  reponse = BI_CreerArbre( BI_ArbreAlerte, nbr_objets, 52, haut_boite );
  if (reponse != 0) {
    (void) fprintf(stderr, "BI_Demande_Texte : erreur dans BI_creer_arbre() [code %d]\n", reponse);
    (void) fprintf(stderr, "   Message : %s\n", texte_traite);
    free(texte_traite);
    free(lignes);
    return NULL;
  }
  arbres[ BI_ArbreAlerte ].drapeaux.interne = VRAI; /* On ne libère pas les chaines en meme temps que l'arbre */
  objets = arbres[ BI_ArbreAlerte ].adresse;

  /* Les boutons */
  BI_creer_boutons( BI_ArbreAlerte, haut_boite, contenu->nbr_boutons, 1, contenu->bouton_ok - 1, contenu->boutons );
  arbres[ BI_ArbreAlerte ].objet_annuler = contenu->bouton_annuler;

  /* Le champ éditable */
  obj_editable = contenu->nbr_boutons + 1;
  ARB_CreerObjet( BI_ArbreAlerte, obj_editable, G_TXT_INFINI, 1, (int) (haut_boite - 4), 50, 1 );

  /* Les lignes de texte */
  (void) BI_generer_texte( BI_ArbreAlerte, lignes, nbr_lignes, obj_editable + 1 );

  /* Appel */
  ARB_Initialiser( BI_ArbreAlerte, NULL, ( int (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );

#ifdef _DEVELOPPE_
  ARB_AfficherStructure( BI_ArbreAlerte );
#endif

  do {
    met_chaine( BI_ArbreAlerte, (int) obj_editable, 0, texte_initial, TXT_COPIER );

    reponse = ARB_Bloquant( BI_ArbreAlerte, contenu->titre, mere, BI_ArbreAlerte );
    if ( ( 2 == reponse ) && ( 3 == contenu->nbr_boutons ) ) { /* On remet le texte par défaut */
      reponse = 5;
      continue;
    }
  } while ( reponse > 3 );

  /* Libération de l'arbre et de ses objets */
  free( texte_traite ); texte_traite = NULL;
  free( lignes ); lignes = NULL;

  /* On récupère le résultat */
  if ( 1 == reponse ) {
    resultat = strdup( lit_chaine( BI_ArbreAlerte, (int) obj_editable, 0, NULL, TXT_COPIER ) );
    if ( NULL == resultat ) {
      (void) fprintf(stderr, "BI_Demande_Texte : mémoire insuffisante pour récupérer le texte indiqué...\n");
    }
  } else {
    resultat = NULL;
  }

  /* On termine : plus besoin de tout ça... */
  (void) BI_LibererArbre( BI_ArbreAlerte );

  return resultat;
}

/* ~-~-~-~-~-~-~-~- Demande d'une seule valeur ~-~-~-~-~-~-~-~- */

/* -------- Valeur numérique --------
   Renvoie <0 en cas d'erreur, le numéro du bouton choisi sinon
   Attention, dans tous les cas les valeurs sont lues. À l'appelant de gérer le Annuler éventuel comme il faut !
 */

int BI_DemandeValeur(Fenetre *mere, const BI_Boite_Demande *contenu, struct infos_demande *valeur,...)
{
  char *texte_traite = NULL, **lignes = NULL;
  char *texte_min = NULL, *texte_max = NULL;
  int reponse, retour;
  int y_editable = 1, y_min = 0, y_max = 0;
  unsigned int larg_boite, larg_texte, haut_boite;
  unsigned int num_editable;
  unsigned int nbr_objets;
  unsigned long nbr_lignes;

  /* On contrôle que la boîte est convenable & peut être ouverte */
  retour = BI_ControlerBoiteDemande( contenu );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Incohérence de la boîte de demande [code %d ; texte : %s]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, contenu->texte );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -20;
  }

  /* Autres contrôles */
  if ( NULL == valeur ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR, pas de boîte ou d'infos de valeur\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* Taille de la boîte */
  haut_boite =  5; /* Avant, valeur, après, boutons, après */
  larg_boite = 22; /* 20 pour l'éditable, 1 de chaque côté */
  if ( valeur->texte != NULL ) {
    larg_texte = strlen( valeur->texte );
    if ( larg_texte > 40 ) larg_texte = 40;

    larg_boite += 1 + larg_texte;
  } else {
    larg_texte = 0;
  }
  if ( larg_boite < 52 ) larg_boite = 52; /* 52 caractères minimum (donc 50 caractères pour la chaîne) */

  /* Combien d'objets faudra-t-il ? */
  nbr_objets = 2 + contenu->nbr_boutons; /* Racine, boutons, éditable */
  num_editable = contenu->nbr_boutons + 1; /* L'éditable suit les boutons */

  /* Objets facultatifs */
  if ( valeur->proprietes & BI_EDITABLE_MIN ) {
    nbr_objets++; /* Une ligne en plus : valeur minimale admise */
    haut_boite++;
    y_min = 1;
    y_editable++;
    num_editable++;
  }
  if ( valeur->proprietes & BI_EDITABLE_MAX ) {
    nbr_objets++; /* Une ligne en plus : valeur maximale admise */
    haut_boite++;
    y_editable++;
    y_max = y_min + 1; /* Donc 1 si pas de min, 2 sinon */
    num_editable++;
  }
  /* S'il y a un texte avant, on le prépare */
  if ( NULL != valeur->texte ) {
    nbr_objets++;
    num_editable++; /* num_editable contient _vraiment_ le numéro de l'objet éditable */
  }
  /* S'il y a un texte introductif, on le prépare */
  if ( NULL != contenu->texte ) {
    va_list parametres;

    va_start( parametres, valeur );
    texte_traite = TXT_vGenereChaine( contenu->texte, parametres );
    va_end( parametres );

    if ( NULL == texte_traite ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de préparer le texte de la boîte [%s]. Travail sans texte...\n",
		     __FILE__, __FUNCTION__, __LINE__, contenu->texte );
    } else {
      unsigned int larg_ligne;

      larg_ligne = larg_boite - 2;

      lignes = TXT_GenereLignes( texte_traite, larg_ligne, '|', &nbr_lignes );
      if (NULL == lignes) {
	(void) fprintf(stderr, "%s, %s l.%u : Impossible de découper le texte en lignes\n   Message : %s\n",
		       __FILE__, __FUNCTION__, __LINE__, texte_traite);
	free(texte_traite); texte_traite = NULL;
      } else {
	y_editable += nbr_lignes + 1;
	haut_boite += nbr_lignes + 1; /* Séparateur */
	nbr_objets += nbr_lignes + 1;
	num_editable += nbr_lignes + 1;
	y_min += nbr_lignes + 1;
	y_max += nbr_lignes + 1;
      }
    }
  }

  /* On crée l'arbre correspondant */
  reponse = BI_CreerArbre( BI_ArbreAlerte, nbr_objets, larg_boite, haut_boite );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans BI_creer_arbre() [code %d]\n   Message : %s\n",
		   __FILE__, __FUNCTION__, __LINE__, reponse, texte_traite);
    free( texte_traite ); texte_traite = NULL;
    free( lignes ); lignes = NULL;
    return -25;
  }
#ifdef _DEVELOPPE_
/*   printf("%s : numéro de l'éditable = %u\n", __FUNCTION__, num_editable); */
#endif

  /* L'arbre est interne, créé au vol à partir de textes statiques [ou en tout cas, à ne pas libérer] */
  arbres[ BI_ArbreAlerte ].drapeaux.interne = VRAI;

  /* Les boutons (objets 1 à n) ==> permet identité avec la description dans la structure */
  BI_creer_boutons( BI_ArbreAlerte, haut_boite, contenu->nbr_boutons, 1, contenu->bouton_ok - 1, contenu->boutons);
  arbres[ BI_ArbreAlerte ].objet_annuler = contenu->bouton_annuler;

  /* Les lignes de texte */
  if ( lignes != NULL ) {
    /* Le séparateur */
    ARB_CreerSeparateur( BI_ArbreAlerte, contenu->nbr_boutons + 1, 1, nbr_lignes + 1, larg_boite - 2, ASC_HORIZONTAL,
			 " Valeur... " );

    /* Le texte */
    (void) BI_generer_texte( BI_ArbreAlerte, lignes, nbr_lignes, contenu->nbr_boutons + 2 );
  }
  if ( valeur->proprietes & BI_EDITABLE_MIN ) {
    unsigned int obj_min;

    /* Quel est l'objet concerné ? */
    obj_min = num_editable - 1;
    if ( valeur->texte != NULL ) obj_min--;
    if ( valeur->proprietes & BI_EDITABLE_MAX ) obj_min--;

    /* Créer le texte */
    if ( valeur->proprietes & BI_EDITABLE_ENTIER )
      texte_min = TXT_genere_chaine( "Valeur minimale : %u", (unsigned int) (valeur->min) );
    else {
      if ( valeur->unite != NULL )
	texte_min = TXT_genere_chaine( "Valeur minimale : %f %s", valeur->min, valeur->unite );
      else
	texte_min = TXT_genere_chaine( "Valeur minimale : %f", valeur->min );
    }

    /* Créer l'objet */
    (void) ARB_CreerTexte( BI_ArbreAlerte, obj_min, 1, y_min, larg_boite - 2, texte_min );
  }
  if ( valeur->proprietes & BI_EDITABLE_MAX ) {
    unsigned int obj_max;

    /* Quel est l'objet concerné ? */
    obj_max = num_editable - 1;
    if ( valeur->texte != NULL ) obj_max--;

    /* Créer le texte */
    if ( valeur->proprietes & BI_EDITABLE_ENTIER )
      texte_max = TXT_genere_chaine( "Valeur maximale : %u", (unsigned int) (valeur->max) );
    else {
      if ( valeur->unite != NULL )
	texte_max = TXT_genere_chaine( "Valeur maximale : %f %s", valeur->max, valeur->unite );
      else
	texte_max = TXT_genere_chaine( "Valeur maximale : %f", valeur->max );
    }

    /* Créer l'objet */
    (void) ARB_CreerTexte( BI_ArbreAlerte, obj_max, 1, y_max, larg_boite - 2, texte_max );
  }

  /* L'éditable */
  (void) BI_CreerChampEditable( BI_ArbreAlerte, num_editable - 1, valeur, 1, y_editable, larg_texte, ( larg_boite - (3 + larg_texte) ) );
  if ( valeur->texte != NULL ) {
    (void) BI_CreerChampEditable( BI_ArbreAlerte, num_editable - 1, valeur, 1, y_editable, larg_texte, ( larg_boite - (3 + larg_texte) ) );
  } else {
    (void) BI_CreerChampEditable( BI_ArbreAlerte, num_editable, valeur, 1, y_editable, larg_texte, ( larg_boite - (3 + larg_texte) ) );
  }

#ifdef _DEVELOPPE_
/*   { */
/*     unsigned int i; */
/*     printf("\n%s : CARACTÉRISTIQUES DE L'ARBRE\n", __FUNCTION__ ); */
/*     for ( i = 0 ; i < nbr_objets ; i++ ) { */
/*       printf(" --- Objet %u ---\n", i); */
/*       OBJ_AfficherContenu( arbres[ BI_ArbreAlerte ].adresse + i ); */
/*     } */
/*   } */
#endif

  /* Arbre maintenant créé... Appel */
  ARB_Initialiser( BI_ArbreAlerte, NULL, ( int (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );

  do {
    reponse = ARB_Bloquant( BI_ArbreAlerte, contenu->titre, mere, BI_ArbreAlerte );
    if ( reponse == (int) contenu->bouton_reinit ) { /* On remet le texte par défaut */
      OBJ_MettreNombre( BI_ArbreAlerte, num_editable, valeur->valeur_defaut.reel, 0.0 );
      reponse = 1974;
      continue;
    }
    if ( reponse == (int) contenu->bouton_annuler ) {
      valeur->valeur_donnee = valeur->valeur_initiale;
      break; /* On sort et la valeur n'a pas été modifiée ! */
    }

    /* Si l'on est là : on a cliqué sur le bouton pour OK */

    /* On lit la valeur pour l'objet éditable */
    if ( valeur->texte ) {
      reponse = BI_LireChampEditable( BI_ArbreAlerte, num_editable - 1, valeur, NULL );
    } else {
      reponse = BI_LireChampEditable( BI_ArbreAlerte, num_editable, valeur, NULL );
    }

    /* On contrôle qu'elle est correcte */
    if ( reponse != 0 ) {
      reponse += 874;
    }
  } while ( reponse > 3 );

  if ( lignes != NULL ) {
    /* Attention, le texte du séparateur ne doit pas être libéré : chaîne constante */
    arbres[ BI_ArbreAlerte ].adresse[ contenu->nbr_boutons + 1 ].ob_spec.txt_malloc = NULL;
    free(lignes); lignes = NULL;
  }

  /* Libération de l'arbre et de ses objets */
  free( texte_max ); texte_max = NULL;
  free( texte_min ); texte_min = NULL;
  (void) BI_LibererArbre( BI_ArbreAlerte );

  /* On a fini */
  free( texte_traite ); texte_traite = NULL;

  /* Tout s'est bien passé */
  if ( 0 == reponse ) {	/* Clic sur OK, lecture des valeurs sans souci */
    reponse = contenu->bouton_ok;
  }
  return reponse;
}

/* ~-~-~-~-~-~-~-~- Demande simultanée de plusieurs valeurs ~-~-~-~-~-~-~-~- */

/* -------- Plusieurs valeurs flottantes --------
   Renvoie : <0 en cas d'erreur
             numéro du bouton sinon

   Attention, dans tous les cas les valeurs sont lues. À l'appelant de gérer le Annuler éventuel comme il faut !
 */

int BI_DemandeMultiple(Fenetre *mere, const BI_Boite_Demande *contenu,
		       unsigned int nbr_valeurs, struct infos_demande *valeurs, ...)
{
  char *texte_traite = NULL, **lignes = NULL;
  int reponse, retour;
  int y_valeurs = 1;
  unsigned int larg_boite, larg_texte, haut_boite;
  unsigned int premier_editable, obj_ascenseur;
  unsigned int i, nbr_objets, objet, tmp;
  unsigned long nbr_lignes;

  /* Contrôles préliminaires */
  retour = BI_ControlerBoiteDemande( contenu );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Incohérence de la boîte de demande [code %d ; texte : %s]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, contenu->texte );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -25;
  }

#ifdef _DEVELOPPE_
  if ( ( NULL == valeurs ) || ( 0.0 == nbr_valeurs ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune valeur éditable demandée !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -3;
  }
#endif

  /* Combien d'objets faudra-t-il ? [on en profite pour savoir quelle largeur occupent les textes avant */
  nbr_objets = 1 + contenu->nbr_boutons; /* Racine, boutons */
  larg_texte = 0;
  for ( i = 0 ; i < nbr_valeurs ; i++ ) {
    switch ( valeurs[ i ].type )
    {
    case BI_CHAMP_DEMANDE_ENTIER_U:
    case BI_CHAMP_DEMANDE_ENTIER_S:
    case BI_CHAMP_DEMANDE_REEL:
    case BI_CHAMP_DEMANDE_CHAINE:
    case BI_CHAMP_DEMANDE_LISTE:
      if ( NULL == valeurs[ i ].texte ) {
	nbr_objets++;		/* Le champ éditable ou le sous-menu */
      } else {
	nbr_objets += 2;	/* Texte avant + champ éditable/sous-menu */

	tmp = strlen( valeurs[ i ].texte );
	if ( tmp > larg_texte )
	  larg_texte = tmp;
      }
      break;

    case BI_CHAMP_DEMANDE_COMPLEXE:
      if ( NULL == valeurs[ i ].texte ) {
	nbr_objets += 3;	/* Les 2 champs éditables et le texte " + i*" */
      } else {
	nbr_objets += 4;	/* Texte avant + 2 éditables + " + i*" */

	tmp = strlen( valeurs[ i ].texte );
	if ( tmp > larg_texte )
	  larg_texte = tmp;
      }
      break;

    default:
      AVERTISSEMENT( "Type de champ inconnu -- ignoré [%d]", valeurs[ i ].type );
      break;
    }
  }
  premier_editable = contenu->nbr_boutons + 1;

  /* Taille de la boîte */
  haut_boite = nbr_valeurs + 4; /* Une ligne avant et après chaque valeur éditable, + une par valeur, + 1 boutons, + 1 après */
  if ( larg_texte > 40 ) larg_texte = 40; /* 40 caractères au plus pour les textes */

  larg_boite = 1 + larg_texte + 1 + 20 + 1; /* 20 au moins pour les textes éditables */

  /* S'il y a un texte, on le prépare */
  if ( NULL != contenu->texte ) {
    va_list parametres;

    va_start( parametres, valeurs );
    texte_traite = TXT_vGenereChaine( contenu->texte, parametres );
    va_end( parametres );

    if ( NULL == texte_traite ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de préparer le texte de la boîte [%s]. Travail sans texte...\n",
		     __FILE__, __FUNCTION__, __LINE__, contenu->texte );
    } else {
      unsigned int larg_ligne;

      larg_ligne = larg_boite - 2;
      if ( larg_ligne < 50 ) {
	larg_ligne = 50;
	larg_boite = 52;
	if ( larg_texte > 29 ) /* Peut arriver pour un texte entre 29 et 40 */
	  larg_texte = 29;
      }

      lignes = TXT_GenereLignes( texte_traite, larg_ligne, '|', &nbr_lignes );
      if (NULL == lignes) {
	(void) fprintf(stderr, "%s, %s l.%u : Impossible de découper le texte en lignes\n   Message : %s\n",
		       __FILE__, __FUNCTION__, __LINE__, texte_traite);
	free(texte_traite); texte_traite = NULL;
      } else {
	y_valeurs += nbr_lignes + 1;
	haut_boite += nbr_lignes + 1; /* Séparateur */
	nbr_objets += nbr_lignes + 1;
	premier_editable += nbr_lignes + 1;
      }
    }
  }

  /* Faut-il un ascenseur ? */
  if ( nbr_valeurs > 10 ) {
    nbr_objets += 4;
    larg_texte--; /* On rogne sur le texte ! */
    obj_ascenseur = nbr_objets - 5;
  } else {
    obj_ascenseur = ROOT;
  }

  /* On crée l'arbre correspondant */
  retour = BI_CreerArbre( BI_ArbreAlerte, nbr_objets, larg_boite, haut_boite );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans BI_creer_arbre() [code %d]\n   Message : %s\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, texte_traite);
    free( texte_traite ); texte_traite = NULL;
    free( lignes ); lignes = NULL;
    return -25;
  }

  /* L'arbre est interne, créé au vol à partir de textes statiques [ou en tout cas, à ne pas libérer] */
  arbres[ BI_ArbreAlerte ].drapeaux.interne = VRAI;

  /* Les boutons */
  BI_creer_boutons( BI_ArbreAlerte, haut_boite, contenu->nbr_boutons, 1, contenu->bouton_ok - 1, contenu->boutons);
  arbres[ BI_ArbreAlerte ].objet_annuler = contenu->bouton_annuler;

  /* Les lignes de texte */
  if ( lignes != NULL ) {
    /* Le séparateur */
    ARB_CreerSeparateur( BI_ArbreAlerte, contenu->nbr_boutons + 1, 1, nbr_lignes + 1, larg_boite - 2, ASC_HORIZONTAL,
			 " Valeurs... " );

    /* Le texte */
    (void) BI_generer_texte( BI_ArbreAlerte, lignes, nbr_lignes, contenu->nbr_boutons + 2 );
  }

  /* Les éditables */
  for ( i = 0, objet = premier_editable ; i < ( ( nbr_valeurs > 10 ) ? ( 10 ) : ( nbr_valeurs ) ) ; i++ ) {
    objet += BI_CreerChampEditable( BI_ArbreAlerte, objet, valeurs + i, 1, y_valeurs + i, larg_texte, ( larg_boite - (3 + larg_texte) ) );
  }

  /* L'ascenseur éventuel */
  if ( obj_ascenseur > ROOT ) {
    ARB_CreerAscenseur( BI_ArbreAlerte, obj_ascenseur, larg_boite - 2, y_valeurs, ASC_VERTICAL, 10 );
  }

  /* Arbre maintenant créé... Appel */
  ARB_Initialiser( BI_ArbreAlerte, NULL, ( int (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );

  do {
    reponse = ARB_Bloquant( BI_ArbreAlerte, contenu->titre, mere, BI_ArbreAlerte );
    if ( reponse == (int) contenu->bouton_reinit ) { /* On remet le texte par défaut */
      for ( i = 0, objet = premier_editable ; i < nbr_valeurs ; i++, objet += 2 ) {
	OBJ_MettreNombre( BI_ArbreAlerte, objet + 1, valeurs[ i ].valeur_initiale.reel, valeurs[ i ].erreur );
      }
      reponse = 5;
    }
  } while ( reponse > 3 );

  /* On lit les valeurs pour chaque objet éditable (si OK) */
  for ( i = 0, objet = premier_editable ; i < nbr_valeurs ; i++ ) {
    unsigned int nbr_objets = 0;

    retour = BI_LireChampEditable( BI_ArbreAlerte, objet, valeurs + i, &nbr_objets );
    objet += nbr_objets;

    if ( retour != 0 ) {
      AVERTISSEMENT( "Problème en lisant la valeur %u : reponse = %d", i, retour );
    }
  }

  /* Attention, texte du séparateur statique -> oter le lien pour éviter une libération abusive */
  arbres[ BI_ArbreAlerte ].adresse[ contenu->nbr_boutons + 1 ].ob_spec.texte = NULL;

  /* Libération de l'arbre et de ses objets */
  (void) BI_LibererArbre( BI_ArbreAlerte );

  /* On a fini */
  free( texte_traite ); texte_traite = NULL;
  free( lignes ); lignes = NULL;

  /* Tout s'est bien passé */
  return reponse;
}

/* ~-~-~-~-~-~-~-~- Les demandes de choses de base ~-~-~-~-~-~-~-~- */

/* -------- Demander le style de ligne, parmi ceux disponibles par défaut sur le système -------- */

short int BI_DemanderStyleLigne(unsigned int numero_arbre_pere, unsigned int numero_objet_pere)
{
  int i, retour;
  short int style_choisi;
  unsigned int largeur = 10;  	/* 10 caractères de large par défaut */

  /* Contrôle préliminaire : pas d'appel récursif */
  if ( arbres[ BI_ArbreSousMenu ].adresse != NULL ) {
    (void) fprintf(stderr, "%s,%s l.%u : il y a déjà un sous-menu ouvert !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* On adapte la largeur du sous-menu */
  if ( numero_objet_pere != ROOT ) { /* L'objet parent existe */
    largeur = (unsigned int) OB_WIDTH( arbres[ numero_arbre_pere ].adresse, numero_objet_pere );
    largeur /= BI_larg_car; /* Car arbre déjà converti en coordonnées écran ! */
  }

  /* On crée l'arbre contenant le menu */
  retour = BI_CreerArbre( BI_ArbreSousMenu, BI_NOMBRE_STYLES_LIGNES + 2, largeur, BI_NOMBRE_STYLES_LIGNES + 1 ); /* NPO ROOT comme objet ! */
  if (retour != 0) {
    (void) fprintf(stderr, "%s,%s l.%u : erreur dans BI_creer_arbre()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -4;
  }
  /* Arbre interne, textes statiques ! */
  arbres[ BI_ArbreSousMenu ].drapeaux.interne = VRAI;

  /* Premier style : spécial, texte "aucune ligne tracée" */
  ARB_CreerTexte( BI_ArbreSousMenu, 1, 0, 0, largeur, "Aucune ligne" );

  /* On crée ensuite un objet par style de ligne disponible [en commençant à 0 : pas de ligne -> texte associé] */
  for ( i = 1 ; i <= BI_NOMBRE_STYLES_LIGNES ; i++ ) {
    ARB_CreerObjet( BI_ArbreSousMenu, 1 + i, G_STYLELIGNE, 0, i, largeur, 1 );
    OBJ_MettreStyle( BI_ArbreSousMenu, 1 + i, i );
  }

  /* On appelle le sous-menu ainsi créé */
  retour = GER_ssmenu( BI_ArbreSousMenu, numero_arbre_pere, numero_objet_pere, 0 );
  if ( retour <= 0 ) { /* Problème ou annulation */
    style_choisi = -1;
  } else {
    style_choisi = retour - 1;

    if ( numero_objet_pere != ROOT ) { /* L'objet parent existe */
      OBJ_MettreStyle( numero_arbre_pere, numero_objet_pere, style_choisi );
    }
  }

  /* On a fini */
  BI_LibererArbre( BI_ArbreSousMenu );
  return style_choisi;
}

/* -------- Demander le style de marque, parmi ceux disponibles par défaut sur le système -------- */

short int BI_DemanderMarque(unsigned int numero_arbre_pere, unsigned int numero_objet_pere)
{
  int i, retour;
  short int marque_choisie;
  unsigned int largeur = 10;  	/* 10 caractères de large par défaut */

  /* Contrôle préliminaire : pas d'appel récursif */
  if ( arbres[ BI_ArbreSousMenu ].adresse != NULL ) {
    (void) fprintf(stderr, "%s,%s l.%u : il y a déjà un sous-menu ouvert !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* On adapte la largeur du sous-menu */
  if ( numero_objet_pere != ROOT ) { /* L'objet parent existe */
    largeur = (unsigned int) OB_WIDTH( arbres[ numero_arbre_pere ].adresse, numero_objet_pere );
    largeur /= BI_larg_car; /* Car arbre déjà converti en coordonnées écran ! */
  }

  /* On crée l'arbre contenant le menu */
  retour = BI_CreerArbre( BI_ArbreSousMenu, BI_NOMBRE_MARQUES + 2, largeur, BI_NOMBRE_MARQUES + 1 ); /* NPO ROOT comme objet ! */
  if (retour != 0) {
    (void) fprintf(stderr, "%s,%s l.%u : erreur dans BI_creer_arbre()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -4;
  }
  /* Arbre interne, textes statiques ! */
  arbres[ BI_ArbreSousMenu ].drapeaux.interne = VRAI;

  /* Premier style : spécial, texte "aucune ligne tracée" */
  ARB_CreerTexte( BI_ArbreSousMenu, 1, 0, 0, largeur, "Aucune marque" );

  /* On crée ensuite un objet par style de marque disponible [en commençant à 0 : pas de marque -> texte associé] */
  for ( i = 1 ; i <= BI_NOMBRE_MARQUES ; i++ ) {
    ARB_CreerObjet( BI_ArbreSousMenu, 1 + i, G_STYLEMARQUE, 0, i, largeur, 1 );
    OBJ_MettreStyle( BI_ArbreSousMenu, 1 + i, i );
  }

  /* On appelle le sous-menu ainsi créé */
  retour = GER_ssmenu( BI_ArbreSousMenu, numero_arbre_pere, numero_objet_pere, 0 );
  if ( retour <= 0 ) { /* Problème ou annulation */
    marque_choisie = -1;
  } else {
    marque_choisie = retour - 1;
  }

  /* On a fini */
  BI_LibererArbre( BI_ArbreSousMenu );
  return marque_choisie;
}
