/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des fentres : contenu [routines par dfaut et tout a...]
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_
# define _LIBRAIRIE_X_ /* Pour avoir accs aux fonctions internes */
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "bib_interface.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean ARB_EstDansFenetre(Arbre *arbre, Fenetre *fenetre);
static boolean FEN_EstFilleDe(Fenetre *mere, Fenetre *fenetre);

/*  ~-~-~-~-~-~-~-~- Lien avec d'autres objets ~-~-~-~-~-~-~-~- */

/* -------- Renvoie la fentre associe  un arbre ---------------------\
 |                                                                      |
 | arbre  : adresse de la structure associe  l'arbre nous intressant |
 | fermee : VRAI = on cherche aussi dans les fentres fermes...        |
 \--------------------------------------------------------------------- */

Fenetre *ARB_QuelleFenetre(Arbre *arbre, boolean fermee)
{
  Fenetre *fenetre;
  boolean est_dans_fenetre;

  ENTREE_FONCTION( __FUNCTION__, "arbre=%p, fermee=%d", (void *) arbre, fermee );

  if ( NULL == arbre ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : arbre = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    SORTIE_FONCTION( __FUNCTION__, "%p [arbre=NULL]", NULL );
    return NULL; /* Pas d'arbre associ */
  }
  if ( NULL == BI_premiere_fenetre ) {
    SORTIE_FONCTION( __FUNCTION__, "%p [BI_premiere_fenetre=NULL]", NULL );
    return NULL; /* Aucune fentre ouverte */
  }

  for ( fenetre = BI_premiere_fenetre; fenetre ; fenetre = fenetre->suivante ) {
    est_dans_fenetre = ARB_EstDansFenetre( arbre, fenetre );

    if ( VRAI == est_dans_fenetre ) { /* L'arbre est dans cette fentre */
      if ( FAUX == fenetre->drapeaux.ouverte ) { /* Fentre ferme */
	if ( FAUX == fermee ) {
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : Fentre trouve, mais ferme !\n",
			 __FILE__, __FUNCTION__, __LINE__);
#endif
	  SORTIE_FONCTION( __FUNCTION__, "%p [Trouve, mais ferme]", NULL );
	  return NULL;
	} else {
	  SORTIE_FONCTION( __FUNCTION__, "%p [Trouve, ferme]", fenetre );
	  return fenetre;
	}
      } else {
	SORTIE_FONCTION( __FUNCTION__, "%p [Trouve, ouverte]", fenetre );
	return fenetre;
      }      
    }
  }

  /* Aucune fentre associe */
  SORTIE_FONCTION( __FUNCTION__, "%p [Non trouve]", NULL );
  return NULL;
}

/* -------- Renvoie VRAI si l'arbre est dans la fentre indique -------- */

static boolean ARB_EstDansFenetre(Arbre *arbre, Fenetre *fenetre)
{
  /* Controle :fenetre != NULL (peut arriver si la boite d'alerte est affichee pendant la construction d'une fenetre multiple :
     elle n'est pas encore initialise compltement ==> certaines sous-fenetres sont  NULL) */
  if ( NULL == fenetre ) return FAUX;

  /* 1er test : fentre contient directement l'arbre */
  if ( arbre == fenetre->arbre ) {
    return VRAI;
  }

  /* 2e test : l'arbre pourrait-il tre dans une des sous-fentres ? */
  if ( fenetre->nbr_ss_fenetres > 0 ) {
    boolean est_ds_ss_fenetre;
    unsigned int i;

    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      est_ds_ss_fenetre = ARB_EstDansFenetre( arbre, fenetre->ss_fenetres[ i ] );
      if ( VRAI == est_ds_ss_fenetre ) {
	return VRAI;
      }
    }
  }

  /* Si on est l, l'arbre n'est pas dans la fentre indique */
  return FAUX;
}

/* -------- Indique quel est l'arbre contenu dans une fentre --------
   Renvoie -1 si aucun arbre n'est dans la fentre
           le numro de l'arbre sinon
 */

int FEN_QuelArbre(Fenetre *fenetre)
{
  if ( NULL == fenetre ) return -1;

  if ( BI_FENETRE_ARBRE == fenetre->type ) {
    return fenetre->numero_arbre;
  }

  if ( fenetre->ss_fenetres != NULL ) {
    unsigned int i;
    int n_arbre;

    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      n_arbre = FEN_QuelArbre( fenetre->ss_fenetres[ i ] );
      if ( n_arbre >= 0 ) {
	return n_arbre;
      }
    }
  }

  /* Si on est l : pas d'arbre dans la fentre ! */
  return -1;
}

/*  ~-~-~-~-~-~-~-~- Organisation des sous-fentres  l'intrieur ~-~-~-~-~-~-~-~- */

/* -------- Rechercher la fenetre de plus bas niveau -------- */

Fenetre *FEN_FenetreMere( Fenetre *fenetre )
{
  Fenetre *mere;

  if ( NULL == fenetre ) return NULL;

  for ( mere = BI_premiere_fenetre ; mere != NULL ; mere = mere->suivante ) {
    if ( mere == fenetre ) return mere;

    if ( VRAI == FEN_EstFilleDe( mere, fenetre ) )
      return mere;
  }

  (void) fprintf(stderr, "FEN_FenetreMere : impossible de trouver la mre de la fenetre\n");
  return NULL;
}

/* -------- Renvoie VRAI si la fenetre indique est la fille de l'autre fenetre -------- */

static boolean FEN_EstFilleDe(Fenetre *mere, Fenetre *fenetre)
{
  boolean retour;
  Fenetre **ss_fenetres;
  unsigned int i;

  if ( NULL == mere ) return FAUX;

  ss_fenetres = mere->ss_fenetres;
  if ( NULL == ss_fenetres ) {
    return FAUX;
  }

  for ( i = 0 ; i < mere->nbr_ss_fenetres ; i++ ) {
    if ( ss_fenetres[ i ] == fenetre )
      return VRAI;

    retour = FEN_EstFilleDe( ss_fenetres[ i ], fenetre );
    if ( VRAI == retour ) {
      return VRAI;
    }
  }

  /* Non trouve */
  return FAUX;
}

/* -------- Optimiser les dimensions des sous-fenetres -------- */

int FEN_OptimiserDimensionsInternes( Fenetre *fenetre )
{
  unsigned int lg_fenetre, ht_fenetre;
  unsigned int i = 0, nbr_fenetres, nbr_colonnes;
  boolean nombre_impair;
  Fenetre **ss_fenetres;

  if ( NULL == fenetre ) return -1;

  if ( fenetre->type != BI_FENETRE_MULTIPLE ) {
    (void) fprintf(stderr, "FEN_OptimiserDimensionsInternes : la fentre n'est pas une fentre multiple !\n");
    return -2;
  }
  if ( NULL == fenetre->ss_fenetres ) {
    (void) fprintf(stderr, "FEN_OptimiserDimensionsInternes : Pas de table des sous-fentres !\n");
    return -3;
  }
  ss_fenetres = fenetre->ss_fenetres;

  /* Cas spciaux : 0, 1, 2 fentres [donc une seule range] */
  if ( 0 == fenetre->nbr_ss_fenetres ) {
    (void) fprintf(stderr, "FEN_OptimiserDimensionsInternes : Aucune sous-fentre (mais table) !\n");
    return -4;
  }
  if ( 1 == fenetre->nbr_ss_fenetres ) { /* Elle occupe toute la place */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FEN_OptimiserDimensionsInternes : Une seule fentre [ou 0]\n");
#endif
    FEN_dimensions( ss_fenetres[ 0 ], fenetre->w_x, fenetre->w_y, fenetre->w_larg, fenetre->w_haut );
    return 0;
  }
  if ( 2 == fenetre->nbr_ss_fenetres ) { /* Scission en colonnes */
    ht_fenetre = fenetre->w_haut;
    lg_fenetre = fenetre->w_larg >> 1;

    lg_fenetre--; /* Pour avoir une ligne verticale grise au milieu */

    FEN_dimensions( ss_fenetres[ 0 ], fenetre->w_x, fenetre->w_y, lg_fenetre, ht_fenetre );
    FEN_dimensions( ss_fenetres[ 1 ], fenetre->w_x + lg_fenetre + 2, fenetre->w_y, lg_fenetre, ht_fenetre );
    return 0;
  }

  /* Plus de deux fentres : on agence comme on peut... Ranges en lignes */ 
  ht_fenetre = fenetre->w_haut >> 1;

  if ( fenetre->nbr_ss_fenetres & 1 ) { /* Nombre impair de fentres */
    nombre_impair = VRAI;
    nbr_fenetres = fenetre->nbr_ss_fenetres - 1;
  } else {
    nbr_fenetres = fenetre->nbr_ss_fenetres;
    nombre_impair = FAUX;
  }

  /* On calcule le nombre de colonnes par range */
  nbr_colonnes = nbr_fenetres >> 1;

  /* Largeur d'une colonne */
  if ( VRAI == nombre_impair )
    lg_fenetre = fenetre->w_larg / nbr_colonnes;
  else
    lg_fenetre = fenetre->w_larg / nbr_colonnes;
  if ( lg_fenetre < 10 ) {
    (void) fprintf(stderr, "FEN_OptimiserDimensionsInternes : Trop de fentres ou fentre trop troite\n");
    return -5;
  }

  /* On place les fentres */
  for ( i = 0 ; i < nbr_colonnes ; i++ ) {
    FEN_dimensions( ss_fenetres[ i ], fenetre->w_x + i * lg_fenetre, fenetre->w_y, lg_fenetre, ht_fenetre );
    FEN_dimensions( ss_fenetres[ nbr_colonnes + i ], fenetre->w_x + i * lg_fenetre, fenetre->w_y + ht_fenetre, lg_fenetre, ht_fenetre );
  }
  if ( VRAI == nombre_impair ) {
    /* La dernire occupe alors la hauteur totale, sur la colonne */
    FEN_dimensions( ss_fenetres[ nbr_fenetres ], fenetre->w_x + i * lg_fenetre, fenetre->w_y, lg_fenetre, 2 * ht_fenetre );
  }

  /* Initialisation russie */
  return 0;
}

/* -------- Adapter la sous-fentre contenant un arbre dans la fentre mre -------- */

int FEN_AdapterArbre(Fenetre *contenant, Fenetre *fenetre_arbre)
{
  OBJECT *racine;

  if ( ( NULL == contenant ) || ( NULL == fenetre_arbre ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Une des deux fentres vaut NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* On rcupre l'objet racine */
  racine = fenetre_arbre->arbre->adresse;
  if ( NULL == racine ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'arbre associ. Adaptation impossible !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  switch ( fenetre_arbre->position )
  {
  case BI_FEN_ARBRE_AUCUN:
  case BI_FEN_ARBRE_COIN_HG:
    fenetre_arbre->x    = contenant->w_x;
    fenetre_arbre->y    = contenant->w_y;
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  case BI_FEN_ARBRE_COIN_HD:
    fenetre_arbre->x    = contenant->w_x - OB_WIDTH( racine, ROOT );
    fenetre_arbre->y    = contenant->w_y;
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  case BI_FEN_ARBRE_COIN_BG:
    fenetre_arbre->x    = contenant->w_x;
    fenetre_arbre->y    = contenant->w_y - OB_HEIGHT( racine, ROOT );
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  case BI_FEN_ARBRE_COIN_BD:
    fenetre_arbre->x    = contenant->w_x - OB_WIDTH( racine, ROOT );
    fenetre_arbre->y    = contenant->w_y - OB_HEIGHT( racine, ROOT );
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  case BI_FEN_ARBRE_GAUCHE:	/* Arbre align  gauche : occupe toute la hauteur, largeur gale  celle de l'arbre */
    fenetre_arbre->x    = contenant->w_x;
    fenetre_arbre->y    = contenant->w_y;
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_DROITE:
    fenetre_arbre->x    = contenant->w_x + contenant->w_larg - OB_WIDTH( racine, ROOT );
    fenetre_arbre->y    = contenant->w_y;
    fenetre_arbre->larg = OB_WIDTH( racine, ROOT );
    fenetre_arbre->haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_HAUT:
    fenetre_arbre->x    = contenant->w_x;
    fenetre_arbre->y    = contenant->w_y;
    fenetre_arbre->larg = contenant->w_larg;
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  case BI_FEN_ARBRE_BAS:
    fenetre_arbre->x    = contenant->w_x;
    fenetre_arbre->y    = contenant->w_y + contenant->w_haut - OB_HEIGHT( racine, ROOT );
    fenetre_arbre->larg = contenant->w_larg;
    fenetre_arbre->haut = OB_HEIGHT( racine, ROOT );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Position inconnue [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre_arbre->position );
    return -5;
  }

  /* La zone de travail a les mmes dimensions que la fentre, forcment */
  fenetre_arbre->w_x = fenetre_arbre->x;
  fenetre_arbre->w_y = fenetre_arbre->y;
  fenetre_arbre->w_larg = fenetre_arbre->larg;
  fenetre_arbre->w_haut = fenetre_arbre->haut;

  /* On met la fentre aux nouvelles dimensions [pas forcment la fentre de l'objet racine, c'est bizarre...] */
  FEN_dimensions( fenetre_arbre, fenetre_arbre->x, fenetre_arbre->y, fenetre_arbre->larg, fenetre_arbre->haut );

  /* On met l'arbre aussi  ces dimensions */
  /* Attention, on le laisse en (0, 0) car il est toujours dans sa propre fentre ! */
  OBJ_taille  ( fenetre_arbre->numero_arbre, ROOT, fenetre_arbre->larg, fenetre_arbre->haut, NULL, NULL);

  /* On a termin sans encombre */
  return 0;
}

/* -------- Adapter la sous-fentre contenant la partie Open GL  --------
   Renvoie    0 si tout s'est bien pass
           != 0 en cas de problme

   contenant  : la fentre mre (dimensions globales)
   opengl     : la fentre contenant l'arbre [c'est par rapport  lui que se place la partie utilisateur !]
   proportion : la taille de la fentre OpenGL relativement au contenant [en pour milles]
 */

int FEN_AdapterOpenGL(Fenetre *contenant, Fenetre *opengl, unsigned int proportion)
{
  unsigned int dimension = 0;
  int nv_x, nv_y;
  unsigned int nv_larg, nv_haut;

  if ( ( NULL == contenant ) || ( NULL == opengl ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Une des trois fentres vaut NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* Par dfaut, on ne change pas de dimension */
  nv_x = opengl->x; nv_larg = opengl->larg;
  nv_y = opengl->y; nv_haut = opengl->haut;

  /* En fonction de la position de la fentre Open GL */
  switch ( opengl->position )
  {
  case BI_FEN_ARBRE_AUCUN:
  case BI_FEN_ARBRE_COIN_HG:
    OPTION_A_FAIRE( "Open GL en haut,  gauche" );
    break;

  case BI_FEN_ARBRE_COIN_HD:
    OPTION_A_FAIRE( "Open GL en haut,  droite" );
    break;

  case BI_FEN_ARBRE_COIN_BG:
    OPTION_A_FAIRE( "Open GL en bas,  gauche" );
    break;

  case BI_FEN_ARBRE_COIN_BD:
    OPTION_A_FAIRE( "Open GL en bas,  droite" );
    break;

  case BI_FEN_ARBRE_GAUCHE:	/* Open GL align  gauche : occupe toute la hauteur, largeur indique */
    dimension = (unsigned int) ( (float) contenant->w_larg * (float) proportion / 1000.0 );
    nv_x    = contenant->w_x;
    nv_y    = contenant->w_y;
    nv_larg = dimension;
    nv_haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_DROITE:	/* Open GL align  droite : occupe toute la hauteur, largeur indique */
    dimension = (unsigned int) ( (float) contenant->w_larg * (float) proportion / 1000.0 );
    printf("%s : ancienne largeur =%u, nouvelle = %u [contenant : %u]\n", __FUNCTION__, opengl->larg, dimension, contenant->w_larg );
    nv_x    = contenant->w_x + contenant->w_larg - dimension;
    nv_y    = contenant->w_y;
    nv_larg = dimension;
    nv_haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_HAUT:	/* Open GL align en haut */
    dimension = (unsigned int) ( (float) contenant->w_haut * (float) proportion / 1000.0 );
    nv_x    = contenant->w_x;
    nv_y    = contenant->w_y;
    nv_larg = contenant->w_larg;
    nv_haut = dimension;
    break;

  case BI_FEN_ARBRE_BAS:	/* Open GL align en bas */
    nv_x    = contenant->w_x + contenant->w_haut - dimension;
    nv_y    = contenant->w_y;
    nv_larg = contenant->w_larg;
    nv_haut = dimension;
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Position OpenGL inconnue [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, opengl->position );
    return -2;
  }

  FEN_dimensions( opengl, nv_x, nv_y, nv_larg, nv_haut );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Adapter la sous-fentre contenant la partie utilisateur  --------
   Renvoie    0 si tout s'est bien pass
           != 0 en cas de problme

   contenant           : la fentre mre (dimensions globales)
   fenetre_arbre       : la fentre contenant l'arbre [c'est par rapport  lui que se place la partie utilisateur !]
   fenetre_utilisateur : la fentre  placer
 */

int FEN_AdapterSousFenetre(Fenetre *contenant, Fenetre *fenetre_arbre, Fenetre *fenetre_utilisateur)
{
  int nv_x = 0, nv_y = 0;
  unsigned int nv_larg = 0, nv_haut = 0;

  if ( ( NULL == contenant ) || ( NULL == fenetre_arbre ) || ( NULL == fenetre_utilisateur ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Une des trois fentres vaut NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* En fonction de la position de l'arbre */
  switch ( fenetre_arbre->position )
  {
  case BI_FEN_ARBRE_AUCUN:
  case BI_FEN_ARBRE_COIN_HG:
    /* 3 cas possibles. Distinction : suivant position de la fentre indique */
    if ( BI_FEN_ARBRE_COIN_HD == fenetre_utilisateur->position ) {
      nv_x    = contenant->w_x + fenetre_arbre->larg;
      nv_y    = contenant->w_y;
      nv_larg = contenant->w_larg - fenetre_arbre->larg;
      nv_haut = fenetre_arbre->haut;
    } else if ( BI_FEN_ARBRE_COIN_BG == fenetre_utilisateur->position ) {
      nv_x    = contenant->w_x;
      nv_y    = contenant->w_y    + fenetre_arbre->haut;
      nv_larg = fenetre_arbre->larg;
      nv_haut = contenant->w_haut - fenetre_arbre->haut;
    } else if ( BI_FEN_ARBRE_DROITE == fenetre_utilisateur->position ) {
      nv_x    = contenant->w_x + fenetre_arbre->larg;
      nv_y    = contenant->w_y;
      nv_larg = contenant->w_larg - fenetre_arbre->larg;
      nv_haut = contenant->w_haut;
    } else {
      AVERTISSEMENT( "Combinaison inattendue ou non-prvue : arbre en %d, utilisateur en %d", fenetre_arbre->position, fenetre_utilisateur->position );
    }
    break;

  case BI_FEN_ARBRE_COIN_HD:
    /* 3 cas possibles. Distinction : suivant position de la fentre indique */
    break;

  case BI_FEN_ARBRE_COIN_BG:
    /* 3 cas possibles. Distinction : suivant position de la fentre indique */
    break;

  case BI_FEN_ARBRE_COIN_BD:
    /* 3 cas possibles. Distinction : suivant position de la fentre indique */
    break;

  case BI_FEN_ARBRE_GAUCHE:	/* Arbre align  gauche : occupe toute la hauteur, largeur gale  celle de l'arbre */
    nv_x    = contenant->w_x + fenetre_arbre->larg;
    nv_y    = contenant->w_y;
    nv_larg = contenant->w_larg - fenetre_arbre->larg;
    nv_haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_DROITE:
    nv_x    = contenant->w_x;
    nv_y    = contenant->w_y;
    nv_larg = contenant->w_larg - fenetre_arbre->larg;
    nv_haut = contenant->w_haut;
    break;

  case BI_FEN_ARBRE_HAUT:
    nv_x    = contenant->w_x;
    nv_y    = contenant->w_y + fenetre_arbre->haut;
    nv_larg = contenant->w_larg;
    nv_haut = contenant->w_haut - fenetre_arbre->haut;
/*     printf("%s : contenant [%p], H=%u ; arbre [%p], H=%u\n", */
/* 	   __FUNCTION__, (void *) contenant, contenant->w_haut, (void *) fenetre_arbre, fenetre_arbre->w_haut ); */
    break;

  case BI_FEN_ARBRE_BAS:
    nv_x    = contenant->w_x;
    nv_y    = contenant->w_y;
    nv_larg = contenant->w_larg;
    nv_haut = contenant->w_haut - fenetre_arbre->haut;
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Position d'arbre [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, contenant->position );
    return -2;
  }

  /* On applique le changement de dimensions */
  FEN_dimensions( fenetre_utilisateur, nv_x, nv_y, nv_larg, nv_haut );

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Gestion des vnements pour la fentre ~-~-~-~-~-~-~-~- */

/* -------- Associer des routines de gestion d'vnements  la fentre considre --------
 *  Seules les routines non-nulles sont modifies !
 */

void FEN_AssocierEvenements(Fenetre *fenetre,
			    void (*redessin) (Fenetre *, int, int, unsigned int, unsigned int),
			    void (*ascenseur)(Fenetre *, int, int, int),
			    int  (*clavier)  (Fenetre *, unsigned int, int),
			    int  (*clic)     (Fenetre *, int, int, int, int, int),
			    int  (*message)  (Fenetre *, Message *),
			    void (*rect)     (Fenetre *, int, int, int, int),
			    int  (*temps)    (Fenetre *) )
{
  if ( NULL == fenetre ) return;

  if ( redessin != ( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL ) {
    fenetre->redessin = redessin;
    /* Pour modifier rds_post, utiliser FEN_GL_evntsgl() */
    FEN_redessin( fenetre, BI_RDS_TOUT );
  }

  if ( ascenseur != ( void (*)(Fenetre *, int, int, int) ) NULL )
    fenetre->ascenseur = ascenseur;

  if ( clavier != ( int (*)(Fenetre *, unsigned int, int) ) NULL ) {
    fenetre->clavier = clavier;

    /* Et on l'assigne aussi aux sous-fenetres */
    if ( fenetre->nbr_ss_fenetres > 0 ) {
      unsigned int i;

      if ( NULL == fenetre->ss_fenetres ) {
	(void) fprintf( stderr, "%s,%s l.%u : Pas de sous-fentres mais %lu indiques !\n",
			__FILE__, __FUNCTION__, __LINE__, fenetre->nbr_ss_fenetres );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return;
      }

      for ( i = 0 ; i < fenetre->numero_arbre ; i++ ) {
	FEN_AssocierEvenements( fenetre->ss_fenetres[ i ],
				( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL,
				( void (*)(Fenetre *, int, int, int) ) NULL,
				clavier,
				( int  (*)(Fenetre *, int, int, int, int, int) ) NULL,
				( int  (*)(Fenetre *, Message *) ) NULL,
				( void (*)(Fenetre *, int, int, int, int) ) NULL,
				( int  (*)(Fenetre *) ) NULL );
      }
    }
  }

  if ( clic != ( int (*)(Fenetre *, int, int, int, int, int) ) NULL )
    fenetre->clic = clic;

  if ( message != ( int (*)(Fenetre *, Message *) ) NULL )
    fenetre->message = message;

  if ( rect != ( void (*)(Fenetre *, int, int, int, int) ) NULL ) {
    /* On te l'ancien pointeur */
    if ( fenetre->rect != ( void (*)(Fenetre *, int, int, int, int) ) NULL ) {
      fenetre->rect( fenetre, -1, -1, fenetre->vx_x, fenetre->vx_y );
    }

    /* On met le nouveau */
    fenetre->rect = rect;
    fenetre->rect( fenetre, fenetre->vx_x, fenetre->vx_y, -1, -1 );
  }

  if ( temps != ( int (*)(Fenetre *) ) NULL )
    fenetre->temps = temps;
}

/* -------- Routine de redessin par dfaut -------- */

void FEN_RedessinDefaut(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p x=%d y=%d larg=%u haut=%u", (void *) fenetre, x, y, larg, haut );
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel sans fentre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "[Pas de fentre]" );
    return;
  }

  vswr_mode(_PRG_station->handle, MD_REMPLACE);

  /* On met un fond rouge : bien visible -> signale un problme ! */
  TRC_Couleur( ROUGE );
  TRC_RectanglePlein( fenetre, x, y, larg, haut );

  /* On met un cercle bleu pais */
  TRC_Couleur( BLEU );
  TRC_Epaisseur( 3 );
  TRC_Ellipse( fenetre, fenetre->w_larg / 2, fenetre->w_haut / 2, fenetre->w_larg / 2, fenetre->w_haut / 2 );

  SORTIE_FONCTION( __FUNCTION__, "[Redessin termin]" );
}

/* -------- Routine de redessin OpenGL par dfaut -------- */

void FEN_GL_redessin_defaut(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel sans fentre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

#ifdef _DEVELOPPE_
  printf("FEN_GL_redessin_defaut : fenetre = %p [%s], x = %d, y = %d, larg = %u, haut = %u\n",
	 (void *) fenetre, fenetre->titre, x, y, larg, haut );
#endif

#ifdef AVEC_OPEN_GL_
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  /* Aspect pour la fentre */
  gluPerspective(45.0f, (GLfloat) fenetre->w_larg/ (GLfloat) fenetre->w_haut, 0.1f, 100.0f);

  glMatrixMode(GL_MODELVIEW);

  glShadeModel(GL_SMOOTH);
  glClearDepth(1.0f);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  glClearColor(0.0, 1.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  glTranslatef(-1.5f, 0.0f, -6.0f);
  glBegin(GL_TRIANGLES);
    glVertex3f( 0.0f, 1.0f, 0.0f);
    glVertex3f(-1.0f,-1.0f, 0.0f);
    glVertex3f( 1.0f,-1.0f, 0.0f);
  glEnd();

  glTranslatef(3.0f, 0.0f, 0.0f);
  glBegin(GL_QUADS);
    glVertex3f(-1.0f, 1.0f, 0.0f);
    glVertex3f( 1.0f, 1.0f, 0.0f);
    glVertex3f( 1.0f,-1.0f, 0.0f);
    glVertex3f(-1.0f,-1.0f, 0.0f);
  glEnd();
#endif /* AVEC_OPEN_GL_ */
}

/* -------- Rcuprer une information associe  la fentre -------- */

void *FEN_LireComplement(Fenetre *fenetre, int numero)
{
  if ( NULL == fenetre ) return NULL;

  switch ( numero )
  {
  case FEN_COMPLEMENT_1:
    return fenetre->info_1;

  case FEN_COMPLEMENT_2:
    return fenetre->info_2;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Code d'information incorrect [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }
}

/* -------- Associer des informations  une fentre ET SES SOUS-FENTRES --------
    fenetre    = la fentre  laquelle associer des informations
    numero     = dans quel pointeur mettre cette information ?
    complement = l'information  mettre
 */

void FEN_StockerComplement(Fenetre *fenetre, int numero, const void *complement)
{
  unsigned int i;

  if ( NULL == fenetre ) return;

  switch ( numero )
  {
  case FEN_COMPLEMENT_1:
#ifdef _DEVELOPPE_
    if ( ( fenetre->info_1 != NULL ) && ( fenetre->info_1 != complement ) ) {
      AVERTISSEMENT( "Une information tait dj associe  la fentre [%p, info_1=%p]", fenetre, fenetre->info_1 );
    }
#endif
    fenetre->info_1 = (void *) complement; /* On fait sauter le const, pour pouvoir en lecture ne pas tre gn... */
    break;

  case FEN_COMPLEMENT_2:
#ifdef _DEVELOPPE_
    if ( ( fenetre->info_2 != NULL ) && ( fenetre->info_2 != complement ) ) {
      AVERTISSEMENT( "Une information tait dj associe  la fentre [%p, info_2=%p]", fenetre, fenetre->info_2 );
    }
#endif
    fenetre->info_2 = (void *) complement; /* On fait sauter le const, pour pouvoir en lecture ne pas tre gn... */
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Code d'information incorrect [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* On stocke aussi dans les sous-fentres, si ncessaire */
  if ( fenetre->ss_fenetres != NULL ) {
    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      FEN_StockerComplement( fenetre->ss_fenetres[ i ], numero, complement );
    }
#ifdef _DEVELOPPE_
  } else if ( fenetre->nbr_ss_fenetres > 0 ) {
    AVERTISSEMENT( "Pas de table de sous fentres alors que %lu sous-fentres sont indiques !", fenetre->nbr_ss_fenetres );
#endif
  }
}
