/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des fentres
   Remarque : comme c'est assez bas niveau, trs peu de routines sont indpendantes du systme !
*/


#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_
# define _LIBRAIRIE_X_ /* Pour avoir accs aux fonctions internes */
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "bib_interface.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~- Gestion des structures ~-~-~-~-~-~-~-~- */

/* -------- ter une fentre du chanage -------- */

int FEN_DechainerFenetre( Fenetre *fenetre )
{
  if ( NULL == fenetre ) return 0;

  if ( NULL == BI_premiere_fenetre ) {
    (void) fprintf(stderr, "FEN_DechainerFenetre : Pas de chanage de fentres !\n");
    return -1;
  }

  if ( fenetre == BI_premiere_fenetre ) {
    BI_premiere_fenetre = BI_premiere_fenetre->suivante;
  } else {
    Fenetre *avant;

    avant = BI_premiere_fenetre;
    while ( avant->suivante != fenetre ) {
      avant = avant->suivante;
      if ( NULL == avant ) {
#ifdef _DEVELOPPE_
	(void) fprintf(stderr, "FEN_DechainerFenetre : Fentre hors chanage !\n");
#endif
	return +1; /* Erreur, mais minime */
      }
    }

    avant->suivante = fenetre->suivante;
  }

  /* La fentre est maintenant hors-chanage */
  fenetre->suivante = NULL;

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Librer le contenu d'une fentre -------- */

int FEN_LibererFenetre( Fenetre *fenetre )
{
  if ( NULL == fenetre ) return 0;

  if ( fenetre->suivante != NULL ) {
    (void) fprintf(stderr, "FEN_LibererFenetre : fentre dans un chanage --- libration impossible\n");
    return -1;
  }

  /* On doit librer les sous-fentres */
  if ( fenetre->ss_fenetres != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      FEN_LibererFenetre( fenetre->ss_fenetres[ i ] ); 
    }
    free( fenetre->ss_fenetres );
    fenetre->ss_fenetres = NULL;

    /* Plus de sous-fentre donc */
    fenetre->nbr_ss_fenetres = 0;
  }

  /* On fait disparatre la fentre pour l'interface graphique [+ tout ce qui lui est spcifique !]
     Sans mmoriser la taille puisque l'on perdra la structure de toute faon... */
  FEN_DetruireFenetre( fenetre, VRAI );

  /* Plus besoin du titre */
  free( fenetre->titre ); fenetre->titre = NULL;

  /* Plus besoin des infos */
  free( fenetre->infos ); fenetre->infos = NULL;

  /* Plus besoin de la structure */
  free( fenetre );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Fermeture & destruction d'une fentre -------- */

int FEN_Detruire( Fenetre *fenetre )
{
  if (fenetre == NULL) return -1; /* Rien  faire... */
#ifdef _DEVELOPPE_
  printf("FEN_Detruire(fenetre = %p)\n", (void *) fenetre);
  BI_AfficherFenetre( fenetre );
#endif

  /* On te la fentre du chanage */
  FEN_DechainerFenetre( fenetre );

  /* On libre son contenu */
  FEN_LibererFenetre( fenetre );

  return 0; /*Tout s'est bien pass */
}

/* ~-~-~-~-~-~-~-~- Aides  la fermeture ~-~-~-~-~-~-~-~- */

int FEN_MessageFermeture(Fenetre *fenetre, int mode_fermeture)
{
  if ( NULL == fenetre ) return BI_EVENEMENT_ERREUR;

  if ( fenetre->message != NULL ) {
    Message message;
    int retour;

    if ( FAUX == fenetre->msg_masque.fermeture ) {
      AVERTISSEMENT( "Fonction de messages, mais pas pour les messages de fermeture [fentre %s]", fenetre->titre );
    }

    /* Initialisation du message */
    message.type = WM_CLOSED;
    message.fenetre.application = 0;
    message.fenetre.en_plus = 0;
    message.fenetre.x    = mode_fermeture; message.fenetre.y    = mode_fermeture;
    message.fenetre.larg = mode_fermeture; message.fenetre.haut = mode_fermeture; /* Fermeture suivant le mode indiqu */

    /* On traite le message */
    retour = fenetre->message( fenetre, &message );    
    if ( retour != BI_EVENEMENT_NON_TRAITE ) {
      return retour;
    }
  }

  /* vnement non-trait : pas de routine de messages ou ne grant pas ces messages ==> fermeture simple */
  FEN_Fermer( fenetre, FAUX );
  return BI_EVENEMENT_FEN_FERMEE;
}
