/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système

   Travail sur les objets (boîtes de dialogue)
   Déclarations (indépendantes de LASE)
*/

#ifndef BI_OBJETS_COMMUNS_H_
# define BI_OBJETS_COMMUNS_H_ /* Pas d'inclusion multiple */

/* -------- Arborescence -------- */

int OBJ_TrouverPere(unsigned int arbre, unsigned int objet);

/* -------- Contenu des objets -- */

int OBJ_LibererComplement (unsigned int arbre, unsigned int objet);
int OBJ_PreparerComplement(unsigned int arbre, unsigned int objet);

int OBJ_OptionsEditable(unsigned int arbre, unsigned int objet, unsigned int proprietes, double min, double max, const char *unite);

int           OBJ_LireCaractere(unsigned int arbre, unsigned int objet);
int           OBJ_LireDate     (unsigned int arbre, unsigned int objet, const char *format, struct tm *date);
long          OBJ_LireEntier_S (unsigned int arbre, unsigned int objet);
unsigned long OBJ_LireEntier_U (unsigned int arbre, unsigned int objet);
double        OBJ_LireNombre   (unsigned int arbre, unsigned int objet, double *erreur);
unsigned int  OBJ_LireStyle    (unsigned int arbre, unsigned int objet);

int OBJ_MettreCaractere(unsigned int arbre, unsigned int objet, int caractere);
int OBJ_MettreDate    (unsigned int arbre, unsigned int objet, const char *format, struct tm *date);
int OBJ_MettreEntier_S(unsigned int arbre, unsigned int objet, long valeur);
int OBJ_MettreEntier_U(unsigned int arbre, unsigned int objet, unsigned long valeur);
int OBJ_MettreNombre  (unsigned int arbre, unsigned int objet, double valeur, double erreur);
int OBJ_MettreStyle   (unsigned int arbre, unsigned int objet, unsigned int style);
int OBJ_MettreUnite   (unsigned int arbre, unsigned int objet, const char *unite);

int OBJ_CopierTexte   (unsigned int arbre, unsigned int objet, const char *texte, unsigned long lg_max);
int OBJ_RemplacerTexte(unsigned int arbre, unsigned int objet, const char *texte, boolean copier_original);

int     BI_AjouterCaractere_TexteInfini(Texte_Infini *texte_infini, unsigned int position, const char *c, boolean inserer);
int     BI_OterCaractere_TexteInfini   (Texte_Infini *texte_infini, unsigned int position);
boolean BI_ValiderCaractere_Nombre(struct infos_editable *infos, unsigned int position, int code, boolean mode_insertion);

unsigned long OBJ_OptionMenu(unsigned int arbre, unsigned int objet, unsigned long option, int mode);

/* -------- État des objets ----- */

#define FORCE_NON FAUX /* On force pour que l'objet ne soit pas dans cet état */
#define FORCE_OUI VRAI /* On force l'objet à être dans cet état               */
#define ECHANGE   2    /* On change l'objet d'état                            */
#define QUESTION  3    /* On ne fait que demande quel est l'état actuel       */
#define METTRE    4    /* On demande l'installation des paramètres            */

/* Remarque : les déclarations sont communes, mais la réalisation est spécifique ! */

/* ob_state */
int OBJ_active (unsigned int arbre, unsigned int objet, int mode);
int OBJ_coche  (unsigned int arbre, unsigned int objet, int mode);
int OBJ_choisit(unsigned int arbre, unsigned int objet, int mode);

/* ob_flags */
int OBJ_cache  (unsigned int arbre, unsigned int objet, int mode);

/* Autres */

int OBJ_ChangerType(unsigned int arbre, unsigned int objet, int nv_type, ...);
int OBJ_MettreMenu (unsigned int arbre, unsigned int objet, unsigned long nombre_options, const char **options, const char *titre,
		    void (*gestion)(unsigned int arbre, unsigned int objet, unsigned long option_choisie) );

#ifdef _DEVELOPPE_
void OBJ_AfficherContenu(OBJECT *objet);
#else
# define OBJ_AfficherContenu(objet) /* Version de développement seulement */
#endif

#endif
