/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2007, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système

   Gestion du sélecteur de fichier [si interne]
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_memorise = NULL;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Appel du sélecteur pour le chargement d'un ou plusieurs fichiers -------- */

int BI_SelecteurChargement(BI_InfosSelecteur *infos, unsigned long *nombre_fichiers, char ***noms_fichiers, Fenetre *fenetre_mere)
{
  if ( ( NULL == infos ) || ( NULL == nombre_fichiers ) || ( NULL == noms_fichiers ) ) {
    AVERTISSEMENT( "Problème avec l'un des paramètres (infos=%p, nombre_fichiers=%p, noms_fichiers=%p)", (void *) infos, (void *) nombre_fichiers, (void *) noms_fichiers);
    return -254;
  }

  /* On contrôle les propriétés */
  infos->proprietes &= ~( BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_CHEMIN_SEULEMENT );

  return BI_SelecteurDeFichiers( infos, nombre_fichiers, noms_fichiers, fenetre_mere );
}

/* -------- Appel du sélecteur pour le chargement d'un ou plusieurs fichiers [ANCIENNE VERSION] -------- */

char **CH_selecteur(const char *masque, const char *titre, unsigned int nbr_ext,
		    const char **extensions, unsigned int *nbr_fichiers, Fenetre *fenetre_mere)
{
  BI_InfosSelecteur infos = { "Charger un/des fichier(s)",
			      BI_SELECTEUR_MULTIPLE | BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AVEC_JOKERS | BI_SELECTEUR_MEMORISER_CHEMIN,
			      NULL, 0, NULL, 0, NULL, (void (*)(unsigned int, unsigned int, const char *)) NULL };
  int reponse;
  char **liste_fichiers = NULL;
  unsigned long nombre_fichiers = 0;

  /* Contrôles préliminaires */
  if ( NULL == nbr_fichiers ) {
    (void) fprintf(stderr, "%s, %s l.%u : nbr_fichiers = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }
  if ( ( NULL == extensions ) && ( nbr_ext > 0 ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Incohérence, aucune extension donnée mais nbr_ext = %u\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_ext);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    nbr_ext = 0;
  }

  if ( NULL == titre ) {
    AVERTISSEMENT( "Pas de titre indiqué [%s]", titre );
  } else {
    infos.titre = (char *) titre;
  }

  /* On initialise les infos */
  if ( masque != NULL ) {
    OPTION_A_FAIRE( "Masque indiqué" );
  } else {
    infos.nbr_filtres = nbr_ext;
    infos.filtres = extensions;
  }

  /* On rappelle le chemin par défaut */
  infos.chemin = chemin_memorise;

  /* On appelle le sélecteur de fichiers */
  reponse = BI_SelecteurDeFichiers( &infos, &nombre_fichiers, &liste_fichiers, fenetre_mere );

  /* On analyse la réponse */
  if ( ( BI_SELECTEUR_ANNULER == reponse ) || ( 0 == nombre_fichiers ) ) {
    /* On a annulé le chargement : rien à faire ! */
    *nbr_fichiers = 0;
    return NULL;
  } else if ( reponse != 0 ) {
    AVERTISSEMENT( "Erreur survenue dans BI_SelecteurDeFichiers, code %d", reponse );
    *nbr_fichiers = 0;
    return NULL;
  }

  /* On mémorise le nouveau chemin */
  chemin_memorise = infos.chemin;

  /* Si on est là : la liste de fichiers est correcte, on la renvoie donc */
  *nbr_fichiers = nombre_fichiers;
  return liste_fichiers;
}

/* -------- Appel du sélecteur pour la sauvegarde d'un fichier -------- */

char *BI_SelecteurSauvegarde(BI_InfosSelecteur *infos, int *code_erreur, Fenetre *fenetre_mere)
{
  FILE *verif = NULL;
  int reponse;
  unsigned long nombre_fichiers = 0;
  char **fichiers_choisis = NULL;
  char *nom_fichier;

  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : pas d'infos de sélecteur indiquées\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* On contrôle les propriétés */
  infos->proprietes &= ~( BI_SELECTEUR_MULTIPLE | BI_SELECTEUR_AVEC_JOKERS | BI_SELECTEUR_CHEMIN_SEULEMENT );

  /* On appelle le sélecteur de fichiers */
  reponse = BI_SelecteurDeFichiers( infos, &nombre_fichiers, &fichiers_choisis, fenetre_mere );

  /* On analyse la réponse */
  if ( code_erreur != NULL ) *code_erreur = reponse;
  if ( ( BI_SELECTEUR_ANNULER == reponse ) || ( 0 == nombre_fichiers ) ) {
    /* On a annulé la sauvegarde : rien à faire ! */
    return NULL;
  } else if ( reponse != 0 ) {
    AVERTISSEMENT( "Erreur survenue dans BI_SelecteurDeFichiers, code %d", reponse );
    return NULL;
  }

  /* On ne garde que le premier */
  if ( nombre_fichiers > 1 ) {
    unsigned long i;

    AVERTISSEMENT( "Plusieurs fichiers sélectionnés [%lu]", nombre_fichiers );
    for ( i = 1 ; i < nombre_fichiers ; i++ ) {
      free( fichiers_choisis[ i ] );
      fichiers_choisis[ i ] = NULL;
    }
  }

  /* On récupère le nom du fichier */
  nom_fichier = fichiers_choisis[ 0 ]; fichiers_choisis[ 0 ] = NULL;
  free( fichiers_choisis ); fichiers_choisis = NULL;

  /* Contrôles */
  verif = fopen( nom_fichier, "r" );
  if ( verif != NULL ) { /* On a pu ouvrir le fichier -> on le renomme en .bak */
    char *nom_ancien;
    (void) fclose( verif );

    nom_ancien = (char *) calloc( strlen( nom_fichier ) + 5, sizeof( char ) );
    if ( NULL == nom_ancien ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de changer le nom de l'ancienne version !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return nom_fichier;
    }

    strcpy( nom_ancien, nom_fichier );
    strcat( nom_ancien, ".bak" );

    reponse = rename( nom_fichier, nom_ancien);
    if ( reponse != 0 ) {
      (void) fprintf( stderr, "%s, %s l.%u : Impossible de renommer l'ancienne version, code %d\n",
		      __FILE__, __FUNCTION__, __LINE__, errno );
    }
    free( nom_ancien ); nom_ancien = NULL;
  }

  /* On renvoie le nom du fichier */
  return nom_fichier;
}

/* -------- Appel du sélecteur pour la sauvegarde d'un fichier -------- */

char *SV_selecteur(const char *masque, const char *titre, Fenetre *fenetre_mere)
{
  BI_InfosSelecteur infos = { "Sauver dans un fichier",
			      BI_SELECTEUR_AJOUTER_PREDEFINI,
			      NULL, 0, NULL, 0, NULL, (void (*)(unsigned int, unsigned int, const char *)) NULL };
  int reponse;
  char *nom_fichier;
  char **liste_fichiers = NULL;
  unsigned long nombre_fichiers = 0;
  FILE *verif = NULL;

  /* Contrôles */
  if ( NULL == titre ) {
    AVERTISSEMENT( "Pas de titre indiqué [%s]", titre );
  } else {
    infos.titre = (char *) titre;
  }

  /* On initialise la structure d'infos */
  if ( masque != NULL ) {
    infos.nbr_filtres = 1;
    infos.filtres = &masque;
  }

  /* On rappelle le chemin par défaut */
  infos.chemin = chemin_memorise;

  /* On appelle le sélecteur */
  reponse = BI_SelecteurDeFichiers( &infos, &nombre_fichiers, &liste_fichiers, fenetre_mere );

  /* On analyse la réponse */
  if ( ( BI_SELECTEUR_ANNULER == reponse ) || ( 0 == nombre_fichiers ) ) {
    /* On a annulé la sauvegarde : rien à faire ! */
    return NULL;
  } else if ( reponse != 0 ) {
    AVERTISSEMENT( "Erreur survenue dans BI_SelecteurDeFichiers, code %d", reponse );
    return NULL;
  }

  /* On mémorise le nouveau chemin */
  chemin_memorise = infos.chemin;

  /* Si on est là : la liste de fichiers est correcte, on peut l'utiliser ; il y a forcément 1 et 1 seul nom dans la liste. */
  nom_fichier = liste_fichiers[ 0 ]; free( liste_fichiers ); liste_fichiers = NULL;

  /* Vérification : le fichier existe-t-il déjà ? */
  verif = fopen( nom_fichier, "r" );
  if ( verif != NULL ) { /* On a pu ouvrir le fichier -> on le renomme en .bak */
    char *nom_ancien;
    (void) fclose( verif );

    nom_ancien = (char *) calloc( strlen( nom_fichier ) + 5, sizeof( char ) );
    if ( NULL == nom_ancien ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de changer le nom de l'ancienne version !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return nom_fichier;
    }

    strcpy( nom_ancien, nom_fichier );
    strcat( nom_ancien, ".bak" );

    reponse = rename( nom_fichier, nom_ancien);
    if ( reponse != 0 ) {
      (void) fprintf( stderr, "%s, %s l.%u : Impossible de renommer l'ancienne version, code %d\n",
		      __FILE__, __FUNCTION__, __LINE__, errno );
    }
    free( nom_ancien ); nom_ancien = NULL;
  }

  /* On renvoie le nom du fichier finalement retenu */
  return nom_fichier;
}

/* -------- Demande le chemin pour travailler ensuite dans le dossier indiqué -------- */

char *CHEM_selecteur(const char *titre, Fenetre *fenetre_mere)
{
  BI_InfosSelecteur infos = { "Choisir un dossier",
			      BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_CHEMIN_SEULEMENT | BI_SELECTEUR_MEMORISER_CHEMIN,
			      NULL, 0, NULL, 0, NULL, (void (*)(unsigned int, unsigned int, const char *)) NULL };
  char **fichiers_choisis = NULL; unsigned long nombre_choisis;  /* Juste pour l'appel correct du sélecteur, ignorés ! */
  int reponse;

  /* Contrôles */
  if ( NULL == titre ) {
    AVERTISSEMENT( "Pas de titre indiqué [%s]", titre );
  } else {
    infos.titre = (char *) titre;
  }

  /* On appelle le sélecteur de fichiers */
  reponse = BI_SelecteurDeFichiers( &infos, &nombre_choisis, &fichiers_choisis, fenetre_mere );
  if ( reponse != 0 ) {
    return NULL;		/* On a annulé ou il y a un problème */
  }

  /* On renvoie le nouveau chemin */
  return infos.chemin; /* Contient l'ancien chemin... Doit être libéré par la fonction appelante */
}

/* -------- Fonctions utilitaires du sélecteur de fichier -------- */

/* Ajouter un chemin à la table des chemins prédéfinis -------- */

int BI_Selecteur_AjouterChemin(char ***table_chemins, unsigned int *nombre_chemins, const char *chemin_a_ajouter)
{
  char **tableau;
  unsigned int i, n_chemins;

  if ( NULL == chemin_a_ajouter ) {
    return 0;			/* Rien de particulier à faire... */
  }

  /* Contrôles de validité */
  if ( ( NULL == table_chemins ) || ( NULL == nombre_chemins ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec NULL pour les variables à modifier !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* Reprise des variables */
  tableau = *table_chemins;
  n_chemins = *nombre_chemins;

  /* On vérifie que le chemin indiqué n'est pas déjà dans la liste */
  if ( tableau != NULL ) {
    for ( i = 0 ; i < n_chemins ; i++ ) {
      if ( strcmp( tableau[ i ], chemin_a_ajouter ) == 0 ) {
	/* Chemin déjà dans la liste : ignoré... */
	AVERTISSEMENT( "Chemin déjà présent : %s", chemin_a_ajouter );
	return +1;
      }
    }
  }

  /* On modifie le tableau pour ajouter un élément */
  tableau = realloc( tableau, sizeof(char *) * (n_chemins + 1) );
  if ( NULL == tableau ) {
    AVERTISSEMENT( "Impossible d'ajouter le chemin %s", chemin_a_ajouter );
    return -2;
  }

  /* On ajoute le nouvel élément : COPIE de la chaîne pour être tranquille */
  tableau[ n_chemins ] = strdup( chemin_a_ajouter );

  /* On mémorise les nouvelles valeurs des variables */
  *nombre_chemins = n_chemins + 1;
  *table_chemins = tableau;

  /* Terminé sans souci */
  return 0;
}
