/* Bibliothèque GEM / X_windows étendue
   (C) 1998-2003 Emmanuel CURIS

   Gestion de l'état ds objets (changement en particulier)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "Bib_Commun/bib_interface.h"

/* -------- Fonctions locales -------- */

#ifdef _DEVELOPPE_
static char *TXT_TypeObjet(int type);
#endif

/* -------- Réalisation pratique ----- */

/* -------- Effacer les compléments associés à un objet [dépendent du type] -------- */

int OBJ_LibererComplement(unsigned int num_arbre, unsigned int num_objet)
{
  OBJECT *objet;
  int retour;

  /* Vérifications préliminaires */
  retour = OBJ_ControlerValeurs( num_arbre, num_objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : valeurs incorrectes [arbre = %u, objet = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, num_objet);
    return -1;
  }

  objet = arbres[ num_arbre ].adresse + num_objet;

  switch( objet->ob_type )
  {
  case G_SSMENU:
    /* On vérifie que la table des options a bien été enlevée */
    /* Remarque : on ne peut pas la libérer, car peut être en texte dans le programme... */
    if( objet->ob_spec.menu->options != NULL ) {
      AVERTISSEMENT( "Table des options encore liée à l'objet [arbre %u, objet %u]", num_arbre, num_objet);
      objet->ob_spec.menu->options = NULL;
    }
    free( objet->ob_spec.menu ); objet->ob_spec.menu = NULL;
    break;

  case G_TXT_INFINI:
    free( objet->ob_spec.txt_infini->texte );
    free( objet->ob_spec.txt_infini );
    objet->ob_spec.txt_infini = NULL;
    break;

  case G_NOMBRE:
    free( objet->ob_spec.nombre->texte.texte );
    free( objet->ob_spec.nombre->unite );
    free( objet->ob_spec.nombre );
    objet->ob_spec.nombre = NULL;
    break;

  case G_USERDEF:
    free( objet->ob_spec.util );
    objet->ob_spec.util = NULL;
    break;

  case G_TEXT:  case G_BOXTEXT:
  case G_FTEXT: case G_FBOXTEXT:
    if ( objet->ob_spec.tedinfo != NULL ) {
      TEDINFO *tedinfo;

      tedinfo = objet->ob_spec.tedinfo;

      free( tedinfo->te_ptext.editable );
      if ( FAUX == arbres[ num_arbre ].drapeaux.interne ) {
	/* Si le drapeau est mis, arbre créé à partir d'objets dans un tableau du programme
	   ==> ces deux chaînes sont déclarées en constantes */
	free( tedinfo->te_ptmplt );
	free( tedinfo->te_pvalid );
      }
    }
    free( objet->ob_spec.tedinfo );
    objet->ob_spec.tedinfo = NULL;
    break;

  case G_BOX:
  case G_IBOX:
  case G_BOXCHAR:
  case G_STYLELIGNE:
  case G_STYLEMARQUE:
    /* Aucun complément à libérer */
    break;

  case G_BUTTON:
  case G_STRING:
  case G_TITLE:
  case G_SEPARATEUR:
    objet->ob_spec.texte = NULL; /* Pour être sûr de ne pas avoir une valeur absurde ! */
    break;

  case G_IMAGE:
    (void) fprintf(stderr, "%s,%s l.%u : À FAIRE --- G_IMAGE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case G_ICON:
    (void) fprintf(stderr, "%s,%s l.%u : À FAIRE --- G_ICON\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case G_CICON:
    (void) fprintf(stderr, "%s,%s l.%u : À FAIRE --- G_CICON\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Type d'objet inconnu [%d] \n",
		   __FILE__, __FUNCTION__, __LINE__, objet->ob_type);
    return +1;
  }

  return 0;
}

/* -------- Créer les compléments associés à un objet -------- */

int OBJ_PreparerComplement(unsigned int num_arbre, unsigned int num_objet)
{
  OBJECT *objet;

  /* Vérifications préliminaires */
  if ( num_objet >= arbres[ num_arbre ].nbr_objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : objet incorrect [%u, seulement %u dans l'arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_objet, arbres[ num_arbre ].nbr_objets, num_arbre );
    return -1;
  }

  objet = arbres[ num_arbre ].adresse + num_objet;

  switch( objet->ob_type )
  {
  case G_TXT_INFINI:
    objet->ob_spec.txt_infini = (Texte_Infini *) calloc( 1, sizeof(Texte_Infini) );
    if ( NULL == objet->ob_spec.txt_infini ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'allouer la structure Texte_Infini\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    objet->ob_spec.txt_infini->texte = NULL;
    /* Caractères visibles : quand on appelle, on suppose que l'on a déjà converti en coordonnées écran */
    objet->ob_spec.txt_infini->max_visibles = ( objet->ob_width ) / BI_larg_car;
    break;

  case G_NOMBRE:
    objet->ob_spec.nombre = (struct infos_editable *) malloc( sizeof( struct infos_editable ) );
    if ( NULL == objet->ob_spec.nombre ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible d'allouer la structure pour l'éditable\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    objet->ob_spec.nombre->texte.texte = NULL;
    objet->ob_spec.nombre->texte.max_cars = 0;
    objet->ob_spec.nombre->texte.prem_car = 0;
    /* Caractères visibles : quand on appelle, on suppose que l'on a déjà converti en coordonnées écran */
    objet->ob_spec.nombre->texte.max_visibles = ( objet->ob_width ) / BI_larg_car;

    /* Par défaut, valeur réelle, sans erreur, sans limites, sans unité, fonctions interdites */
    objet->ob_spec.nombre->proprietes = 0;
    objet->ob_spec.nombre->min = 0.0; objet->ob_spec.nombre->max = 0.0;
    objet->ob_spec.nombre->unite = NULL;
    break;

  case G_SSMENU:
    objet->ob_spec.menu = (struct infos_ssmenu *) malloc( sizeof( struct infos_ssmenu ) );
    if ( NULL == objet->ob_spec.menu ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de réserver les infos pour le sous-menu !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    objet->ob_spec.menu->nombre_options = 0;
    objet->ob_spec.menu->option_choisie = 0;
    objet->ob_spec.menu->texte   = NULL;
    objet->ob_spec.menu->options = NULL;
    objet->ob_spec.menu->gestion = ( void (*)(unsigned int, unsigned int, unsigned long) ) NULL;
    break;

  case G_USERDEF:
    objet->ob_spec.util = (USERBLK *) malloc(sizeof(USERBLK));
    if ( NULL == objet->ob_spec.util ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'allouer la structure USERBLK\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    objet->ob_spec.util->ub_code = ( int (*)(struct parm_blk *) ) NULL;
    objet->ob_spec.util->ub_parm = 0L;
    break;

  case G_TEXT:  case G_BOXTEXT:
  case G_FTEXT: case G_FBOXTEXT:
    objet->ob_spec.tedinfo = (TEDINFO *) calloc( 1, sizeof( TEDINFO ) );
    if ( NULL == objet->ob_spec.tedinfo ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'allouer la structure TEDINFO\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    } else {
      TEDINFO *tedinfo;

      tedinfo = objet->ob_spec.tedinfo;
      tedinfo->te_font = IBM;
      tedinfo->te_just = TE_CNTR;
      tedinfo->te_color =  0x1180;
    }
    break;

  case G_BOX:
  case G_IBOX:
  case G_BOXCHAR:
    break;

  case G_BUTTON:
  case G_STRING:
  case G_TITLE:
  case G_SEPARATEUR:
    objet->ob_spec.texte = NULL; /* Pour être sûr de ne pas avoir une valeur absurde ! */
    break;

  case G_STYLELIGNE:
    objet->ob_spec.style_ligne = BI_CONTINU;
    break;

  case G_STYLEMARQUE:
    objet->ob_spec.style_marque = PM_POINT;
    break;

  case G_IMAGE:
    OPTION_A_FAIRE( "G_IMAGE" );
    break;

  case G_ICON:
    OPTION_A_FAIRE( "G_ICON" );
    break;

  case G_CICON:
    OPTION_A_FAIRE( "G_CICON" );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Type d'objet inconnu [%d] \n",
		   __FILE__, __FUNCTION__, __LINE__, objet->ob_type);
    return +1;
  }

  return 0;
}

/* -------- Changer les options associées à un objet éditable (G_NOMBRE) -------- */

int OBJ_OptionsEditable(unsigned int num_arbre, unsigned int num_objet,
			unsigned int proprietes, double min, double max, const char *unite)
{
  OBJECT *objet;
  struct infos_editable *infos;
  double valeur, erreur;

  /* Vérifications préliminaires */
  if ( num_objet >= arbres[ num_arbre ].nbr_objets ) {
    (void) fprintf( stderr, "%s,%s l.%u : objet incorrect [%u, seulement %u dans l'arbre %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, num_objet, arbres[ num_arbre ].nbr_objets, num_arbre );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  objet = arbres[ num_arbre ].adresse + num_objet;
  if ( objet->ob_type != G_NOMBRE ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s, %s l.%u : Type d'objet incorrect [%d = %s ; objet %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, objet->ob_type, TXT_TypeObjet( objet->ob_type ), num_objet );
    OBJ_AfficherContenu( objet );
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  infos = objet->ob_spec.nombre;
  if ( NULL == infos ) {
    (void) fprintf( stderr, "%s, %s l.%u : Pas d'informations associées !\n",
		    __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }

  /* Valeur originale */
  valeur = OBJ_LireNombre( num_arbre, num_objet, &erreur );

  /* On installe les nouvelles propriétés */
  infos->proprietes = proprietes;
  if ( proprietes & BI_EDITABLE_MIN ) {
    infos->min = min;
  }
  if ( proprietes & BI_EDITABLE_MAX ) {
    infos->max = max;
  }
  if ( proprietes & BI_EDITABLE_UNITE ) {
    if ( infos->unite != NULL ) {
      /* Libérer l'ancienne unité */
      free( infos->unite );
      infos->unite = NULL;
    }
    if ( NULL == unite ) { /* En fait, il n'y a pas d'unité */
      infos->proprietes &= ~BI_EDITABLE_UNITE;
    } else {
      /* On fait une copie de l'unité pour pouvoir la libérer sans souci */
      infos->unite = strdup( unite );
      if ( NULL == infos->unite ) { /* En fait, il n'y a pas d'unité */
	infos->proprietes &= ~BI_EDITABLE_UNITE;
      }
    }
  }

  /* Comme les propriétés ont changé, le texte doit être mis à jour
     [unité différente ou valeur dans une plage incorrecte...] */
  OBJ_MettreNombre( num_arbre, num_objet, valeur, erreur );

  /* Tout s'est bien passé */
  return 0;
}

#ifdef _DEVELOPPE_
/* -------- Afficher le contenu d'un objet (pour déboguage) -------- */

void OBJ_AfficherContenu(OBJECT *objet)
{
  printf("Suivant      : %d\n", objet->ob_next);
  printf("Premier fils : %d\n", objet->ob_head);
  printf("Dernier fils : %d\n", objet->ob_tail);
  printf("Type         : %d [%s]\n", objet->ob_type, TXT_TypeObjet( objet->ob_type ) );
  printf("Flags        : %d\n", objet->ob_flags);
  printf("État         : %d\n", objet->ob_state);
  printf("Complément   : ");
  switch ( objet->ob_type )
  {
  case G_SEPARATEUR:
  case G_BUTTON:
  case G_STRING:
  case G_TITLE:
    printf("chaîne = \"%s\"", objet->ob_spec.texte);
    break;
  }
  printf("\n");
  printf("Position     : X = %d, Y = %d\n", objet->ob_x, objet->ob_y );
  printf("Dimensions   : L = %u, H = %u\n", objet->ob_width, objet->ob_height );
}

static char *TXT_TypeObjet(int type)
{
  switch ( type )
  {
  case G_CLOSER:    return "Bouton de fermeture";
  case G_INFOS:     return "Barre d'informations";
  case G_FULLER:    return "Bouton d'agrandissement";
  case G_FLECHE:    return "Flèche";
  case G_ASCENSEUR: return "Ascenseur [fenêtre]";

  case G_SEPARATEUR: return "Séparateur";

  case G_TXT_INFINI: return "Texte infini";
  case G_NOMBRE:     return "Nombre éditable";

  case G_BOX:        return "Boîte";
  case G_TEXT:       return "Texte simple";
  case G_BOXTEXT:    return "Texte dans une boîte";
  case G_IMAGE:      return "Image";
  case G_USERDEF:    return "Objet utilisateur";
  case G_IBOX:       return "Case à cocher";
  case G_BUTTON:     return "Bouton";
  case G_BOXCHAR:    return "Caractère encadré";
  case G_STRING:     return "Chaîne";
  case G_FTEXT:      return "Texte formaté";
  case G_FBOXTEXT:   return "Texte formaté encadré";
  case G_ICON:       return "Icône monochrome";
  case G_TITLE:      return "Titre de menu";
  case G_CICON:      return "Icône coloré";
  }

  return "Inconnu";
}
#endif
