/**** Bibliothque mixte GEM / X_windows
  (C) 1998-2003 Emmanuel CURIS

  Gestion des fentres : cration des fentres sous X-Window
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"
#ifdef AVEC_XPM_
# include <X11/xpm.h>
#endif

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  FEN_creation         (Fenetre *fenetre, Window mere, XVisualInfo *visual, Colormap *palette);
static int  FEN_InitialiserOpenGL(Fenetre *fenetre);
/* static void FEN_creation_opengl  (Fenetre *fenetre, int x_gl, int y_gl, unsigned int larg_gl, unsigned int haut_gl); */

#ifdef AVEC_OPEN_GL_
static void installe_colormap(Fenetre *fenetre);
#endif

/* ~~~~~~~~~~~~~~~ Routines de cration de fentres ~~~~~~~~~~~~~~~~ */

/* -------- Cration d'une fentre pour X-Window [interface gnrale] -------- */

int FEN_CreerFenetre(Fenetre *fenetre, Fenetre *mere)
{
  Window fen_mere = None;
  int retour;

  if ( NULL == fenetre ) return -1;
  if ( mere != NULL ) {
    /* Il s'agit d'une sous-fentre */
    fen_mere = mere->handle;
  } else {
    /* Il s'agit d'une vraie fentre, native */
    fen_mere = BI_Fen_racine;
  }

  /* Cration proprement dite */
  if ( BI_FENETRE_GL == fenetre->type ) {
#ifdef AVEC_OPEN_GL_
    if ( FAUX == BI_OpenGL_actif ) {
      /* Open GL non install : on ne peut donc pas crer la fentre */
      return -1;
    }
# ifdef _DEVELOPPE_
    glXGetConfig( BI_Display, &XGL_Visual, GLX_USE_GL, &retour );
    if ( retour != True ) {
      /* Erreur : on analyse pourquoi */
      if ( GLX_NO_EXTENSION == retour ) {
	(void) fprintf( stderr, "%s : Extension GLX non-supporte ! Utilisation Open-GL douteuse...", __FUNCTION__ );
      } else {
	(void) fprintf( stderr, "*** Gros problme : Visual pas support ! [code %d] ***\n", retour );
	(void) fprintf( stderr, " fentre racine associe : %ld\n", RootWindow( BI_Display, XGL_Visual.screen ) );
	(void) fprintf( stderr, " visual associ : %p - id = %ld\n", (void *) XGL_Visual.visual, XGL_Visual.visualid );
	(void) fprintf( stderr, " profondeur : %d\n", XGL_Visual.depth );
	(void) fprintf( stderr, " type : %d\n", XGL_Visual.class );

	return -1; /* On ne pourra donc pas crer la fentre ! */
      }
    }
# endif
    retour = FEN_creation( fenetre, fen_mere, &XGL_Visual, &XGL_Palette );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "FEN_CreerFenetre : ERREUR dans FEN_creation / OpenGL\n");
      return -2;
    }

    /* Installer la Colormap et, surtout, prvenir le WM */
    installe_colormap( fenetre );

    /* On choisit le contexte... */
    (void) FEN_GL_choisit_contexte(fenetre, BI_GL_direct);
#else
    retour = -1; /* Fentre OpenGL pure n'a pas de sens sans OpenGL ! */
#endif
  } else {
    retour = FEN_creation( fenetre, fen_mere, NULL, NULL );
  }

  /* Si l'on sait dj qu'il y a un arbre associ : on cre les sous-fentres correspondantes ! */
  if ( fenetre->arbre != NULL ) { /* Fentre de type GEM */
    /* On recre donc les fentres de tous les objets */
    OBJ_creer_fenetres( fenetre->arbre, fenetre->handle, ROOT, ROOT );
  }

  /* On a fini, cela semble s'tre bien pass */
  return retour;
}

/* -------- Cration de la fentre X + attributs --------\
 |  Renvoie 0 si tout s'est bien pass, != 0 sinon       |
 \------------------------------------------------------ */

static int FEN_creation(Fenetre *fenetre, Window mere, XVisualInfo *visual, Colormap *palette)
{
  Visual *visual_reel;
  Window handle;
  XSetWindowAttributes parametres; /* De la fentre X */
  unsigned int unit_car; /* Unit de largeur & hauteur des boutons... */
  unsigned long masque_attr;
  int attributs;
  int n_plans; /* = profondeur du Visual */

  attributs = fenetre->attributs;

  /* Choix du visual */
  if ( NULL == visual ) { /* On utilise le visual par dfaut du programme */
    visual_reel = BI_Visual;
    n_plans = CopyFromParent;
#ifdef _DEVELOPPE_
    /*    printf("Visual finalement choisi = celui de l'cran, par dfaut\n"); */
#endif
  } else { /* Essentiellement utilis par Open GL */
    visual_reel = visual->visual;
    n_plans = visual->depth;
#ifdef _DEVELOPPE_
    /*    printf("Visual finalement choisi (pas celui de l'cran)\n");
	  printf(" fentre racine associe : %ld\n", RootWindow(BI_Display, visual->screen));
	  printf(" visual associ : %p - id = %ld\n", visual->visual, visual->visualid);
	  printf(" profondeur : %d\n", visual->depth);
	  printf(" type : %d\n", visual->class); */
#endif
  }

  /* On essaye d'ouvrir un contexte d'entre des caractres */
  if ( ( BI_Methode_entree != NULL ) && ( BI_FonteUTF8 != NULL ) ) {
    XVaNestedList options_entree;

    options_entree = XVaCreateNestedList( 0, XNFontSet,  BI_FonteUTF8, NULL );
    fenetre->xic = XCreateIC( BI_Methode_entree, /* La mthode d'entre active au lancement */
			      XNInputStyle       , BI_Methode_choisie, /* Le style d'entre retenu */
			      XNPreeditAttributes, options_entree,
			      XNStatusAttributes , options_entree,
			      NULL );
    XFree( options_entree );
    if ( NULL == fenetre->xic ) {
      (void) fprintf( stderr, "%s,%s l.%u : Impossible d'ouvrir un contexte d'entre\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      fenetre->evenements_xic = NoEventMask;
    } else {
      char *retour;

      /* Obtenir la liste des vnements que doit recevoir la mthode d'entre */
      retour = XGetICValues( fenetre->xic, XNFilterEvents, &(fenetre->evenements_xic), NULL );
      if ( retour != NULL ) {
	(void) fprintf( stderr, "%s,%s l.%u : Erreur en voulant obtenir la liste des vnements de l'IC\n",
			__FILE__, __FUNCTION__, __LINE__ );
	fenetre->evenements_xic = NoEventMask;
      }

      fenetre->evenements_xic |= FocusChangeMask;
    }
  } else {
    fenetre->xic = NULL;
    fenetre->evenements_xic = NoEventMask;
  }

  /* On tche d'ouvrir la fentre */
  parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    | StructureNotifyMask
    | PointerMotionMask | EnterWindowMask | LeaveWindowMask
    | ButtonReleaseMask | ExposureMask | fenetre->evenements_xic; /* vnements transmis */

  masque_attr = CWEventMask;
  if ( palette != NULL ) { /* Palette spciale <=> OpenGL */
#ifdef _DEVELOPPE_
    /*    printf("On installe la palette : %lu\n", *palette); */
#endif
    masque_attr |= CWColormap | CWBackPixmap;
    parametres.background_pixmap = None;
    parametres.colormap = *palette;
  } else {
    masque_attr |= CWBackPixel;
    /*    parametres.event_mask |= ColormapChangeMask; */
    parametres.background_pixel = BI_Couleurs[ GRIS ].pixel; /* Fond en blanc */
  }

  handle = XCreateWindow( BI_Display, mere, fenetre->x, fenetre->y,
			  fenetre->larg, fenetre->haut, 0, n_plans,
			  InputOutput, visual_reel, masque_attr, &parametres );
  if ( 0 == handle ) { /* Fentre n'a pas pu tre cre */
    (void) fprintf( stderr, "%s,%s l.%u : erreur dans XCreateWindow !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return -2;
  }
  fenetre->handle = handle;
  if ( palette != NULL ) {
    XSetWindowColormap( BI_Display, handle, *palette );
  }

  /* On installe la mthode choisie pour l'entre des donnes */
  if ( fenetre->xic != NULL ) {
    char *retour;

    retour = XSetICValues( fenetre->xic, XNClientWindow, fenetre->handle, NULL );
    if ( retour != NULL ) {
      (void) fprintf( stderr, "%s,%s l.%u : Erreur en voulant obtenir la liste des vnements de l'IC\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      fenetre->evenements_xic = NoEventMask;
    }
  }

  /* On installe une proprit : WM_DELETE_WINDOW, qui vite que le programme soit dtruit
     quand l'utilisateur demande de fermer une fentre par le WM (menu "close") */

  if ( BI_ProtocoleFermerFenetre != (Atom) None ) {
    if ( XSetWMProtocols( BI_Display, handle, &BI_ProtocoleFermerFenetre, 1 ) == 0 ) {
      (void) fprintf( stderr, "%s,%s l.%u : impossible de dfinir WM_DELETE_WINDOW\n",
		      __FILE__, __FUNCTION__, __LINE__ );
    }
  } else {
    (void) fprintf( stderr, "%s,%s l.%u : attention, fentre non protge contre la fermeture par le WM !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  if ( BI_larg_car > BI_haut_car ) /* Plus grande des 2 dimensions */
    unit_car = BI_larg_car;
  else
    unit_car = BI_haut_car;

  parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    /*  | PointerMotionMask | EnterWindowMask | LeaveWindowMask */
    | ButtonReleaseMask | ExposureMask; /* vnements transmis */
  parametres.do_not_propagate_mask = PointerMotionMask | EnterWindowMask | LeaveWindowMask;

#ifdef XGEM_AVEC_CLOSER_
  if ( attributs & CLOSER ) { /* Bouton de fermeture */
    parametres.background_pixel = BI_Couleurs[GRIS_FONCE].pixel;

    fenetre->closer = XCreateWindow( BI_Display, handle, 0, 0, unit_car, unit_car,
				     0, CopyFromParent, InputOutput, visual_reel,
				     CWBackPixel | CWEventMask, &parametres );
  } else
    fenetre->closer = 0;
#endif

  if ( attributs & NAME ) { /* Il y a un titre  */
    if ( NULL == fenetre->titre ) { /* Pas de titre */
      FEN_titre( fenetre, "LASE pour X_Window, v. 3.2 - (C) 1998-2003, Emmanuel CURIS" );
    } else {
      char *ex_titre = fenetre->titre;

      /* On le spare de la fentre, car FEN_titre() libre fenetre->titre
	 => on essaierai de copier ce que l'on vient de librer ! */
      fenetre->titre = NULL;
      FEN_titre( fenetre, ex_titre );
      free( ex_titre ); /* On peut maintenant le librer */
    }
  }

  if ( attributs & INFO ) { /* Il y a des infos */
    int x_inf = 0;
    unsigned int largeur; /* hauteur : pour accueillir la fonte */

    largeur  = fenetre->larg;
#ifdef XGEM_AVEC_CLOSER_
    if ( attributs & CLOSER ) { /* Il y a aussi un bouton de fermeture */
      x_inf += unit_car + 1; /* Bouton de fermeture carr */
      largeur -= unit_car + 1;
    }
#endif
    if ( ( attributs & VSLIDE ) || ( attributs & UPARROW ) )
      largeur -= unit_car;
    parametres.background_pixel = BI_Couleurs[GRIS].pixel;

    fenetre->barre_d_infos = XCreateWindow( BI_Display, handle,
					    x_inf, 0, largeur, unit_car,
					    0, CopyFromParent, InputOutput, visual_reel,
					    CWBackPixel | CWEventMask, &parametres );
    if ( None == fenetre->barre_d_infos ) {
      (void) fprintf( stderr, "%s,%s l.%u : Impossible de crer la barre d'informations\n",
		      __FILE__, __FUNCTION__, __LINE__ );
    } else {
      XMapWindow( BI_Display, fenetre->barre_d_infos );
    }
#ifdef _DEVELOPPE_
/*     FEN_infos( fenetre, "Barre d'informations"); */
#endif
  } else {
    fenetre->barre_d_infos = None; /* Aucune fentre associe */
  }

  if ( attributs & UPARROW ) {
    int x_fl = 0, y_fl = 0;

    x_fl = fenetre->larg - unit_car;

    parametres.background_pixel = BI_Couleurs[GRIS].pixel;
    fenetre->fleche_haute = XCreateWindow( BI_Display, handle, x_fl, y_fl, unit_car, unit_car,
					   0, CopyFromParent, InputOutput, visual_reel,
					   CWBackPixel | CWEventMask, &parametres );
    if ( None == fenetre->fleche_haute ) {
      (void) fprintf( stderr, "%s,%s l.%u : Impossible de crer la flche vers le haut\n",
		      __FILE__, __FUNCTION__, __LINE__ );
    } else {
      XMapWindow( BI_Display, fenetre->fleche_haute );
    }
  } else
    fenetre->fleche_haute = 0;

  if ( attributs & VSLIDE ) {
    int x_asc = 0, y_asc = 0;
    unsigned int haut_asc;

    x_asc = fenetre->larg - unit_car;
    haut_asc = fenetre->haut;

    if ( attributs & UPARROW ) {
      y_asc += unit_car;
      haut_asc -= unit_car;
    }

    if ( attributs & DNARROW )
      haut_asc -= unit_car;

    if ( attributs & (ASC_HORIZONTAL) )
      haut_asc -= unit_car;

    parametres.background_pixel = BI_Couleurs[GRIS_FONCE].pixel;
    fenetre->fond_asc_vertical = XCreateWindow(BI_Display, handle,
					       x_asc, y_asc, unit_car, haut_asc,
					       0, CopyFromParent, InputOutput, visual_reel,
					       CWBackPixel | CWEventMask, &parametres);
    if ( None == fenetre->fond_asc_vertical ) {
      (void) fprintf(stderr, "FEN_creation : Impossible de crer l'ascenseur vertical\n");
      fenetre->bouton_asc_vertical = None;
    } else {
      XMapWindow( BI_Display, fenetre->fond_asc_vertical );

      parametres.background_pixel = BI_Couleurs[GRIS].pixel;
      fenetre->bouton_asc_vertical = XCreateWindow(BI_Display, fenetre->fond_asc_vertical,
						   0, 0, unit_car, haut_asc,
						   0, CopyFromParent, InputOutput, visual_reel,
						   CWBackPixel | CWEventMask, &parametres);
      if ( None == fenetre->bouton_asc_vertical ) {
	(void) fprintf(stderr, "FEN_creation : Impossible de crer le poussoir de l'ascenseur vertical\n");
	fenetre->bouton_asc_vertical = None;
      } else {
	XMapWindow( BI_Display, fenetre->bouton_asc_vertical );
      }
    }

  } else {
    fenetre->fond_asc_vertical = None;
    fenetre->bouton_asc_vertical = None;
  }

  if ( attributs & DNARROW ) {
    int x_fl, y_fl;

    x_fl = fenetre->larg - unit_car;
    y_fl = fenetre->haut - unit_car;

    if ( attributs & (ASC_HORIZONTAL) )
      y_fl -= unit_car;
    parametres.background_pixel = BI_Couleurs[GRIS].pixel;
    fenetre->fleche_basse = XCreateWindow(BI_Display, handle, x_fl, y_fl, unit_car, unit_car,
					  0, CopyFromParent, InputOutput, visual_reel,
					  CWBackPixel | CWEventMask, &parametres);
    if ( None == fenetre->fleche_basse ) {
      (void) fprintf(stderr, "FEN_creation : Impossible de crer la flche vers le bas\n");
    } else {
      XMapWindow( BI_Display, fenetre->fleche_basse );
    }
  } else
    fenetre->fleche_basse = None;

  if ( attributs & LFARROW ) {
    int x_fl = 0, y_fl;

    y_fl = fenetre->haut - unit_car;

    parametres.background_pixel = BI_Couleurs[GRIS].pixel;
    fenetre->fleche_gauche = XCreateWindow(BI_Display, handle, x_fl, y_fl, unit_car, unit_car,
					   0, CopyFromParent, InputOutput, visual_reel,
					   CWBackPixel | CWEventMask, &parametres);
    if ( None == fenetre->fleche_gauche ) {
      (void) fprintf(stderr, "FEN_creation : Impossible de crer la flche vers la gauche\n");
    } else {
      XMapWindow( BI_Display, fenetre->fleche_gauche );
    }
  } else
    fenetre->fleche_gauche = 0;

  if (attributs & HSLIDE) {
    int x_asc = 0, y_asc;
    unsigned int larg_asc;

    y_asc = fenetre->haut - unit_car;
    larg_asc = fenetre->larg;

    if (attributs & ASC_VERTICAL) {
      larg_asc -= unit_car;
    }

    if (attributs & LFARROW) {
      x_asc += unit_car;
      larg_asc -= unit_car;
    }

    if (attributs & RTARROW)
      larg_asc -= unit_car;

    parametres.background_pixel = BI_Couleurs[GRIS_FONCE].pixel;
    fenetre->fond_asc_horizontal = XCreateWindow(BI_Display, handle,
						 x_asc, y_asc, larg_asc, unit_car,
						 0, CopyFromParent, InputOutput, visual_reel,
						 CWBackPixel | CWEventMask, &parametres);
    if ( None == fenetre->fond_asc_horizontal ) {
      (void) fprintf(stderr, "FEN_creation : Impossible de crer l'ascenseur horizontal\n");
      fenetre->bouton_asc_horizontal = None;
    } else {
      XMapWindow( BI_Display, fenetre->fond_asc_horizontal );

      parametres.background_pixel = BI_Couleurs[GRIS].pixel;
      fenetre->bouton_asc_horizontal = XCreateWindow(BI_Display, fenetre->fond_asc_horizontal,
						     0, 0, larg_asc, unit_car,
						     0, CopyFromParent, InputOutput, visual_reel,
						     CWBackPixel | CWEventMask, &parametres);
      if ( None == fenetre->bouton_asc_horizontal ) {
	(void) fprintf(stderr, "FEN_creation : Impossible de crer le poussoir de l'ascenseur vertical\n");
	fenetre->bouton_asc_horizontal = None;
      } else {
	XMapWindow( BI_Display, fenetre->bouton_asc_horizontal );
      }
    }
  } else {
    fenetre->fond_asc_horizontal = None;
    fenetre->bouton_asc_horizontal = None;
  }

  if ( attributs & RTARROW ) {
    int x_fl, y_fl;

    x_fl = fenetre->larg - unit_car;
    y_fl = fenetre->haut - unit_car;

    if ( attributs & (ASC_VERTICAL) )
      x_fl -= unit_car;
    parametres.background_pixel = BI_Couleurs[GRIS].pixel;
    fenetre->fleche_droite = XCreateWindow(BI_Display, handle, x_fl, y_fl, unit_car, unit_car,
					   0, CopyFromParent, InputOutput, visual_reel,
					   CWBackPixel | CWEventMask, &parametres);
    if ( None == fenetre->fleche_droite ) {
      (void) fprintf(stderr, "FEN_creation : Impossible de crer la flche vers la droite\n");
    } else {
      XMapWindow( BI_Display, fenetre->fleche_droite );
    }
  } else
    fenetre->fleche_droite = None;

  return 0; /* Tout s'est bien pass */
}

/* -------- Initialisation de la fentre [partie spcifique X] -------- */

int FEN_InitialiserFenetre( Fenetre *fenetre )
{
  int unit_car;
  int attributs;

  if ( NULL == fenetre ) return -2;

  /* Pas encore de fentre X-Window */
  fenetre->handle = None;

  /* Calcul des dimensions internes */
  attributs = fenetre->attributs;

  if ( BI_larg_car > BI_haut_car ) /* Plus grande des 2 dimensions */
    unit_car = BI_larg_car;
  else
    unit_car = BI_haut_car;

  /* Zone de travail : par dfaut, toute la fentre sauf ventuellement les attributs */
  fenetre->w_x = 0; fenetre->w_y = 0;
  fenetre->w_larg = BI_larg_defaut; fenetre->w_haut = BI_haut_defaut; /* Zone de travail */
#ifdef XGEM_AVEC_CLOSER_
  if ( ( attributs & CLOSER ) || ( attributs & INFO ) ) {
    fenetre->w_y    += unit_car;
    fenetre->w_haut -= unit_car; /* On perd tout a comme place */
  }
#else
  if ( attributs & INFO ) {
    fenetre->w_y    += unit_car;
    fenetre->w_haut -= unit_car; /* On perd tout a comme place */
  }
#endif
  if ( attributs & ASC_VERTICAL ) { /* Il y a un ascenseur vertical */
    fenetre->w_larg -= unit_car;
  }
  if ( attributs & ASC_HORIZONTAL ) { /* Il y a un ascenseur horizontal */
    fenetre->w_haut -= unit_car;
  }

  if ( BI_FENETRE_GL == fenetre->type ) {
    int retour;

    retour = FEN_InitialiserOpenGL( fenetre );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans FEN_InitialiserOpenGL() [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return -1;
    }
  }

  /* L'initialisation a russi */
  return 0;
}

/* -------- Initialisation des informations OpenGL de la fentre -------- */

static int FEN_InitialiserOpenGL(Fenetre *fenetre)
{
  BI_infos_GL *infos;

  if ( fenetre->open_gl != NULL ) {
    (void) fprintf(stderr, "FEN_InitialisationOpenGL : La fentre a dj des infos OpenGL associes\n");
    return -3;
  }

  /* On rserve les infos OpenGL */
  infos = (BI_infos_GL *) malloc( sizeof(BI_infos_GL) );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "FEN_InitialisationOpenGL : Impossible de rserver les infos OpenGL\n");
    return -2;
  }

  /* On les initialise */
/*   infos->handle = None; */
  infos->fond  = None;
#ifdef AVEC_OPEN_GL_
  infos->image = None;
#endif
  infos->mode  = BI_FEN_ARBRE_AUCUN;
  infos->refaire = VRAI; /* Premier redessin obligatoirement entier */
#ifdef AVEC_OPEN_GL_
  infos->gl_contexte = NULL;
#endif

  fenetre->redessin = FEN_GL_redessin_defaut;
  infos->rds_post = ( void (*)(struct fenetre *, int, int, unsigned int, unsigned int) ) NULL;
  infos->deplace  = ( boolean (*)(struct fenetre *, int, int, int, int) ) NULL;
  infos->temps    = ( boolean (*)(struct fenetre *) ) NULL;

  /* On associe les infos  la fentre */
  fenetre->open_gl = infos;

  /* L'initialisation a russi */
  return 0;
}

/* -------- Imposer la taille d'une fentre X -------- */

int FEN_ImposerTaille(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  if ( fenetre->taille ) {
    AVERTISSEMENT( "Il existe dj des informations de taille pour cette fentre [%p] !", (void *) fenetre );
    return +1;
  }

  /* x<0 ou y<0 : on centre pour la coordonn correspondante */
  if ( x < 0 ) {
    x = ( BI_larg_ecran - larg ) >> 1;
  }

  if ( y < 0 ) {
    y = ( BI_haut_ecran - haut ) >> 1;
  }

  /* Largeur ou hauteur nulle : on ne modifie pas */
  if ( 0 == larg ) larg = fenetre->larg;
  if ( 0 == haut ) haut = fenetre->haut;

  /* On impose la taille, v. X-Window */
  if ( NULL == fenetre->taille ) {
    fenetre->taille = XAllocSizeHints();
  }
  if ( fenetre->taille != NULL ) { /* A pu tre cr */
    /* Placement des fentres : il faut
        - le flag PPosition avec les positions [mais seul : place en (0,0)]
	- l'appel  XMoveWindow [mais seul : ignor par le WM]
    */
    fenetre->taille->flags = PPosition | PMinSize | PMaxSize; /* On ne peut pas en changer la taille */
    fenetre->taille->x = x;
    fenetre->taille->y = y;
    fenetre->taille->min_width  = larg;
    fenetre->taille->min_height = haut;
    fenetre->taille->max_width  = larg;
    fenetre->taille->max_height = haut;

    XSetWMNormalHints( BI_Display, fenetre->handle, fenetre->taille );
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Fentre %s place en x=%d, y=%d\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->titre, x, y );
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver les SizeHints pour la fentre %p\n",
		   __FILE__, __FUNCTION__, __LINE__, (void *) fenetre );
#endif
    return -1;
  }
  XMoveWindow( BI_Display, fenetre->handle, x, y );

  /* On adapte les lments de fentre, si ncessaire */
  change_taille_fenetre( fenetre, larg, haut );

  /* On mmorise ces nouvelles valeurs */
  fenetre->x = x;
  fenetre->y = y;
  fenetre->larg = larg;
  fenetre->haut = haut;

  /* la taille a pu tre impose */
  return 0;
}

#ifdef AVEC_OPEN_GL_
static void installe_colormap(Fenetre *fenetre)
{
  XSetWMColormapWindows(BI_Display, fenetre->handle, &fenetre->handle, 1);
}
#endif

void OBJ_creer_fenetres(Arbre *arbre, Window mere, int objet, int obj_pere)
{
  OBJECT *adresse;
  XSetWindowAttributes attributs;
  int x, y;
  unsigned int larg, haut;
  unsigned int bord = 0, couleur_bord = NOIR, motif;
  unsigned long liste_attributs = CWEventMask | CWBorderPixel | CWBackPixel;
  signed char ep_bord;

  if ( NULL == arbre->objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : PAS de table pour les fentres X associes aux objets.\n    Appel  ARB_Initialiser oubli ?\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }
  if ( arbre->objets[ objet ] != None ) {
    (void) fprintf(stderr, "%s, %s l.%u : Objet %u dj cr ! Problme probable dans la structure de l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__, objet);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  adresse = arbre->adresse; /* Adresse de l'arbre */
  if (NULL == adresse) {
    (void) fprintf(stderr, "%s, %s l.%u : adresse de l'arbre nulle !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    (void) fprintf(stderr, "  arbre : %p - soit le numro %u [sur %u]\n", (void *) arbre, (unsigned int) (arbres - arbre), BI_nbr_arbres - 1);
    (void) fprintf(stderr, "  objet : %d de pre %d - fentre mre : %lu\n", objet, obj_pere, mere);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

#ifdef _DEVELOPPE_
/*   (void) fprintf(stderr, "%s,%s l.%u :\n", __FILE__, __FUNCTION__, __LINE__); */
/*   (void) printf("Objet : %d [X=%d, Y=%d, L=%u, H=%u], 1er fils = %d, dernier fils = %d, suivant = %d\n", */
/* 		objet, OB_X( adresse, objet ), OB_Y( adresse, objet ), OB_WIDTH(adresse, objet), OB_HEIGHT(adresse, objet), */
/* 		OB_HEAD( adresse, objet ), OB_TAIL( adresse, objet ), OB_NEXT( adresse, objet ) ); */
/*   if ( obj_pere > 0 ) { */
/*     (void) printf("Pre  : %d [X=%d, Y=%d, L=%u, H=%u], 1er fils = %d, dernier fils = %d, suivant = %d\n", */
/* 		  obj_pere, OB_X( adresse, obj_pere ), OB_Y( adresse, obj_pere ), OB_WIDTH(adresse, obj_pere), OB_HEIGHT(adresse, obj_pere), */
/* 		  OB_HEAD( adresse, obj_pere ), OB_TAIL( adresse, obj_pere ), OB_NEXT( adresse, obj_pere ) ); */
/*   } */
#endif

  attributs.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    | ButtonReleaseMask | ExposureMask; /* Tous vnements transmis */
  attributs.background_pixmap = None;
  attributs.background_pixel = BI_Couleurs[GRIS].pixel; /* Fond gris clair par dfaut */

  x    = OB_X(adresse, objet);
  y    = OB_Y(adresse, objet);
  larg = OB_WIDTH(adresse, objet);
  haut = OB_HEIGHT(adresse, objet);

  switch ( OB_TYPE(adresse, objet) )
  {
  case G_BOX:
  case G_BOXCHAR:
    couleur_bord = (unsigned int) ( (OB_SPEC(adresse, objet) & 0xF0) >> 12 ); /* Couleur bord */
    ep_bord = (signed char) ( (OB_SPEC(adresse, objet) & 0xFF00) >> 16 ); /* paisseur bord */
    if (ep_bord > 0) { /* paisseur positive -> vers l'extrieur */
      bord = (unsigned int) ep_bord;
      bord --;
    } else
      bord = (unsigned int) (-ep_bord);

    motif = adresse[ objet ].ob_spec.gbox.motif;
    if ( motif == 0 ) /* Motif de remplissage = blanc */
      attributs.background_pixel = BI_Couleurs[ BLANC ].pixel;
    else if ( motif < 7 ) /* Pas plein */
      attributs.background_pixel = BI_Couleurs[ GRIS_FONCE ].pixel;

    bord = 0; /* Carrment plus joli... */
    if (x > 0) {x--; larg++;}
    if (y > 0) {y--; haut++;}
    break;

  case G_USERDEF:
    bord = 0;
    couleur_bord = GRIS;
    break;

  case G_TEXT:  /* Texte ou texte ditable, sans cadre */
  case G_FTEXT:
    bord = 0;
    if (x > 0) {x--; larg++;}
    if (y > 0) {y--; haut++;}
    break;

  case G_BOXTEXT:
  case G_FBOXTEXT: /* Texte ou texte ditable encadr */
    bord = 0;
    break;

  case G_IBOX:
    bord = 1;
    if ( x > 0 ) { x--; }
    if ( y > 0 ) { y--; } 
    break; 

  case G_BUTTON:
    bord = 0;
    if (x > 0) {x--; larg++;}
    if (y > 0) {y--; haut++;}
    break;

  case G_STRING:
    if (y > 0) {y--; haut++;}
  case G_SEPARATEUR:
    bord = 0; /* Pas de bord */
    break;

  case G_ICON:
  case G_IMAGE:
  case G_CICON:
#ifdef AVEC_XPM_
    if ( adresse[objet].ob_spec.icone_xpm != NULL ) {
      Pixmap icone;
      unsigned int lg_icone, ht_icone;

      /* On adapte la taille  celle de l'icne */
      sscanf(adresse[objet].ob_spec.icone_xpm[0], "%u %u", &lg_icone, &ht_icone);
      if (lg_icone < larg)
	larg = lg_icone;
      if (ht_icone < haut)
	haut = ht_icone;
      XpmCreatePixmapFromData(BI_Display, mere, adresse[objet].ob_spec.icone_xpm, &icone, NULL, NULL);

      attributs.background_pixmap = icone;
      liste_attributs |= CWBackPixmap;
      liste_attributs &= ~CWBackPixel;
    }
#endif
    break;

  case G_STYLELIGNE:
  case G_STYLEMARQUE:
  case G_SSMENU:
    couleur_bord = ROUGE_FONCE;
    break;

  case G_TITLE:
  case G_TXT_INFINI:
  case G_NOMBRE:
    break;

  default:
    (void) fprintf(stderr, "OBJ_creer_fenetre : type d'objet inconnu [%d]\n", OB_TYPE(adresse, objet) );
  }

  if ( (OB_FLAGS(adresse, objet) & EXIT) || (OB_FLAGS(adresse, objet) & TOUCHEXIT) ) {
    bord += 1;
    if ( x > 0 ) x--;
    if ( y > 0 ) y--;
  }

  if ( OB_FLAGS(adresse, objet) & DEFAULT ) {
    bord += 1;
    if ( x > 0 ) x--;
    if ( y > 0 ) y--;
  }

  /* Ombr utilis pour les sous-menus => reprs ici par un bord rouge sombre */
  if ( OB_STATE(adresse, objet) & SHADOWED ) {
    couleur_bord = ROUGE_FONCE;
    if (bord < 1) {
      bord = 1;
      if ( x > 0 ) x--;
      if ( y > 0 ) y--;
    }
  }

  /* On cr la fentre de l'objet */
  attributs.border_pixel = BI_Couleurs[ couleur_bord ].pixel;
  if ( larg <= 0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s, %s l.%u : Objet de largeur nulle : %d [arbre %p = %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, objet, (void *) arbre, (unsigned int) (arbre - arbres) );
#endif
    larg = 2;
  }
  if (haut <= 0) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "*** Objet de hauteur nulle : c'est interdit ***\n");
    (void) fprintf(stderr, "Objet numro %d de l'arbre %p\n", objet, (void *) arbre);
#endif
    haut = 2;
  }
#ifdef _DEVELOPPE_
  if ( ( x < 0 ) || ( x > (int) OB_WIDTH(adresse, obj_pere) ) ||
       ( y < 0 ) || ( y > (int) OB_HEIGHT(adresse, obj_pere) ) ) {
    (void) fprintf(stderr, "%s,%s l.%u :\n *** Objet dans une position absurde, arbre %p [%u, %u objets] ***\n",
		   __FILE__, __FUNCTION__, __LINE__, (void *) arbre, (unsigned int) (arbre - arbres), arbre->nbr_objets );
    (void) fprintf(stderr, "Objet : %d [X=%d, Y=%d, L=%u, H=%u], 1er fils = %d, dernier fils = %d, suivant = %d \n",
		   objet, OB_X( adresse, objet ), OB_Y( adresse, objet ), OB_WIDTH(adresse, objet), OB_HEIGHT(adresse, objet),
		   OB_HEAD( adresse, objet ), OB_TAIL( adresse, objet ), OB_NEXT( adresse, objet ) );
    (void) fprintf(stderr, "Pre  : %u [X=%d, Y=%d, L=%u, H=%u], 1er fils = %d, dernier fils = %d, suivant = %d\n",
		   obj_pere, OB_X( adresse, obj_pere ), OB_Y( adresse, obj_pere ), OB_WIDTH(adresse, obj_pere), OB_HEIGHT(adresse, obj_pere),
		 OB_HEAD( adresse, obj_pere ), OB_TAIL( adresse, obj_pere ), OB_NEXT( adresse, obj_pere ) );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
  }
#endif

  (arbre->objets)[objet] = XCreateWindow(BI_Display, mere, x, y, larg, haut, bord,
					 CopyFromParent, InputOutput, CopyFromParent,
					 liste_attributs, &attributs);
  if (attributs.background_pixmap != None)
    XFreePixmap(BI_Display, attributs.background_pixmap);

  if ( !(OB_FLAGS(adresse, objet) & HIDETREE) )   /* Objet visible */
    XMapWindow(BI_Display, (arbre->objets)[objet]); /* On affiche donc la fentre */
#ifdef _DEVELOPPE_
  else {
    (void) printf("OBJ_creer_fenetres : objet invisible [n%u]\n", objet);
  }

/*   printf(" ---> fentre X : %lu = %lx\n", (arbre->objets)[ objet ], (arbre->objets)[ objet ] ); */
#endif

  /* S'il y a des fils */
  if ( OB_HEAD(adresse, objet) != -1 ) {
    if ( ROOT == OB_HEAD(adresse, objet) ) {
      (void) fprintf(stderr, "%s, %s l.%u : Structure d'arbre incorrecte, ROOT ne peut tre fils de l'objet %u [arbre %p, %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, objet, (void *) arbre, (unsigned int) (arbre - arbres) );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }
    OBJ_creer_fenetres(arbre, (arbre->objets)[objet], OB_HEAD(adresse, objet), objet);
  }

  /* S'il y a des objets de mme niveau */
  if (  ( OB_NEXT(adresse, objet) != -1 ) /* un objet  la suite */
     && ( OB_NEXT(adresse, objet) != obj_pere) ) { /* et ce n'est pas le pre */
    if ( ROOT == OB_NEXT(adresse, objet) ) {
      (void) fprintf(stderr, "%s, %s l.%u : Structure d'arbre incorrecte, ROOT ne peut tre l'objet suivant sans tre le pre [objet %u, arbre %p = n%u]\n",
		     __FILE__, __FUNCTION__, __LINE__, objet, (void *) arbre, (unsigned int) (arbre - arbres) );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }
    OBJ_creer_fenetres(arbre, mere, OB_NEXT(adresse, objet), obj_pere);
  }
}

/* -------- Cration des onglets -------- */

int BI_Onglets_Creation(Fenetre *fenetre, unsigned int nombre_onglets, BI_Onglet *onglets)
{
  struct fen_onglets *infos_onglets = NULL;
  unsigned int i, largeur_onglet;
  XSetWindowAttributes parametres; /* De la fentre X correspondant  un onglet */
  unsigned long masque_attr;

  if ( ( 0 == nombre_onglets ) || ( NULL == onglets ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun onglet indiqu ! [n=%u, p=%p]\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre_onglets, (void *) onglets );
    return -5;
  }

  /* On rserve les infos pour les onglets */
  infos_onglets = (struct fen_onglets *) malloc( sizeof( struct fen_onglets ) );
  if ( NULL == infos_onglets ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de rserver les infos\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On rserve la table des fentres des onglets */
  infos_onglets->onglets = (Window *) calloc( nombre_onglets, sizeof( Window ) );
  if ( NULL == infos_onglets->onglets ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer la table des onglets\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( infos_onglets ); infos_onglets = NULL;
    return -2;
  }

  /* L'onglet actif est, par dfaut, le premier */
  infos_onglets->onglet_actif = 0;

  /* Quelle est la largeur d'un onglet ? */
  largeur_onglet = fenetre->w_larg / nombre_onglets;

  /* --- Paramtres des onglets ---  */
  /* a. vnements intressants : clics, entre/sortie souris, redessin */
  parametres.event_mask = ButtonPressMask | EnterWindowMask | LeaveWindowMask | ButtonReleaseMask | ExposureMask;

  /* b. Paramtres de la fentre */
  masque_attr = CWEventMask | CWBackPixel;
  parametres.background_pixel = BI_Couleurs[ BLEU ].pixel; /* Fond en gris clair [bleu pour essais] */

  /* On cre les fentres des onglets */
  for ( i = 0 ; i < nombre_onglets ; i++ ) {
    infos_onglets->onglets[ i ] = XCreateWindow( BI_Display, fenetre->handle,
						 fenetre->w_x + i * largeur_onglet, fenetre->w_y, largeur_onglet, BI_haut_car + 1,
						 0 /* Pas de bord */, CopyFromParent, InputOutput, BI_Visual,
						 masque_attr, &parametres );
    if ( 0 == infos_onglets->onglets[ i ] ) {
      (void) fprintf( stderr, "%s, %s l.%u : La fentre pour l'onglet %u n'a pas pu tre cre !\n",
		      __FILE__, __FUNCTION__, __LINE__, i );
      OPTION_A_FAIRE( "Fermer les fentres X" );
      free( infos_onglets->onglets ); infos_onglets->onglets = NULL;
      free( infos_onglets ); infos_onglets = NULL;
      return -20;
    }

    /* On rend visible la fentre de l'onglet */
    XMapWindow( BI_Display, infos_onglets->onglets[ i ] );

    /* On en profite pour rendre invisible la fentre de l'arbre associ */
    XUnmapWindow( BI_Display, fenetre->ss_fenetres[ i ]->handle );
  }

  /* On associe les infos spcifiques au DEUXIME complment de la fentre */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos_onglets );

  /* On active le premier onglet */
  XMapWindow( BI_Display, fenetre->ss_fenetres[ 0 ]->handle );
  if ( arbres[ onglets[ 0 ].numero_arbre ].objets != NULL ) {
    XMapWindow( BI_Display, arbres[ onglets[ 0 ].numero_arbre ].objets[ ROOT ] );
#ifdef _DEVELOPPE_
  } else {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de table de fentres X pour l'arbre %u [onglet %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, onglets[ 0 ].numero_arbre, 0 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
#endif
  }

  /* La cration s'est bien pass */
  return 0;
}

/* -------- Cration de la partie Open-GL d'une fentre multiple -------------\
 | Ne renvoie rien...                                                         |
 |                                                                            |
 | fenetre : la fentre pour laquelle il faut crer une sous-fentre OpenGL   |
 | x_gl, y_gl       : position de la rgion OpenGL                            |
 | larg_gl, haut_gl : dimensions de cette rgion                              |
 \--------------------------------------------------------------------------- */
#ifdef ancien_
static void FEN_creation_opengl(Fenetre *fenetre, int x_gl, int y_gl, unsigned int larg_gl, unsigned int haut_gl)
{
  int retour;
#ifdef AVEC_OPEN_GL_
  Window fen_x[2];
#endif
  unsigned long masque;
  XSetWindowAttributes parametres;

  retour = FEN_InitialiserOpenGL( fenetre );
  if ( retour < 0 ) {
    (void) fprintf(stderr, "FEN_creation_opengl : ERREUR dans FEN_InitialisationOpenGL() [code %d]\n", retour);
    return;
  }
  /* Rq : code de retour positif -> OpenGL absent, mais on continue tout de mme... */

  /* Cration de la fentre OpenGL */
  masque = CWEventMask | CWColormap | CWBackPixmap | CWBackingStore;
  parametres.background_pixmap = None;
#ifdef AVEC_OPEN_GL_
  parametres.colormap = XGL_Palette;
#endif
  parametres.backing_store = WhenMapped; /* On sauve le fond pour acclrer les dessins */
  parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    | PointerMotionMask | EnterWindowMask | LeaveWindowMask
    | ButtonReleaseMask | ExposureMask; /* vnements transmis */
#ifdef _DEVELOPPE_
  printf("%s : On veut crer la fentre Open-GL\n", __FUNCTION__);
#endif
  if ( BI_OpenGL_actif == FAUX ) { /* OpenGL non support par le Display utilis */
#ifdef _DEVELOPPE_
    printf("... mais OpenGL n'est pas support par ce Display !\n");
#endif
    /* On fait une fentre avec les mmes proprits que la mre */
    fenetre->open_gl->handle
      = XCreateWindow(BI_Display, fenetre->handle, x_gl, y_gl, larg_gl, haut_gl, 0,
		      CopyFromParent, InputOutput, CopyFromParent, masque, &parametres);
#ifdef AVEC_OPEN_GL_
  } else {
# ifdef _DEVELOPPE_
    (void) printf( "%s,%s l.%u : On va appeler XCreateWindow\n",
		   __FILE__, __FUNCTION__, __LINE__);
# endif
    fenetre->open_gl->handle
      = XCreateWindow(BI_Display, fenetre->handle, x_gl, y_gl, larg_gl, haut_gl, 0, 
		      XGL_Visual.depth, InputOutput, XGL_Visual.visual, masque, &parametres);
# ifdef _DEVELOPPE_
    (void) printf( "%s,%s l.%u : XCreateWindow russi, handle = %lu\n  -> On va essayer de mettre la Colormap",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->open_gl->handle);
# endif

    fen_x[0] = fenetre->handle;
    fen_x[1] = fenetre->open_gl->handle;

    if (XSetWMColormapWindows(BI_Display, fenetre->handle, &fenetre->open_gl->handle, 1) == False) {
      (void) fprintf(stderr, "%s,%s l.%u : impossible d'indiquer la ColorMap\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }
# ifdef _DEVELOPPE_
    (void) printf( "%s,%s l.%u : XSetWMColormapWindows, OK\n",
		   __FILE__, __FUNCTION__, __LINE__);
# endif
    XSetWindowColormap(BI_Display, fenetre->open_gl->handle, XGL_Palette);
# ifdef _DEVELOPPE_
    (void) printf( "%s,%s l.%u : XSetWindowColormap, OK\n  -> On va choisir et crer le contexte Open GL !",
		   __FILE__, __FUNCTION__, __LINE__);
# endif

    /* On cre le contexte. change_taille_fenetre ne le fera pas car pas de changement de taille dans ouvre_fenetre */
    (void) FEN_GL_choisit_contexte(fenetre, BI_GL_direct);
#endif
  }
}
#endif
