/**** Bibliothque mixte GEM / X_windows
  (C) 1998-2003 Emmanuel CURIS

  Gestion des fentres - compatible gem_fen.c [rfrence]
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* -------------- Fermeture sans destruction d'une fentre ---------------\
 |  Ne renvoie rien du tout                                               |
 |                                                                        |
 |  fenetre : la fentre que l'on veut fermer                             |
 |  detruit : FAUX ==> on ne fait que fermer la fentre de l'interface    |
 |            VRAI ==> on la dtruit aussi MAIS on garde la structure     |
 \----------------------------------------------------------------------- */

void FEN_Fermer(Fenetre *fenetre, boolean detruit)
{
  if ( NULL == fenetre ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : fenetre=NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return;
  }

  if ( fenetre->arbre != NULL ) {
    (fenetre->arbre)->drapeaux.ouvert = FAUX; /* On ferme l'arbre */
  }

  if ( VRAI == fenetre->drapeaux.ouverte ) /* La fentre est effectivement ouverte */
    XUnmapWindow( BI_Display, fenetre->handle );

  if ( VRAI == detruit ) { /* On veut dtruire la fentre sans perdre les infos */
    if ( fenetre->taille != NULL ) { /* On mmorise aussi la position... */
      Status retour;
      long proprietes;

      retour = XGetWMNormalHints( BI_Display, fenetre->handle, fenetre->taille, &proprietes );
#ifdef _DEVELOPPE_
      if ( 0 == retour ) {
	(void) fprintf(stderr, "%s,%s l.%u : Problme dans XGetWMNormalHints()\n",
		       __FILE__, __FUNCTION__, __LINE__);
      } else {
/* 	(void) fprintf(stderr, "%s,%s l.%u : XGetWMNormalHints, proprietes = %ld, retour = %d\n", */
/* 		       __FILE__, __FUNCTION__, __LINE__, proprietes, retour); */
      }
#endif
    }
    FEN_DetruireFenetre( fenetre, VRAI );
  }

  fenetre->drapeaux.ouverte = FAUX; /* Fentre ferme */
}

/* -------- Dtruire la fentre pour X [et toutes les informations propres associes] -------- */

int FEN_DetruireFenetre( Fenetre *fenetre, boolean memoriser_taille )
{
  if ( NULL == fenetre ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : fenetre=NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return 0;
  }

  /* Est-ce une fentre d'onglets ? */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    struct fen_onglets *infos_onglets = NULL;

    /* On rcupre les informations internes sur les onglets */
    infos_onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
    if ( infos_onglets != NULL ) {
      /* Remarque : l'appel final  XDestroyWindow dtruira les descendantes, donc aussi celles des onglets ! */
      free( infos_onglets->onglets ); infos_onglets->onglets = NULL;
      free( infos_onglets ); infos_onglets = NULL;
    }

    /* Plus de complment 2 associ  la fentre */
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );
  }

  /* Faut-il librer un arbre associ ? */
  if ( fenetre->arbre != NULL ) {
    unsigned int i;
    Window *w = (fenetre->arbre)->objets;

    if ( w != NULL ) { /* La table des objets existe */
      for (i = 0 ; i < fenetre->arbre->nbr_objets ; i++)
	w[i] = None; /* Il n'y a plus de fentre associe, puisque l'on ferme la fentre... */
#ifdef _DEVELOPPE_
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : table des objets inexistante, mais arbre associ  la fentre\n",
		     __FILE__, __FUNCTION__, __LINE__);
#endif
    }
    
    (fenetre->arbre)->drapeaux.ouvert = FAUX; /* On ferme l'arbre */
  }

  /* Faut-il librer des informations OpenGL ? */
  if ( fenetre->open_gl != NULL) {
#ifdef AVEC_OPEN_GL_
    if ( BI_OpenGL_actif == VRAI ) {
      if ( fenetre->open_gl->image != None )
	glXDestroyGLXPixmap( BI_Display, fenetre->open_gl->image );
      if ( fenetre->open_gl->fond != None )
	XFreePixmap( BI_Display, fenetre->open_gl->fond );
      glXDestroyContext( BI_Display, fenetre->open_gl->gl_contexte );
      fenetre->open_gl->fond = None;
      fenetre->open_gl->image = None;
      fenetre->open_gl->gl_contexte = NULL;
    }
#endif
    free( fenetre->open_gl ); fenetre->open_gl = NULL;
  }

  /* On perd les informations de taille, mais seulement si on le demande */
  if ( FAUX == memoriser_taille ) {
    if ( fenetre->taille != NULL ) { /* Il y a des infos de taille */
      XFree( fenetre->taille );
      fenetre->taille = NULL;
    }
  }

  /* Remarques :
     - les sous-fentres sont libres dans FEN_LibererFenetre(), version indpendante de l'interface ! */

  /* Fermeture pour X */
  if ( fenetre->handle != None ) {
    FEN_ote_evenement( fenetre->handle ); /* Supprime tout vnement pour la fentre _et ses filles_ */

    XDestroyWindow(BI_Display, fenetre->handle); /* Supprime aussi les filles */
    fenetre->handle = None;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -- Supprime tous les vnements qui concernent une fentre et ses filles */

int FEN_ote_evenement(Window fenetre)
{
  Window pere, racine, *enfants = NULL;
  XEvent evenement;
  unsigned int nbr_enfants;
  long masque = 0xFFFFL; /* Tous les vnements, si je ne m'abuse... */

  /*   long masque = KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask 
       | EnterWindowMask | LeaveWindowMask | PointerMotionMask | PointerMotionHintMask
       | Button1MotionMask | Button2MotionMask | Button3MotionMask | Button4MotionMask
       | Button5MotionMask | ButtonMotionMask | KeymapStateMask | ExposureMask
       | VisibilityChangeMask | StructureNotifyMask | ResizeRedirectMask
       | SubstructureNotifyMask | SubstructureRedirectMask | FocusChangeMask 
       | PropertyChangeMask | ColormapChangeMask | OwnerGrabButtonMask; */

#ifdef _DEVELOPPE_
/*   static unsigned int niveau = 0U, i; */

/*   for (i = 0 ; i < niveau ; i++) */
/*     printf(" "); */
/*   printf("FEN_ote_evenement(fenetre = %lu) :\n", (unsigned long) fenetre ); */
/*   fflush( stdout ); */
/*   niveau++; */
#endif

  /* on rcupre les enfants */
  XQueryTree(BI_Display, fenetre, &racine, &pere, &enfants, &nbr_enfants);
  if ( enfants != NULL ) { /* Il y a des enfants */
    unsigned int i;

    for (i = 0 ; i < nbr_enfants ; i++)
      FEN_ote_evenement( enfants[i] ); /* On s'occupe du fils */

    XFree(enfants); /* On libre la mmoire */
  }
    
  /* On limine tout vnement se rapportant  cette fentre */
  while ( XCheckWindowEvent(BI_Display, fenetre, masque, &evenement) );

#ifdef _DEVELOPPE_
/*   niveau--; */
#endif
  return 0; /* Tout s'est bien pass */
}
