/* LASE - Logiciel d'analyse des spectres EXAFS
   (C) Emmanuel CURIS, 2004-1998

   Sauvegarde au format postscript
   L'unité de base est le 1/100 de mm = taille d'un point donc 
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "serie.h"
#include "tracer.h"
#include "ps.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~~~~ */

int PS_BlocFigure(FILE *fichier, struct infos_ps *infos, struct figure *figure, struct bloc_figure *bloc)
{
  double echelle_x __attribute__((__unused__)) = 0.0, echelle_y __attribute__((__unused__)) = 0.0;
  int retour;
  struct rect_ps rectangle;

  if ( ( NULL == figure ) || ( NULL == bloc ) ) return -1;

  /* Sauvegarde de l'état graphique : on va pouvoir jouer avec l'origine */
  (void) fprintf( fichier, "gsave\n" );

  /* Le rectangle contenant ce bloc de figure [en 1/100 mm, or larg_dessin en mm]
     Conversion en changeant l'origine et le sens de l'axe des Y : BG en % de la figure (en mm) dans le bloc, BG en 1/100 mm dans le PS
     ==> x(PS) = x_BG * larg_figure [= 100 * 0,... * mm => en 1/100 mm]
         y(PS) = y_BG * haut_figure
  */
  rectangle.x_BG = bloc->dessin.x_BG_pc * (double) figure->largeur;
  rectangle.y_BG = bloc->dessin.y_BG_pc * (double) figure->hauteur;
  rectangle.larg = bloc->dessin.larg_pc * (double) figure->largeur;
  rectangle.haut = bloc->dessin.haut_pc * (double) figure->hauteur;

  /* Mis à part cela, les paramètres de dessin sont les mêmes */
  rectangle.cadre = bloc->dessin;

  /* en tenant compte du tracé non-écran */
  rectangle.cadre.axe_X.proprietes &= ~AXE_ECRAN;
  rectangle.cadre.axe_X.debord_gauche =   0.0;
  rectangle.cadre.axe_X.debord_droite = 600.0;

  rectangle.cadre.axe_Y.proprietes &= ~AXE_ECRAN;
  rectangle.cadre.axe_Y.debord_gauche =   0.0;
  rectangle.cadre.axe_Y.debord_droite = 600.0;

#ifdef _DEVELOPPE_
  /* Contour du bloc figure en bleu pour contrôle */
  (void) fprintf( fichier, "%% Contour du bloc figure en bleu\n" );
  (void) PS_Rectangle( fichier, infos->proprietes & PS_NIVEAU_2, infos->epaisseur, BLEU, BLANC,
		       rectangle.x_BG, rectangle.y_BG, rectangle.larg, rectangle.haut );
#endif

  /* On se place au coin de ce rectangle */

  /* On dessine, en fonction du type de bloc */
  switch ( bloc->type )
  {
  case BLOCFIGURE_TYPE_GRAPHES:
    /* On dessine le cadre entourant */
    retour = PS_Cadre( fichier, infos, &rectangle );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans PS_Cadre, code %d\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return -1;
    }

    /* On dessine tous les graphes */
    if ( bloc->donnees.graphes.nbr_graphes > 0 ) {
      unsigned int erreurs = 0;
      unsigned long i;
      struct axes_graphe *inf_graphe;
      double x_min = 0.0, x_max = 0.0, y_min = 0.0, y_max = 0.0;

      for ( i = 0, inf_graphe = bloc->donnees.graphes.infos_graphe ; i < bloc->donnees.graphes.nbr_graphes ; i++, inf_graphe++ ) {
	/* Les bornes des axes */
	if ( inf_graphe->axes_secondaires & AXE_X ) {
	  OPTION_A_FAIRE( "Tracer un graphe selon l'axe secondaire en X" );
	} else {
	  if ( AXE_ValiderBornes( &(bloc->dessin.axe_X) ) != 0 ) {
	    /* Problème sur l'axe principal, en X */
	    continue;
	  }
	  x_min = bloc->dessin.axe_X.min;
	  x_max = bloc->dessin.axe_X.max;
	}

	if ( inf_graphe->axes_secondaires & AXE_Y ) {
	  OPTION_A_FAIRE( "Tracer un graphe selon l'axe secondaire en Y" );
	} else {
	  if ( AXE_ValiderBornes( &(bloc->dessin.axe_Y) ) != 0 ) {
	    /* Problème sur l'axe principal, en Y */
	    continue;
	  }
	  y_min = bloc->dessin.axe_Y.min;
	  y_max = bloc->dessin.axe_Y.max;
	}

	/* Échelles de tracé */
	echelle_x = rectangle.larg / ( x_max - x_min );
	echelle_y = rectangle.haut / ( y_max - y_min );

	/* Doit-on tracer les erreurs ? */
	if ( ( VRAI == inf_graphe->graphes->drapeaux.erreur_x ) && ( inf_graphe->graphes->erreur_x != NULL ) ) {
	  erreurs |= AXE_X;
	}
	if ( ( VRAI == inf_graphe->graphes->drapeaux.erreur_y ) && ( inf_graphe->graphes->erreur_y != NULL ) ) {
	  erreurs |= AXE_Y;
	}

	/* Tracé proprement dit */
	retour = PS_Graphe( fichier, infos, &rectangle, inf_graphe->graphes, erreurs, -1, -1, -1 );
	if ( retour < 0 ) {
	  return -10; /* Erreur grave survenue */
	}
      }
    }
    break;

  default:
    OPTION_A_FAIRE( "Bloc de type ..." );
    break;
  }

  (void) fprintf(fichier, "grestore\n");
  return 0;
}
