/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 1999 Emmanuel CURIS
   Cration : 8 fvrier 1999

   Export des rsultats  divers formats

   Export d'un graphe sous forme de fichier texte ASCII
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "fichiers.h"
#include "graphes.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Export proprement dit ~-~-~-~-~-~-~-~-~ */

/* -------- Export d'un graphe rel ---------------------------\
 | Place les bons lments dans les bonnes colonnes            |
 |  si un numro de colonne vaut 0, l'info n'est pas utilise  |
 |  si une colonne n'est pas utilise, on y met 0              |
 |  utilise les fins de ligne PC                               |
 |  s'arrange pour avoir au moins 6 chiffres significatifs     |
 |                                                             |
 | fichier : le fichier o crire                              |
 | graphe  : le graphe  exporter, suppos rel                |
 | c_x     : numro de colonne o mettre les X                 |
 | c_y     : numro de colonne o mettre les Y                 |
 | c_ex    : numro de colonne o mettre les erreurs sur les X |
 | c_ey    : numro de colonne o mettre les erreurs sur les Y |
 | total   : nombre total de colonnes  mettre                 |
 \------------------------------------------------------------ */

int EA_GrapheReel(FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes, unsigned int nc_X, unsigned int nc_Y, unsigned int nc_sX, unsigned int nc_sY)
{
  double *x, *y, *x_err = NULL, *y_err = NULL;
  int n_ecrit;
  unsigned int i;
  unsigned long j;

  if ( ( NULL == fichier ) || ( NULL == graphe ) ) return -1;

  if ( separateur < ' ' ) separateur = '\t';

  x = graphe->x; y = graphe->y;
  if ( graphe->erreur_x != NULL ) x_err = graphe->erreur_x->y;
  if ( graphe->erreur_y != NULL ) y_err = graphe->erreur_y->y;

  /* Une ligne par point dans le graphe */
  for ( j = 0 ; j < graphe->nombre_point ; j++, x++, y++ ) {
    for (i = 1 ; i <= nbr_colonnes ; i++) {
      double valeur = 0.0;

      if ( i == nc_X )
	valeur = *x;
      else if ( i == nc_Y )
	valeur = *y;
      else if ( ( i == nc_sX ) && ( x_err != NULL ) ) {
	valeur = *x_err;
	x_err++;
      } else if ( (i == nc_sY) && (y_err != NULL) ) {
	valeur = *y_err;
	y_err++;
      }

      /* Le but est une relecture par un ordinateur -> sauvegarde brutale avec 10 chiffres, notation scientifique */
      n_ecrit = fprintf( fichier, "%.10e", valeur );
      if (n_ecrit <= 0) {/* On n'a rien crit */
	(void) SYS_ERREUR(ERREUR_ECRITURE, "Fichier en cours", fichier, NULL);
	return -20;
      }

      if ( i < nbr_colonnes ) fputc( separateur, fichier );
    }

    /* Ligne termine */
    (void) fprintf(fichier, "\r\n");
  }

  /* Tout s'est bien pass... */
  return 0;
}

/* -------- Export d'un graphe complexe ---------------------------\
 | Place les bons lments dans les bonnes colonnes                |
 |  si un numro de colonne vaut 0, l'info n'est pas utilise      |
 |  si une colonne n'est pas utilise, on y met 0                  |
 |  utilise les fins de ligne PC                                   |
 |  s'arrange pour avoir au moins 6 chiffres significatifs         |
 |                                                                 |
 | fichier : le fichier o crire                                  |
 | graphe  : le graphe  exporter, suppos rel                    |
 | c_x     : numro de colonne o mettre les X                     |
 | c_y_re  : numro de colonne o mettre les Re(Y)                 |
 | c_y_im  : numro de colonne o mettre les Im(Y)                 |
 | c_ex    : numro de colonne o mettre les erreurs sur les X     |
 | c_ey_re : numro de colonne o mettre les erreurs sur les Re(Y) |
 | c_ey_im : numro de colonne o mettre les erreurs sur les Im(Y) |
 | total   : nombre total de colonnes  mettre                     |
 \---------------------------------------------------------------- */


int EA_GrapheComplexe(FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes,
		      unsigned int nc_X, unsigned int nc_Re, unsigned int nc_Im,
		      unsigned int nc_sX, unsigned int nc_sRe, unsigned int nc_sIm)
{
  register double *x, *x_err = NULL;
  register Complexe *y, *y_err = NULL;
  int n_ecrit = 0;
  unsigned int i;
  unsigned long j;

  if ( ( NULL == fichier ) || ( NULL == graphe ) ) return -1;

  if ( separateur < ' ' ) separateur = '\t';

  x = graphe->x; y = (Complexe *) graphe->y;
  if (graphe->erreur_x != NULL)
    x_err = graphe->erreur_x->y;
  if (graphe->erreur_y != NULL)
    y_err = (Complexe *) graphe->erreur_y->y;

  /* Une ligne par point dans le graphe */
  for ( j = 0 ; j < graphe->nombre_point ; j++, x++, y++, y_err++) {
    for (i = 1 ; i <= nbr_colonnes ; i++) {
      double valeur = 0.0;

      if ( i == nc_X )
	valeur = *x;
      else if ( i == nc_Re )
	valeur = y->re;
      else if ( i == nc_Im )
	valeur = y->im;
      else if ( ( i == nc_sX ) && ( x_err != NULL ) ) {
	valeur = *x_err;
	x_err++;
      } else if ( ( i == nc_sRe ) && ( graphe->erreur_y != NULL ) ) {
	valeur = y_err->re;
      } else if ( ( i == nc_sIm ) && ( graphe->erreur_y != NULL ) ) {
	valeur = y_err->im;
      } 

      /* Le but est une relecture par un ordinateur -> sauvegarde brutale avec 10 chiffres, notation scientifique */
      n_ecrit = fprintf( fichier, "%.10e", valeur );
      if (n_ecrit <= 0) {/* On n'a rien crit */
	(void) SYS_ERREUR(ERREUR_ECRITURE, "Fichier en cours", fichier, NULL);
	return -20;
      }

      if ( i < nbr_colonnes ) fputc( separateur, fichier );
    }

    (void) fprintf(fichier, "\r\n");
  }

  /* Tout s'est bien pass... */
  return 0;
}

int EA_GrapheSerie(FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes, unsigned int nc_X, unsigned int nc_Y, unsigned int nc_sY)
{
  double *y, *y_err = NULL;
  int n_ecrit;
  unsigned int i;
  unsigned long j;

  if ( ( NULL == fichier ) || ( NULL == graphe ) ) return -1;

  if ( separateur < ' ' ) separateur = '\t';

  y = graphe->y;
  if ( graphe->erreur_y != NULL ) y_err = graphe->erreur_y->y;

  /* Une ligne par point dans le graphe */
  for ( j = 0 ; j < graphe->nombre_point ; j++, y++ ) {
    for (i = 1 ; i <= nbr_colonnes ; i++) {
      double valeur = 0.0;

      if ( i == nc_X ) { /* X implicites ! */
	fprintf(fichier, "%6lu", j); /* Largeur 6 -> suppose rarement plus de 1000000 points */
	if ( i < nbr_colonnes ) fputc( separateur, fichier );
	continue;
      } else if ( i == nc_Y ) {
	valeur = *y;
      } else if ( (i == nc_sY) && (y_err != NULL) ) {
	valeur = *y_err;
	y_err++;
      }

      /* Le but est une relecture par un ordinateur -> sauvegarde brutale avec 10 chiffres, notation scientifique */
      n_ecrit = fprintf( fichier, "%.10e", valeur );
      if (n_ecrit <= 0) {/* On n'a rien crit */
	(void) SYS_ERREUR(ERREUR_ECRITURE, "Fichier en cours", fichier, NULL);
	return -20;
      }

      if ( i < nbr_colonnes ) fputc( separateur, fichier );
    }

    /* Ligne termine */
    (void) fprintf(fichier, "\r\n");
  }

  /* Tout s'est bien pass... */
  return 0;
}
