/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 2003, Emmanuel CURIS

   Tout ce qui concerne l'import de fichiers contenant des graphes  des formats autres

   Import des fichiers _EXAFS_ crs  l'ESRF
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "systeme.h"

#include "serie.h"

#include "esrf.h"
#include "fichiers.h"
#include "import_xas.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~- Microfluorescence (ID21) ~-~-~-~-~-~-~-~- */

/* -------- Nombre de spectres  la suite -------- */

unsigned long ID21_CompterSpectres( FILE *fichier, const char *commande )
{
  unsigned long nbr_spectres = 0, num_scan;
  char *commande_lue;
  boolean scan_trouve;
#ifdef _DEVELOPPE_
  unsigned long nbr_scans = 0;
#endif

  do {
    scan_trouve = ESRF_TrouverScanSuivant( fichier );
    if ( FAUX == scan_trouve ) break; /* Plus de scan aprs */

    (void) fgetc( fichier ); /* On saute le #S */
    (void) fgetc( fichier );

    fscanf( fichier, "%lu", &num_scan ); /* Numro de scan */
#ifdef _DEVELOPPE_
    nbr_scans++;
#endif

    commande_lue = lit_commande( fichier ); /* Commande utilise */
    if ( NULL == commande_lue ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme, #S sans commande ? Fichier tronqu ?\n",
		     __FILE__, __FUNCTION__, __LINE__);
      break;
    } else {
      if ( 0 == strcmp( commande, commande_lue ) ) {
	nbr_spectres++; /* On a trouv un spectre du type cherch */
      }

      free( commande_lue ); /* Plus besoin d'elle */
    }

    /* On ignore la fin de la ligne, pour passer correctement au scan suivant */
    (void) ignore_ligne( fichier );
  } while ( scan_trouve == VRAI );

#ifdef _DEVELOPPE_
  printf("%s : %lu scans dans le fichier, dont %lu XANES\n", __FUNCTION__, nbr_scans, nbr_spectres );
#endif

  rewind( fichier );
  return nbr_spectres;
}

/* -------- Trouver le dbut du scan suivant -------- */

int ID21_TrouverDebut( FILE *fichier, const char *commande )
{
  boolean xanes = FAUX, scan_trouve;
  char *commande_lue;
  unsigned long num_scan;
  int c;

  /* On cherche le scan suivant qui contient un XANES */
  do {
    /* On cherche le scan suivant */
    scan_trouve = ESRF_TrouverScanSuivant( fichier );
    if ( FAUX == scan_trouve ) return EOF; /* Plus de scan aprs */

    (void) fgetc( fichier ); /* On saute le #S */
    (void) fgetc( fichier );

    fscanf( fichier, "%lu", &num_scan ); /* Numro de scan */

    commande_lue = lit_commande( fichier ); /* Commande utilise */
    if ( NULL == commande_lue ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme, #S sans commande ? Fichier tronqu ?\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return EOF;
    } else {
      if ( 0 == strcmp( commande, commande_lue ) ) {
	xanes = VRAI; /* Spectre trouv */
      }

      free( commande_lue );
    }

    /* On ignore la fin de la ligne, pour passer correctement au scan suivant */
    c = ignore_ligne( fichier );
  } while ( xanes == FAUX );

  /* On saute les commentaires initiaux */
  while ( '#' == c ) {
    c = ignore_ligne( fichier );
    if ( EOF == c ) return EOF;
  }

  return 0;
}

/* ~-~-~-~-~-~-~-~- Microfluorescence (ID22) ~-~-~-~-~-~-~-~- */

/* -------- Nombre de spectres  la suite -------- */

unsigned long ID22_CompterSpectres( FILE *fichier, const char *commande __attribute__((__unused__)) )
{
  unsigned long nbr_spectres = 0, num_scan;
  char *commande_lue;
  boolean scan_trouve;
#ifdef _DEVELOPPE_
  unsigned long nbr_scans = 0;
#endif

  do {
    scan_trouve = ESRF_TrouverScanSuivant( fichier );
    if ( FAUX == scan_trouve ) break; /* Plus de scan aprs */

    (void) fgetc( fichier ); /* On saute le #S */
    (void) fgetc( fichier );

    fscanf( fichier, "%lu", &num_scan ); /* Numro de scan */
#ifdef _DEVELOPPE_
    nbr_scans++;
#endif

    commande_lue = lit_commande( fichier ); /* Commande utilise */
    if ( NULL == commande_lue ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme, #S sans commande ? Fichier tronqu ?\n",
		     __FILE__, __FUNCTION__, __LINE__);
      break;
    } else {
      if ( 0 == strcmp( commande_lue, "ascan" ) ) {
	char *ss_commande;

	ss_commande = lit_commande( fichier );
	if ( NULL == ss_commande ) {
	  (void) fprintf(stderr, "%s,%s l.%u : Problme, ascan sans mouvement. Fichier tronqu ou mmoire insuffisante ?\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  free( commande_lue );
	  break;
	}
	if ( strcmp( ss_commande, "energy" ) == 0 ) {
	  nbr_spectres++;
#ifdef _DEVELOPPE_
	  printf("%s : Scan XAS energy, numro %lu\n", __FUNCTION__, num_scan );
#endif
	} else if ( strcmp( ss_commande, "menergy" ) == 0 ) {
	  nbr_spectres++;
#ifdef _DEVELOPPE_
	  printf("%s : Scan XAS menergy, numro %lu\n", __FUNCTION__, num_scan );
#endif
	}

	free( ss_commande );
      }

      free( commande_lue ); /* Plus besoin d'elle */
    }

    /* On ignore la fin de la ligne, pour passer correctement au scan suivant */
    (void) ignore_ligne( fichier );
  } while ( scan_trouve == VRAI );

#ifdef _DEVELOPPE_
  printf("%s : %lu scans dans le fichier, dont %lu XANES\n", __FUNCTION__, nbr_scans, nbr_spectres );
#endif

  rewind( fichier );
  return nbr_spectres;
}

/* -------- Trouver le dbut du scan suivant -------- */

int ID22_TrouverDebut( FILE *fichier, const char *commande __attribute__((__unused__)) )
{
  boolean xanes = FAUX, scan_trouve;
  char *commande_lue;
  unsigned long num_scan;
  int c;

  /* On cherche le scan suivant qui contient un XANES */
  do {
    /* On cherche le scan suivant */
    scan_trouve = ESRF_TrouverScanSuivant( fichier );
    if ( FAUX == scan_trouve ) {
#ifdef _DEVELOPPE_
      printf("%s : Dernier scan, plus de scan aprs !\n", __FUNCTION__ );
#endif
      return EOF; /* Plus de scan aprs */
    }

    (void) fgetc( fichier ); /* On saute le #S */
    (void) fgetc( fichier );

    fscanf( fichier, "%lu", &num_scan ); /* Numro de scan */

    commande_lue = lit_commande( fichier ); /* Commande utilise */
    if ( NULL == commande_lue ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme, #S sans commande ? Fichier tronqu ?\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return EOF;
    } else {
      if ( 0 == strcmp( "ascan", commande_lue ) ) {
	char *ss_commande;

	ss_commande = lit_commande( fichier );
	if ( NULL == ss_commande ) {
	  (void) fprintf(stderr, "%s,%s l.%u : Problme, ascan sans mouvement. Fichier tronqu ou mmoire insuffisante ?\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  free( commande_lue );
	  return EOF;
	}
	if ( strcmp( ss_commande, "energy" ) == 0 ) {
	  xanes = VRAI; /* Spectre trouv */
#ifdef _DEVELOPPE_
	  printf("%s : Scan XAS energy, numro %lu\n", __FUNCTION__, num_scan );
#endif
	} else if ( strcmp( ss_commande, "menergy" ) == 0 ) {
	  xanes = VRAI; /* Spectre trouv */
#ifdef _DEVELOPPE_
	  printf("%s : Scan XAS menergy, numro %lu\n", __FUNCTION__, num_scan );
#endif
	}

	free( ss_commande );
      }

      free( commande_lue );
    }

    /* On ignore la fin de la ligne, pour passer correctement au scan suivant */
    c = ignore_ligne( fichier );
  } while ( xanes == FAUX );

  /* On saute les commentaires initiaux */
  while ( '#' == c ) {
    c = ignore_ligne( fichier );
    if ( EOF == c ) return EOF;
  }

  return 0;
}

/* ~-~-~-~-~-~-~-~- Swiss and Norvegian Beam Line (BM-01B, SNBL) ~-~-~-~-~-~-~-~- */

/* -------- Nombre de lignes dans un seul spectre -------- */

unsigned long SNBL_CompterLignes( FILE *fichier, unsigned int col_X )
{
  unsigned long nbr_lignes;
  long position;
  int c;
  unsigned int idx_colonne;
  double e_depart, energie;

  /* On saute d'ventuels commentaires inutiles */
  c = fgetc( fichier );
  ungetc( c, fichier ); /* Pour tre compatible avec ignore_ligne() et ne pas perdre d'info si on n'est pas sur un commentaire ! */
  if ( '#' == c ) {
    while ( c == '#' ) {
      c = ignore_ligne( fichier );
      if ( EOF == c ) {
	(void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte avant d'avoir trouv une ligne de donnes\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return 0;
      }
    }

  }

  /* Position de dpart relle */
  position = ftell( fichier );

  /* nergie du premier point */
  for ( idx_colonne = 1 ; idx_colonne < col_X ; idx_colonne++ )
    fscanf( fichier, "%*s" );
  fscanf( fichier, "%lf", &e_depart );
  nbr_lignes = 1;
/*   printf("Es = %g", e_depart); */

  /* On lit les nergies, pour contrler si l'on passe au suivant ou non */
  do {
    c = ignore_ligne( fichier ); /* On passe  la ligne suivante */

    if ( '#' == c ) {
      (void) fgetc( fichier ); /* On saute le # */
      c = fgetc( fichier );
      if ( EOF == c ) break;
      if ( 'M' == c ) {
	/* Coupure de faisceau : rien  faire de spcial,
	   la question sera pose dans la routine gnrique, en retombant dessus... */
      } else if ( 'S' == c ) {
	/* On change de scan => on s'arrte */
	break;
      }
    } else if ( EOF == c ) {
      break;
    } else {
      /* nergie du point */
      for ( idx_colonne = 1 ; idx_colonne < col_X ; idx_colonne++ )
	fscanf( fichier, "%*s" );
      fscanf( fichier, "%lf", &energie );
/*       printf(", %g", energie); */

      if ( energie > e_depart ) { /* On est bien dans le mme spectre */
	nbr_lignes++; /* Une ligne relle en plus */
	e_depart = energie;
      } else {
	break;
      }
    }
  } while ( c != EOF );

  /* On se remet au dbut */
  fseek( fichier, position, SEEK_SET );
/*   printf("\n"); */

  /* On connat le nombre de lignes */
  return nbr_lignes;
}

/* -------- Nombre de spectres  la suite -------- */

unsigned long SNBL_CompterSpectres( FILE *fichier )
{
  unsigned long nbr_spectres = 0;
  double e_avant = -1.0, energie;
  int c;

  /* On saute l'en-tte initial */
  c = fgetc( fichier );
  while ( '#' == c ) {
    c = ignore_ligne( fichier );
  }

  /* On lit la premire colonne, qui contient les nergies */
  do {
    fscanf( fichier, "%lf", &energie );
    if ( ( energie < e_avant ) || ( e_avant < 0.0 ) ) { /* On commence un nouveau spectre */
      AVERTISSEMENT("Nouveau spectre commence : energie = %lf, avant = %lf", energie, e_avant );
      nbr_spectres++;
    }
    e_avant = energie;

    /* On passe  la ligne suivante */
    c = ignore_ligne( fichier );
    if ( EOF == c ) break; /* On a fini */

    while ( '#' == c ) {
      (void) fgetc( fichier ); /* Pour faire sauter le # */
      c = fgetc( fichier ); /* Code aprs */
      if ( 'M' == c ) {
	/* Coupure de faisceau : rien  faire de spcial,
	   la question sera pose dans la routine gnrique, en retombant dessus... */
      } else if ( 'S' == c ) {
	/* On commence un nouveau scan => d'autres spectres ! */
	e_avant = -1.0;
      }
      /* On ignore la suite de la ligne de commentaire */
      c = ignore_ligne( fichier );
    }
  } while ( c != EOF );

  rewind( fichier ); /* On se remet au dbut du fichier */
  AVERTISSEMENT( "On a compte %lu spectres", nbr_spectres );
  return nbr_spectres;
}
