/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2003 Emmanuel CURIS
   Cration : le 5 fvrier 1999

   Rcupration des donnes exprimentales pour obtenir  total

   Extraction proprement dite, qui redirige vers les bonnes routines
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "serie.h"

#include "import_xas.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int PreparerTableGraphes(Graphe **table, unsigned int nbr_graphes, Intervalle_I *intervalles, unsigned int nbr_intervalles,
				Serie_Graphes *serie, const char *nom_fichier, const char *complement,
				unsigned long nbr_points, int type, unsigned int passage);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Prparer les graphes ncessaires pour recevoir le rsultat -------- */

int EXTRAIT_PreparerGraphes(Infos_Extraction *infos, unsigned int nbr_spectres, unsigned long nbr_points, const char *nom_fichier,
			    EXT_ListeGraphes *graphes, Serie_Graphes *serie, int style, unsigned int passage)
{
  char *titre;

  graphes->x = NULL; /* Pas encore de table des X */
  graphes->nbr_spectres = nbr_spectres;

  /* On cre les tables de graphes
     1. pour les spectres */
  if ( infos->que_garder & DETECT_GARDER_SIGNAL ) {
    graphes->spectres = (Graphe **) calloc( sizeof(Graphe *), nbr_spectres );
    if ( NULL == graphes->spectres ) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/preparer.c", "EXTRAIT_PreparerGraphes", "graphes->spectres");
    }
  } else {
    graphes->spectres = NULL;
  }

  /* 2. pour I1 */
  if ( infos->que_garder & ( DETECT_GARDER_TRANS | DETECT_GARDER_FLUO | DETECT_GARDER_EL ) ) {
    graphes->i_1 = (Graphe **) calloc( sizeof(Graphe *), nbr_spectres );
    if ( NULL == graphes->i_1 ) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/preparer.c", "EXTRAIT_PreparerGraphes", "graphes->i_1");
    }
  } else {
    graphes->i_1 = NULL;
  }

  /* On cre les graphes proprement dit
     1. graphe de I0 si ncessaire */
  if ( infos->que_garder & DETECT_GARDER_I0 ) {
    titre = TXT_genere_chaine("%F : I0, passage %u", nom_fichier, passage);
    if ( NULL == serie ) {
      /* On fait donc des graphes isols */
      graphes->i_0 = GRF_nv_isole( nbr_points, GR_I0, GRF_FCT_REELLE );
      GRF_titre( graphes->i_0, titre );
    } else {
      graphes->i_0 = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I0, GRF_FCT_REELLE, titre );
    }
    free( titre );
  } else {
    graphes->i_0 = NULL;
  }

  /* 2. graphes de I1 si ncessaire */
  if ( ( DETECT_MULTI_MOYENNER == infos->MD_traitement ) ||
       ( DETECT_MULTI_MOY_LOG  == infos->MD_traitement ) ) {
    if ( ( infos->que_garder & DETECT_GARDER_TRANS ) && ( NULL != infos->colonnes_trans ) ) {
      titre = TXT_genere_chaine( "%F : I transmis, moyenne, passage %u", nom_fichier, passage );
      graphes->i_1[ DETECT_TRANSMISSION ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I1, GR_NORMAL, titre );
      free( titre );
    }
    if ( ( infos->que_garder & DETECT_GARDER_FLUO ) && ( NULL != infos->colonnes_fluo ) ) {
      titre = TXT_genere_chaine( "%F : I fluoresc"e_AIGU", moyenne, passage %u", nom_fichier, passage );
      graphes->i_1[ DETECT_FLUORESCENCE ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I1, GR_NORMAL, titre );
      free( titre );
    }
    if ( ( infos->que_garder & DETECT_GARDER_EL ) && ( NULL != infos->colonnes_det ) ) {
      titre = TXT_genere_chaine( "%F : courant Auger, moyenne, passage %u", nom_fichier, passage );
      graphes->i_1[ DETECT_ELECTRONS ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I1, GR_NORMAL, titre );
      free( titre );
    }
  } else {
    if ( ( infos->que_garder & DETECT_GARDER_TRANS ) && ( NULL != infos->colonnes_trans ) ) {
      PreparerTableGraphes( graphes->i_1, nbr_spectres, infos->colonnes_trans, infos->nbr_I_trans, serie, nom_fichier,
			    "I1 trans.", nbr_points, GR_I1, passage );
    }
    if ( ( infos->que_garder & DETECT_GARDER_FLUO ) && ( NULL != infos->colonnes_fluo ) ) {
      PreparerTableGraphes( graphes->i_1, nbr_spectres, infos->colonnes_fluo, infos->nbr_I_fluo, serie, nom_fichier,
			    "I1 fluo.", nbr_points, GR_I1, passage );
    }
    if ( ( infos->que_garder & DETECT_GARDER_EL ) && ( NULL != infos->colonnes_det ) ) {
      PreparerTableGraphes( graphes->i_1, nbr_spectres, infos->colonnes_det, infos->nbr_I_det, serie, nom_fichier,
			    "I1 det.", nbr_points, GR_I1, passage );
    }
  }

  /* 3. graphes des spectres si ncessaire */
  if ( ( DETECT_MULTI_MOYENNER == infos->MD_traitement ) ||
       ( DETECT_MULTI_MOY_LOG  == infos->MD_traitement ) ) {
    if ( NULL != infos->colonnes_trans ) {
      titre = TXT_genere_chaine( "%F : transmission, spectre moyen, passage %u", nom_fichier, passage );
      graphes->spectres[ DETECT_TRANSMISSION ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_BRUT, GR_NORMAL, titre );
      free( titre );
    }
    if ( NULL != infos->colonnes_fluo ) {
      titre = TXT_genere_chaine( "%F : fluorescence, spectre moyen, passage %u", nom_fichier, passage );
      graphes->spectres[ DETECT_FLUORESCENCE ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_BRUT, GR_NORMAL, titre );
      free( titre );
    }
    if ( NULL != infos->colonnes_det ) {
      titre = TXT_genere_chaine( "%F : courant Auger, spectre moyen, passage %u", nom_fichier, passage );
      graphes->spectres[ DETECT_ELECTRONS ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_BRUT, GR_NORMAL, titre );
      free( titre );
    }
  } else {
    PreparerTableGraphes( graphes->spectres, nbr_spectres, infos->colonnes_trans, infos->nbr_I_trans, serie, nom_fichier,
			  "trans.", nbr_points, GR_BRUT, passage );
    PreparerTableGraphes( graphes->spectres, nbr_spectres, infos->colonnes_fluo, infos->nbr_I_fluo, serie, nom_fichier,
			  "fluo.", nbr_points, GR_BRUT, passage );
    PreparerTableGraphes( graphes->spectres, nbr_spectres, infos->colonnes_det, infos->nbr_I_det, serie, nom_fichier,
			  "det.", nbr_points, GR_BRUT, passage );
  }

  /* On cherche la table des X */
  (void) EXTRAIT_TrouverTableX( infos, graphes );
  if ( NULL == graphes->x ) {
    (void) fprintf(stderr, "EXTRAIT_PreparerGraphes : aucun graphe n'a t cr !\n");
    return +5;
  }

  /* On adapte couleurs et styles des graphes */
  GRF_couleur( graphes->i_0, NOIR, style, -1 );
  if ( graphes->i_1 != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < nbr_spectres ; i++ ) {
      GRF_couleur( graphes->i_1[ i ], NOIR + i, style, -1 );
    }
  }
  if ( graphes->spectres != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < nbr_spectres ; i++ ) {
      GRF_couleur( graphes->spectres[ i ], NOIR + i, style, -1 );
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

static int PreparerTableGraphes(Graphe **table, unsigned int nbr_graphes, Intervalle_I *intervalles, unsigned int nbr_intervalles,
				Serie_Graphes *serie, const char *nom_fichier, const char *complement,
				unsigned long nbr_points, int type, unsigned passage)
{
  Graphe *avant = NULL;
  unsigned int i;

  if ( NULL == table ) /* Rien de spcial  faire! */
    return 0;

  if ( ( NULL == intervalles ) || ( 0 == nbr_intervalles ) ) {
    return -1;
  }

  /* Les graphes sont ajouts dans l'ordre  la fin de la srie */
  avant = serie->dernier_graphe;

  for ( i = 0 ; i < nbr_graphes ; i++ ) {
    char *titre;

    if ( table[ i ] != NULL ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "PreparerTableGraphes : graphe %u,table %p dj existant\n", i, (void *) table);
#endif
      continue;
    }

    if ( FAUX == TXT_est_dans_intervalle( i + 1, intervalles, nbr_intervalles ) )
      continue;

    titre = TXT_genere_chaine( "%F : colonne %u (%s) passage %u", nom_fichier, i + 1, complement, passage );
    table[ i ] = Serie_nv_graphe( serie, avant, nbr_points, type, GR_NORMAL, titre );
    free( titre );
    if ( NULL == table[ i ] ) {
      (void) fprintf(stderr, "PreparerTableGraphes : attention, impossible de crer le graphe %i / table %p \n",
		     i, (void *) table);
    }

    avant = table[ i ];
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Trouver o mmoriser les X lus ou calculs -------- */

int EXTRAIT_TrouverTableX(Infos_Extraction *infos, EXT_ListeGraphes *graphes)
{
  if ( ( NULL == infos ) || ( NULL == graphes ) ) return -1;

  /* Par dfaut, pas de table */
  graphes->x = NULL;

  if ( infos->que_garder & DETECT_GARDER_I0 ) {
    if ( graphes->i_0 != NULL ) {
      graphes->x = graphes->i_0->x;
      return 0; /* On a trouv la table, tout va bien */
    }
    /* REMARQUE : on peut avoir le graphe I0 qui existe sans que l'on soit intress par lui,
       si sa cration facilite le chargement [fichier NSLS par exemple].
       On ne doit pas alors le prendre pour les X, car alors il peut tre effac  tout moment ! 
    */
  }

  if ( infos->que_garder & ( DETECT_GARDER_TRANS | DETECT_GARDER_FLUO | DETECT_GARDER_EL ) ) {
    if ( graphes->i_1 != NULL ) {
      unsigned int i;

      for ( i = 0 ; i < graphes->nbr_spectres ; i++ ) {
	if ( graphes->i_1[ i ] != NULL ) {
	  graphes->x = graphes->i_1[ i ]->x;
	  return 0;
	}
      }
    }
    /* Mme remarque que prcdemment */
  }

  if ( infos->que_garder & DETECT_GARDER_SIGNAL ) {
    if ( graphes->spectres != NULL ) {
      unsigned int i;

      for ( i = 0 ; i < graphes->nbr_spectres ; i++ ) {
	if ( graphes->spectres[ i ] != NULL ) {
	  graphes->x = graphes->spectres[ i ]->x;
	  return 0;
	}
      }
    }
  }

  /* Si on est l, c'est qu'aucun spectre n'est conserv ! Le chargement est donc inutile... */
  (void) fprintf(stderr, "EXTRAIT_TrouverTableX : aucune table trouve\n");
  return +5;
}

/* -------- effacer les graphes, si l'on n'en n'a plus besoin -------- */

int EXTRAIT_DetruireGraphes(Infos_Extraction *infos, EXT_ListeGraphes *graphes)
{
  unsigned long nbr_spectres;

  if ( ( NULL == infos ) || ( NULL == graphes ) ) return -1;
  nbr_spectres = graphes->nbr_spectres;

  /* On efface et on met  NULL pour viter des ennuis */
  GRF_efface( graphes->i_0, VRAI, VRAI ); graphes->i_0 = NULL;
  if ( graphes->i_1 != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < nbr_spectres ; i++ ) {
      GRF_efface( graphes->i_1[ i ], VRAI, VRAI ); graphes->i_1[ i ] = NULL;
    }
  }
  if ( graphes->spectres != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < nbr_spectres ; i++ ) {
      GRF_efface( graphes->spectres[ i ], VRAI, VRAI ); graphes->spectres[ i ] = NULL;
    }
  }

  /* La table des X n'a plus de sens*/
  graphes->x = NULL;

  /* Tout s'est bien pass */
  return 0;
}
