/* LASE : Logiciel d'Analyse des Spectres Exprimentaux
   (c) Emmanuel CURIS 1998-2002

   Interface FEFF
   Modles d'ajustement par chemins, prvus pour FEFF

   Paramtres gnraux jouant peu sur la structure (nom,...)
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "exafs.h"

#include "feff.h"
#include "interface.h"

#define FPG_NOMBRE_SORTIES_MODULE_1 6 /* Module POT/POTPH */
#define FPG_NOMBRE_SORTIES_MODULE_2 4 /* Module XSPH */
#define FPG_NOMBRE_SORTIES_MODULE_3 1 /* Module FMS */
#define FPG_NOMBRE_SORTIES_MODULE_4 6 /* Module PATH */
#define FPG_NOMBRE_SORTIES_MODULE_5 2 /* Module GENFMT */
#define FPG_NOMBRE_SORTIES_MODULE_6 4 /* Module FF2X */

#include "feff_general.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static const char *versions_FEFF[ FEFF_NOMBRE_VERSIONS ] = {
  "FEFF 3.00",
  "FEFF 4.00",
  "FEFF 5.00",
  "FEFF 6.00",
  "FEFF 6 lite",
  "FEFF 7.00",
  "FEF 7.02X",
  "FEFF 8.00",
  "FEFF 8.10",
  "FEFF 8.20",
  "FEFF 8.40"
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void      I_FEFF_PG_Activer_Module(short int niveau, unsigned int objet_module, unsigned int objet_sortie);
static void      I_FEFF_PG_Changer_Module(unsigned int objet_module, unsigned int objet_sortie);
static short int I_FEFF_PG_Lire_Module   (unsigned int objet_module, unsigned int objet_sortie);

static void I_FEFF_PreparerParametresGeneraux(void);
static int  I_FEFF_GererParametresGeneraux   (unsigned int objet, int x, int y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

void FEFF_ParametresGeneraux(Modele_FEFF *modele)
{
  boolean retour = FAUX;
  FEFF_INP *params_feff;

  if ( NULL == modele ) { /* On modifie le modle par dfaut */
    modele = FEFF_ChoisirModele( VRAI ); /* On demande si l'on veut en crer un nouveau */
    if ( NULL == modele )
      return;
  }

  params_feff = &(modele->parametres);

  /* Prparation de l'arbre */
  if ( FAUX == arbres[ FEFF_PARAM_GENE ].drapeaux.prepare ) {
    I_FEFF_PreparerParametresGeneraux();
  }

  /* --- On met  jour le contenu de l'arbre */
  /* a. Contrle des modules  */
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ POTPH  ], FEFF_POTPHA , FPG_PRINT_MOD1 );
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ XSPH   ], FEFF_XSPH   , FPG_PRINT_MOD2 );
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ FMS    ], FEFF_FMS    , FPG_PRINT_MOD3 );
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ PATHS  ], FEFF_PATHS  , FPG_PRINT_MOD4 );
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ GENFMT ], FEFF_GENFMT , FPG_PRINT_MOD5 );
  I_FEFF_PG_Activer_Module( params_feff->CONTROL[ FF2CHI ], FEFF_FF2CHI , FPG_PRINT_MOD6 );

  /* b. Champs  TITLE  */
  if ( NULL == params_feff->titre[ 0 ] ) {
    /* Voir  proposer un nom  partir de la molcule et de l'absorbeur ? */
    met_chaine( FEFF_PARAM_GENE, FEFF_COMPOSE, 0, txt_compose_inconnu, TXT_COPIER );
  } else
    met_chaine( FEFF_PARAM_GENE, FEFF_COMPOSE, 0, params_feff->titre[ 0 ], TXT_COPIER );

  if ( NULL == params_feff->titre[ 1 ] ) {
    const char *nom_utilisateur = NULL;

#ifdef _UNIX_
    nom_utilisateur = getenv( "LOGNAME" );
#endif
    if ( NULL == nom_utilisateur ) {
      nom_utilisateur = txt_utilisateur_inconnu;
    }

    met_chaine( FEFF_PARAM_GENE, FG_UTILISATEUR, 0, nom_utilisateur, TXT_COPIER );
  } else
    met_chaine( FEFF_PARAM_GENE, FG_UTILISATEUR, 0, params_feff->titre[ 1 ], TXT_COPIER );

  OBJ_choisit( FEFF_PARAM_GENE, FPG_SANS_GEOM, modele->drapeaux.av_NOGEOM );
  OBJ_choisit( FEFF_PARAM_GENE, FPG_AVEC_XANES, modele->drapeaux.av_XANES );
  OBJ_MettreEntier_U( FEFF_PARAM_GENE, FEFF_NEMAXE, params_feff->NEMAX );

  retour = ( params_feff->POLARIZATION.presente != ABSENT );
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_X_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Y_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Z_POLARISE, retour);
  OBJ_MettreCoordonnees( FEFF_PARAM_GENE, FEFF_X_POLARISE, FEFF_Y_POLARISE, FEFF_Z_POLARISE, &(params_feff->POLARIZATION.vecteur), NULL );

  retour = ( params_feff->ELLIPTICITY.circulaire != ABSENT );
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_ELIPTICITE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_X_ELLIPSE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Y_ELLIPSE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Z_ELLIPSE, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEF_POLAR_RECT, params_feff->ELLIPTICITY.circulaire == FAUX);
  OBJ_choisit(FEFF_PARAM_GENE, FEF_POLAR_CIRC, params_feff->ELLIPTICITY.circulaire == VRAI);
  OBJ_MettreCoordonnees( FEFF_PARAM_GENE, FEFF_X_ELLIPSE, FEFF_Y_ELLIPSE, FEFF_Z_ELLIPSE, &(params_feff->ELLIPTICITY.vecteur), NULL );

  /* On lance l'arbre */
  if ( ARB_Bloquant( FEFF_PARAM_GENE, "Param"e_GRAVE"tres g"e_AIGU"n"e_AIGU"raux", principale, 0 )
       == FEFF_GENE_ANNULE ) /* On annule : rien d'autre  faire */
    return;

  /* On lit les valeurs des paramtres nouvellement dfinis */
  params_feff->CONTROL[ POTPH  ] = I_FEFF_PG_Lire_Module( FEFF_POTPHA, FPG_PRINT_MOD1 );
  params_feff->CONTROL[ XSPH   ] = I_FEFF_PG_Lire_Module( FEFF_XSPH  , FPG_PRINT_MOD2 );
  params_feff->CONTROL[ FMS    ] = I_FEFF_PG_Lire_Module( FEFF_FMS   , FPG_PRINT_MOD3 );
  params_feff->CONTROL[ PATHS  ] = I_FEFF_PG_Lire_Module( FEFF_PATHS , FPG_PRINT_MOD4 );
  params_feff->CONTROL[ GENFMT ] = I_FEFF_PG_Lire_Module( FEFF_GENFMT, FPG_PRINT_MOD5 );
  params_feff->CONTROL[ FF2CHI ] = I_FEFF_PG_Lire_Module( FEFF_FF2CHI, FPG_PRINT_MOD6 );

  if ( NULL == params_feff->titre[ 0 ] ) {
    params_feff->titre[ 0 ] = (char *) malloc( sizeof(char) * 76 );
    if ( NULL == params_feff->titre[ 0 ] )
      SYS_ERREUR(MANQUE_MEMOIRE, "feff_sh.c", "FEF_gene", "params_feff->titre[ 0 ]");
  }
  lit_chaine( FEFF_PARAM_GENE, FEFF_COMPOSE, 75, params_feff->titre[ 0 ], TXT_COPIER );

  if ( params_feff->titre[1] == NULL ) {
    params_feff->titre[1] = (char *) malloc( sizeof(char) * 75 );
    if ( params_feff->titre[1] == NULL )
      SYS_ERREUR(MANQUE_MEMOIRE, "feff_sh.c", "FEF_gene", "params_feff->titre[1]");
  }
  lit_chaine(FEFF_PARAM_GENE, FG_UTILISATEUR, 75, params_feff->titre[1], TXT_COPIER);

  modele->drapeaux.av_NOGEOM = OBJ_choisit( FEFF_PARAM_GENE, FPG_SANS_GEOM, QUESTION );
  modele->drapeaux.av_XANES  = OBJ_choisit( FEFF_PARAM_GENE, FPG_AVEC_XANES, QUESTION );
  params_feff->NEMAX = OBJ_LireEntier_U( FEFF_PARAM_GENE, FEFF_NEMAXE );

  if ( OBJ_choisit( FEFF_PARAM_GENE, FEFF_ELIPTICITE, QUESTION ) == VRAI ) {
    if ( OBJ_choisit( FEFF_PARAM_GENE, FEF_POLAR_RECT, QUESTION ) == VRAI ) {
    /* Polarisation rectiligne */
      params_feff->ELLIPTICITY.circulaire = FAUX;
    } else {
      params_feff->ELLIPTICITY.circulaire = VRAI;
    }
  } else
    params_feff->ELLIPTICITY.circulaire = ABSENT;

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, QUESTION) == VRAI )
    params_feff->POLARIZATION.presente = VRAI;
  else
    params_feff->POLARIZATION.presente = ABSENT;

  OBJ_LireCoordonnees( FEFF_PARAM_GENE, FEFF_X_POLARISE, FEFF_Y_POLARISE, FEFF_Z_POLARISE, &(params_feff->POLARIZATION.vecteur), NULL );
  OBJ_LireCoordonnees( FEFF_PARAM_GENE, FEFF_X_ELLIPSE, FEFF_Y_ELLIPSE, FEFF_Z_ELLIPSE, &(params_feff->ELLIPTICITY.vecteur), NULL );
}

/* -------- Procdure de gestion de cet arbre --------\
 |    Renvoie VRAI si la gestion a pu tre faite      |
 |                                                    |
 | objet : l'objet sur lequel on a cliqu             |
 | x, y  : position de la souris lors du clic         |
 \--------------------------------------------------- */

static int I_FEFF_GererParametresGeneraux(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  boolean retour;

  switch ( objet )
  {
  case FEFF_POTPHA:  /* Quels modules doit-on lancer ? */
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD1 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_XSPH:
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD2 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_FMS:
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD3 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_PATHS:
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD4 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_GENFMT:		/* Doit-on lancer genfmt ? */
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD5 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_FF2CHI:		/* Doit-on lancer ff2chi ? */
    I_FEFF_PG_Changer_Module( objet, FPG_PRINT_MOD6 );
    return BI_EVENEMENT_TRAITE;

  case FEFF_ELIPTICITE:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_ELIPTICITE, ECHANGE);
    OBJ_active (FEFF_PARAM_GENE, FEFF_X_ELLIPSE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Y_ELLIPSE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Z_ELLIPSE, retour);
    return BI_EVENEMENT_TRAITE;

  case FEFF_POLARISE:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, ECHANGE);
    OBJ_active (FEFF_PARAM_GENE, FEFF_X_POLARISE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Y_POLARISE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Z_POLARISE, retour);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Sous-routines de gestion de l'arbre -------- */

static void I_FEFF_PG_Activer_Module(short int niveau, unsigned int objet_module, unsigned int objet_sortie)
{
  if ( NON_LANCE == niveau ) {
    OBJ_choisit( FEFF_PARAM_GENE, objet_module, FORCE_NON );
    OBJ_active ( FEFF_PARAM_GENE, objet_sortie, FORCE_NON );
  } else {
    OBJ_choisit( FEFF_PARAM_GENE, objet_module, FORCE_OUI );
    OBJ_active ( FEFF_PARAM_GENE, objet_sortie, FORCE_OUI );
    OBJ_OptionMenu( FEFF_PARAM_GENE, objet_sortie, (unsigned long) niveau, METTRE );
  }  
}

static void I_FEFF_PG_Changer_Module(unsigned int objet_module, unsigned int objet_sortie)
{
  int ancien_etat;

  /* On change d'tat */
  ancien_etat = OBJ_choisit( FEFF_PARAM_GENE, objet_module, QUESTION );
  if ( ancien_etat ) {
    /* Choisi : on va donc le dsactiver */
    I_FEFF_PG_Activer_Module( NON_LANCE, objet_module, objet_sortie );
  } else {
    /* Non-choisi : on va donc l'activer ; le sous-menu garde son option */
    I_FEFF_PG_Activer_Module( OBJ_OptionMenu( FEFF_PARAM_GENE, objet_sortie, 0, QUESTION ),
			      objet_module, objet_sortie );
  }
}

static short int I_FEFF_PG_Lire_Module(unsigned int objet_module, unsigned int objet_sortie)
{
  int etat;

  etat = OBJ_choisit( FEFF_PARAM_GENE, objet_module, QUESTION );
  if ( 0 == etat ) {
    /* Module non lanc */
    return NON_LANCE;
  }

  /* Module lanc : avec quel mode de sortie ? */
  return OBJ_OptionMenu( FEFF_PARAM_GENE, objet_sortie, 0, QUESTION );
}

/* -------- Prparation de l'arbre -------- */

static void I_FEFF_PreparerParametresGeneraux(void)
{
  /* On installe les menus */
  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD1, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD1, FPG_NOMBRE_SORTIES_MODULE_1, fpg_texte_sortie_module1, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD2, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD2, FPG_NOMBRE_SORTIES_MODULE_2, fpg_texte_sortie_module2, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD3, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD3, FPG_NOMBRE_SORTIES_MODULE_3, fpg_texte_sortie_module3, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD4, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD4, FPG_NOMBRE_SORTIES_MODULE_4, fpg_texte_sortie_module4, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD5, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD5, FPG_NOMBRE_SORTIES_MODULE_5, fpg_texte_sortie_module5, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_PRINT_MOD6, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_PRINT_MOD6, FPG_NOMBRE_SORTIES_MODULE_6, fpg_texte_sortie_module6, NULL, NULL );

  OBJ_ChangerType( FEFF_PARAM_GENE, FPG_VERSION_FEFF, G_SSMENU );
  OBJ_MettreMenu ( FEFF_PARAM_GENE, FPG_VERSION_FEFF, FEFF_NOMBRE_VERSIONS, versions_FEFF, NULL, NULL );
  OBJ_OptionMenu ( FEFF_PARAM_GENE, FPG_VERSION_FEFF, VERSION_FEFF_PAR_DEFAUT, METTRE );

  if ( FAUX == arbres[ FEFF_PARAM_GENE ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( FEFF_PARAM_GENE, "menu_feff.html#general", I_FEFF_GererParametresGeneraux, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  /* On dplace les objets pour enjoliver */
  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_UTIL  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FG_UTILISATEUR, 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FEFF_MODULES_TXT, 0, 500, DEPLACE_RELATIF );

  OBJ_agrandit( FEFF_PARAM_GENE, FEFF_MODS_FOND, 0, 5 );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_XSPH  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_FMS   , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_PATHS , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_GENFMT, 0, +4, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_FF2CHI, 0, +5, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_SORTIE2, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_PRINT_MOD2 , 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_SORTIE3, 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_PRINT_MOD3 , 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_SORTIE4, 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_PRINT_MOD4 , 0, +3, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_SORTIE5, 0, +4, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_PRINT_MOD5 , 0, +4, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_SORTIE6, 0, +5, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_PRINT_MOD6 , 0, +5, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FPG_AVEC_XANES, 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FPG_TXT_XANES , 0, +3, DEPLACE_PIXELS );

  OBJ_deplace( FEFF_PARAM_GENE, FEFF_POLARISE  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_X_POLARISE, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_Y_POLARISE, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEFF_Z_POLARISE, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( FEFF_PARAM_GENE, FEF_POLAR_CIRC , 0, +1, DEPLACE_PIXELS );

#ifdef AVEC_TRADUCTION_
  OPTION_A_FAIRE( "Traduction de l'arbre FEFF/paramtres gnraux" );
#endif

  arbres[ FEFF_PARAM_GENE ].drapeaux.prepare = VRAI;
}
