/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 1999, Emmanuel CURIS
   Cration : 10 fvrier 1999

   Interfaage avec FEFF
   Lancement de FEFF directement depuis le programme

   Cette option n'est disponible que dans la version unix / X-Window
*/

#ifndef _POSIX_SOURCE
# define _POSIX_SOURCE /* Pour pouvoir utiliser waitpid() et autres
			  Indispensable avec HP-Unix, pas sur DEC-Unix
		       */
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h> /* Pour waitpid */
#include <unistd.h>   /* Pour fork, chdir, execl */

#include "systeme.h"

#include "feff.h"
#include "fichiers.h"

#include "feff_lancer.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *nom_feff = EXECUTABLES"/lancer_feff"; /* Comment lancer feff */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Interface pour le lancement : prpare le lancement et tout et tout -------- */

int lancer_feff(void)
{
  char *chemin, *c;
  unsigned int nbr_choisis, i;
  int retour;
  char **fichiers;
  struct infos_import_grf infos_chidat = { IA_MULTI_COLONNES, 0, 0, 0, 4, 1, 2, 0, 0, 0, "2", NULL, 0, 0, 0, 0 /*GR_EXAFS*/, NULL };

  fichiers = CH_selecteur( "feff.inp", "Fichier de commandes FEFF", 0, NULL, &nbr_choisis, principale );
  if ( (nbr_choisis == 0) || (fichiers == NULL) ) /* Aucun fichier choisi */
    return FAUX; /* Lancement a chou */

  for (i = 1 ; i < nbr_choisis ; i++) {
    free(fichiers[i]);
    fichiers[i] = NULL;
  }

  chemin = fichiers[0]; /* Contient le chemin o est le fichier */

  /* On lance feff et on attend qu'il ait fini */
  if (attend_feff(chemin) == FAUX) /* Si a a chou */
  {
    free(fichiers[0]);
    free(fichiers);
    return FAUX;
  }

  /* Maintenant, on essaye de lire le chi.dat */

  c = strrchr(chemin, '/') + 1;
  *c = 0; /* On tronque pour ne garder que le chemin d'accs */
  strcat(chemin, "chi.dat"); /* Moyennement propre, mais normalement, avant,
				il y a feff.inp plus long... */

  /* On prpare pour ne rcuprer que le signal EXAFS */
  retour = ASCII_Generique( chemin, ASCII_FORMAT_FEFF_C, &infos_chidat );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le chargement de %s [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, chemin, retour);
  }

  free(fichiers[0]); /* On ne peut librer que maintenant, car chemin = fichiers[0] utilis
			jusque l ! */
  free(fichiers);
  return 0;
}

/* -------- Lancement effectif de Feff -------- */

int attend_feff(const char *chemin)
{
  pid_t num_processus, retour;
  boolean termine = FAUX;
  int etat;

  num_processus = fork(); /* On ddouble */
  if ( 0 == num_processus ) { /* On est dans le fils : on lance feff */
    if ( execl(nom_feff, nom_feff, chemin, NULL) ) { /* La fonction revient -> erreur */
      (void) BI_ERR_alerte( NULL, &ERR_ImpossibleDeLancerFEFF, FAUX, "execl()", etat );
#ifdef _DEVELOPPE_
      perror("lancer FEFF, erreur execl : ");
#endif
      exit(-1); /* On quitte le processus fils */
    }
  }

  if ( num_processus < 0 ) { /* Erreur */
    (void) BI_ERR_alerte( NULL, &ERR_ImpossibleDeLancerFEFF, FAUX, "fork()", etat );
    return FAUX;
  }

  /* Ici, on est dans le pre et on attend que feff finisse */
  do {
    retour = waitpid( num_processus, &etat, WNOHANG ); /* On vrifie o il en est */
    if ( retour == num_processus ) { /* Il a termin */
      if ( 0 == etat )
	FEFF_ERREUR(FEFF_TERMINE, chemin);
      else {
	(void) BI_ERR_alerte( NULL, &ERR_FEFFAEchoue, FAUX, etat );
      }
      termine = VRAI;
    }

    BI_AttendEvenement( BI_EVENEMENT_TIMER,      /* vnements attendus */
			0, 0, 0, 0, 0, /* rectangle 2 : aucun */
			0,             /* Compteur  0 = on traite les vnements en attente et on revient */
			NULL, NULL, NULL, NULL, NULL, NULL);
  } while ( FAUX == termine );

  if ( etat != 0 ) /* Fin incorrecte */
    return FAUX;

  return VRAI; /* Lancement a fonctionn, tout va bien */
}
