/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1999, Emmanuel CURIS
   Ce fichier : cration le 12 janvier 1999

   Cration d'un modle thorique pour FEFF

   Paramtrage du modle structural
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"
#include "exafs.h"

#include "atomes.h"
#include "feff.h"
#include "interface.h"
#include "science.h" /* Pour M_PI s'il n'est pas dfini dans math.h */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_NOM_MOLECULE 23 /* Nom d'une molcule dans le slecteur pour tout choisir */

#define LGMAX_NOM_ATOME  6 /* Nom de l'atome dans le dfileur */
#define LGMAX_NOM_SS    14 /* Nom de la couche dans le dfileur */

#define NBR_ATM_VISIBLES (FEF_NOM_DERN_ATM - FEF_NOM_PREM_ATM + 1)
#define NBR_OVL_VISIBLES ()
#define NBR_SS_VISIBLES  (FEF_DERN_SS - FEF_PREM_SS + 1)

typedef struct { /* infos ncessaires  cela */
  unsigned int prem_atome; /* 0 = premier de la liste */
  unsigned int atm_choisi; /* Atome slectionn dans le dfileur   */
  unsigned int prem_ss;    /* Premire couche de rflexion simple  */
  unsigned int ss_choisie; /* Couche slectionne dans le dfileur */
  unsigned int prem_ovl;   /* Premieres infos de recouvrement      */
  unsigned int ovl_choisi; /* Recouvrement choisi dans le dfileur */
  unsigned int ovl_etudie; /* Le potentiel pour lequel on tudie le recouvrement */
  Modele_FEFF *modele; /* Le modle en cours de modification */
  Molecule *mol_feff; /* La molcule o ajouter les atomes */
  Fragment *frg_feff; /* Le fragment o ajouter les atomes */
} FEFF_InfosDefileurs;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int     GER_structure(unsigned int num_objet, int x, int y);
static void    prepare_arbre_structure(void);

static void FEFF_AfficheListeAtomes (FEFF_InfosDefileurs *infos);
static void FEFF_AfficheListeCouches(FEFF_InfosDefileurs *infos);
static void FEFF_EnregistreAtome (FEFF_InfosDefileurs *infos);
static void FEFF_EnregistreCouche(FEFF_InfosDefileurs *infos);

static FEFF_atome *FEFF_AtomeActif  (FEFF_InfosDefileurs *infos);
static FEFF_SS    *FEFF_CoucheActive(FEFF_InfosDefileurs *infos);

static void SS_nouvelle(Modele_FEFF *modele);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Appel et enregistrement de ces paramtres ----\
 |    Ne renvoie rien                                     |
 \------------------------------------------------------- */

void FEFF_DefinirStructure(Modele_FEFF *modele)
{
  int retour = FAUX;
  FEFF_InfosDefileurs infos;
  FEFF_INP *params_feff;

  if (NULL == modele) {
    modele = FEFF_ChoisirModele(FAUX); /* s'il y en a un, on le modifie sans demander */
    if (NULL == modele)
      return;
  }

  if ( NULL == modele->absorbeur ) {
    (void) fprintf(stderr, "FEFF_DefinirPotentiels : il n'y a aucun absorbeur associ au modle !\n");
    return;
  }
  params_feff = &(modele->parametres);

  /* On prpare l'affichage de l'arbre */

  OBJ_MettreNombre( FEFF_PAR_STRUCT, FEF_RMAX,  params_feff->RMAX, 0.0 );
  OBJ_MettreNombre( FEFF_PAR_STRUCT, FEF_RMULT, params_feff->RMULTIPLIER, 0.0 );
  OBJ_MettreEntier_U(FEFF_PAR_STRUCT, FEFF_NLEG, params_feff->NLEG);

/*  OBJ_active(FEFF_PAR_STRUCT, FEF_AJOUTER,      FORCE_OUI); */
/*  OBJ_active(FEFF_PAR_STRUCT, FEF_TOUT_CHOISIR, FORCE_OUI); */

  infos.modele = modele;
  infos.prem_atome = 0;
  infos.atm_choisi = 0; /* On commence ascenseur en haut, 1er atome choisi */  
  FEFF_AfficheListeAtomes( &infos );

  infos.prem_ss = 0;    /* On commence en haut */
  infos.ss_choisie = 0; /* Et c'est celle-ci qui est slectionne */
  FEFF_AfficheListeCouches( &infos );

  /* On lance l'arbre */
  if ( arbres[FEFF_PAR_STRUCT].drapeaux.prepare == FAUX )
    prepare_arbre_structure();

  infos.mol_feff = NULL; /* On ajoute  la molcule active, par dfaut */
  infos.frg_feff = NULL; /* Dans un nouveau fragment, inutile tant qu'il n'y a pas cration */

  ARB_MettreComplement( FEFF_PAR_STRUCT, &infos );
  retour = ARB_Bloquant( FEFF_PAR_STRUCT, "Mod"e_GRAVE"le structural", principale, 0 );
  ARB_MettreComplement( FEFF_PAR_STRUCT, NULL );
  if (infos.mol_feff != NULL) { /* On a ajout des atomes */
    /* On rinitialise donc la vue de la molcule, par scurit */

    MOL_cherche_barycentre(infos.mol_feff, FAUX);
    MOL_initialise_vue(infos.mol_feff);
    MOL_PrepareListe();
  }

  if (retour == FEF_STRCT_ANNULE) /* On annule : rien d'autre  faire */
    return;

  /* Atome ventuellement modifi */
  FEFF_EnregistreAtome ( &infos );
  FEFF_EnregistreCouche( &infos );
  
  /* On lit les valeurs des paramtres nouvellement dfinis */  
  params_feff->RMAX        = OBJ_LireNombre( FEFF_PAR_STRUCT, FEF_RMAX, NULL );
  params_feff->RMULTIPLIER = OBJ_LireNombre( FEFF_PAR_STRUCT, FEF_RMULT, NULL);
  params_feff->NLEG        = OBJ_LireEntier_S( FEFF_PAR_STRUCT, FEFF_NLEG );
}

/* ------------------------- Gestion effective de la bote de paramtrage ------------- */

static int GER_structure(unsigned int num_objet, int x, int y)
{
  FEFF_InfosDefileurs *infos;
  int nv_etat, etat_clavier;

  infos = (FEFF_InfosDefileurs *) ARB_LireComplement( FEFF_PAR_STRUCT );
  if (NULL == infos) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune information de structure associe  l'arbre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_NON_TRAITE;
  }

  if ( (num_objet >= FEF_NOM_PREM_ATM) && (num_objet <= FEF_NOM_DERN_ATM) ) {
    FEFF_EnregistreAtome( infos );
    infos->atm_choisi = (num_objet - FEF_NOM_PREM_ATM) + infos->prem_atome;
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;
  }

  if ( (num_objet >= FEF_PREM_SS) && (num_objet <= FEF_DERN_SS) ) {
    FEFF_EnregistreCouche( infos );
    infos->ss_choisie = (num_objet - FEF_PREM_SS) + infos->prem_ss;
    FEFF_AfficheListeCouches( infos );
    return BI_EVENEMENT_TRAITE;
  }

  BI_EtatSouris( NULL, NULL, NULL, NULL, &etat_clavier);

  switch (num_objet)
  {
  case FEF_ATM_AVANT:  /* Faire dfiler d'un atome vers le haut */
  case FEF_ATM_FND:    /* Faire dfiler de 10 atomes */
  case FEF_ATM_BOUTON: /* Faire dfiler : ascenseur glisse */
  case FEF_ATM_APRES:  /* Faire dfiler d'un atome vers le bas */
    infos->prem_atome = ARB_ascenseur(FEFF_PAR_STRUCT, FEF_ATM_AVANT, FEF_ATM_FND, FEF_ATM_BOUTON, FEF_ATM_APRES, ASC_VERTICAL,
				      infos->modele->parametres.nbr_atomes, NBR_ATM_VISIBLES, infos->prem_atome,
				      num_objet, x, y, ECHANGE);
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_AJOUTER:    /* Ajouter un atome  la liste, aprs l'atome slectionn */
    OPTION_A_FAIRE( "Choisir un atome  ajouter" );
    return BI_EVENEMENT_TRAITE;

  case FEF_EFFACER:    /* Effacer l'atome choisi */
    FEFF_EffacerAtome(infos->modele, FEFF_AtomeActif(infos), etat_clavier & K_CTRL);
    if ( infos->atm_choisi >= infos->modele->parametres.nbr_atomes ) {
      infos->atm_choisi = infos->modele->parametres.nbr_atomes - 1;
    }
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_TT_EFFACE:  /* Effacer tous les atomes choisis */
    if (FEFF_ERREUR(FATM_TT_EFFACER) == 2)
      return BI_EVENEMENT_TRAITE;

    while (infos->modele->parametres.prem_atome) /* Tant qu'il y a un atome */
      FEFF_EffacerAtome(infos->modele, infos->modele->parametres.prem_atome, VRAI);
    infos->atm_choisi = 0;
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_TOUT_CHOISIR: /* Tous les atomes en mmoire deviennent des atomes FEFF */
    FEFF_AjouterVoisins( infos->modele );
    infos->prem_atome = 0;
    infos->atm_choisi = 0; /* On commence ascenseur en haut, 1er atome choisi */
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_CREER:      /* Crer un atome et l'ajouter  la liste */
    {
      Molecule *molecule;
      Atome *atome;

      molecule = ATM_QuelleMolecule( infos->modele->absorbeur );
      if (NULL == molecule) {
	(void) fprintf(stderr, "Modle FEFF avec un absorbeur hors-molcule !\n");
	return BI_EVENEMENT_TRAITE;
      }

      /* Cration du nouvel atome [unique] */
      atome = I_ATM_CreerAtome_Absolu( molecule, FAUX, ARB_QuelleFenetre( arbres + FEFF_PAR_STRUCT, FAUX ) );
      if (NULL == atome) return BI_EVENEMENT_TRAITE; /* Annulation ou problme */

      /* Intgration au modle FEFF, avec potentiel "optimal" */
      FEFF_AjouterAtome(infos->modele, atome, -1, VRAI);
    }
    FEFF_AfficheListeAtomes( infos );
    return BI_EVENEMENT_TRAITE;

    /* -~-~-~-~-~- On passe aux recouvrements (OVERLAP) -~-~-~-~-~-~ */

  case FEF_OVERLAP_OK: /* Utilise-t-on cette option ? */
    nv_etat = OBJ_choisit(FEFF_PAR_STRUCT, FEF_OVERLAP_OK, ECHANGE);
    OPTION_A_FAIRE( "Activer/Dsactiver le reste en fonction du choix" );
    return BI_EVENEMENT_TRAITE;

  case FEF_NV_OVL:     /* Nouveau recouvrement pour le potentiel de la couche trait */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_EFF_OVL:    /* Effacer le recouvrement slectionn */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_AVANT:  /* Recouvrement prcdent */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_FOND:   /* Recouvrement : dfilement de +/- 10 */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_BOUTON: /* Recouvrement : dfilement par le bouton */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_APRES:  /* Recouvrement suivant */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_POT_AV: /* Potentiel  traiter prcdent */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;
    
  case FEF_OVL_POT_FND:/* Potentiel : dfilement de +/- 10 */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_CHX_POT:/* Potentiel : dfilement par le bouton */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

  case FEF_OVL_POT_AP: /* Potentiel  traiter suivant */
    OPTION_A_FAIRE( "Gestion des recouvrements" );
    break;

    /* -~-~-~-~-~- On passe aux rflexions simples (SS) -~-~-~-~-~-~ */

  case FEF_SS_OK:      /* Utilise-t-on cette option ? */
    nv_etat = OBJ_choisit(FEFF_PAR_STRUCT, FEF_SS_OK, ECHANGE);
    if ( VRAI == nv_etat ) { /* On veut crer une structure de rflexions simple */
      SS_nouvelle( infos->modele );
    } else { /* Au contraire, on dtruit celles existant dj */
      while (infos->modele->parametres.prem_ss)
	FEFF_EffacerCouche(infos->modele, infos->modele->parametres.prem_ss, VRAI); /* On efface toutes les couches */
    }
    infos->prem_ss    = 0;
    infos->ss_choisie = 0;
    FEFF_AfficheListeCouches( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_NV_SS:      /* Crer une nouvelle couche de rflexions simples */
    SS_nouvelle( infos->modele );
    FEFF_AfficheListeCouches( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_SS_AVANT:   /* Couche prcdente */
  case FEF_SS_FOND:    /* Couche : dfilement de +/- 10 */
  case FEF_SS_BOUTON:  /* Couche : dfilement par le bouton */
  case FEF_SS_APRES:   /* Couche suivante */
    infos->prem_ss = ARB_ascenseur(FEFF_PAR_STRUCT, FEF_SS_AVANT, FEF_SS_FOND, FEF_SS_BOUTON, FEF_SS_APRES, ASC_VERTICAL,
				   infos->modele->parametres.nbr_ss, NBR_SS_VISIBLES, infos->prem_ss,
				   num_objet, x, y, ECHANGE);
    FEFF_AfficheListeCouches( infos );
    return BI_EVENEMENT_TRAITE;

  case FEF_EFF_SS:     /* Effacer la couche courante */
    (void) FEFF_EffacerCouche( infos->modele, FEFF_CoucheActive( infos ), FAUX);
    if ( infos->ss_choisie >= infos->modele->parametres.nbr_ss ) {
      infos->ss_choisie = infos->modele->parametres.nbr_ss - 1;
    }
    FEFF_AfficheListeCouches( infos );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion avance de l'arbre de structure... -------- */

/* Trouver l'atome actuellement slectionn dans le dfileur */
static FEFF_atome *FEFF_AtomeActif(FEFF_InfosDefileurs *infos)
{
  unsigned int i;
  FEFF_atome *atome;

  for (i = 0, atome = infos->modele->parametres.prem_atome ; i < infos->atm_choisi ; i++, atome = atome->suivant) {
    if ( NULL == atome ) {
      (void) fprintf(stderr, "FEFF_EnregistreAtome : valeur de infos->atm_choisi absurde !\n");
      infos->atm_choisi = 0;
      return NULL;
    }
  }

  return atome;
}

/* Trouver la couche SS actuellement slectionne dans le dfileur */
static FEFF_SS *FEFF_CoucheActive(FEFF_InfosDefileurs *infos)
{
  unsigned int i;
  FEFF_SS *couche;

  for (i = 0, couche = infos->modele->parametres.prem_ss ; i < infos->ss_choisie ; i++, couche = couche->suivante) {
    if ( NULL == couche ) {
      (void) fprintf(stderr, "FEFF_EnregistreCouche : valeur de infos->ss_choisie absurde !\n");
      infos->ss_choisie = 0;
      return NULL;
    }
  }

  return couche;
}

/* Enregistrer les paramtres de l'atome actif */
static void FEFF_EnregistreAtome(FEFF_InfosDefileurs *infos)
{
  int nv_pot;
  FEFF_atome *atome;

  if (NULL == infos) return;
  if ( NULL == infos->modele->parametres.prem_atome ) return;

  /* On se place sur l'atome actif */
  atome = FEFF_AtomeActif(infos);
  if ( NULL == atome ) return;

  /* On enregistre les modifs */

  /* Remarque : il peut y avoir quelques erreurs d'arrondi :-(
     Ceci dit, joue sur le 1/1000e d'Angstrom donc... */
  OBJ_LireCoordonnees( FEFF_PAR_STRUCT, FEF_X_ATOME, FEF_Y_ATOME, FEF_Z_ATOME, &(atome->atome->position), NULL );

  /* Remarque : on n'a plus  s'embter avec l'absorbeur, puisqu'il n'est plus modifiable... */
  nv_pot = OBJ_LireEntier_S(FEFF_PAR_STRUCT, FEF_ATM_POT);
  if ( (nv_pot > 0) && (nv_pot < 8) ) { /* Potentiel FEFF valide */
    atome->potentiel = nv_pot;
  }
}

/* Enregistrer les paramtres de la couche active */
static void FEFF_EnregistreCouche(FEFF_InfosDefileurs *infos)
{
  unsigned int nv_valeur;
  FEFF_SS *couche;

  if (NULL == infos) return;
  if ( NULL == infos->modele->parametres.prem_ss ) return;

  /* On se met sur la couche choisie */
  couche = FEFF_CoucheActive( infos );
  if (NULL == couche) return;

  /* On enregistre les modifs */
  couche->degenerescence = OBJ_LireEntier_S( FEFF_PAR_STRUCT, FEF_SS_DEG );  /* Dgnrescence de la couche */
  couche->distance       = OBJ_LireNombre( FEFF_PAR_STRUCT, FEF_SS_DIST, NULL ); /* Distance couche - absorbeur */
  nv_valeur = OBJ_LireEntier_S (FEFF_PAR_STRUCT, FEF_SS_POT);  /* Potentiel de la couche */
  if ( (nv_valeur > 0) && (nv_valeur < 9) ) { /* Potentiel valide */
    couche->potentiel = nv_valeur;
  }
  nv_valeur = OBJ_LireEntier_U( FEFF_PAR_STRUCT, FEF_SS_INDEX ); /* Nouvel index de la couche */
  if ( nv_valeur != couche->index ) { /* On change l'index de couche */
    FEFF_SS *autre = infos->modele->parametres.prem_ss;
      
    while (autre) {
      if ( (autre != couche) && (autre->index == nv_valeur) ) {
	(void) FEFF_ERREUR(FSS_EXISTE_DEJA, nv_valeur);
	return;
      }
      autre = autre->suivante;
    }

    /* Si l'on est l, l'index est plausible */
    couche->index = nv_valeur;
  }
}

/* Le dfileur des atomes inclus au modle */

static void FEFF_AfficheListeAtomes(FEFF_InfosDefileurs *infos)
{
  FEFF_atome *atome;
  Modele_FEFF *modele;
  unsigned int i, j;

  if (NULL == infos) return;
  modele = infos->modele;

  if ( NULL == modele->parametres.prem_atome ) { /* Aucun atome dans le modle */
    /* L'ascenseur */
    (void) ARB_ascenseur(FEFF_PAR_STRUCT, FEF_ATM_AVANT, FEF_ATM_FND, FEF_ATM_BOUTON, FEF_ATM_APRES, ASC_VERTICAL,
			 0, NBR_ATM_VISIBLES, 0, ROOT, 0, 0, METTRE);

    for (i = FEF_NOM_PREM_ATM; i <= FEF_NOM_DERN_ATM ; i++) { /* La liste des noms */
      OBJ_active (FEFF_PAR_STRUCT, i, FORCE_NON);
      OBJ_choisit(FEFF_PAR_STRUCT, i, FORCE_NON);
      met_chaine (FEFF_PAR_STRUCT, i, LGMAX_NOM_ATOME, "", TXT_COPIER);
    }

    OBJ_active(FEFF_PAR_STRUCT, FEF_X_ATOME, FORCE_NON); /* Affichage des coordonnes */
    OBJ_active(FEFF_PAR_STRUCT, FEF_Y_ATOME, FORCE_NON);
    OBJ_active(FEFF_PAR_STRUCT, FEF_Z_ATOME, FORCE_NON);
    OBJ_active(FEFF_PAR_STRUCT, FEF_ATM_POT, FORCE_NON);
    
    OBJ_active(FEFF_PAR_STRUCT, FEF_EFFACER,   FORCE_NON); /* Oprations sur les atomes */
    OBJ_active(FEFF_PAR_STRUCT, FEF_TT_EFFACE, FORCE_NON);

    infos->prem_atome = 0;
    infos->atm_choisi = 0;
    return;
  }

  /* L, il y a forcment des atomes */
  OBJ_active(FEFF_PAR_STRUCT, FEF_X_ATOME, FORCE_OUI); /* Affichage des coordonnes */
  OBJ_active(FEFF_PAR_STRUCT, FEF_Y_ATOME, FORCE_OUI);
  OBJ_active(FEFF_PAR_STRUCT, FEF_Z_ATOME, FORCE_OUI);
  OBJ_active(FEFF_PAR_STRUCT, FEF_ATM_POT, FORCE_OUI);
    
  OBJ_active(FEFF_PAR_STRUCT, FEF_EFFACER,   FORCE_OUI); /* Oprations sur les atomes */
  OBJ_active(FEFF_PAR_STRUCT, FEF_TT_EFFACE, FORCE_OUI);

  /* La liste des noms
     0. Contrle */
  if ( modele->parametres.nbr_atomes <= NBR_ATM_VISIBLES ) {
    infos->prem_atome = 0; /* On affiche tout ! */
  } else if ( infos->prem_atome + NBR_ATM_VISIBLES > modele->parametres.nbr_atomes ) {
    infos->prem_atome = modele->parametres.nbr_atomes - NBR_ATM_VISIBLES;
  }

  /* 1. On se place sur le premier atome affich */
  for (i = 0, atome = modele->parametres.prem_atome ; i < infos->prem_atome ; i++, atome = atome->suivant) {
    if ( NULL == atome ) {
      (void) fprintf(stderr, "FEFF_AfficheListeAtomes : valeur de infos->prem_atome absurde !\n");
      atome = modele->parametres.prem_atome;
      infos->prem_atome = 0;
      break;
    }
  }

  /* 2. On affiche les noms */
  for (j = FEF_NOM_PREM_ATM ; j <= FEF_NOM_DERN_ATM ; j++) { /* La liste des noms */
    if ( NULL == atome ) {
      OBJ_active (FEFF_PAR_STRUCT, j, FORCE_NON);
      OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_NON);
      met_chaine (FEFF_PAR_STRUCT, j, LGMAX_NOM_ATOME, "", TXT_COPIER);
    } else {
      OBJ_active (FEFF_PAR_STRUCT, j, FORCE_OUI);
      if (i == infos->atm_choisi) { /* Les infos sur l'atome actuel */
	OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_OUI);
	OBJ_MettreCoordonnees( FEFF_PAR_STRUCT, FEF_X_ATOME, FEF_Y_ATOME, FEF_Z_ATOME, &(atome->atome->position), NULL );
	OBJ_MettreEntier_U(FEFF_PAR_STRUCT, FEF_ATM_POT, atome->potentiel);
      } else {
	OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_NON);
      }
      met_chaine (FEFF_PAR_STRUCT, j, LGMAX_NOM_ATOME, ATM_Z_vers_nom(atome->atome->num_atomique), TXT_COPIER);
      atome = atome->suivant;
      i++;
    }
  }

  /* L'ascenseur */
  (void) ARB_ascenseur(FEFF_PAR_STRUCT, FEF_ATM_AVANT, FEF_ATM_FND, FEF_ATM_BOUTON, FEF_ATM_APRES, ASC_VERTICAL,
		       modele->parametres.nbr_atomes, NBR_ATM_VISIBLES, infos->prem_atome, ROOT, 0, 0, METTRE);
}

/* Le dfileur pour les couches de rflexion simple */

static void FEFF_AfficheListeCouches(FEFF_InfosDefileurs *infos)
{
  FEFF_SS *couche;
  Modele_FEFF *modele;
  unsigned int i, j;

  if (NULL == infos) return;
  modele = infos->modele;

  if ( NULL == modele->parametres.prem_ss) { /* Aucune couche SS dans le modle */
    (void) ARB_ascenseur(FEFF_PAR_STRUCT, FEF_SS_AVANT, FEF_SS_FOND, FEF_SS_BOUTON, FEF_SS_APRES, ASC_VERTICAL,
			 0, NBR_SS_VISIBLES, 0, ROOT, 0, 0, METTRE);

    for (i = FEF_PREM_SS; i <= FEF_DERN_SS ; i++) { /* La liste des noms */
      OBJ_active (FEFF_PAR_STRUCT, i, FORCE_NON);
      OBJ_choisit(FEFF_PAR_STRUCT, i, FORCE_NON);
      met_chaine (FEFF_PAR_STRUCT, i, LGMAX_NOM_SS, "", TXT_COPIER);
    }

    OBJ_choisit(FEFF_PAR_STRUCT, FEF_SS_OK, FORCE_NON);
    OBJ_active (FEFF_PAR_STRUCT, FEF_SS_INDEX, FORCE_NON);
    OBJ_active (FEFF_PAR_STRUCT, FEF_SS_POT, FORCE_NON);
    OBJ_active (FEFF_PAR_STRUCT, FEF_SS_DEG, FORCE_NON);
    OBJ_active (FEFF_PAR_STRUCT, FEF_SS_DIST, FORCE_NON);
    OBJ_active (FEFF_PAR_STRUCT, FEF_EFF_SS, FORCE_NON);

    return;
  }

  /* L, on est sr qu'il y a des couches de rflexion simple */
  OBJ_choisit(FEFF_PAR_STRUCT, FEF_SS_OK, FORCE_OUI);
  OBJ_active (FEFF_PAR_STRUCT, FEF_SS_INDEX, FORCE_OUI);
  OBJ_active (FEFF_PAR_STRUCT, FEF_SS_POT, FORCE_OUI);
  OBJ_active (FEFF_PAR_STRUCT, FEF_SS_DEG, FORCE_OUI);
  OBJ_active (FEFF_PAR_STRUCT, FEF_SS_DIST, FORCE_OUI);
  OBJ_active (FEFF_PAR_STRUCT, FEF_EFF_SS, FORCE_OUI);

  /* Liste des couches
     1. on va sur la premire couche */
  for (i = 0, couche = modele->parametres.prem_ss ; i < infos->prem_ss ; i++, couche = couche->suivante) {
    if ( NULL == couche ) {
      (void) fprintf(stderr, "FEFF_AfficheListeCouches : valeur de infos->prem_ss absurde !\n");
      couche = modele->parametres.prem_ss;
      infos->prem_ss = 0;
      break;
    }
  }

  /* 2. on affiche la liste des noms */
  for (j = FEF_PREM_SS ; j <= FEF_DERN_SS ; j++) { /* La liste des noms */
    if (NULL == couche) {
      OBJ_active (FEFF_PAR_STRUCT, j, FORCE_NON);
      OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_NON);
      met_chaine (FEFF_PAR_STRUCT, j, LGMAX_NOM_SS, "", TXT_COPIER);
    } else {
      char nom_couche[15];

      sprintf(nom_couche, "Couche %d", infos->prem_ss + i - FEF_PREM_SS + 1);
      met_chaine (FEFF_PAR_STRUCT, i, LGMAX_NOM_SS, nom_couche, TXT_COPIER);
      OBJ_active (FEFF_PAR_STRUCT, j, FORCE_OUI);
      if (i == infos->ss_choisie) {
	OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_OUI);
	OBJ_MettreEntier_U( FEFF_PAR_STRUCT, FEF_SS_INDEX, couche->index );
	OBJ_MettreEntier_U( FEFF_PAR_STRUCT, FEF_SS_POT  , couche->potentiel );
	OBJ_MettreEntier_U( FEFF_PAR_STRUCT, FEF_SS_DEG  , couche->degenerescence );
	OBJ_MettreNombre( FEFF_PAR_STRUCT, FEF_SS_DIST , couche->distance, 0.0 );
      } else {
	OBJ_choisit(FEFF_PAR_STRUCT, j, FORCE_NON);
      }

      couche = couche->suivante;
      i++;
    }
  }

  /* L'ascenseur */
  (void) ARB_ascenseur(FEFF_PAR_STRUCT, FEF_SS_AVANT, FEF_SS_FOND, FEF_SS_BOUTON, FEF_SS_APRES, ASC_VERTICAL,
		       modele->parametres.nbr_ss, NBR_SS_VISIBLES, infos->prem_ss, ROOT, 0, 0, METTRE);
}

/* ~~~~~~~~ Prparation de l'interface ~~~~~~~~ */

static void prepare_arbre_structure(void) /* Arbre de paramtrage de la structure */
{
#ifndef _DEVELOPPE_
  unsigned int i;
#endif

  if ( FAUX == arbres[ FEFF_PAR_STRUCT ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( FEFF_PAR_STRUCT, "menu_feff.html#structure", GER_structure, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  OBJ_deplace(FEFF_PAR_STRUCT, FEF_Y_ATOME, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEF_Z_ATOME, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEF_ATM_POT, 0, +3, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PAR_STRUCT, FEF_EFFACER, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEF_TOUT_CHOISIR, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PAR_STRUCT, FEF_SS_POT , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEF_SS_DEG , 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEF_SS_DIST, 0, +3, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PAR_STRUCT, FEF_EFF_OVL, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PAR_STRUCT, FEF_RMAX , 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PAR_STRUCT, FEFF_NLEG, 0, +1, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVERLAP_OK, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_POT_AV, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_POT_FND, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_CHX_POT, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_POT_AP, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_AVANT, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_FOND, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_BOUTON, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_OVL_APRES, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_NV_OVL, FORCE_NON);
  OBJ_active(FEFF_PAR_STRUCT, FEF_EFF_OVL, FORCE_NON);

  for (i = FEF_OVL_P_POT ; i <= FEF_OVL_D_DIST ; i++)
    OBJ_active(FEFF_PAR_STRUCT, i, FORCE_NON);
#endif

  arbres[FEFF_PAR_STRUCT].drapeaux.prepare = VRAI;
}

/* -~-~-~-~-~-~-~-~ Interface pour la cration de sous-ensembles de la structure -~-~-~-~-~-~-~-~ */

/* -------- Paramtrage d'une couche de rflexion simple --------\
 |  Correspond juste  l'interface GEM avant appel de SS_creer   |
 \-------------------------------------------------------------- */

static void SS_nouvelle(Modele_FEFF *modele)
{
  unsigned int numero;
  int deg, potentiel;
  float distance;

  if ( ARB_Bloquant( SS_NOUVELLE, "Nouvelle couche de r"e_AIGU"flexion simple", NULL, FEFF_PAR_STRUCT ) == NVSS_ANNULER )
    return;

  numero    = OBJ_LireEntier_S( SS_NOUVELLE, NVSS_INDEX     );
  potentiel = OBJ_LireEntier_S( SS_NOUVELLE, NVSS_POTENTIEL );
  deg       = OBJ_LireEntier_S( SS_NOUVELLE, NVSS_DEGENERE  );
  distance  = OBJ_LireNombre ( SS_NOUVELLE, NVSS_DIST, NULL );

  FEFF_AjouterCouche(modele, numero, potentiel, deg, distance);
}
