/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Ce fichier : création le 24 juillet 2002

   Création d'un modèle théorique pour FEFF

   Paramétrage du logiciel xanes, fondé sur feff
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "feff.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int gere_arbre_feff(unsigned int objet, int x_souris, int y_souris);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* ------ Fonction d'appel : installe et relit les valeurs des paramètres ---------- */

void FEFF_ParametresXANES(Modele_FEFF *modele)
{
  FEFF_INP *params_feff;
  int retour;
  int i;

  if (NULL == modele) { /* On cherche le modèle par défaut */
    modele = FEFF_ChoisirModele(FAUX); /* s'il y en a un, on le modifie sans demander */
    if (NULL == modele)
      return;
  }
  params_feff = &(modele->parametres);

  OBJ_choisit(FEFF_MODIFIE_XAN, FXAN_ACTIF, params_feff->Utiliser_Xanes);
  OBJ_active (FEFF_MODIFIE_XAN, FX_XANES, params_feff->XANES >= 0 );
  OBJ_MettreEntier_U( FEFF_MODIFIE_XAN, FX_XANES, params_feff->XANES );
  OBJ_MettreEntier_U( FEFF_MODIFIE_XAN, FX_VINTFIX, params_feff->VINTFIX );
  OBJ_MettreEntier_U( FEFF_MODIFIE_XAN, EGRID_NBR_PTS, params_feff->EGRID.nombre );
  (void) gere_ascenseur(FEFF_MODIFIE_XAN, EGRID_MOINS, EGRID_FOND, EGRID_NBR_PTS, EGRID_PLUS, ASC_HORIZONTAL,
			0, 5, 1, 5, ROOT, 0, 0);
  for (i = 0 ; i < 5 ; i++) {
    OBJ_MettreNombre( FEFF_MODIFIE_XAN, FX_EGRID_POINT1 + i, params_feff->EGRID.position[i], 0.0 );
    OBJ_active(FEFF_MODIFIE_XAN, FX_EGRID_POINT1 + i, params_feff->EGRID.nombre > i);
    OBJ_MettreNombre( FEFF_MODIFIE_XAN, FX_EMESH_P1 + i, params_feff->EGRID.delta_x[i], 0.0 );
    OBJ_active(FEFF_MODIFIE_XAN, FX_EMESH_P1 + i, params_feff->EGRID.nombre > i);
  }

  if ( FAUX == arbres[ FEFF_MODIFIE_XAN ].drapeaux.init ) {
    retour = ARB_Initialiser( FEFF_MODIFIE_XAN, "menu_feff.html#xanes_feff", gere_arbre_feff, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  ARB_MettreComplement( FEFF_MODIFIE_XAN, modele );
  retour = ARB_Bloquant( FEFF_MODIFIE_XAN, "Extensions xanes de FEFF", principale, 0 );
  ARB_MettreComplement( FEFF_MODIFIE_XAN, NULL );
  if ( FXAN_ANNULE == retour )
    return; /* On annule les modifications */

  params_feff->Utiliser_Xanes = OBJ_choisit(FEFF_MODIFIE_XAN, FXAN_ACTIF, QUESTION);
  if (params_feff->XANES >= 0)
    params_feff->XANES = OBJ_LireEntier_U( FEFF_MODIFIE_XAN, FX_XANES );
  params_feff->VINTFIX = OBJ_LireEntier_U( FEFF_MODIFIE_XAN, FX_VINTFIX );

  params_feff->EGRID.nombre = OBJ_LireEntier_U( FEFF_MODIFIE_XAN, EGRID_NBR_PTS );
  for (i = 0 ; i < 5 ; i++) {
    params_feff->EGRID.position[i] = OBJ_LireNombre( FEFF_MODIFIE_XAN, FX_EGRID_POINT1 + i, NULL );
    params_feff->EGRID.delta_x[i]  = OBJ_LireNombre( FEFF_MODIFIE_XAN, FX_EMESH_P1 + i, NULL );
  }
}

/* ------ Fonction de gestion de l'arbre associé (adaptation "temps réel") --------- */

static int gere_arbre_feff(unsigned int objet, int x_souris, int y_souris)
{
  FEFF_INP *params_feff __attribute__((__unused__));
  Modele_FEFF *modele;

  modele = (Modele_FEFF *) ARB_LireComplement( FEFF_MODIFIE_XAN );
  if (NULL == modele) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'informations associées à l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return VRAI;
  }
  params_feff = &(modele->parametres);

  switch (objet)
  {
  case EGRID_MOINS: /* Ascenseur : nombre de points pour EGRID */
  case EGRID_FOND:
  case EGRID_NBR_PTS:
  case EGRID_PLUS:
    {
      int ancien, nouveau;

      ancien = OBJ_LireEntier_U( FEFF_MODIFIE_XAN, EGRID_NBR_PTS );
      nouveau = gere_ascenseur(FEFF_MODIFIE_XAN, EGRID_MOINS, EGRID_FOND, EGRID_NBR_PTS, EGRID_PLUS, ASC_HORIZONTAL,
				   0, 5, 1, -1, objet, x_souris, y_souris);
      if (nouveau != ancien) {
	int i;

	for (i = 0 ; i < 5 ; i++) {
	  OBJ_active(FEFF_MODIFIE_XAN, FX_EGRID_POINT1 + i, nouveau > i);
	  OBJ_active(FEFF_MODIFIE_XAN, FX_EMESH_P1 + i, nouveau > i);
	}
      }
    }
    return VRAI;

  }

  return FAUX;
}
