/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Normalisation simple d'un graphe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "graphes.h"

#include "grf_normaliser.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int GRF_Normaliser(Graphe *graphe, int methode, double val_0, double val_1)
{
  unsigned long i;
  double facteur = 0.0;
  double *y;
  char *titre;

  /* Vrification prliminaire */
  if ( NULL == graphe ) return -1;

  if ( ( graphe->genre_donnees != GRF_FCT_REELLE ) && ( graphe->genre_donnees != GRF_SERIE ) ) {
    (void) BI_ERR_alerte( NULL, &ERR_TypeIncorrect, VRAI );
    return -2;
  }

  if ( VRAI == graphe->drapeaux.normalise ) {
    AVERTISSEMENT( "graphe %s dj normalis !", graphe->nom );
  }

  /* Calcul du facteur de normalisation, en fonction du mode choisi */
  switch ( methode )
  {
  case GRF_NORM_MIN_MAX: /* minimum  0, maximum  1 */
    val_0 = graphe->y_min;
    val_1 = graphe->y_max;
    break;

  case GRF_NORM_MIN_UTIL: /* minimum  0, valeur indique devient 1 */
    val_0 = graphe->y_min;
    break;

  case GRF_NORM_UTIL_MAX: /* valeur indique devient 0, maximum  1 */
    val_1 = graphe->y_max;
    break;

  case GRF_NORM_UTIL_UTIL:/* valeurs indiques deviennent 0 et 1 */
    break;

  default:
    (void) fprintf(stderr, "GRF_Normaliser : mthode de normalisation inconnue [%d]\n", methode);
    return -3;
  }

  facteur = val_1 - val_0;
  if ( facteur < 0.0 ) {
    AVERTISSEMENT( "Facteur de normalisation ngatif (%g)", facteur );
  }
  if ( 0.0 == facteur ) {
    (void) BI_ERR_alerte( NULL, &ERR_NormalisationImpossible, VRAI, val_0, val_1 );
    return -5;
  }

  /* Normalisation proprement dite
     1. on translate pour avoir 0 en val_0 (additif : ne joue pas sur les erreurs)
  */
  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++) {
    *y -= val_0;
  }
  graphe->y_min -= val_0;
  graphe->y_max -= val_0;

  /* 2. On divise par le facteur --- la routine s'occupe des erreurs et tout */
  GRF_FacteurMultiplicatif( graphe, 1.0 / facteur, AXE_Y, NULL );

  /* 2bis. On marque le graphe comme normalis */
  graphe->drapeaux.normalise = VRAI;

  /* 3. On dfinit un titre... */
  titre = TXT_genere_chaine( titre_graphe_normalise, graphe->nom );
  GRF_titre( graphe, titre );
  free( titre ); titre = NULL;

  /* Aprs normalisation, on est sans unit ! */
  GRF_ChangerUnite( graphe, AXE_Y, "" );

  /* On met  jour l'interface si ncessaires */
  GRF_MiseAJour( graphe, VRAI );

  /* Tout s'est bien pass */
  return 0;
}
