/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Aller  une image,  une srie d'images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"
#include "interface.h"
#include "serie.h" /* Pour trouver la srie ! */
#include "tracer.h" /* Pour DES_image */

#include "img_aller.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Marquer une image comme tant l'image courante -------- */

void IMG_AllerImage(Image *image)
{
  Serie_Images *serie;
  Image *img;
  DES_image *dessin_image;
  unsigned long num_image;

  /* On enregistre les modifications ventuelles des paramtres de l'ancienne image courante */
  LIMG_EnregistrerParametres();

  /* Si NULL : est-ce qu'il reste des images en mmoire ? */
  if ( NULL == image ) {
    if ( ( NULL == IMG_serie_courante ) || ( NULL == IMG_serie_courante->premiere_image ) ) {
      image_courante = NULL;

      FEN_ascenseur( fen_images, ASC_VERTICAL, 0, 1000);
      FEN_redessin( fen_images, BI_RDS_TOUT);
      if ( NULL == IMG_serie_courante ) { /* Aucune image, ni srie */
	FEN_titre( fen_images, txt_err_aucune_image );
	FEN_ascenseur(fen_images, ASC_HORIZONTAL, 0, 1000);
      } else { /* Aucune image dans la srie courante */
	FEN_titre( fen_images, txt_err_serie_vide );
      }

      LIMG_RafraichirListe( VRAI );
    }

    return;
  }

  /* Doit-on changer de srie ? */
  serie = IMG_QuelleSerie( image );
  if ( NULL == serie ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Appel pour une image isole [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }
  if ( serie != IMG_serie_courante ) {
    IMG_AllerSerie( serie );
  }

  /* Maintenant, on sait que l'on est dans la bonne srie */
  /* 1. Numro de l'image dans la srie */
  for ( num_image = 0, img = serie->premiere_image ; img != image ; img = img->suivante, num_image++ );

  /* 2. Mmorise que l'on est bien sur l'image actuelle */
  image_courante = image;

  /* 3. Mise  jour de la liste des images */
  LIMG_RafraichirListe( VRAI );

  /* 4. Si ncessaire, on ouvre la fentre d'affichage */
  if ( NULL == fen_images ) {
    int retour;

    retour = I_OuvrirPrincipale_IMG();
    if ( ( retour != 0 ) || ( NULL == fen_images ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR, impossible d'ouvrir la fentre d'affichage [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }
  }

  /* 5. Installation de l'ascenseur */
  if ( serie->nbr_images < 2 ) {
    FEN_ascenseur( fen_images, ASC_VERTICAL, 0, 1000);
  } else {
    float pos_asc, taille_asc;

    pos_asc  = (float) (num_image * 1000.0);
    pos_asc /= (float) (serie->nbr_images - 1);
    taille_asc = 1000.0 / (float) ( serie->nbr_images );
    FEN_ascenseur( fen_images, ASC_VERTICAL, (int) pos_asc, (int) taille_asc);
  }

  /* 6. Titre de la fentre */
  FEN_titre( fen_images, image_courante->nom );

  /* 7. Paramtres pour le dessin */
  dessin_image = ( DES_image * ) FEN_LireComplement( fen_images, FEN_COMPLEMENT_1 );
  if ( NULL == dessin_image ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, pas d'infos pour le trac des images !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  dessin_image->image = image;

  IMG_AdapterAffichage( dessin_image );

  if ( image->infos_affichage != NULL ) {
    dessin_image->couleur = image->infos_affichage->couleur;
    dessin_image->proprietes = image->infos_affichage->proprietes;
    dessin_image->proprietes |= IMG_AVEC_COUPE; /* On affiche une coupe ventuelle */
  } else {
    dessin_image->couleur = IMG_FAUSSES_COULEURS;
    dessin_image->proprietes = IMG_AVEC_COUPE | IMG_AVEC_CENTRE | IMG_AVEC_CADRE | IMG_AVEC_GRADUATION | IMG_AVEC_AXES;
  }

  /* 8. On force le redessin */
  FEN_redessin( fen_images, BI_RDS_TOUT );
}

/* ~-~-~-~-~-~-~-~-~ Trouver des images ou quivalent ~-~-~-~-~-~-~-~-~ */

/* -------- Trouver la srie de graphes associe  une image -------- */

struct serie *IMG_TrouverSerieGraphes(Image *image)
{
  Graphe **graphes;
  Serie_Graphes *serie;
  unsigned long i, nbr_valeurs;

  if ( NULL == image ) return NULL;
  if ( ( IMG_GENRE_REELLE == image->genre ) || ( IMG_GENRE_COMPLEXE == image->genre ) || ( IMG_GENRE_PHOTO == image->genre ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Image sans graphes dans le contenu [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom);
#ifdef _DEVELOPPE_
    abort();
#endif
    return NULL;
  }

  /* 1. Recherche rapide : lien... */
  for ( serie = premiere_serie ; serie != NULL ; serie = serie->suivante ) {
    if ( serie->carto == image ) {
      return serie;
    }
  }

  /* 2. On essaye plus brutalement... */
  nbr_valeurs = image->larg * (unsigned long) image->haut;
  for ( i = 0, graphes = image->valeurs.G ; i < nbr_valeurs ; i++, graphes++ ) {
    if ( *graphes != NULL ) {
      return GRF_QuelleSerie( *graphes );
    }
  }

  /* Si on est l : aucun graphe mmoris ! */
  return NULL;
}
