/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 11 fevrier 2002

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Pour une carto complete, que doit-on afficher comme image ?
     => recherche des tranches...
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"

#include "exafs.h"
#include "images.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* static void    RCT_tranche(Fenetre *, int, int, int, int); */
static void    RDS_tranche(Fenetre *, int, int, unsigned int, unsigned int);
static int CLC_tranche(Fenetre *, int, int, int, int, int);
/* static int CLV_tranche(Fenetre *fen, unsigned int touche, int etat); */
static int MSG_tranche(Fenetre *fen, Message *message);

/* ~~~~~~~~ Realisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Calculs ~-~-~-~-~-~-~-~- */

int IMG_moyenne_rectangle(Image *source, Infos_tranche *infos, boolean erreurs)
{
  Graphe *moyenne;
  unsigned long i, j;
  unsigned long nbr_cases = 0;
  double *y;

  if (infos == NULL) return FAUX;
  if (source == NULL) return FAUX;
  if (infos->tranche == NULL) return FAUX;

  moyenne = infos->spectre;
  if (moyenne == NULL) {
    return FAUX;
  }

  /* On met a zero le spectre moyen/somme */
  for (i = 0, y = moyenne->y ; i < moyenne->nombre_point ; i++, y++) {
    *y = 0.0;
  }

  if ( (erreurs == VRAI) && (moyenne->erreur_y == NULL) ) {
    (void) GRF_creer_erreurs( moyenne, AXE_Y );
    if (moyenne->erreur_y == NULL) {
      (void) fprintf(stderr, "IMG_moyenne_rectangle : Impossible de crer les erreurs...\n");
      erreurs = FAUX; /* Impossible de calculer l'erreur */
    }
  }
  if (erreurs == VRAI) { /* On remet les erreurs a zero */
    double *y_err;

    for (i = 0, y_err = moyenne->erreur_y->y ; i < moyenne->nombre_point ; i++, y_err++)
      *y_err = 0.0;
  }

  /* On fait la somme sur toutes les cases choisies */
  nbr_cases = (unsigned long) infos->larg * (unsigned long) infos->haut;
  /*  printf("Nombre de cases : %lu\n", nbr_cases); */
  if (nbr_cases == 0) {
    printf(" AUCUNE CASE CHOISIE \n");
    moyenne->y_min = 0.0;
    moyenne->y_max = 0.0;
    return FAUX;
  } else if (nbr_cases == 1) {
    unsigned long num_case;
    int retour;
    Graphe *graphe;

    retour = IMG_IJ_v_N( source, infos->x, infos->y, &num_case );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_IJ_v_N()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return FAUX;
    }
    graphe = source->valeurs.G[ num_case ];
    if ( NULL == graphe ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas de graphe dans la case %lu (image %s)\n",
		     __FILE__, __FUNCTION__, __LINE__, num_case, source->nom);
      return VRAI;
    }

    memcpy(moyenne->y, graphe->y, sizeof(double) * (graphe->nombre_point));
    moyenne->y_min = graphe->y_min;
    moyenne->y_max = graphe->y_max;

    if (erreurs == VRAI) {
      moyenne->erreur_y->y_min = 0.0; /* Pas d'erreur avec une seule mesure ! */
      moyenne->erreur_y->y_max = 0.0;
    }

    return VRAI;
  }

  /*  printf("cases :\n"); */
  for (i = 0 ; i < infos->larg ; i++) {
    for (j = 0 ; j < infos->haut ; j++) {
      int retour;
      unsigned long numero_case, k;
      double *y_partiel;
      Graphe *graphe;

      retour = IMG_IJ_v_N( source, infos->x + i, infos->y + j, &numero_case );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_IJ_v_case()\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return FAUX;
      }

      graphe = source->valeurs.G[ numero_case ];
      if ( graphe != NULL ) {
	/*      printf("%lu [graphe : %p]\n", numero_case, graphe); */
	for ( k = 0, y = moyenne->y, y_partiel = graphe->y ; k < graphe->nombre_point ; k++, y++, y_partiel++ ) {
	  *y += *y_partiel / (double) (nbr_cases);
	}
      }
    }
  }
  /*  printf("Fin cases\n"); */

  if (erreurs == VRAI) { /* On calcule les erreurs en meme temps */
    double *y_partiel, *y_err;
    unsigned long k;
    int *ddl;

    for (i = 0 ; i < infos->larg ; i++) {
      for (j = 0 ; j < infos->haut ; j++) {
	int retour;
	unsigned long numero_case;
	Graphe *graphe;

	retour = IMG_IJ_v_N( source, infos->x + i, infos->y + j, &numero_case );
	if ( retour != 0 ) {
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_IJ_v_case()\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  return FAUX;
	}

	graphe = source->valeurs.G[ numero_case ];
	if ( graphe != NULL ) {
	  for ( k = 0, y_partiel = graphe->y, y_err = moyenne->erreur_y->y, y = moyenne->y;
		k < graphe->nombre_point ;
		k++, y_partiel++, y_err++, y++) {
	    double delta;

	    delta = (*y_partiel - *y);
	    delta *= delta;
	    *y_err += delta;
	  }
	}
      }
    }

    /* y_erreur contient la somme des carres des ecarts a la moyenne */
    for (k = 0, y_err = moyenne->erreur_y->y, ddl = moyenne->erreur_y->z.ddl ;
	 k < moyenne->nombre_point ; k++, y++, y_err++, ddl++) {
      *y_err = sqrt(*y_err / (double) (nbr_cases - 1));
      *ddl = nbr_cases - 1;
    }

    GRF_bornes(moyenne->erreur_y, NE_PAS_FAIRE, DESORDONNE);
  }

  GRF_bornes(moyenne, NE_PAS_FAIRE, DESORDONNE);
  return VRAI;
}

int IMG_moyenne_tranches(Image *source, Infos_tranche *infos)
{
  double *x;
  unsigned long i, nbr_cases;

  if (infos == NULL) return FAUX;
  if (infos->tranche == NULL) return FAUX;
  nbr_cases = source->larg * source->haut;

  if (nbr_cases != infos->tranche->larg * infos->tranche->haut ) {
#ifdef _DEVELOPPE_
    fprintf(stderr, "IMG_moyenne_tranches : images de taille differente !\n");
#endif
    return FAUX;
  }

  /* On met a zero l'image moyenne */
  for (i = 0 ; i < nbr_cases ; i++) {
    infos->tranche->valeurs.R[i] = 0.0;
  }

  /* On fait la moyenne */
  for (i = 0, x = infos->spectre->x ; i < infos->spectre->nombre_point ; i++, x++) {
    if ( (*x >= infos->x_min) && (*x <= infos->x_max) ) { /* Le point convient */
      unsigned long j;
      Graphe *graphe;

      for (j = 0 ; j < nbr_cases ; j++) {
	graphe = source->valeurs.G[ j ];
	if ( graphe != NULL ) {
	  infos->tranche->valeurs.R[ j ] += *(graphe->y + i);
	} else {
	  infos->tranche->valeurs.R[ j ] += 0.0;
	}
      }
    }
  }

  IMG_bornes(infos->tranche);
  return VRAI;
}

/* ~-~-~-~-~-~-~-~- Interface ~-~-~-~-~-~-~-~- */

void IMG_choisit_tranche(Image *image)
{
  Fenetre *fenetre;
  Infos_tranche *infos;
  unsigned int i;

  if (image == NULL) return;

  if ( image->genre != IMG_GENRE_GRAPHE ) {
    fprintf(stderr, "IMG_choisit_tranche : image de type incorrect [%s, type %d]\n", image->nom, image->type);
    return;
  }

  infos = (Infos_tranche *) malloc( sizeof(Infos_tranche) );
  if (infos == NULL) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Images/cartos.c", "IMG_choisit_tranche", "infos / tranche");
#ifdef _DEVELOPPE_
    fprintf(stderr, "IMG_choisit_tranche : infos impossibles a reserver\n");
    fprintf(stderr, "   taille demandee : %lu\n", (unsigned long) sizeof(Infos_tranche));
#endif
    return;
  }

  /* Sert pour l'apercu => initialement, image de dpart */
  infos->tranche = IMG_CreerImage(NULL, image, IMG_GENRE_REELLE, image->type, image->larg, image->haut);
  if (infos->tranche == NULL) {
    (void) fprintf(stderr, "IMG_choisit_tranche : impossible de crer l'image rsultat\n");
    free(infos);
    return;
  }
  infos->tranche->val_max = image->val_max;
  infos->tranche->val_min = image->val_min;
  infos->tranche->l_reel = image->l_reel;
  infos->tranche->h_reel = image->h_reel; 

  /* On copie, GRF_Copier se charge de crer la nouvelle srie contenant le spectre rsultat */
  for ( i = 0 ; image->valeurs.G[ i ] == NULL ; i++ );
  infos->spectre = GRF_Copier( image->valeurs.G[ i ], NULL, FAUX );
  if (infos->spectre == NULL) {
    (void) fprintf(stderr, "IMG_choisit_tranche : impossible de crer le graphe copie du premier (graphe moyen)\n");
    IMG_EffacerImage( infos->tranche, FAUX );
    free(infos);
    return;
  }

  /* On s'assure que l'on a bien un graphe  valeurs relles ! */
  if ( GRF_SERIE == infos->spectre->genre_donnees ) {
    GRF_ChangerGenre( infos->spectre, GRF_FCT_REELLE );
  }

  infos->x_min = infos->spectre->x_min;
  infos->x_max = infos->spectre->x_max;

  infos->x = 0; infos->y = 0; /* On prend toute l'image, par defaut */
  infos->larg = image->larg; infos->haut = image->haut;

  /* On prepare l'image initiale */
  IMG_moyenne_tranches (image, infos);
  IMG_moyenne_rectangle(image, infos, FAUX); /* Sans calcul d'erreur actuellement */

  fenetre = FEN_CreerFenetreSimple( TOUS_SF_ASC, "Cartographie moyenne", NULL, RDS_tranche, NULL, 
				    NULL /* Clavier */, CLC_tranche, MSG_tranche,
				    NULL/*RCT_pointeur*/, NULL);
  if (fenetre == NULL) {
    free(infos); infos = NULL;
    return;
  }
  FEN_Ouvrir( fenetre );

  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, image );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos );
  fenetre->drapeaux.redessin = VRAI; /* Redessin si changement de taille */
  fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_tranche */
}

/* -------- Redessin de la fenetre de travail -------- */

static void RDS_tranche(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  DES_image dessin_image;
  Infos_tranche *infos;
  double x_echelle, y_echelle;
  int x_image, y_image;

  if (fenetre == NULL) return;

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* Separation entre les deux parties */
  TRC_Separateur( fenetre, VERTICAL, fenetre->w_larg / 2, 0, fenetre->w_haut );

  infos = (Infos_tranche *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if (infos == NULL) return;

  /* Apercu de l'image moyenne */
  x_image = fenetre->w_x + (fenetre->w_larg >> 1) + 5;
  y_image = fenetre->w_y + 5;
  TRC_CreerDegrade( fenetre, VERT );
  dessin_image.image   = infos->tranche;
  dessin_image.couleur = VERT;
  /*  dessin_image.type_grille = GRILLE_AUCUNE; */
  TRC_image( fenetre, &dessin_image, x_image, y_image, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, IMG_TRACE_NORMAL);

  /* Rectangle pour la zone de moyenne */
  {
    double l_case, h_case;

    x_echelle = IMG_dim_trace(infos->tranche, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, NULL, NULL);
    l_case = infos->tranche->l_reel * x_echelle;
    h_case = infos->tranche->h_reel * x_echelle;

    TRC_Couleur( ROUGE );
    TRC_Rectangle( fenetre, x_image + (infos->x ) * l_case, y_image + (infos->y ) * h_case, infos->larg * l_case, infos->haut * h_case );
  }

  /* Apercu du graphe moyen */
  if (infos->spectre) {
    int x_barre;

    if ( infos->spectre->x_max == infos->spectre->x_min ) {
      TRC_Z_Message( fenetre, 0, 0, fenetre->w_larg >> 1, fenetre->w_haut,
		     "Spectre moyen d"e_AIGU"fini sur un intervalle nul" );
      return;
    }
    if ( infos->spectre->y_max == infos->spectre->y_min ) {
      TRC_Z_Message( fenetre, 0, 0, fenetre->w_larg >> 1, fenetre->w_haut,
		     "Spectre moyen sans contraste" );
      return;
    }

    x_echelle = (double) ((fenetre->w_larg >> 1) - 4) / (infos->spectre->x_max - infos->spectre->x_min);
    y_echelle = (double)  (fenetre->w_haut -  4)      / (infos->spectre->y_max - infos->spectre->y_min);

    TRC_Graphe( fenetre, infos->spectre, NOIR, 1, 0, 0,
		fenetre->w_x + 2, fenetre->w_y + 2, infos->spectre->x_min, infos->spectre->x_max,
		infos->spectre->y_min, infos->spectre->y_max, x_echelle, y_echelle );

    /* Les marques de la rgion  moyenner */
    x_barre = TRC_ConvertitX_PvE( infos->x_min, infos->spectre->x_min, 2, x_echelle );
    TRC_Couleur( ROUGE ); /* Barre rouge : E0 */
    TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );

    x_barre = TRC_ConvertitX_PvE( infos->x_max, infos->spectre->x_min, 2, x_echelle );
    TRC_Couleur( VERT ); /* Barre rouge : E0 */
    TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );
  } else {
    TRC_Z_Message( fenetre, 0, 0, fenetre->w_larg >> 1, fenetre->w_haut,
		   "Aucun spectre moyen" );
  }
}

/* -------- Gestion des messages -------- */

static int MSG_tranche(Fenetre *fenetre, Message *message)
{
  Image *source;
  Infos_tranche *infos;

  source = (Image *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (Infos_tranche *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  if ( message->type == WM_CLOSED ) { /* On veut fermer la fentre */
    int retour;

    if ( (source == NULL) || (message->fenetre.haut != 0) ) /* Demande de fermeture force */
      retour = 2; /* On ferme sans rien demander et sans rien faire */
    else
      retour = EXF_ERREUR(FIN_MODULE); /* On demande s'il faut fermer */

    if (retour == 3) return BI_EVENEMENT_TRAITE; /* Fermeture annulee */
    if (retour == 2) { /* Calculs annules */
      /* On dlie de la fentre pour limiter les risques */
      FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, NULL );
      FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

      IMG_EffacerImage( infos->tranche, FAUX );
      GRF_efface(infos->spectre, VRAI, VRAI);
      free( infos ); infos = NULL;
      FEN_Detruire( fenetre );
      return BI_EVENEMENT_FEN_FERMEE;
    }

    OPTION_A_FAIRE( "Demander que garder (image ou graphe ?)..." );
    free( infos ); infos = NULL;
    FEN_Detruire( fenetre );
    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non trait [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE; /* Message non traite */
}

/* -------- Clic de la souris sur la zone utilisateur -------- */

static int CLC_tranche(Fenetre *fenetre, int bouton, int n_clic __attribute__((__unused__)), int etat __attribute__((__unused__)), int x, int y)
{
  Image *source;
  Infos_tranche *infos;

  if ( NULL == fenetre ) return BI_EVENEMENT_TRAITE;

  source = (Image *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (Infos_tranche *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( ( NULL == source ) || ( NULL == infos ) ) {
    /* Pas d'info associe : rien  faire... */
    return BI_EVENEMENT_TRAITE;
  }

  if ( x < (int) (fenetre->w_larg >> 1) ) { /* Clic sur la partie graphe */
    double x_echelle;
    Graphe *spectre = infos->spectre;

    if (spectre->x_max == spectre->x_min) return VRAI;
    x_echelle = ( (double) ((fenetre->w_larg >> 1) - 4.0) ) / (spectre->x_max - spectre->x_min);

    if (bouton == 1) { /* Bouton gauche -> x_min */
      infos->x_min = spectre->x_min + (x - (fenetre->w_x + 2)) / x_echelle;
      /*met_fvaleur(ELIM_MU0_1, VALEUR_E0, infos->x_min);*/
    } else { /* Bouton droit -> x_max */
      infos->x_max = spectre->x_min + (x - (fenetre->w_x + 2)) / x_echelle;
      /*    met_fvaleur(ELIM_MU0_1, VALEUR_E2, infos->x_max); */
    }
    IMG_moyenne_tranches(source, infos); /* Calcul */
    FEN_redessin(fenetre, BI_RDS_TOUT);
  } else { /* Clic sur la partie image */
    float l_case, h_case;
    float echelle;
    unsigned int x_image, y_image;
    int i_1, j_1, i_2, j_2;

    x_image = fenetre->w_x + (fenetre->w_larg >> 1) + 5;
    y_image = fenetre->w_y + 5;
    echelle = IMG_dim_trace(infos->tranche, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, NULL, NULL);
    l_case = infos->tranche->l_reel * echelle;
    h_case = infos->tranche->h_reel * echelle;

    /* Premier point clique */
    i_1 = (int) ( (x - x_image)/l_case );
    j_1 = (int) ( (y - y_image)/h_case );
    if ( prepare_zoom(fenetre, x, y, &i_2, &j_2) == VRAI ) {
      i_2 = (int) ( (i_2 - x_image)/l_case );
      j_2 = (int) ( (j_2 - y_image)/h_case );

      if (i_2 < i_1) { int i = i_2; i_2 = i_1; i_1 = i;}
      if (j_2 < j_1) { int j = j_2; j_2 = j_1; j_1 = j;}

      if ( (i_2 < 0) || (j_2 < 0) || (i_1 >= (int) source->larg) || (j_1 >= (int) source->haut) )
	return VRAI; /* Clic en-dehors, on ne fait rien ! */
      if (i_1 < 0) i_1 = 0; /* On se recadre dans la zone permise */
      if (j_1 < 0) j_1 = 0;
      if (i_2 >= (int) source->larg) i_2 = source->larg - 1;
      if (j_2 >= (int) source->haut) j_2 = source->haut - 1;

      infos->x = i_1; infos->y = j_1;
      /*      printf("Coordonnees debut : %u, %u\n", infos->x, infos->y); */
      infos->larg = i_2 - i_1 + 1; infos->haut = j_2 - j_1 + 1;

      IMG_moyenne_rectangle(source, infos, FAUX); /* On calcule */
      FEN_redessin(fenetre, BI_RDS_TOUT); /* On affiche */
    }
  }

  return BI_EVENEMENT_TRAITE;
}
