/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS
   Cration : 13 juin 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Chargement des images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "bin_utils.h"

#include "fichiers.h"
#include "images.h"
#include "serie.h"

/* -------- Chargement d'une image [donnes brutes] -------- */

Image *IMG_Charger(FILE *fichier, int version, const char *nom_fichier )
{
  Image *image = NULL;
  char *nom;
  int type, genre;
  unsigned int larg, haut;

  if ( NULL == fichier ) return NULL;

  /* Le genre de l'image */
  if ( version < 32 ) {
    genre = IMG_GENRE_REELLE;
  } else {
    fread( &genre, sizeof( int ), 1, fichier );
  }

  /* Le type d'image */
  fread( &type, sizeof( int ), 1, fichier );

  /* Le nom de l'image */
  nom = BIN_CH_chaine( fichier );

  /* Le nombre de cases en X, en Y */
  fread( &larg, sizeof( unsigned int ), 1, fichier );
  fread( &haut, sizeof( unsigned int ), 1, fichier );

  /* On essaye de crer l'image */
  image = IMG_CreerImage( NULL, NULL, genre, type, larg, haut );
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans creer_image\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( nom );
    return NULL;
  }
  IMG_Titre( image, nom );
  free( nom ); nom = NULL;

  if ( version < 32 ) {
    unsigned long i, nbr_valeurs;
    float tmp, axe_X, axe_Y;

    /* Tailles de la grille */
    fread( &axe_X, sizeof( float ), 1, fichier );
    fread( &axe_Y, sizeof( float ), 1, fichier );
    IMG_ChangerEchelle( image, AXE_X | AXE_Y, (double) axe_X, (double) axe_Y );

    /* Position du coin bas-gauche dans le repre absolu */
    fread( &axe_X, sizeof( float ), 1, fichier ); image->x_min = (double) tmp;
    fread( &axe_Y, sizeof( float ), 1, fichier ); image->y_min = (double) tmp;
    IMG_ChangerRepere( image, AXE_X | AXE_Y, (double) axe_X, (double) axe_Y );

    /* Valeurs extrmes */
    fread( &tmp, sizeof( float ), 1, fichier ); image->val_min = (double) tmp;
    fread( &tmp, sizeof( float ), 1, fichier ); image->val_max = (double) tmp;

    /* Table des valeurs */
    nbr_valeurs = image->larg * image->haut;
    for ( i = 0 ; i < nbr_valeurs ; i++ ) {
      fread( &tmp, sizeof( float ), 1, fichier );
      image->valeurs.R[ i ] = (double) tmp;
    }

    /* On calcule les bornes, qui ne sont pas sauves */
    IMG_bornes( image );

    /* Termin pour cette version */
    return image;
  }

  /* Tailles de la grille */
  fread( &(image->l_reel), sizeof( double ), 1, fichier );
  fread( &(image->h_reel), sizeof( double ), 1, fichier );
  IMG_ChangerEchelle( image, AXE_X | AXE_Y, image->l_reel, image->h_reel ); /* Pour mettre  jour les dessins... */

  /* Position du coin bas-gauche dans le repre absolu */
  fread( &(image->x_min), sizeof( double ), 1, fichier );
  fread( &(image->y_min), sizeof( double ), 1, fichier );
  IMG_ChangerRepere( image, AXE_X | AXE_Y, image->x_min, image->y_min ); /* Pour mettre  jour les dessins... */

  /* Valeurs extrmes */
  fread( &(image->val_min), sizeof( double ), 1, fichier );
  fread( &(image->val_max), sizeof( double ), 1, fichier );

  /* Table des valeurs */
  if ( IMG_GENRE_REELLE == genre ) {
    fread( image->valeurs.R, sizeof( double ), image->larg * image->haut, fichier );
  } else if ( IMG_GENRE_COMPLEXE == genre ) {
    fread( image->valeurs.C, sizeof( Complexe ), image->larg * image->haut, fichier );
  } else if ( IMG_GENRE_GRAPHE ==  genre ) {
    Serie_Graphes *serie = NULL;
    char *titre;
    unsigned long i, nbr_valeurs;

    /* Prparation de la srie qui contient ces graphes */
    titre = TXT_genere_chaine( "Graphes de l'image %s", image->nom );
    serie = SERIE_GRF_CreerSerie( NULL, titre );
    free( titre );
    if ( NULL == serie ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la srie des graphes\n",
		     __FILE__, __FUNCTION__, __LINE__);
      IMG_EffacerImage( image, VRAI );
      return NULL;
    }

    /* Chargement des graphes dans cette srie */
    nbr_valeurs = image->larg * image->haut;
    for (  i = 0 ; i < nbr_valeurs ; i++ ) {
      image->valeurs.G[ i ] = CH_Graphe( fichier, version, nom_fichier, serie, FAUX );
    }

    /* On protge la srie et l'associe  l'image */
    serie->drapeaux.protegee = VRAI;
    serie->carto = image;
  } else {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Chargement images de type\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  /* On calcule les bornes, qui ne sont pas sauves */
  IMG_bornes( image );

  /* Chargement termin */
  return image;
}
