/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 6 septembre 2002

   Imports/export, des fichiers exprimentaux entre autres
   Import d'un fichier, ou d'une carto, enregistr sur la ligne D15 du LURE (fluo X, P. Chevallier)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"

#include "images.h"
#include "fichiers.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Image *D15_Cartographie(const char *nom_fichier);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

static Image *D15_Cartographie(const char *nom_fichier)
{
  FILE *fichier;
  Image *image = NULL;
  Serie_Graphes *spectres;
  char *titre, *radical, *nom_fichier_pca;
  float vx_x, x, vx_y, y;
  long position_debut;
  unsigned long nbr_X = 1, nbr_Y = 1;

  fichier = fopen(nom_fichier, "rb");
  if (NULL == fichier) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, NULL);
    (void) fprintf(stderr, "D15_Cartographie : impossible d'ouvrir le fichier %s\n", nom_fichier);
    return NULL;
  }

  if ( FAUX == sauter_lignes(fichier, 3) ) {  /* On saute les trois premires lignes */
    (void) fprintf(stderr, "D15_Cartographie : fin de fichier atteinte en sautant l'en-tte ! [%s]\n", nom_fichier);
    (void) fclose(fichier);
    return NULL;
  }

  titre = TXT_genere_chaine("Spectres de fluorescence X / carte %F", nom_fichier);
  spectres = SERIE_GRF_CreerSerie(NULL, titre);
  free(titre);
  if (NULL == spectres) {
    (void) fprintf(stderr, "D15_Cartographie : impossible de crer la srie\n");
    (void) fclose(fichier);
    return NULL;
  }

  position_debut = ftell(fichier);

  /* Premire ligne de la carte */
  fscanf(fichier, "%*d %*d %f %f", &vx_x, &vx_y);
  ignore_ligne(fichier);

  /* Deuxime ligne de la carte */
  fscanf(fichier, "%*d %*d %f %f", &x, &y);
  ignore_ligne(fichier);

  /* On prpare pour le chargement global */
  radical = strdup(nom_fichier);
  if (NULL == radical) {
    (void) fprintf(stderr, "D15_Cartographie : impossible de crer le radical des fichiers !\n");
    (void) fclose(fichier);
    SERIE_GRF_EffacerSerie( spectres, VRAI );
    return NULL;
  }
  if ( strchr(radical, '.') != NULL ) {
    *( strchr(radical, '.') ) = 0; /* On te le(s) suffixe(s), point compris */
  }

  nom_fichier_pca = (char *) calloc(strlen(radical) + 9, sizeof(char)); /* radical + 4 chiffres + .spc + 0 final */
  if (NULL == nom_fichier_pca) {
    (void) fprintf(stderr, "D15_Cartographie : impossible de crer le radical des fichiers !\n");
    (void) fclose(fichier);
    SERIE_GRF_EffacerSerie( spectres, VRAI );
    free( radical ); radical = NULL;
    return NULL;
  }

  if ( x == vx_x ) { /* Travail  X constant = en colonnes */
    fprintf(stderr, "D15_Cartographie :  FAIRE, cartographie en colonnes [X constant]\n");
  } else if (y == vx_y) { /* Travail  Y constant = en lignes */
    double pas_x, pas_y;

    pas_x = x - vx_x;
    /* On cherche le nombre de X pour chaque Y */
    do {
      nbr_X++;
      fscanf(fichier, "%*d %*d %*f %f", &y);
      ignore_ligne(fichier);
    } while (y == vx_y);

    pas_y = y - vx_y;
    /* On cherche ensuite le nombre de Y */
    nbr_Y = compte_lignes(fichier, 0);
    nbr_Y /= nbr_X; /* Chaque ligne doit avoir le mme nombre de points ! */
    nbr_Y++;

    printf("Carte : %lu * %lu, en lignes - pas X : %f microns, Y : %f microns\n", nbr_X, nbr_Y, pas_x, pas_y);

    image = IMG_CreerImage( NULL, NULL, IMG_GENRE_GRAPHE, IMG_TYPE_CARTOGRAPHIE, nbr_X, nbr_Y);
    if (NULL == image) {
      (void) fprintf(stderr, "D15_Cartographie : impossible de crer l'image\n");
      (void) fclose(fichier);
      SERIE_GRF_EffacerSerie( spectres, VRAI );
      free( radical ); radical = NULL;
      free( nom_fichier_pca ); nom_fichier_pca = NULL;
      return NULL;
    }
    image->l_reel = fabs(pas_x);
    image->h_reel = fabs(pas_y);

    if (pas_y > 0) {
      unsigned long i, j, numero_case;
      unsigned int num_bloc, num_fichier;

      for (i = 0 ; i < nbr_Y ; i++) {

	if (pas_x > 0) {
	  for (j = 0 ; j < nbr_X ; j++) {
	    Graphe *fluoX;
	    
	    fscanf(fichier, "%*d %*d %*f %*f %*d %u %u", &num_fichier, &num_bloc);
	    ignore_ligne(fichier);
	    sprintf(nom_fichier_pca, "%s%04d.SPC", radical, num_fichier);
	    fluoX = Import_PCA3(nom_fichier_pca, spectres, num_bloc);
	    if (NULL == fluoX) {
	      (void) fprintf(stderr, "D15_Cartographie : erreur dans Import_PCA3 [fichier %s : n%u, bloc %u]\n", nom_fichier_pca, num_fichier, num_bloc);
	      (void) fclose(fichier);
	      free(radical);
	      free(nom_fichier_pca);
	      return NULL;
	    }

	    IMG_IJ_v_N( image, j, i, &numero_case );
	    image->valeurs.G[ numero_case ] = fluoX;
	  }
	} else {
	  for (j = nbr_X ; j > 0 ; j--) {
	    Graphe *fluoX;
	    
	    fscanf(fichier, "%*d %*d %*f %*f %*d %u %u", &num_fichier, &num_bloc);
	    ignore_ligne(fichier);
	    sprintf(nom_fichier_pca, "%s%04d.SPC", radical, num_fichier);
	    fluoX = Import_PCA3(nom_fichier_pca, spectres, num_bloc);
	    if (NULL == fluoX) {
	      (void) fprintf(stderr, "D15_Cartographie : erreur dans Import_PCA3 [fichier %s : n%u, bloc %u]\n", nom_fichier_pca, num_fichier, num_bloc);
	      (void) fclose(fichier);
	      free(radical);
	      free(nom_fichier_pca);
	      return NULL;
	    }

	    IMG_IJ_v_N( image, j - 1, i, &numero_case );
	    image->valeurs.G[ numero_case ] = fluoX;
	  }
	}
      }
    } else {
      printf("D15_Cartographie : carte en lignes, pas Y ngatif :  FAIRE\n");
    }

    fseek(fichier, SEEK_SET, position_debut); /* On se remet au dbut */
  }

  IMG_bornes(image);

  free(radical);
  free(nom_fichier_pca);

  return image;
}

/* -------- Interface gnrique -------- */

void Import_LURED15(void)
{
  char **noms;      /* Les adresses des noms des fichiers  charger */
  const char *extensions[] = { "*.spc,*.SPC" };
  char masque[12] = "*.spc";
  unsigned int i, nbr_fichier = 0;

  noms = CH_selecteur( masque, "Fichier LURE D-15 PCA3", 1, extensions, &nbr_fichier, principale );
  if (noms == NULL) {  /* Aucun fichier  traiter : on essaye une carto */
    noms = CH_selecteur( "*.POS", "Cartographie LURE D-15 PCA3", 0, NULL, &nbr_fichier, principale );
    if (NULL == noms) return;

    for (i = 0 ; i < nbr_fichier ; i++) {
      (void) D15_Cartographie(noms[i]);
      free(noms[i]); /* Plus besoin de ce nom */
    }
    free(noms); /* Plus besoin de la liste de noms */

    return;
  }

  for (i = 0 ; i < nbr_fichier ; i++) {
    (void) Import_PCA3(noms[i], NULL, 0);

    free(noms[i]); /* Plus besoin de ce nom */
  }
  free(noms); /* Plus besoin de la liste de noms */
}
