/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (c) Emmanuel CURIS 1998 - 2007
   Cration : 23 janvier 2007
   
   Toutes les fonctions d'automatisation des calculs
   FICHIER  INCLURE
*/

#ifndef LASE_AUTOMATISE_H_
# define LASE_AUTOMATISE_H_ /* Protection contre l'inclusion multiple */

# include "fonctions.h"		/* On aura de toute faon besoin de la liste des fonctions ! */

/* -------- Les structures intermdiaires -------- */

/* Les structures dfinies ailleurs */
struct graphe;			/* Dans graphes.h */
struct image;			/* Dans imagesh */
struct molecule;		/* Dans atomes.h */

/* L'objet sur lequel porte l'automatisation */
struct objet_generique {
  struct graphe *graphe;
  struct image  *image;
  struct molecule *molecule;
};

/* Les informations ncessaire  une automatisation */
struct automatiser {
  unsigned int type_objet;	/* Sur quel type d'objet travaille-t-on ? */
  unsigned int classe_fonction;	/* Quelle type de fonction doit-on appliquer dessus ? */
  unsigned int genre, type;	/* Genre et type, si ncessaire */
  unsigned int numero_fonction;	/* Quelle fonction exacte appliquer dessus ? */

  struct objet_generique reference; /* L'objet servant de rfrence */

  unsigned long nombre_objets;	     /* Sur combien d'objets appliquer l'automatisation ? */
  struct objet_generique *objets;    /* Table dynamique : les objets  traiter */
  struct objet_generique *resultats; /* Table dynamique cre par les fonctions : les rsultats */

  unsigned int methode;		/* Comment automatiser ? */
  void *infos_necessaires;	/* Infos pour l'automatisation, spcifiques de la fonction considre */

  char *nom_serie;		/* Si non-NULL : nouvelle srie demande, avec le nom indiqu */
};

/* -------- Les constantes de codage des protocoles d'automatisation -------- */

/* Types d'objet traits */
#define AUTOMATISE_GRAPHE   0
#define AUTOMATISE_IMAGE    1
#define AUTOMATISE_MOLECULE 2

/* Classe et numro de fonction : cf. fonctions.h */

/* Mthodes d'automatisation */
#define AUTOMATISE_METHODE_ROBOT     0 /* Absolument identique  la fonction originale */
#define AUTOMATISE_METHODE_ADAPTE    1 /* Mme mthode, mais recalcule pour chaque objet */
#define AUTOMATISE_METHODE_OPTIMISE  2 /* Mme mthode, adapte pour reproduire au mieux la rfrence */
#define AUTOMATISE_METHODE_REDEMANDE 3 /* On redemande  chaque fois [intrt limit...] */

#define AUTOMATISE_NOMBRE_METHODES 4

/* -------- Les fonctions associes  l'automatisation -------- */

struct automatiser *AUTO_GRF_PreparerStructure(unsigned long nombre_graphes, struct graphe **graphes);
struct automatiser *AUTO_IMG_PreparerStructure(unsigned long nombre_images, struct image **images);
struct automatiser *AUTO_MOL_PreparerStructure(unsigned long nombre_molecules, struct image **molecules);

int AUTO_LibererStructure(struct automatiser *structure);

int AUTO_EffectuerCalculs(struct automatiser *structure);
int AUTO_EffectuerCalculs_GRF(struct automatiser *structure);
int AUTO_EffectuerCalculs_IMG(struct automatiser *structure);
int AUTO_EffectuerCalculs_MOL(struct automatiser *structure);

#endif /* LASE_AUTOMATISE_H_ */
