/* LASE : logiciel d'analyse des spectres EXAFS
   (c) 1996-2001, Emmanuel CURIS
   Ce fichier : 29 septembre 1998

   Module de  Calcul formel 

   Dclarations des fonctions
*/

#ifndef CALCUL_FORMEL_H_

#define CALCUL_FORMEL_H_ /* Protection contre l'inclusion multiple */

/* -------- Qu'est-ce qu'une expression ? ------------------------------- */

struct parametre;		/* Dans modeles.h */
struct variable;		/* Ci-dessous */

typedef struct expr
{
  int type;
  int code;

  union
  {
    double valeur;
    struct expr *expression;
    struct variable *variable;
  } arg_1;
  union
  {
    double valeur;
    struct expr *expression;
    struct variable *variable;
  } arg_2;
} Expression;

/* Types d'expression rencontrs */
#define EXP_CTE_CTE    0 /* Les deux arguments sont des nombres    */
#define EXP_CTE_ARG    1 /* Le premier argument est une expression, le second un nombre */
#define EXP_ARG_ARG    2 /* Les deux arguments sont des relations  */
#define EXP_CTE_VAR    3 /* Premier argument est une variable, le second un nombre */
#define EXP_VAR_VAR    4 /* Les deux arguments sont des variables */
#define EXP_ARG_VAR    5 /* Le premier argument est une expression, le second une variable */
#define EXP_VAR_ARG    6 /* Le premier argument est une variable, le second une expression */
#define EXP_ARG_CTE    7 /* Le premier argument est une expression, le second un nombre */
#define EXP_VAR_CTE    8 /* Le premier argument est une variable, le second un nombre */

/* Les codes de traitement */
#define EXP_INDEFINIE -1 /* Expression en cours de traitement */

#define EXP_NOMBRE_ISOLE 0 /* Dans le premier argument ; devrait tre vit, logiquement... */

/* Oprations  deux arguments */
#define EXP_ADDITION     1
#define EXP_SOUSTRACTION 2
#define EXP_PRODUIT      3
#define EXP_DIVISION     4
#define EXP_PUISSANCE    5

/* Oprations  un seul argument */
#define EXP_SINUS       10
#define EXP_COSINUS     11
#define EXP_TANGENTE    12
#define EXP_COTANGENTE  13
#define EXP_SECANTE     14
#define EXP_COSECANTE   15

#define EXP_COSH        20
#define EXP_SINH        21
#define EXP_TANH        22
#define EXP_COTANH      23
#define EXP_SECH        24
#define EXP_COSECH      25

#define EXP_EXP         30
#define EXP_EXP10       31
#define EXP_LOG         32
#define EXP_LOG10       33

#define EXP_RACINE      40
#define EXP_CARRE       41
#define EXP_CUBE        42
#define EXP_INVERSE     43

#define EXP_ASIN        50
#define EXP_ACOS        51
#define EXP_ATAN        52

#define EXP_ASINH       60
#define EXP_ACOSH       61
#define EXP_ATANH       62

#define EXP_LAMBERTW0   70	/* Fonction W de Lambert, branche 0 --- modle de Michaelis intgr, p. ex. */

#define OperationADeuxParametres(code) ( (code == EXP_ADDITION) || (code == EXP_SOUSTRACTION) || \
					 (code == EXP_PRODUIT) || (code == EXP_DIVISION) || (code == EXP_PUISSANCE) )

/* Fonctions pour grer les expressions */

Expression *construit_arbre(const char *formule);
Expression *CF_ConstruireArbre(const char *formule, unsigned long n_parametres, struct parametre *parametres);
boolean     calcule_arbre(Expression *arbre, double *resultat);
void        effacer_arbre(Expression *arbre);

double     *TABLE_Creer_Formule(double *table_reference, unsigned long nombre_valeurs, const char *formule, unsigned long n_parametres, struct parametre *parametres);

/* -------- Qu'est-ce qu'une grandeur que l'on peut faire varier ? ------ */

typedef struct variable
{
  double valeur_initiale;
  double min, max;        /* Intervalle de variation */
  double delta;           /* Pas minimal de variation (0 = au choix) */
  int type;               /* Voir les types possibles ci-dessous     */
  union
  {
    double valeur;             /* C'est une valeur numrique */
    struct variable *original; /* Fantme                    */
    Expression expression;     /* Dfinie par une relation   */
  } contenu;
} Variable;

#define VAR_CONSTANTE  1 /* Fausse variable... = valeur initiale, toujours */
#define VAR_LIBRE      2 /* Variable simple    */
#define VAR_COPIE      3 /* Variable fantme, toujours gale  une autre */
#define VAR_EXPRESSION 4 /* Valeur = expression(autres variables), calcule APRS ajustement */
#define VAR_RELATION   5 /* Idem, mais intervient pour chercher le minimum (+ raliste mais + complexe... */

#define VARIABLE_LIRE_VALEUR_INITIALE 0 /* On renvoie la valeur initiale de la variable */
#define VARIABLE_LIRE_VALEUR_MINIMALE 1 /* On renvoie la valeur minimale autorisee pour la variable */
#define VARIABLE_LIRE_VALEUR_MAXIMALE 2 /* On renvoie la valeur maximale autorisee pour la variable */
#define VARIABLE_LIRE_VALEUR_AJUSTEE  3 /* On renvoie la valeur "reelle" de la variable */

int    VAR_Initialiser   (Variable *variable, unsigned int type, double valeur, double min, double max, double pas);
int    VAR_FixerValeur   (Variable *variable, unsigned int classe_valeur, double nouvelle_valeur);
double VAR_LireValeur    (Variable *variable, unsigned int classe_valeur);
int    VAR_ControleBornes(Variable *variable);

/* -------- Macro pour l'historique -------- */

void HIST_VAR_ValeurVariable(Variable *variable, const char *texte);

#ifndef VARIABLES_CALCUL_FORMEL
extern Variable Var_X, Var_Y, Var_Z, Var_T, Var_I, Var_R; /* Dans variables.c */
extern Variable Var_sX, Var_sY;
#endif

#endif /* CALCUL_FORMEL_H_ */
