/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1998-2002

   Dfinitions pour les chemins d'lectrons

   Le chemin : ensemble ordonn d'atomes que parcours formellement le photolectron
   avant de revenir sur l'atome absorbeur. Un chemin commence et finit toujours par
   l'atome absorbeur -> il n'est jamais prcis ici...

   Les chemins ne peuvent pas tre partags entre plusieurs modles
*/

#ifndef CHEMINS_H_
# define CHEMINS_H_  /* Protection contre l'inclusion multiple */

/* -------- La structure chemin : fondamentale... -------- */

# define LG_MAX_NOM_CHEMIN 63 /* avec le 0 final, donne un multiple de 8 */

struct couche; /* Dfinie dans couches.h */

typedef struct chemin
{
  char nom[LG_MAX_NOM_CHEMIN + 1];
  unsigned int max_atomes; /* Nombre maximal d'atomes dans le chemin pour le tableau rserv */
  unsigned int nbr_atomes; /* Nombres d'atomes dans le chemin, absorbeur non compris (fin comme dbut) */
  struct atome   **atomes; /* Ensemble des atomes du chemin. Attention, l'ordre compte ! Les atomes
			      sont supposs parcourus dans l'ordre indiqu.
			      C'est en fait un tableau dynamique. */

  struct graphe *exf_calcule; /* Spectre EXAFS calcul pour ce chemin */

  struct graphe *amplitude; /* Amplitude et phase du chemin */
  struct graphe *phase;

  double multiplicite;    /* Multiplicit du chemin */
  double longueur_reelle; /* Longueur relle du trajet effectu par l'lectron */
  double pseudo_longueur; /*  utiliser dans la formule de l'EXAFS */
  double dw_global;       /* Facteur de Debye-Waller global associ */

  unsigned int nbr_equivalents;/* Nombre de chemins quivalents par symtrie (ou pseudo-symtrie) */
  struct chemin **equivalents; /* Chemins quivalents par symtrie, qui ont t regroups dans celui-ci
				  Tableau dynamique (devrait y en avoir peu, une dizaine au mieux */
} Chemin;

#define LG_MAX_TXT_CHEMIN 32 /* Nom du chemin dans la fentre d'affichage 3d */

Chemin *creer_chemin  (unsigned int nbr_atomes);
int     CHEMIN_AjouterEquivalent(Chemin *reference, Chemin *equivalent);

#define EFF_CHEMIN_SANS_GRAPHES 0
#define EFF_CHEMIN_TOUS_GRAPHES 1
#define EFF_CHEMIN_GRAPHES_EQU  2 /* Effacer seulement les graphes pour les chemins quivalents */

boolean CHEMIN_Effacer(struct couche *couche, Chemin *chemin, boolean forcer, boolean atomes, int graphes);

boolean CHEMIN_Combiner();
/*
int combiner_chemins(Absorbeur *absorbeur);
int modifier_chemins(Absorbeur *absorbeur, Chemin *chemin);

Absorbeur *CHM_quel_absorbeur(Chemin *chemin);
*/

void nommer_chemin(Chemin *chemin, char *nom);

#endif /* CHEMINS_H_ */
