/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Cration              : 16 septembre 1998
   Dernire modification :  5 novembre  1998

   Module d'ajustement par un modle  n couches

   Dclarations pour l'ensemble de ce module
*/

#ifndef N_COUCHES_

#define N_COUCHES_ /* Protection contre l'inclusion multiple */

#include "serie.h" /* Sries de graphes */
#include "c_formel.h" /* Dclaration de ce qu'est une variable */

#define LG_MAX_NOM_COUCHE 95 /* Longueur maximale d'un nom de couche, zro final non compris
			       95 + 1 = 96 caractres, multiple de 8 : mieux pour l'alignement */

struct chemin;        /* Dans chemins.h */
struct modele_ajuste; /* Dans modeles.h */
struct modele_feff;   /* Dans feff.h */

typedef struct couche
{
  struct couche *suivante; /* Pointeur sur la couche suivante dans le modle */
  char nom[LG_MAX_NOM_COUCHE + 1]; /* Nom de la couche */

  boolean utilisee;        /* VRAI s'il faut la prendre en compte dans le modle */

  Variable N;              /* Nombre de voisins = dgnrescence     */
  Variable R;              /* Longueur du chemin                     */
  Variable Debye_Waller;   /* Facteur de Debye-Waller                */
  Variable Cumul_1;        /* Cumulants pour le dsordre             */
  Variable Cumul_3;
  Variable Cumul_2;        /* Cumulants pour la phase, si dsordonn */
  Variable Cumul_4;
  Variable Delta_E;        /* Delta_E, si un par couche              */
  Variable Gamma;          /* Libre parcours moyen, mme remarque    */

  Graphe *signal;          /* Signal EXAFS associ au chemin, s'il existe  */
  Graphe *amplitude;       /* Amplitude associe au chemin, si elle existe */
  Graphe *phase;           /* Dphasage associ au chemin, s'il existe     */

  double *b_amp, *c_amp, *d_amp; /* Coefficients de lissage pour l'amplitude */
  double *b_pha, *c_pha, *d_pha; /* Coefficients de lissage pour la phase    */

  struct chemin *chemin; /* Le chemin rel (moyen) associ  cette couche */
} Couche;

#define VAR_SO2          1 /* Type de la variable pour l'ajustement */
#define VAR_DELTAE_GENE  2
#define VAR_GAMMA_GENE   3
#define VAR_CONTRIBUTION 4

#define VAR_N           10
#define VAR_DEBYE       11
#define VAR_R           12
#define VAR_CUMUL_1     13
#define VAR_CUMUL_2     14
#define VAR_CUMUL_3     15
#define VAR_CUMUL_4     16
#define VAR_DELTAE      17
#define VAR_GAMMA       18

/* ------ Routines de gestion des couches (couches.c) ------ */

int choisit_couche(Couche *prem_couche);

/* ------ Routines de paramtrage de l'ajustement ---------- */

/*void    ASC_nc_couches(Fenetre *fenetre, int type, int ss_type, int position);*/
int GER_nc_couches(unsigned int objet, int x, int y);

void lance_general(void);

#ifndef VARIABLES_GENERALES_

extern Couche *couche_editee;

extern double val_eta;            /* Valeur de ta, si ncessaire  */

#endif /* VARIABLES_GENERALES_ (dfinies dans variables.c) */

/* ------ Routines de calcul effectif de l'ajustement ------ */

/* ---- Quelques routines globales */

void    COUCHE_Aller                (struct modele_feff *modele, Couche *couche);
Couche *COUCHE_Choisir              (struct modele_feff *modele, const char *titre, int arbre, int objet);
Couche *COUCHE_Creer                (struct modele_feff *modele, Couche *avant );
int     COUCHE_Effacer              (struct modele_feff *modele, Couche *couche, boolean forcer, boolean eff_graphes);
void    COUCHE_EnregistrerModele    (void);
void    COUCHE_EnregistrerParametres(Couche *couche);
int     COUCHE_Inserer              (struct modele_feff *modele, Couche *avant, Couche *couche);
int     COUCHE_Liberer              (Couche *couche, boolean eff_graphes);
void    COUCHE_Nommer               (Couche *couche, const char *nom);
void    COUCHE_OuvrirEditeur        (struct modele_feff *modele);

int     COUCHE_CalculerContribution (struct modele_feff *modele_feff, Couche *couche, struct graphe *modele);

/* ---- Les briques de base pour construire des trucs plus complexes */

int NC_ControleModele(struct modele_feff   *modele, double x_min, unsigned int *nbr_variables);
int NC_LibereModele  (struct modele_ajuste *modele, boolean graphes);
int NC_PreparerModele(struct modele_feff   *modele);

Fenetre *NC_ouvre    (struct modele_ajuste *modele);

void  NC_AfficheValeurs(struct modele_ajuste *modele);
char *NC_InfosVariable (int type, struct couche *couche);
char *NC_UniteVariable (int type, struct couche *couche);

/* ---- Les fonctions de calcul du modle EXAFS  partir des chemins ---- */

int NC_CalculeEXAFS      (struct modele_ajuste *modele);
int NC_CalculeEXAFSCouche(struct modele_feff *feff, Couche *couche, struct graphe *exafs);

double calc_derivee        (double k, Couche *couche, struct modele_feff *feff, int type, double valeur,
			    double val_khi, double delta_e);
int    calc_exafs_ncouches (Couche *prem_couche, Graphe *exafs);
double calc_exafs_1point   (double k, struct modele_feff *feff, Couche *couche, double delta_e);

struct serie *NC_CalculeContributions(struct modele_feff *feff, struct graphe *modele);

/* ---- Macros d'ajustement -------- */

int NC_AfficherResultats(struct modele_feff *modele, boolean historique);

int NC_AjusterEXAFS  (struct modele_ajuste *modele, Fenetre *fenetre);
int NC_AjusterModele (struct modele_ajuste *modele);
int NC_AjusterSpectre(struct graphe *reference);

int NC_IncrementsPG  (struct modele_ajuste *modele);

int NC_PGInitVariables(struct modele_ajuste *modele, unsigned int prem_variable);

int NC_LibererModele (struct modele_feff *modele, boolean eff_coef_splines);

/* void   reinit_rapide (unsigned int nbr_var); */
/* double val_rapide    (unsigned int j); */

int NC_Verifier(struct modele_feff *modele, int methode); /* Cohrence d'un rsultat d'ajustement */

/*boolean NC_covariance(Infos_ncouches *infos);*/ /* Calcul de la matrice de covariance */

/* ~~~~~~~~ Travail sur le modle ~~~~~~~~ */

#define COMBINER_COUCHES_TOTAL        1 /* VRAI : on considre toutes les couches, meme avec intercalations d'autres */
#define COMBINER_COUCHES_DISTANCE     2 /* VRAI : on utilise la distance comme critre */
#define COMBINER_COUCHES_ANGLE        4 /* VRAI : on utilise l'angle comme critre */
#define COMBINER_COUCHES_TOLERANCE_Z  8 /* VRAI : tolrance Z/Z+1/Z-1 dans les chemins */
#define COMBINER_COUCHES_EFFACER     16	/* VRAI : on efface les anciennes couches */
#define COMBINER_COUCHES_MOYENNER    32	/* VRAI : on moyenne les phases & amplitudes aussi */
#define COMBINER_COUCHES_DISTANCE_TOTALE 64 /* VRAI : la tolrance de distance porte sur la distance entre le premier et le dernier chemin */
#define COMBINER_COUCHES_ESTIMER_DW 128	/* VRAI : on estime la valeur du facteur de Debye-Waller */
#define COMBINER_COUCHES_N_CONSTANT 256	/* VRAI : le N de la couche cre est constant */
#define COMBINER_COUCHES_R_CONSTANT 512	/* VRAI : le R de la couche cre est constant [2^9] */

struct infos_combiner_couches {
  unsigned int drapeaux;	/* Masque de bits : cf ci-dessus */
  double distance_limite;	/* Diffrence de distance maximale tolre [en A] */
  double angle_limite;		/* Diffrence d'angle maximale tolre [en degrs] */

  /* Les rsultats */
  unsigned long nombre_groupes;	/* Nombre de groupes de couches au final */
  unsigned long nombre_uniques;	/* Nombre de couches sans quivalent ("uniques") */
};

int NC_CombinerCouches(struct modele_feff *modele, struct infos_combiner_couches *parametres);

#if defined(_PURE_C_) || defined(_TURBO_C_)
# include "atomes.h" /* Pour struct chemin, sinon Pure C / Turbo C n'aiment pas... */
#endif

#endif /* N_COUCHES_ */
