/* Traitement de l'E.X.A.F.S. sur ATARI
   Extraction du signal EXAFS
   (c) 1998 - 2002, Emmanuel CURIS
*/

#ifndef EXTRACTION_H_

#define EXTRACTION_H_ /* Protection contre l'inclusion multiple */

struct automatiser;		/* Dans Include/automatiser.h */
struct graphe;			/* Dans Include/graphes.h */

/* ---------- Recomposition du signal brut ---------------------- *
 *      (Module Extrait/calcul.c)                                 *
 ******************************************************************/

struct graphe *XAN_extrait(void);

struct graphe *MU_Fluorescence(struct graphe *i0, struct graphe *i1);
struct graphe *MU_Transmission(struct graphe *i0, struct graphe *i1);

/* ---------- Extraction du signal E.X.A.F.S. ------------------- *
 *      (Modules EXAFS/...)                                       *
 ******************************************************************/

/* ---------- Reproduction de l'extraction  l'identique -------- */

void EXF_reproduit(struct graphe *graphe);

struct infos_reproduit_filtre; /* Dans filtrage.h */
boolean EXF_chparams(struct infos_reproduit_filtre *infos); /* Charger - sauver params. d'extraction */
boolean EXF_svparams(void);

/* ---------- limination de 0, premire tape -------------- */

#define XAS_MU01_DEGRE_MAXIMAL 10  /* Degr maximal du polynme utilisable */

typedef struct
{
  double e_0;       /* Valeur de E0                     */
  double e_2;       /* Valeur de E2                     */
  double e_3;       /* Valeur de E3                     */
  int methode;      /* Type de calcul du polynome       */
  struct graphe *exafs;    /* Le signal EXAFS extrait          */
  struct graphe *mu_0;     /* Le mu_0 utilis                  */
  int deg_affiche;  /* Degr pour affichage : k^n * spectre */
  int avec_e3;      /* VRAI si on utilise le 3e point   */
  unsigned int degre; /* Degr du polynme utilis      */
  double coefs[ XAS_MU01_DEGRE_MAXIMAL + 1 ]; /* Coefficients du polynome utilis */
  double residu;		/* Somme des rsidus au carr */
  struct graphe *reference;	/* Graphe de rfrence pour la recherche du 0 */
} MU1_infos;

struct graphe *MU1_ExtraitEXAFS(struct graphe *brut, MU1_infos *infos, boolean recalcule);

int XAS_MU01_CalculerMu0_I (const struct graphe *brut, MU1_infos *infos);
int XAS_MU01_ExtraireEXAFS (const struct graphe *brut, MU1_infos *infos);
int XAS_MU01_PostExtraction(const struct graphe *brut, struct graphe *exafs_resultat, MU1_infos *infos);

int XAS_MU01_Automatisation(struct automatiser *lot);

/* ---------- limination de 0, deuxime tape -------------- */

typedef struct
{
  double coefs[11]; /* Coefficients du polynme utilis     */
  unsigned int degre; /* Degr du polynme utilis          */
  int methode;      /* Mthode de calcul du polynme        */
  struct graphe *exafs;    /* Le signal EXAFS extrait              */
  struct graphe *mu_0;     /* Le polynme soustrait                */
  struct graphe *mu_0_ref; /* Le polynme soustrait, tape d'avant */
} MU2_infos;

struct graphe *MU2_ouvre(struct graphe *brut, struct graphe *mu_0, boolean chainee, struct graphe **mu0);
struct graphe *MU2_ExtraitEXAFS(struct graphe *brut, MU2_infos *infos, boolean recalcule);

void MU2_calc   (struct graphe *brut, MU2_infos *infos);
void MU2_extrait(struct graphe *brut, MU2_infos *infos);

/* ---------- limination de [mu]0, troisime tape ------------- */

typedef struct
{
  double parametre; /* Valeur du paramtre de lissage       */
  struct graphe *exafs;    /* Le signal EXAFS extrait              */
  struct graphe *mu_0;     /* Le polynme soustrait                */
  struct graphe *mu_0_ref; /* Le polynme soustrait, tape d'avant */
} MU3_infos;

struct graphe *MU3_ouvre(struct graphe *brut, struct graphe *mu_0, boolean chainee);
struct graphe *MU3_ExtraitEXAFS(struct graphe *brut, MU3_infos *infos, boolean recalcule);

void MU3_calc   (struct graphe *brut, MU3_infos *infos);
void MU3_extrait(struct graphe *brut, MU3_infos *infos);

/* ---------- limination d'un [mu]0 calcul par ailleurs ------- */

typedef struct
{
  double e_0;        /* nergie de seuil         */
  int methode;       /* Mthode de calcul du [mu]0 (en nergie, en nombre d'onde ?) */
  struct graphe *mu_0;      /* Le graphe choisi...      */
  struct graphe *exafs;     /* Le spectre EXAFS extrait */
  double *b, *c, *d; /* Tableaux de coefficients pour le lissage par splines */
  char degre;        /* Puissance  l'affichage  */
} MU0_infos;

void MU0_ouvre(struct graphe *brut, struct graphe *mu_0);

void MU0_extrait(struct graphe *brut, MU0_infos *infos);
void MU0_prepare(MU0_infos *infos);

/* ---------- Normalisation d'un spectre exafs ------------------ */

#define NORM_AUCUNE      -1
#define NORM_EXPERIENCE  -2

struct graphe *XAS_NormaliserEXAFS(struct graphe *source);

/* Codes d'erreur : <0 => le calcul n'a pas pu avoir lieu
                    >0 => le calcul a eu lieu, mais douteux */

#define NORMEXAFS_NULL              -1 /* Un graphe est NULL */
#define NORMEXAFS_X_INCOHERENTS     -2 /* Les X sont diffrents entre 0 et 0*khi(k) */
#define NORMEXAFS_ERR_INTERPOLATION -3 /* Erreur en voulant interpoler... */
#define NORMEXAFS_FACTEUR_INCORRECT -4 /* Valeur absurde d'un paramtre */
#define NORMEXAFS_METHODE_INCONNUE  -5 /* Mthode de normalisation inconnue */
#define NORMEXAFS_UNITE_INCORRECTE  -6	/* Unit en X (ou Y) inconnue */

#define NORMEXAFS_DIV_ZERO            1 /* Division par 0 => qq valeurs fausses */
#define NORMEXAFS_ARRONDIS            2 /* Division par < 1e-4 ==> arrondis douteux */
#define NORMEXAFS_ERREURMU0_NEGLIGEES 3	/* Erreurs sur 0 ngliges */
#define NORMEXAFS_K_NEGATIF           4	/* Des valeurs en E sont avant le seuil ! */

int EXF_NormaliseExp(struct graphe *exafs, struct graphe *mu0, boolean coherent);
int EXF_NormaliseTh (struct graphe *exafs, unsigned int methode, double e0, int seuil, int Z,
		     double h_seuil, double *avant, double *apres);

/* ---------- Diverses "macros" de calcul ----------------------- */

void      corr_energie(struct graphe *spectre);
struct graphe *convert(struct graphe *spectre); /* Conversion k <-> E */
double    cherche_e0  (struct graphe *spectre, boolean affiche);

struct graphe *EXF_ponderer(struct graphe *spectre, int proposition_exposant); /* Multiplier par x^n */

int EXF_e_v_k(struct graphe *spectre, double e_0, boolean historique); /* Conversion E (eV)  -> k (1/A) */
int EXF_k_v_e(struct graphe *spectre, double e_0, boolean historique); /* Conversion k (1/A) -> E (eV)  */
int EXF_k_p_n(struct graphe *spectre, int degre, boolean historique);  /* Multiplication des Y par X^n */

double CONV_E_vers_k(double energie, double e0, const char *unite_e0, const char *unite_k);
double CONV_k_vers_E(double k, const char *unite_k, double e0, const char *unite_e0);

#define E_V_K_PAS_GRAPHE   -1 /* Pas de graphe pass            */
#define E_V_K_E0_FAUX      -2 /* Valeur de E0 transmise absurde */
#define E_V_K_VAL_NEGATIVE -3 /* On a pris E0 > x_min -> on a des k ngatifs ! */

#endif /* EXTRACTION_H_ */
