/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1998-2001, Emmanuel CURIS

   Tout ce qui concerne le chargement/la sauvegarde de fichiers rutilisables
   Fichier des dclarations locales  ce module
*/

#ifndef FICHIERS_H_

#define FICHIERS_H_ /* Protection contre l'inclusion multiple */

#include "txt_utils.h"   /* Intervalles... */
#include "ascii_utils.h" /* Fonctions gnriques de lecture en ASCII */

#if defined(_PURE_C_) || defined(_TURBO_C_)
#  include "atomes.h"  /* Pour la structure Molecule */
#  include "images.h"  /* Pour la structure Image    */
#  include "serie.h"   /* Pour la structure Graphe et Serie_Graphes */
#  include "tracer.h"  /* Pour la structure Figure   */
#else
struct figure;			/* Dfinie dans tracer.h */
struct molecule; /* Dfinie dans atomes.h  */
struct image;    /* Dfinie dans images.h  */
struct infos_lecture; /* Dfinie dans cristallo.h */
struct infos_PDB;/* Dfinie dans lit_pdb.h */
struct infos_superpose; /* Dfinie dans tracer.h */
struct graphe;   /* Dfinie dans graphes.h */
struct serie;    /* Dfinie dans serie.h   */
#endif

/* ~~~~~~~~~~~~~~~ Chargement et sauvegarde au format interne ~~~~~~~~~~~~~~~ */

#define FICHIER_GRAPHE      0 /* Le fichier contient un seul graphe */
#define FICHIER_MOLECULE    1 /* le fichier contient une molcule (et ce qui va autour) */
#define FICHIER_IMAGE       2 /* Le fichier contient une image 2D */
#define FICHIER_EXPERIENCE  3 /* Le fichier contient la description d'une exprience */
#define FICHIER_SERIE       4 /* Le fichier contient une srie de graphes */
#define FICHIER_SERIE_IMAGE 5 /* Le fichier contient une srie d'images */

struct graphe *charger   (char *nom);   /* Charger un graphe */
struct graphe *ch_interne(char *nom);   /* Routine effective */
struct molecule *charge_molecule(char *nom); /* Charger un modle structural */
struct molecule *MOL_charger    (char *nom); /* Routine effective */

/* ------ Les briques pour le chargement ------ */

inline struct graphe *CH_Graphe(FILE *fichier, int version, const char *nom_fichier, struct serie *serie, boolean isole);
inline struct image  *CH_Image (FILE *fichier, int version, const char *nom_fichier, boolean isolee);

/* ------ Les briques pour la sauvegarde ------ */

inline int SV_EnTete(FILE *fichier, int type); /* L'en-tte de fichier LASE */
inline int SV_Graphe(FILE *fichier, struct graphe *graphe); /* Un graphe avec flag d'existence */
inline int SV_Image (FILE *fichier, struct image *image); /* Une image avec flag d'existence */

/* ~~~~~~~~ Import de fichiers ASCII ~~~~~~~~~~~~~~~~~~~~ */

/* Formats de graphes */
#define ASCII_FORMAT_UNIVERSEL 0 /* Format ASCII universel : x colonnes rel multigraphes ventuel */
#define ASCII_FORMAT_EDA_XY    1 /* Format EDA original : 2 colonnes Y puis X (!) */
#define ASCII_FORMAT_EDA_YX    2 /* Nouveau format EDA  : 2 colonnes X puis Y */
#define ASCII_FORMAT_GNUPLOT   3 /* Format gnuplot 2D */
#define ASCII_FORMAT_LASE_R    4 /* Lase, graphe rel */
#define ASCII_FORMAT_LASE_C    5 /* Lase, graphe complexe */
#define ASCII_FORMAT_FEFF_F    6 /* FEFF : fichier feff____.dat */
#define ASCII_FORMAT_FEFF_C    7 /* FEFF : fichier chi.dat */
#define ASCII_FORMAT_FEFF_Cp   8 /* FEFF : fichier chip____.dat */
#define ASCII_FORMAT_FEFF_X    9 /* FEFF : fichier xmu.dat */
#define ASCII_FORMAT_SPEC     10 /* Spec, comme ESRF */
#define ASCII_FORMAT_HDF      11 /* Fichiers HDF v. 4 */

#define NOMBRE_FORMATS_ASCII 10

void GRF_ImportASCII(void);

#define IA_UTILISER_ERREURS 1 /* Bit mis -> on prend en compte les erreurs de la colonne */
#define IA_SERIE_X_COMMUNS  2 /* Bit mis -> tous les graphes ayant les mme X, ils utilisent les X de la srie */
#define IA_MULTI_SERIE      4 /* Bit mis -> un graphe multi-srie... */
#define IA_AVEC_HISTORIQUE  8 /* Bit mis -> on garde une trace dans l'historique */
#define IA_MULTI_COLONNES  16 /* Bit mis -> on fait un graphe par colonne (sauf celle des X) */

struct infos_import_grf {
  unsigned int proprietes; /* Masque de bits : voir ci-dessus */
  int commentaire; /* Caractre qui introduit les lignes de commentaire */
  int separateur;  /* Caractre sparant les colonnes (0 = espace, tab,...) */
  unsigned long lg_en_tete;   /* Nombre de lignes d'en-tte */
  unsigned int nbr_colonnes; /* Nombre de colonnes dans le fichier */
  unsigned int colonne_X, colonne_Y, colonne_Z; /* Agencement des colonnes dans le fichier */
  unsigned int colonne_Re, colonne_Im;
  char *filtre; /* Texte qui contient le filtre  utiliser pour choisir les colonnes [NULL = on charge tout] */
  boolean *charger_grf;       /* Table : pour chaque colonne, VRAI si on doit charger le graphe correspondant */
  unsigned int colonne_sX, colonne_sY, colonne_sIm;
  int type_import;     /* Type du graphe import */
  struct serie *serie; /* Srie dans laquelle charger le graphe */
};

int ASCII_Generique     (const char *nom_fichier, int format, struct infos_import_grf *infos);
int ASCII_PostChargement(int type, unsigned int nombre, struct graphe **table);
int ASCII_SauterEnTete  (FILE *fichier, unsigned long nbr_lg_en_tete, int commentaire);

struct graphe  *IA_Complexe  (FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos);
struct graphe  *IA_ReelUnique(FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos);
struct graphe **IA_ReelsMulti(FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos);

/* formats FEFF */

int FEFF_PostChargement(int format, unsigned int colonne_X, unsigned int nombre, struct graphe **table, const char *nom_fichier);
int FEFF_SauterEnTete  (FILE *fichier);

boolean fef_feff(char *nom);               /* Fichiers FEFFnnnn.DAT            */
boolean fef_chip(char *nom, boolean mode); /* Fichiers CHIPnnnn.DAT et CHI.DAT */
boolean fef_xmu (char *nom);               /* Fichiers XMU.DAT                 */

/* Formats de structure 3D */

int MOL_ImportASCII(void); /* Interface d'import */

#define FORMAT3D_ASCII_XYZ    0
#define FORMAT3D_ASCII_CARINE 1
#define FORMAT3D_ASCII_PDB    2
#define FORMAT3D_ASCII_CCDB   3
#define FORMAT3D_ASCII_CIF    4

#define NOMBRE_FORMATS_3D 5

int MOL_Import_CCDC(const char *nom_fichier, struct infos_lecture *infos); /* Fichier CCDC -> Atomes/lit_ccdc.c */
int MOL_Import_PDB (const char *nom_fichier, struct infos_PDB *options);   /* Fichier PDB  -> Atomes/lit_pdb.c  */
int MOL_Import_XYZ (const char *nom_fichier, unsigned int format, struct infos_lecture *infos); /* Fichier ASCII gnrique -> Atomes/lit_ascii.c */

/* format acr carine v. 3 */

int MOL_Import_Init_Carine(FILE *fichier, struct infos_lecture *infos); 

/* ~~~~~~~~ Export de fichiers  divers formats ~~~~~~~~ */

void    GRF_ExportASCII(struct graphe *graphe);

int EA_SerieComplete(struct serie *serie);

int EA_GrapheComplexe(FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes,
		      unsigned int nc_X, unsigned int nc_Re, unsigned int nc_Im,
		      unsigned int nc_sX, unsigned int nc_sRe, unsigned int nc_sIm);
int EA_GrapheReel    (FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes,
		      unsigned int nc_X, unsigned int nc_Y, unsigned int nc_sX, unsigned int nc_sY);
int EA_GrapheSerie   (FILE *fichier, struct graphe *graphe, int separateur, unsigned int nbr_colonnes,
		      unsigned int nc_X, unsigned int nc_Y, unsigned int nc_sY);

boolean convertit(FILE *fichier, unsigned long nbr_lignes); /* Conversion effective */

void export_pov(struct molecule *molecule, int mode);

int IMG_export_gnuplot(struct image *image);

#ifdef _TOS_
void EXP_image(boolean type); /* Diffrents formats seulement dans la version GEM */
#endif

#ifdef AVEC_PNG_
int sv_image(Fenetre *fenetre); /* Au format PNG */
#endif

/* Figures PostScript */
int EXP_PS_GrapheUnique(struct graphe *graphe);
int EXP_PS_Figure      (struct figure *figure);
int EXP_PS_Superposition(struct infos_superpose *infos_superposition);

/* -------- Fichiers de la ligne D15 (fluo. X) ---------- */

void Import_LURED15(void);

struct graphe *Import_PCA3(const char *nom_fichier, struct serie *serie, unsigned int num_bloc);

/* ~~~~~~~~~~~~~~~ Routines de gestion GEM associes ~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean GER_feff(unsigned int num_objet, int x_souris, int y_souris);
boolean GER_svgem(unsigned int num_objet, int x_souris, int y_souris);

/* ~~~~~~~~~~~~~~~ Extraction de donnes des fichiers de l'ESRF ~~~~~~~~~~~~~~ */

int import_ESRF(unsigned int ligne);

int import_id22(void); /* Fichiers de la ligne ID-22 */
int lit_id22(const char *nom, Intervalle_I *intervalles, unsigned long nbr_intervalles); /* Idem */

struct image *import_edf(void);
struct image *IMG_esrf_edf(const char *nom_fichier);

/* ~~~~~~~~~~~~~~~ Extraction de donnes : Hasylab ~~~~~~~~ */

/* struct graphe *IMP_Hasylab(const char *nom_fichier, Infos_Extraction *infos); */

/* ~~~~~~~~~~~~~~~ Extraction de donnes : formats divers ~~~~~~~~~~~~~~~~~~~~ */

struct image *PIXE_Bordeaux_fc(const char *nom, double pas_x, double pas_y);
struct image *PIXE_Bordeaux   (void);

/* -------- Chanes de caractres qui reviennent souvent : elles sont dans Textes/variable.* -------- */

extern const char *txt_err_format_incorrect;

#endif /* FICHIERS_H_ */
