/* LASE - Logiciel d'Analyse des Spectres EXAFS
   Déclarations concernant l'interface graphique [surcouche indépendante de l'OS]

   © Emmanuel Curis, février 2000
     18 décembre 2016 : renommer la fonction d'ouverture de la vue en 3D des molécules
*/

#ifndef INTERFACE_H_
# define INTERFACE_H_  /* Protection contre l'inclusion multiple */

/* ~-~-~-~-~-~-~-~- Routines spécifiques de LASE ~-~-~-~-~-~-~-~- */

struct absorbeur;		/* Définie dans atomes.h */
struct automatiser;		/* Définie dans automatise.h */
struct bloc_figure;		/* Définie dans tracer.h */
struct decomposition;		/* Définie dans modeles.h */
struct err_coord_3d;		/* Définie dans Utilitaires/utils_3d.h */
struct experience;		/* Définie dans experience.h */
struct fragment;		/* Définie dans atomes.h */
struct graphe;			/* Définie dans graphes.h */
struct image;			/* Définie dans images.h */
struct infos_axe;		/* Définie dans tracer.h */
struct infos_dessin2D;		/* Définie dans tracer.h */
struct infos_import_xas;	/* Définie dans import_xas.h */
struct modele_ajuste;		/* Définie dans modeles.h */
struct modele_generique;	/* Définie dans modeles.h */
struct molecule;		/* Définie dans atomes.h */
struct serie;			/* Définie dans serie.h */
struct serie_images;		/* Définie dans images.h */
struct systeme_cristallin;	/* Définie dans cristallo.h */
struct test_stats;		/* Définie dans statistiques.h */
struct vect_3d;		        /* Définie dans Utilitaires/utils_3d.h */

/* -------- Conversions de chaînes de caractères -------- */

#define DIESE_INT    1
#define DIESE_LONG   2
#define DIESE_FLOAT  3
#define DIESE_DOUBLE 4

char *conv_chaine(const char *chaine, int type_var, int *nbr_params);

/* -------- Objets Userdef (routines de dessin) ~~~~~~~~~~~~~~~~~~~~~~ */

int cdecl USR_graphe(USR_FONCTION *parametres); /* Affichage d'un graphe        */
int cdecl USR_image (USR_FONCTION *parametres); /* Affichage d'une image        */

/* -------- Menus ---------------------------------------------------- */

int MEN_ajuste   (unsigned int option); /* Menu des options d'ajustement               */
int MEN_extrait  (unsigned int option); /* Menu des options de traitement expérimental */
int MEN_theorie  (unsigned int option); /* Menu des options de calcul théorique        */
int MEN_structure(unsigned int option); /* Menu des options du modèle structural       */
int MEN_banque   (unsigned int option); /* Menu des options de la base de données      */
int MEN_fluoX    (unsigned int option); /* Menu des options de traitement des cartos X */

/* -------- Fenêtres évoluées spécifiques au programme --------------- */

void RCT_pointeur(Fenetre *fen, int x, int y, int vx_x, int vx_y);

/* -------- Diverses "macros" de gestion de l'interface -------- */

/* ~-~-~-~-~-~-~-~- Gestion du sélecteur universel ~-~-~-~-~-~-~-~- */

typedef struct {
  unsigned long nbr_objets; /* Nombre d'objets dans la liste */
  unsigned long premier;    /* Index du premier affiché */
  unsigned long obj_choisi; /* Index de l'objet sélectionné */
  int (*nouveau)(unsigned long avant);  /* Routine à appeler pour créer un nouvel objet */
  int (*copier)(unsigned long original);/* Routine à appeler pour copier l'objet choisi */
  int (*modifier)(unsigned long objet); /* Routine à appeler pour modifier un objet */
  int (*sauver)(unsigned long objet);   /* Routine à appeler pour sauvegarder un objet */
  int (*charger)(void);                 /* Routine à appeler pour charger des objets */
  int (*effacer)(unsigned long objet);  /* Routine à appeler pour effacer un objet */
  unsigned long (*compter)(void);       /* Routine donnant le nombre d'objets dans la liste */
  const char *(*objet_v_texte)(unsigned int arbre, unsigned long objet);
} InfosSelecteur;

int I_SelecteurUniversel(const char *titre, InfosSelecteur *infos, Fenetre *fenetre_mere);

/* ~-~-~-~-~-~-~-~- Gestion du sélecteur de graphes "choix d'un graphe" ~-~-~-~-~-~-~-~- */

/* -------- Partie commune avec le sélecteur d'images -------- */

void I_GRFIMG_CHX_InitialiserArbre(unsigned int options);
void I_GRFIMG_CHX_PreparerArbre   (void);

/* -------- Informations nécessaires pour la gestion du sélecteur -------- */

typedef struct {
  /* Comment sélectionner les graphes à afficher dans la liste ? */
  const struct graphe *modele;	/* Les valeurs de référence [NULL impossible !] */
  unsigned int masque;		/* Parmi ces valeurs, quelles sont celles à prendre en compte ? */
  unsigned int options;		/* Options de la liste : masque de bits, GRF_LISTE_TYPES_... dans interface.h */
  const struct serie *serie;	/* La série à considérer pour construire la liste [NULL impossible !] */

  /* Paramètres pour l'affichage de la liste */
  unsigned long nombre_graphes;	/* Nombre de graphes dans cette liste */
  unsigned long premier_graphe; /* Le premier graphe affiché dans le sélecteur */
  unsigned long numero_choisi;  /* Numéro du graphe sélectionné [0 à n-1 ; si >= nombre_graphes : aucun sélectionné */
  unsigned long nombre_choisis;	/* Si sélection multiple : nombre de graphes sélectionnés */
  unsigned int  delta_x;	/* Décalage en x pour l'affichage des noms */
  unsigned int  min, max;	/* Sélection multiple : intervalle valide pour OK */
  size_t        lgmax_txt;	/* Longueur du plus long nom de la liste */

  /* Liste des graphes & de leur état (choisi ou non) */
  boolean      *choisi;		/* Table des choisis, pour la sélection multiple [NULL : sélection unique */
  struct graphe **graphes;	/* Les graphes à afficher dans le sélecteur */
} Infos_ChoixGraphe;

typedef struct {
  unsigned long nbr_graphe;  /* Nombre de graphes dans cette liste            */
  unsigned long prem_graphe; /* Le premier graphe affiché dans le sélecteur   */
  unsigned long num_select;  /* Numéro du graphe sélectionné                  */
  unsigned int  delta_x;     /* Décalage en x pour l'affichage des noms       */
  unsigned int  min, max;    /* Sélection multiple : intervalle valide pour OK*/
  size_t        lgmax_txt;   /* Longueur du plus long nom de la liste         */
  boolean      *choisi;      /* Table des choisis, pour la sélection multiple */
  struct graphe **graphes;     /* Les graphes à afficher dans le sélecteur      */
} GRF_liste;

/* Préparation de la liste : graphes d'un même type */

#define GRF_LISTE_TYPES_STRICT         1 /* Bit mis : on fait une recherche stricte */
#define GRF_LISTE_TYPES_MULTISELECTION 2 /* Bit mis : on prépare pour une sélection multiple */
#define GRF_LISTE_TYPES_MULTISERIE     4 /* Bit mis : on recherche dans toutes les séries */
#define GRF_LISTE_TYPES_MESSAGES       8 /* Bit mis : on affiche des messages en cas d'erreur */

GRF_liste *GRF_par_type(int type, unsigned int options, struct serie *serie);
inline int GRF_LibererListe( GRF_liste *liste );

boolean GRF_affiche(GRF_liste *liste, unsigned int arbre,
                    unsigned int prem_nom, unsigned int der_nom, unsigned int max_lg,
                    unsigned int avant, unsigned int apres, unsigned int fond, unsigned int bouton);

boolean GRF_selecteur(GRF_liste *liste, unsigned int arbre,
                      unsigned int prem_nom, unsigned int der_nom, unsigned int max_lg,
                      unsigned int avant, unsigned int apres, unsigned int fond, unsigned int bouton,
                      unsigned int num_objet, int x, int y);

void    MEN_adapte  (struct graphe *graphe); /* Adapte le menu en fonction du type du graphe courant */

/* ~-~-~-~-~-~-~-~- Préparation des arbres en fenêtre ~-~-~-~-~-~-~-~- */

int  InitialiserInterface(void);

void ARB_PrepareListeAtomes   (void);
void ARB_PrepareListeMolecules(void);

/* ~-~-~-~-~-~-~-~- Préparation des arbres à usage multiple ~-~-~-~-~-~-~-~- */

#define LGMAX_TXT_NOM_OBJET_SELECTEUR 65
void ARB_PrepareSelecteurObjets(boolean multiple, int (*gestion)(unsigned int, int, int), int (*clavier)(unsigned int, int, int *) );

/* ~-~-~-~-~-~-~-~- Gestion "à distance" de certains arbres ~-~-~-~-~-~-~-~- */

int ARB_AppelerInformations( void );
int GER_infos(unsigned int objet, int x_souris, int y_souris); /* utilisée dans les menus */

/* -------- Les listes, dans la boîte à onglets -------- */

#define BI_ONGLET_GRAPHES 0
#define BI_ONGLET_IMAGES  1
#define BI_ONGLET_MODELES 2
#define BI_ONGLET_MOLECULES 3

void LGRF_PreparerArbre(void);

void LIMG_PreparerArbre        (void);
void LIMG_EnregistrerParametres(void);
void LIMG_RafraichirListe      (boolean active_a_change);

#define LM_FAMILLE_GENERIQUE_R 0 /* Générique, y=f(x) */
#define LM_FAMILLE_MOLECULE    1 /* De la molécule courante [hors absorbeur courant] */
#define LM_FAMILLE_ABSORBEUR   2 /* De l'absorbeur courant de la molécule courante */
#define LM_FAMILLE_DECOMPOSER  3 /* De décomposition d'un graphe sur une base */
#define LM_NOMBRE_FAMILLES 4

void LMOD_PreparerArbre(void);
void LMOD_MettreAJour  (unsigned int numero_famille);
void LMOD_EnregistrerModele(struct modele_ajuste *modele_demande);
void LMOD_ChangerFamille   (unsigned int nv_famille);

void LMOL_PreparerArbre(void);

#ifndef VARIABLES_GENERALES_
extern Fenetre *fenetre_onglets;
#endif

/* ~-~-~-~-~-~-~-~- Macros de gestion de certains objets "fréquents" ~-~-~-~-~-~-~-~- */

int OBJ_ChoisirAffichage  (unsigned int arbre, unsigned int objet, int genre, int affichage, int mode);	/* Dans graphes/infos.c */
int OBJ_ChoisirGenreGraphe(unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int genre, int mode);	/* Dans graphes/infos.c */
int OBJ_ChoisirGenreImage (unsigned int arbre, unsigned int objet, int genre, int mode);
int OBJ_ChoisirGroupeEsp  (unsigned int arbre, unsigned int objet, unsigned long lgmax_texte,
			   int actuel, int mode, struct systeme_cristallin *systeme);
int OBJ_ChoisirModeleMu0  (unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int modele, int mode);
int OBJ_ChoisirSeuil      (unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int seuil, int mode);
int OBJ_ChoisirSystemeCr  (unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int actuel, int mode,
			   struct systeme_cristallin *systemes, unsigned long nbr_systemes,
			   unsigned int obj_a, unsigned int obj_b, unsigned int obj_c,
			   unsigned int obj_alpha, unsigned int obj_beta, unsigned int obj_gamma);
int OBJ_ChoisirTypeGraphe (unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int type, int mode, boolean avec_internes);
int OBJ_ChoisirTypeImage  (unsigned int arbre, unsigned int objet, int type, int mode, boolean avec_internes);

/* ~-~-~-~-~-~-~-~-~ Lecture d'informations revenant souvent ~-~-~-~-~-~-~-~-~ */

int OBJ_NumeroAtomique(unsigned int arbre, unsigned int objet);

double OBJ_LireAngle  (unsigned int arbre, unsigned int objet, double *erreur);
int    OBJ_MettreAngle(unsigned int arbre, unsigned int objet, double angle, double erreur);

int OBJ_LireCoordonnees  (unsigned int arbre, unsigned int obj_x, unsigned int obj_y, unsigned int obj_z,
			  struct vect_3d *point, struct err_coord_3d *erreurs);
int OBJ_MettreCoordonnees(unsigned int arbre, unsigned int obj_x, unsigned int obj_y, unsigned int obj_z,
			  struct vect_3d *point, struct err_coord_3d *erreurs);

int OBJ_LireIntervalle(unsigned int arbre, unsigned int obj_min, unsigned int obj_max, unsigned int obj_pas,
		       unsigned int obj_nbr_points, double *min, double *max, double *pas, unsigned long *nbr_points);
int OBJ_MettreIntervalle(unsigned int arbre, unsigned int obj_min, unsigned int obj_max, unsigned int obj_pas,
			 unsigned int obj_nbr_points, double min, double max, double pas, unsigned long nbr_points);

/* ~-~-~-~-~-~-~-~- Sous-menus contextuels, en fonction du type de graphe ~-~-~-~-~-~-~-~- */

int OPT_GraphesSerie  (struct graphe *graphe);
int OPT_CreationGraphe(void);

/* ~-~-~-~-~-~-~-~- Paramétrages des tracés ~-~-~-~-~-~-~-~- */

#define BORNES_XY     0
#define BORNES_X      1
#define BORNES_Y      2
#define BORNES_XpourY 3
#define BORNES_YpourX 4
#define NOMBRE_MODES_AUTOMATIQUES 5

int I_ParametrerBornes  (struct infos_axe *axe_X, struct infos_axe *axe_Y, struct infos_axe *axe_Z,
			 int (*automatique)(unsigned int mode, double bornes[ 6 ], void *infos), void *infos, Fenetre *mere);
int I_ParametrerLegendes(struct infos_axe *axe, int orientation, boolean complet, Fenetre *mere);

const char *TXT_LegendeParDefaut(int type_grandeur);

int  I_BLOC_ChangerTaille_Souris(Fenetre *fenetre, struct infos_dessin2D *region2D,
				 int xBG_zone, int yBG_zone, unsigned int lg_zone, unsigned int ht_zone,
				 unsigned int position);
void I_BLOC_ChangerAxes_Souris  (Fenetre *fenetre, struct bloc_figure *bloc, unsigned int position);
int  I_BLOC_Deplacer_Souris     (Fenetre *fenetre, struct infos_dessin2D *region2D,
				 int xBG_zone, int yBG_zone, unsigned int lg_zone, unsigned int ht_zone, int delta_x, int delta_y);
int  I_BLOC_ZoomerSouris        (Fenetre *fenetre, struct infos_dessin2D *region2D, unsigned int axes_secondaires, int x_clic, int y_clic);

/* ~-~-~-~-~-~-~-~- Interface des fonctions d'import, export + charger, sauver ~-~-~-~-~-~-~-~- */

/* ------ Formats internes ------ */

int I_SERIE_GRF_Sauver(struct serie *serie);
int I_SERIE_IMG_Sauver(struct serie_images *serie_a_sauver);

int I_EXP_Sauver(struct experience *experience); /* Sauver l'expérience indiquée */
int I_GRF_Sauver(struct graphe *graphe); /* Sauver le graphe indiqué */
int I_IMG_Sauver(struct image *image); /* Sauver l'image indiquée */
int I_MOL_Sauver(struct molecule *molecule); /* Sauver la molécule indiquée */

/* ------ Import ------ */

int I_IMPORT_DonneesXAS  (void); /* Interface utilisateur... */
int I_ES_ImportXAS_Bessy2(struct infos_import_xas *infos);

/* ------ Export (données) ------ */

int I_EXPORT_GRF_ASCII(struct graphe *graphe);
int I_EXPORT_IMG_ASCII(struct image *image);

/* ------ Export (figures) ------ */

int I_EXPORT_PS_GrapheUnique(struct graphe *graphe);
int I_EXPORT_PS_ImageUnique (struct image *image);

/* ~-~-~-~-~-~-~-~- Interface graphique des fonctions internes [graphes] ~-~-~-~-~-~-~-~- */

/* -------- Fonctions génériques -------- */

#define I_GRF_RAPPELMODULE_ERREUR -1 /* Une erreur est survenue */
#define I_GRF_RAPPELMODULE_ANNULER 0 /* On annule le rappel */
#define I_GRF_RAPPELMODULE_ANCIEN  1 /* On rappelle, mais avec l'ancien graphe */
#define I_GRF_RAPPELMODULE_NOUVEAU 2 /* On rappelle, mais avec le nouveau graphe => la fenêtre a été fermée */

int I_GRF_RappelerModule(unsigned int numero_arbre, struct graphe *graphe, const char *nom_module,
			 boolean (*fonction_validation)(struct graphe *graphe, BI_Boite_Alerte **message), int (*fonction_fermeture)(Fenetre *fenetre));

/* -------- Fonctions sur les séries de graphes -------- */

struct serie *I_SERIE_GRF_Choisir(int arbre, unsigned int objet, unsigned long lgmax_texte);
struct serie *I_SERIE_GRF_Creer  (struct serie *avant, const char *nom_propose);

void I_SERIE_GRF_Aller       (struct serie *serie_ou_aller);
void I_SERIE_GRF_EffacerSerie(struct serie *serie_a_effacer, boolean demander);

/* -------- Fonctions sur les graphes -------- */

struct serie *I_GRF_InsererDansSerie(struct serie *serie_cible, struct graphe *graphe_avant, struct graphe *graphe_a_inserer);

int I_GRF_Copier  (struct graphe *graphe, boolean multiple); /* Copier un graphe, éventuellement dans une nouvelle série */
int I_GRF_Deplacer(struct graphe *graphe, boolean multiple); /* Déplacer un graphe dans une autre série */
int I_GRF_Isoler  (struct graphe *graphe, boolean multiple); /* Isoler un graphe, en créant une nouvelle série */
int I_GRF_Trier   (struct graphe *graphe); /* Trier un graphe : 0 -> redessiner */

int I_GRF_Effacer(struct graphe *graphe, boolean demander, boolean demander_si_non_sauve );

#define I_FILTREGRAPHE_GENRE   1 /* Bit mis : on filtre sur le genre */
#define I_FILTREGRAPHE_TYPE    2 /* Bit mis : on filtre sur le type */
#define I_FILTREGRAPHE_UNITE_X 4 /* Bit mis : on filtre sur l'unité de l'axe des X */
#define I_FILTREGRAPHE_UNITE_Y 8 /* Bit mis : on filtre sur l'unité de l'axe des Y */

struct graphe *I_GRF_ChoisirUnique(struct graphe *modele, unsigned int masque, boolean strict, boolean multi_serie, struct serie *serie_a_chercher, const char *titre_choix, Fenetre *fenetre_mere);

void I_GRFIMG_CHX_PreparerArbreCommun(void);

int I_GRF_ConvertirUnites(struct graphe *graphe, unsigned int axe, const char *nouvelle_unite);

int I_GRF_AfficherLimites(struct graphe *graphe );

struct graphe *I_GRF_Generer       (struct graphe *avant);
struct graphe *I_GRF_CreerAleatoire(struct graphe *avant);
struct graphe *I_GRF_CreerCouples  (struct graphe *avant);
struct graphe *I_GRF_CreerSerie    (struct graphe *avant);

struct graphe *I_GRF_TronquerGraphe(struct graphe *graphe);

struct graphe *I_GRF_SemiLog       (struct graphe *avant);
struct graphe *I_GRF_Transformer   (struct graphe *graphe);

struct graphe *I_GRF_Derivee  (struct graphe *graphe);
struct graphe *I_GRF_Primitive(struct graphe *graphe);

struct graphe *I_GRF_Bruiter      (struct graphe *source);
struct graphe *I_GRF_Interpolation(struct graphe *source);
struct graphe *I_GRF_Normaliser   (struct graphe *source);

struct graphe *I_GRF_AjusterDroite(struct graphe *source);
int            I_GRF_Decomposer   (struct graphe *source);
struct graphe *I_GRF_ModeliserGenerique(struct graphe *experience);

struct graphe *I_GRF_Moyenner(void);

struct graphe *I_GRF_ComparerGraphes(struct graphe *graphe_1);

/* -------- Fonctions statistiques -------- */

/* Les sous-menus génériques [dans *.txt inclus par stats_gene.c] */
#ifndef I_STAT_DEFINITION_DES_MENUS_
extern const char *I_STAT_options_sens_test[ 3 ];
#endif

struct graphe *I_STAT_FonctionRepartition(struct graphe *valeurs);
struct graphe *I_STAT_DroiteDeHenry      (struct graphe *valeurs);

int I_ANVA_1facteur(struct graphe *reference);

int I_STAT_CMP_OptionsTemoin(unsigned int *methode, unsigned int *approximation, int *sens, double *alpha_global);

#define STATS_ACCEPTER_H0 1
#define STATS_ACCEPTER_H1 2

int I_STAT_TEST_AfficherResultat(struct test_stats *test);

/* -------- Fonctions d'analyse XAS -------- */

#define I_XAS_CHAINE_AUCUNE 0	/* Opération isolée */
#define I_XAS_CHAINE_XANES  1	/* Chaîne d'analyse complète du XANES    : µm, dérivée 1re, dérivée 2nde */
#define I_XAS_CHAINE_XAS    2	/* Chaîne d'analyse complète de l'EXAFS  : µm, µ0 I, II, III, normalisation, TF */
#define I_XAS_CHAINE_EXAFS  3	/* Chaîne d'analyse partielle de l'EXAFS : µ0 I, II, III, normalisation, TF */

struct graphe *I_XAN_OterBlanc         (struct graphe *brut);
struct graphe *I_XAN_OterFondContinu   (struct graphe *brut, unsigned int type_chaine);
int            I_XAN_ControleEffacement(Fenetre *fenetre, struct graphe *graphe_efface);

struct graphe *I_XAS_OterMu0_Etape1 (struct graphe *brut, unsigned int type_chaine, struct graphe **mu0_estime);
struct graphe *I_XAS_NormaliserEXAFS(struct graphe *source);\

/* -------- Fonctions de calculs theoriques XAS -------- */

struct graphe *I_TH_XAS_ConstruireMu0(void);

/* -------- Fonctions d'analyse pharmacocinétique -------- */

int I_PHARMACO_ParametresBasiques(struct graphe *profil);

/* -------- Traitement par lot ------------- */

#define I_AUTO_AJOUT_POSSIBLE          (1) /* Bit mis : on peut ajouter/enlever des objets */
#define I_AUTO_MULTI_SERIE_POSSIBLE    (1 << 1)	/* Bit mis : on peut choisir dans une autre série */
#define I_AUTO_CHOIX_FONCTION_POSSIBLE (1 << 2)	/* Bit mis : on peut choisir la fonction à appliquer */
#define I_AUTO_REFERENCE_POSSIBLE      (1 << 3)	/* Bit mis : on peut choisir la référence à reproduire */

int I_AUTOMATISE_SurListeConnue(struct automatiser *infos, unsigned int options);

/* ~-~-~-~-~-~-~-~- Interface graphique des fonctions internes [images] ~-~-~-~-~-~-~-~- */

/* -------- Fonctions sur les séries d'images -------- */

struct serie_images *I_SERIE_IMG_Choisir(int arbre, unsigned int objet, unsigned long lgmax_texte);
struct serie_images *I_SERIE_IMG_Creer  (struct serie_images *avant, const char *nom_propose);

/* -------- Création d'images -------- */

struct image *I_IMG_Generer       (struct image *avant);
struct image *I_IMG_CreerAleatoire(struct image *avant);

struct serie_images *I_IMG_InsererDansSerie(struct serie_images *serie_cible, struct image *image_avant, struct image *image_a_inserer);

/* -------- Manipulations sur l'agencement des images -------- */

int I_IMG_Copier  (struct image *image, boolean multiple); /* Copier une image, éventuellement dans une nouvelle série */
int I_IMG_Deplacer(struct image *image, boolean multiple); /* Déplacer une image dans une autre série */
int I_IMG_Isoler  (struct image *image, boolean multiple); /* Isoler une image, en créant une nouvelle série */

int I_IMG_Effacer (struct image *image, boolean demander);

int I_IMG_ChangerGenre(struct image *image);
int I_IMG_ChangerType (struct image *image);

int I_IMG_ConvertirUnites(struct image *image, unsigned int axe, const char *nouvelle_unite);

#define I_FILTREIMAGE_GENRE    1 /* Bit mis : on tient compte du genre du modèle */
#define I_FILTREIMAGE_TYPE     2 /* Bit mis : on tient compte du type du modèle */
#define I_FILTREIMAGE_UNITE_X  4 /* Bit mis : on tient compte des unités en X */
#define I_FILTREIMAGE_UNITE_Y  8 /* Bit mis : on tient compte des unités en Y */
#define I_FILTREIMAGE_UNITE_Z 16 /* Bit mis : on tient compte des unités en Z */

struct image  *I_IMG_ChoisirUnique   (struct image *modele, unsigned int masque, boolean strict, boolean multi_serie, struct serie_images *serie_a_chercher);
struct image **I_IMG_ChoisirMultiples(int type, boolean strict, boolean multi_serie, unsigned long *nombre_choisies);

struct image  *I_IMG_SortirErreurs   (struct image *image, unsigned int axe);

/* -------- Transformations simples -------- */

/* ~-~-~-~-~-~-~-~- Interface graphique des fonctions internes [modèles] ~-~-~-~-~-~-~-~- */

/* -------- Communes à tous les modèles -------- */

int  I_MOD_AfficherResidusAbsolus (struct modele_ajuste *modele);
int  I_MOD_AfficherResidusPonderes(struct modele_ajuste *modele);
int  I_MOD_AfficherResidusRelatifs(struct modele_ajuste *modele);

int  I_MOD_LancerAjustement(struct modele_ajuste *modele);

int  I_MOD_VALIDITE_ControlerValidite(struct modele_ajuste *modele, int methode);

/* -------- Spécifiques des modèles génériques -------- */

void I_ModeleGenerique        (struct modele_generique **modele);
int  I_ModeleGenerique_Nouveau(unsigned long numero_avant);
struct modele_ajuste *I_ModeleGenerique_Premier(void);

/* -------- Spécifiques des modèles de decomposition -------- */

void I_ModeleDecomposition        (struct decomposition **modele);
struct modele_ajuste *I_ModeleDecomposition_ChoisirCourant(struct graphe *a_decomposer);
struct modele_ajuste *I_ModeleDecomposition_Nouveau       (unsigned long numero_avant, struct graphe *graphe_a_decomposer);
struct modele_ajuste *I_ModeleDecomposition_Premier       (void);

/* -------- Spécifiques des modèles d'absorbeur/EXAFS [FEFF] -------- */

int I_ModeleAbsorbeur_Nouveau    (struct absorbeur *absorbeur, unsigned long numero_avant);
int I_ModeleAbsorbeur_NouveauFEFF(struct absorbeur *absorbeur, unsigned long numero_modele_avant);

struct modele_ajuste *I_FEFF_DemanderPathsDat(struct absorbeur *absorbeur);
struct modele_ajuste *I_FEFF_LirePathsDat    (struct absorbeur *absorbeur, const char *chemin_de_travail, const char *nom_fichier);

/* ~-~-~-~-~-~-~-~- Interface graphique des fonctions internes [molécules] ~-~-~-~-~-~-~-~- */

/* -------- Gestion des molécules -------- */

void I_MOL_RefaireListeFragments(void);

int I_MOL_EffacerAbsorbeur(struct molecule *molecule, unsigned int numero_absorbeur);
int I_MOL_EffacerFragment (struct molecule *molecule, struct fragment *fragment);
int I_MOL_EffacerMolecule (struct molecule *molecule);

struct molecule *I_MOL_CreerMolecule(struct molecule *avant);

/* -------- Gestion des absorbeurs ---- */

int I_MOL_EditerAbsorbeur(struct absorbeur *absorbeur);

/* -------- Gestion des atomes -------- */

struct atome *I_ATM_CreerAtome_Absolu       (struct molecule *molecule, boolean multiple, Fenetre *mere);
struct atome *I_ATM_CreerAtome_Fractionnaire(struct molecule *molecule, boolean multiple, unsigned long nombre_mailles, struct fragment **mailles, Fenetre *mere);

/* -------- Gestion des chemins ------- */

void I_CHEM_CombinerChemins(struct modele_ajuste *modele);

/* -------- Gestion de la vue en 3D -------- */

boolean I_V3D_ChangerVue(struct molecule *molecule);

/* ~-~-~-~-~-~-~-~- Interface graphique des fonctions internes [simulations d'expériences] ~-~-~-~-~-~-~-~- */

int I_EXP_LancerSimulation(struct experience *experience);

/* ~-~-~-~-~-~-~-~- Gestion des fenêtres principales ~-~-~-~-~-~-~-~- */

int  I_OuvrirPrincipale_GRF (void);
int  I_OuvrirPrincipale_IMG (void);
void I_MettreAJourPrincipale(boolean remise_a_lechelle);
void I_PrincipaleGRF_RefaireAscenseurs(boolean horizontal, boolean vertical);

#endif /* INTERFACE_H_ */
