/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1999
   Cration du fichier   : 14 dcembre 1999

   Module de calculs thoriques divers
   Les dfinitions des fonctions et apparents
*/

#ifndef THEORIE_H_

#define THEORIE_H_ /* Protection contre l'inclusion multiple */

struct graphe; /* Dfinie dans graphes.h */

/* ~~~~~~~~ lecture de tables de valeurs numriques ~~~~~~~~ */

float *TH_Creer_Table  ( unsigned int Z_min, unsigned int Z_max );
int    TH_LireLigne    ( FILE *fichier, unsigned int nbr_colonnes, double *valeurs, double *erreurs );
FILE  *TH_OuvrirFichier( const char *nom, unsigned long *nbr_lignes, unsigned int *nbr_colonnes, double **valeurs );
float  TH_Table_Lire   ( float *table, int Z, unsigned int Z_min, unsigned int Z_max );
int    TH_Table_Remplir( float *table, int Z, unsigned int Z_min, unsigned int Z_max, double valeur );

/* ~~~~~~~~ Calcul des composantes autres que l'EXAFS ~~~~~ */
/* -------- Fonctions de base ------ */

struct graphe *CTH_mu0(void); /* Calcul d'un mu0 thorique */

/* ~~~~~~~~ Calcul des composantes du signal EXAFS ~~~~~~~~ */

/* ---- Calcul des dphasages ---- */

struct graphe *CTH_phase(void);   /* Calcul d'un dphasage pour une couple d'atomes */
int GER_phase(unsigned int num_obj, int x_souris, int y_souris); /* Arbre associ */

/* ---- Libre parcours moyen ---- */

#define MODELE_LPM_GAMMA     0 /* l(k) = G/k */
#define MODELE_LPM_GAMMA_ETA 1 /* l(k) = */
#define MODELE_LPM_GRAPHE    2 /* l(k) inclus dans un graphe */
#define MODELE_LPM_INCLUS    3 /* l(k) inclus aux amplitudes <=> ignorer */

struct graphe *TH_calc_exafs(void);

/* ~~~~~~~~ Calcul des effets systematiques ~~~~~~~~ */

struct graphe *TH_evolution_temporelle(void);
struct graphe *TH_Transformation(struct graphe *ideal, struct graphe *cible, struct graphe *profil_concentration,
				 double t_mesure, double t_attente, unsigned long nbr_passes);
struct graphe *TH_degrade_lineaire(struct graphe *seuil, double a, double b, double t0, double pas);

/* ~~~~~~~~ Bruiter un modle thorique ~~~~~~~~~~~~ */

struct graphe *TH_AjouterBruit(struct graphe *graphe, int distribution, boolean en_place);
struct graphe *TH_bruiter_graphe(struct graphe *graphe); /* Interface utilisateur */

#endif /* THEORIE_H_ */
