/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999, Emmanuel CURIS

   Module de trac des diffrents graphes
   Dclarations des fonctions
*/

#ifndef LASE_TRACES_H_

#define LASE_TRACES_H_ /* Protection contre l'inclusion multiple */

#include "utils_3d.h" /* Pour Vecteur3D */

/* ~-~-~-~-~-~-~-~-~ lments de base du dessin ~-~-~-~-~-~-~-~-~ */

/* -------- Figures gomtriques, 2D -------- */

/* -------- Figures gomtriques, 3D -------- */

void TRC_Boite3D(Vecteur3D *coin_BasGaucheAvant, Vecteur3D *coin_HautDroitFond, int couleur_faces,
		 int couleur_bord, double epaisseur_bord);
void TRC_Couleur_OpenGL(int couleur);

/* ~-~-~-~-~-~-~-~-~ Figures ~-~-~-~-~-~-~-~-~ */

/* -------- Objets de trac (pour la cration de figures) ---------- */

#define TRC_TAILLE_POIGNEE (5)	/* Taille de la zone autour d'un lment, pour slection [en pixels] */

/* --------------- Pour la mise en page avant impression -> divers rectangles */

struct absorbeur;  /* Dfinie dans atomes.h  */
struct atome;      /* Dfinie dans atomes.h  */
struct chemin;     /* Dfinie dans chemins.h */
struct graphe;     /* Dfinie dans graphes.h */
struct molecule;   /* Dfinie dans atomes.h  */
struct image ;     /* Definie dans images.h  */

/* ~-~-~-~-~-~-~-~ Axes gradus ~-~-~-~-~-~-~-~ */

typedef struct infos_axe {
  char *legende;		/* La lgende de l'axe [NULL = automatique] */
  char *unite;			/* L'unit [ ADAPTER AVEC DES VRAIES UNITS !] */
  int proprietes;		/* Masque de bits, cf. ci-dessous */

  int type_graduation;		/* 0 = aucune, 1 = dcimal,... cf. GRADUATION_xxx ci-dessous */
  unsigned int finesse;		/* Jusqu' quel point graduer (1 = 1/10, 2 = 1/100,... ; 0 = auto) */
  double min, max;		/* Valeurs extrmes */
  double pas_graduation;	/* Pas de la graduation (<=0.0 : automatique) [pour les valeurs numriques] */
  char   multiple;		/* Multiple de l'unit  utiliser pour graduer
				   [ex : unite = eV, multiple = k => divise par 10^3 pour afficher des keV] */
  double debord_gauche;		/* Dbord  gauche de l'axe pour une graduation maximale [en pixels ou 1/100 mm] */
  double debord_droite;		/* Dbord  droite de l'axe pour une graduation maximale [en pixels ou 1/100 mm] */

  /* Les couleurs des diffrents lments */
  int couleur_legende;		/* Texte de la lgende */
  int couleur_graduations;	/* Texte & tics des graduations */
  int couleur_axe;		/* Axe lui-mme, s'il est visible */
  int couleur_grille;		/* Couleur de la grille si elle est visible */
  int style_grille;		/* Style de ligne pour la grille (si visible) */
} InfosAxe;

#define AXE_AVEC_BORNES       1 /* Bit mis : afficher les valeurs extrmes */
#define AXE_AVEC_GRADUATIONS  2 /* Bit mis : afficher les valeurs numriques des graduations */
#define AXE_AVEC_LEGENDE      4 /* Bit mis : afficher le texte de la lgende */
#define AXE_AVEC_GRILLE       8	/* Bit mis : afficher une grille */
#define AXE_VERTICAL         16 /* Bit mis : l'axe est vertical */
#define AXE_OBLIQUE          32 /* Bit mis : l'axe est diagonal */
#define AXE_VISIBLE          64 /* Bit mis : on trace l'axe  sa position relle, s'il est visible */
#define AXE_SECONDAIRE      128	/* Bit mis : axe secondaire -> textes  droite ou en haut ! */
#define AXE_ECRAN           256 /* Bit mis : axe pour trac  l'cran -> units en pixels */

#define GRADUATION_AUCUNE        0
#define GRADUATION_DECIMALE      1
#define GRADUATION_LOGARITHMIQUE 2
#define GRADUATION_PARABOLIQUE   3
#define GRADUATION_GAUSSIENNE    4

#ifndef AXE_X  /*  conserver synchrone avec graphes.h */
# define AXE_X  1
# define AXE_Y  2
# define AXE_Z  4
#endif

double AXE_Echelle      (InfosAxe *axe, int debut_ecran, int fin_ecran);
void   AXE_Initialiser  (InfosAxe *axe, boolean vertical);
void   AXE_Liberer      (InfosAxe *axe, boolean av_structure);
char  *AXE_LireLegende  (InfosAxe *axe);
int    AXE_MettreLegende(InfosAxe *axe, const char *texte_legende);
int    AXE_MettreUnite  (InfosAxe *axe, const char *nv_unite);
int    AXE_ValiderBornes(InfosAxe *axe);

double     AXE_GraduationOptimale(double min, double max, double etendue, double haut_texte, boolean vertical);
inline int AXE_GraduationMajeure (double min, double max);
inline int AXE_PrecisionOptimale (double min, double max, double etendue, double espace_minimal);

struct infos_graduation {
  double x_debut, x_fin;	/* Limites de trac en X */
  double y_debut, y_fin;	/* Limites de trac en Y */
  double echelle;               /* chelle  utiliser */
  int niveau;			/* Niveau maximal de graduation */
  int p10_max;			/* Puissance de 10 des graduations de plus haut niveau*/
};

int AXE_PreparerGraduations(double x_d, double y_d, double x_f, double y_f, double delta, InfosAxe *axe,
			    struct infos_graduation *infos);

/* ~-~-~-~-~-~-~-~ Rgion 2D : rectangle cran correspondant  une rgion du plan ~-~-~-~-~-~-~-~ */

typedef struct infos_dessin2D {
  unsigned int x_BG, y_BG;	/* Position du coin infrieur gauche  l'cran [pixels] */
  unsigned int largeur, hauteur; /* Dimensions du rectangle  l'cran [pixels] */

  double x_BG_pc, y_BG_pc, larg_pc, haut_pc; /* Idem, mais en % d'une rgion globale [pour adapter taille en temps rel] */

  unsigned int proprietes;	/* Masque de bits : cf. ci-dessous */

  InfosAxe axe_X;		/* Information sur l'axe X principal */
  InfosAxe axe_Y;		/* Information sur l'axe Y principal */

  InfosAxe *X_secondaire;	/* Axe X secondaire, s'il existe */
  InfosAxe *Y_secondaire;	/* Axe Y secondaire, s'il existe */

  int couleur_bord;		/* Couleur du bord */
  int couleur_fond;		/* Couleur du fond (<0 : transparent) */
} InfosRegion2D;

#define CADRE2D_AVEC_CADRE      1 /* Bit mis : tracer le cadre autour de la rgion considre */
#define CADRE2D_GRADUER_DROITE  2 /* Bit mis : graduer aussi le bord droit du cadre */
#define CADRE2D_GRADUER_HAUT    4 /* Bit mis : graduer aussi le bord haut du cadre  */
#define CADRE2D_ECRAN           8 /* Bit mis : conu pour affichage cran -> exprim en pixels et y croissant en allant vers le bas */

/* -------- Conversion coordonnes cran de la rgion, relatif (%) <-> absolu (pixels) -------- */

int TRC_RelatifVAbsolu(InfosRegion2D *region, int x_BG, int y_BG, unsigned int largeur, unsigned int hauteur);
int TRC_AbsoluVRelatif(InfosRegion2D *region, int x_BG, int y_BG, unsigned int largeur, unsigned int hauteur);

void TRC_PointIntersection(double x_min, double y_min, double x_max, double y_max,
			   double x_dedans, double y_dedans, double x_dehors, double y_dehors,
			   double *x_bord, double *y_bord);
inline boolean TRC_EstDansZone2D(double x_min, double y_min, double x_max, double y_max, double x, double y);

/* -------- Conversion coordonnes cran (pixels) <-> coordonnes relles -------- */

int TRC_Z2D_Initialiser(InfosRegion2D *region, Fenetre *fenetre, struct graphe *graphe);

int TRC_Z2D_Echelles(InfosRegion2D *region, unsigned int axes_secondaires, double *echelle_X, double *echelle_Y);
int TRC_Z2D_EcranVXY(InfosRegion2D *region, unsigned int axes_secondaires, int x_ecran, int y_ecran, double *x, double *y);
int TRC_Z2D_XYVEcran(InfosRegion2D *region, unsigned int axes_secondaires, double x, double y, int *x_ecran, int *y_ecran);

#define TRC_EST_HORS_REGION (0xFFFF)
#define TRC_EST_AU_CENTRE   ( 0)
#define TRC_EST_BORD_GAUCHE ( 1)
#define TRC_EST_BORD_DROIT  ( 2)
#define TRC_EST_BORD_HAUT   ( 4)
#define TRC_EST_BORD_BAS    ( 8)
#define TRC_EST_MILIEU_HOR  (16) /* On est au milieu sur le ct horizontal */
#define TRC_EST_MILIEU_VER  (32) /* On est au milieu sur le ct vertical */
#define TRC_EST_COIN_HG     (TRC_EST_BORD_HAUT | TRC_EST_BORD_GAUCHE)
#define TRC_EST_COIN_BG     (TRC_EST_BORD_BAS  | TRC_EST_BORD_GAUCHE)
#define TRC_EST_COIN_HD     (TRC_EST_BORD_HAUT | TRC_EST_BORD_DROIT)
#define TRC_EST_COIN_BD     (TRC_EST_BORD_BAS  | TRC_EST_BORD_DROIT)
#define TRC_EST_MILIEU_BORD_HAUT   (TRC_EST_BORD_HAUT   | TRC_EST_MILIEU_HOR)
#define TRC_EST_MILIEU_BORD_BAS    (TRC_EST_BORD_BAS    | TRC_EST_MILIEU_HOR)
#define TRC_EST_MILIEU_BORD_GAUCHE (TRC_EST_BORD_GAUCHE | TRC_EST_MILIEU_VER)
#define TRC_EST_MILIEU_BORD_DROIT  (TRC_EST_BORD_DROIT  | TRC_EST_MILIEU_VER)
#define TRC_EST_AU_MILIEU          (TRC_EST_MILIEU_HOR  | TRC_EST_MILIEU_VER)

unsigned int  TRC_PositionDansRegion2D(InfosRegion2D *region, int x, int y, unsigned int marge);

int  TRC_AxeGradue      (Fenetre *fenetre, int x_debut, int y_debut, int x_fin, int y_fin, InfosAxe *axe,
			 int delta, int haut_texte);
void TRC_DessineRegion2D(Fenetre *fenetre, InfosRegion2D *region);
void TRC_Grille         (Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe);
void TRC_PlacerAxes     (Fenetre *fenetre, InfosRegion2D *region);
void TRC_TracerBarre    (Fenetre *fenetre, InfosRegion2D *region, double valeur, int axe, int couleur);

int TRC_LimiterSegment(double x_min, double x_max, double y_min, double y_max, double *x0, double *y0, double *x1, double *y1);

/* boolean TRC_EstDansRegion2D(InfosRegion2D *region, double x, double y) */
#define TRC_EstDansRegion2D(region,x,y) TRC_EstDansZone2D( (region)->axe_X.min, (region)->axe_Y.min, (region)->axe_X.max, (region)->axe_Y.max, (x), (y))

#define PAS_GRADUE     0 /* Types de graduation possible */
#define GRADUE_DECIMAL 1
#define GRADUE_LOG     2

/* ~-~-~-~-~-~-~-~-~ Superposition simple de graphes ~-~-~-~-~-~-~-~-~ */

#define TRC_SUPERPOSE_COCHE 1 /* Bit mis : on tient compte des cases coches */
#define TRC_SUPERPOSE_TYPE  2 /* Bit mis : on tient compte du type actif */
#define TRC_SUPERPOSE_AXES  4 /* Bit mis : on tient compte de la cohrence des axes */
#define TRC_SUPERPOSE_AUTO  8 /* Bit mis : mise  jour automatique active */

struct infos_superpose {
  unsigned int type;		/* Type de superposition : cf. ci-dessus */
  int type_graphe;		/* Type de graphe, si ncessaire */
  struct infos_dessin2D dessin;	/* Comment dessiner cette superposition ? */
};

int  TRC_FermerSuperposition(void); /* Fermeture de la fentre de superposition */
void TRC_OuvrirSuperposition(void); /* Ouverture de la fentre de superposition */
void TRC_MettreAJourSuperposition(void); /* Adapter la fentre de superposition */
boolean TRC_GrapheConvient          (struct infos_superpose *infos, struct graphe *graphe);
boolean TRC_RedessinerSuperposition (struct graphe *graphe, boolean effacement, boolean mise_a_jour);

/* ~-~-~-~-~-~-~-~-~ Cration de figures ~-~-~-~-~-~-~-~-~ */

typedef struct dess_image {
  struct image *image; /* Pointeur sur l'image a dessiner */
  int couleur;         /* Palette de couleur a utiliser */
  int proprietes;      /* Que faut-il dessiner, en plus de l'image ? */
  double x_min, x_max; /* Portion du plan (x, y)  reprsenter */
  double y_min, y_max;
} DES_image;

#define SURFACE_RECTANGLE 0;
#define SURFACE_ELLIPSE   1;

/* -------- Fonctions annexes, servant  plusieurs endroits -------- */

int prepare_barre_horizontale(Fenetre *fenetre, int x_dep, unsigned int larg, int *y_fin);
int prepare_barre_verticale  (Fenetre *fenetre, int y_dep, int *x_fin, unsigned int haut);
int prepare_coin   (Fenetre *fenetre, int x_fixe, int y_fixe, int *x, int *y, int type);
int prepare_deplace(Fenetre *fenetre, int dx, int dy, unsigned int larg, unsigned int haut, int *x, int *y);
int prepare_ligne  (Fenetre *fenetre, int x_fixe, int y_fixe, int *x_mobile, int *y_mobile);
int prepare_zoom   (Fenetre *fenetre, int x_dep, int y_dep, int *x_fin, int *y_fin);

#define NON_TOURNE 0 /* Sens normal : type cran ou imprimante, portrait */
#define TOURNE     1 /* Tourn de 90 degrs : type imprimante, paysage   */

void TRC_gradue(Fenetre *fenetre, int x_deb, int y_deb, int x_fin, int y_fin,
		double min, double max, int type_axe, int precision, int ht_tics, boolean avec_texte);

/* ~-~-~-~-~-~-~-~-~ Objets fondamentaux ~-~-~-~-~-~-~-~-~ */

/* -------- Informations pour tracer une bote  moustache -------- */

struct boite_moustache {
  unsigned int elements;	/* Masque de bits : les lements  tracer --- cf. ci-dessous */
  unsigned int largeur;		/* largeur de la bote */
  double mediane;		/* Position de la mdiane */
  double q1, q3;		/* Position du 1er et du 3e quartile : limites de la bote */
  double d1, d9;		/* Position des deux moustaches (par convention : 1er et 9e dciles) */
  double limite_inf, limite_sup; /* Au del desquelles les valeurs sont supposes aberrantes */
  double moyenne, ecart_type;	/* Position de la moyenne et de l'cart-type */
  /* --- Utilis dans la principale par commodit --- */
  struct graphe *henry;		/* Graphe isol : la droite de Henry */
};

#define BM_AVEC_MEDIANE    1	/* Bit mis : on place la mdiane */
#define BM_AVEC_EXTREMES   2	/* Bit mis : on place les valeurs extrmes */
#define BM_AVEC_ABERRANTES 4	/* Bit mis : on place les valeurs "aberrantes"*/
#define BM_AVEC_MOYENNE    8	/* Bit mis : on place la moyenne */
#define BM_AVEC_SIGMA     16	/* Bit mis : on reprsente l'cart-type */
#define BM_VERTICALE      32	/* Bit mis : on trace verticalement, sinon horizontalement */
#define BM_AVEC_IC        64	/* Bit mis : on indique aussi l'intervalle de confiance exprimental [m-2*s,m+2*s] */

int TRC_BoiteAMoustache(Fenetre *fenetre, InfosRegion2D *region, struct graphe *valeurs,
			struct boite_moustache *infos, int position);
int TRC_GRF_PreparerBoiteAMoustache(struct graphe *valeurs, double f_inferieur, double f_superieur, double f_min, double f_max );

/* -------- Trac d'une image dans un rectangle ----- */

#define IMG_TRACE_NORMAL 0 /* Affichage normal : y vers le bas  */
#define IMG_TRACE_MATHS  1 /* Affichage  maths  : y vers le haut */

void   TRC_image    (Fenetre *fenetre, DES_image *image, int x, int y, unsigned int larg, unsigned int haut, int mode);
double IMG_dim_trace(struct image *image, unsigned int lg_dessin, unsigned int ht_dessin,
		     unsigned int *larg_img, unsigned int *haut_img);

int TRC_AbsoluVersEcran(int x_HG, int y_HG, unsigned int larg, unsigned int haut, double echelle, double x_min, double y_max,
			double x_abs, double y_abs, int *x_ecran, int *y_ecran);
int TRC_EcranVersAbsolu(int x_HG, int y_HG, unsigned int larg, unsigned int haut, double echelle, double x_min, double y_max,
			int x_ecran, int y_ecran, double *x_absolu, double *y_absolu);

double TRC_Echelle2D(double larg_reelle, double haut_reelle, unsigned int lg_zone, unsigned int ht_zone);

int TRC_PortionImageDansRectangle(Fenetre *fenetre, DES_image *des_image, int x, int y, unsigned int larg, unsigned int haut);
int TRC_EchelleImage(Fenetre *fenetre, DES_image *des_image, int x, int y, unsigned int larg, unsigned int haut);

int TRC_ImageDansRectangle (Fenetre *fenetre, struct image *image, int couleur, int x, int y, unsigned int larg, unsigned int haut);
int TRC_ImagePositionCentre(Fenetre *fenetre, struct image *image, int couleur, int x_centre, int y_centre, double echelle);

boolean IMG_ecran_v_image(struct image *image, unsigned int lg_dessin, unsigned int ht_dessin, int mode,
			  int x_ecran, int y_ecran, double *x_image, double *y_image);
boolean IMG_image_v_ecran(struct image *image, unsigned int lg_dessin, unsigned int ht_dessin, int mode,
			  double x_image, double y_image, int *x_ecran, int *y_ecran);

/* -------- Trac des structures 3D des molcules -------- */

/* ---- Masque de bits pour contrler l'affichage de la vue 3D ---- */

#define V3D_VOIR_ATOMES       (1)       /* Mis : on veut voir les atomes      */
#define V3D_VOIR_MODELE       (1 <<  1) /* Mis : on veut voir juste le modle */
#define V3D_VOIR_SELECTION    (1 <<  2) /* Mis : on veut voir juste la slection */
#define V3D_VOIR_LIAISONS     (1 <<  3) /* Mis : on veut voir les liaisons    */
#define V3D_MODELE_SPECIAL    (1 <<  4) /* Mis : on affiche le spcifique du modle (chemins) */
#define V3D_MODELE_RESULTAT   (1 <<  5) /* Mis : on affiche le rsultat de l'ajustement / atome */
#define V3D_MODELE_EQUIV      (1 <<  6) /* Mis : on affiche les chemins quivalents... */
#define V3D_ATOME_REDUIT      (1 <<  7) /* Mis : affichage avec des sphres plus petites */
#define V3D_ATOME_ABSORBEUR   (1 <<  8) /* Mis : affichage spcial pour les absorbeurs */
#define V3D_LIAISON_MULTIPLE  (1 <<  9) /* Mis : tient compte de la multiplicit des liaisons */
#define V3D_LIAISON_SELECTION (1 << 10) /* Mis : on tient compte de la slection pour l'affichage des liaisons */
#define V3D_LIAISON_DEMI      (1 << 11) /* Mis : il suffit qu'un des atomes soit visible pour afficher la liaison */
#define V3D_LIAISON_SPECIALE  (1 << 12) /* Mis : on affiche aussi les liaisons non covalentes */

void TRC_GL_Molecule(struct molecule *molecule, int mode);
void TRC_GL_Chemin  (struct atome *absorbeur, struct chemin *chemin, unsigned int num_equivalent);
void TRC_GL_Liaisons(struct molecule *molecule, int mode);

#ifdef AVEC_OPEN_GL_
# define dessine_molecule TRC_GL_Molecule
# define dessine_chemin   TRC_GL_Chemin
# define dessine_liaisons TRC_GL_Liaisons
#else
# define dessine_molecule(a, b) /* OpenGL absent : sans vue 3D ! */
# define dessine_chemin(a, b, c)/* OpenGL absent : sans vue 3D ! */
# define dessine_liaisons(a, b) /* OpenGL absent : sans vue 3D ! */
#endif

struct atome *ATM_trouver(Fenetre *fenetre, struct molecule *molecule, double x, double y, int mode);

/* -------- Trac d'un graphe proprement dit -------- */

/* Trac simple --- les chelles sont attendues en vrai/pixel ; on donne le coin INFRIEUR GAUCHE [pos de x_min, y_min  l'cran] */

void TRC_Graphe(Fenetre *fenetre, struct graphe *graphe, int couleur, int style_ligne, int style_marque, int mode_dessin,
		int x_BG, int y_BG, double x_min, double x_max, double y_min, double y_max, double echelle_X, double echelle_Y);

void TRC_GrapheDansRectangle(Fenetre *fenetre, struct graphe *graphe, int couleur, int style,
			     int x_HG, int y_HG, unsigned int larg, unsigned int haut);
void TRC_GrapheDansRegion2D (Fenetre *fenetre, InfosRegion2D *region, int axes_secondaires, struct graphe *graphe,
			     int couleur, int style_ligne, int style_marque, int mode_dessin,
			     double *echelle_X, double *echelle_Y);

/* Trac d'un polynome */

void TRC_Polynome(Fenetre *fenetre, InfosRegion2D *region,
		  unsigned int couleur, unsigned int style_ligne, unsigned int style_marque,
		  unsigned int degre, double *coefficients, unsigned long nombre_points, double *X);

#endif /* ndef LASE_TRACES_H_ */
