/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-2016 Emmanuel CURIS
   Création : 25 décembre 2016 par reprise de vue_3d.c

   Affichage 3d d'une molécule à l'aide d'Open-GL
   Contrôle de la vue en 3D
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h"
#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static Molecule *molecule_modifiee = NULL;

#include "int_3d_changer_vue.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  V3D_GER_ChangerVue              (unsigned int objet, int x, int y);
static void V3D_PreparerArbre_ChangerVue    (void);
static void V3D_ChangerVue_MettreCoordonnees(Vecteur3D *vecteur);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Modifier l'angle de vue de la molécule indiquée --------\
 |   Renvoie VRAI si on a changé la vue -> il faut réafficher       |
 |   Travaille en sphérique autour du point visé                    |
 |                                                                  |
 | molecule : la molécule dont on veut modifier la vue              |
 \----------------------------------------------------------------- */

boolean I_V3D_ChangerVue(Molecule *molecule)
{
  Vecteur3D pos_centre;
  double distance, latitude, longitude;
  int retour;

  /* On préparer l'arbre pour que les éditables soient corrects */
  if ( FAUX == arbres[ DEFINIR_VUE_3D ].drapeaux.prepare )
    V3D_PreparerArbre_ChangerVue();

  if ( NULL == molecule ) return FAUX;

  /* Coordonnées du point visé */
  OBJ_MettreCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
			 &(molecule->point_vise), NULL );

  /* Coordonnées de l'œil dans le repère du point visé */
  (void) U3D_VecteurDirecteur( &(molecule->point_vise), &(molecule->pos_oeil), &pos_centre, NULL, NULL, NULL );
  V3D_ChangerVue_MettreCoordonnees( &pos_centre );

  OBJ_active( DEFINIR_VUE_3D, V3D_VISER_ABS, ( molecule->nbr_absorbeurs > 0 ) );

  molecule_modifiee = molecule;

  do {
    retour = ARB_Bloquant( DEFINIR_VUE_3D, titre_arbre_changer_vue3D, fen_vue_3d, 0 );
    if ( V3D_ANNULER == retour ) {
      molecule_modifiee = NULL;
      return FAUX; /* On n'a rien modifié */
    } 

    if ( V3D_CENTRE == retour ) { /* On vise le centre du repère */
      OBJ_MettreCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
			     &pos_centre, NULL );
    } else if ( V3D_BARYCENTRE == retour ) { /* On vise le centre de masse de la molécule */
      OBJ_MettreCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
			     &(molecule->pos_barycentre), NULL );
    }
  } while ( retour != V3D_OK );

  OBJ_LireCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
		       &(molecule->point_vise), NULL );

  distance  = OBJ_LireNombre( DEFINIR_VUE_3D, V3D_DISTANCE , NULL );
  latitude  = OBJ_LireAngle ( DEFINIR_VUE_3D, V3D_LATITUDE , NULL );
  longitude = OBJ_LireAngle ( DEFINIR_VUE_3D, V3D_LONGITUDE, NULL );

  pos_centre.x = distance * sin(latitude) * cos(longitude);
  pos_centre.y = distance * sin(latitude) * sin(longitude);
  pos_centre.z = distance * cos(latitude);

  molecule->pos_oeil.x = pos_centre.x + molecule->point_vise.x;
  molecule->pos_oeil.y = pos_centre.y + molecule->point_vise.y;
  molecule->pos_oeil.z = pos_centre.z + molecule->point_vise.z;
  molecule_modifiee = NULL;

  /* On a changé la façon de regarder la molécule... */
  return VRAI;
}

static void V3D_ChangerVue_MettreCoordonnees(Vecteur3D *position)
{
  double distance, latitude, longitude/*, inclinaison*/;

  if ( NULL == position ) return;

  distance = position->x * position->x + position->y * position->y + position->z * position->z;
  distance = sqrt( distance );
  if ( distance <= 0.0 ) distance = 1.0;

  latitude  = acos ( position->z / distance );
  longitude = atan2( position->y, position->x );

  OBJ_MettreNombre( DEFINIR_VUE_3D, V3D_DISTANCE , distance, 0.0 );
  OBJ_MettreAngle ( DEFINIR_VUE_3D, V3D_LATITUDE , latitude, 0.0 );
  OBJ_MettreAngle ( DEFINIR_VUE_3D, V3D_LONGITUDE, longitude, 0.0 );
}

/* -------- Gestion de l'arbre définissant la vue en 3D -------- */

static int V3D_GER_ChangerVue(unsigned int objet,
			      int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  Atome  *vise = NULL;
  Atome **choisis = NULL;

  if ( V3D_VISER_ABS == objet ) { /* On choisit un absorbeur de la molécule */
    if ( 1 == molecule_modifiee->nbr_absorbeurs ) { /* Un seul absorbeur */
      vise = molecule_modifiee->absorbeurs->absorbeur;
    } else { /* On propose le choix */
      unsigned long i;
      int choix;

      choisis = (Atome **) calloc( molecule_modifiee->nbr_absorbeurs, sizeof( Atome *) );
      if ( NULL == choisis ) {
	SYS_ERREUR( MANQUE_MEMOIRE, "Interface/3d_changer_vue.c", "V3D_GER_ChangerVue", "choisis" );
	return VRAI;
      }

      for ( i = 0 ; i < molecule_modifiee->nbr_absorbeurs ; i++ )
	choisis[ i ] = molecule_modifiee->absorbeurs[ i ].absorbeur;

      ATM_choisir( choisis, molecule_modifiee->nbr_absorbeurs, FAUX, &choix );
      if ( choix >= 0 ) vise = choisis[ choix ];
      free( choisis ); choisis = NULL;
    }

    if ( vise != NULL ) {
      Vecteur3D v_diff;

      (void) U3D_VecteurDirecteur( &(molecule_modifiee->pos_oeil), &(vise->position), &v_diff, NULL, NULL, NULL );
      OBJ_MettreCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
			     &(vise->position), NULL );
      V3D_ChangerVue_MettreCoordonnees( &v_diff );
    }
    return BI_EVENEMENT_TRAITE;
  }

  if ( V3D_ABS_POSSIBLE == objet ) { /* On choisit de viser un absorbeur plausible de la molécule */
    Vecteur3D v_diff;

    vise = ABS_ProposerAbsorbeur( molecule_modifiee, FAUX );
    if ( NULL == vise ) return VRAI;

    (void) U3D_VecteurDirecteur( &(molecule_modifiee->pos_oeil), &(vise->position), &v_diff,
				 NULL, NULL, NULL );
    OBJ_MettreCoordonnees( DEFINIR_VUE_3D, V3D_X_VU, V3D_Y_VU, V3D_Z_VU,
			   &(vise->position), NULL );
    V3D_ChangerVue_MettreCoordonnees( &v_diff );

    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Préparation de l'arbre permettant de définir la vue en 3D -------- */

static void V3D_PreparerArbre_ChangerVue(void)
{
  if ( FAUX == arbres[ DEFINIR_VUE_3D ].drapeaux.init ) {
    int retour;

    OPTION_A_FAIRE( "Documentation, boîte de changement de la vue 3D / molécules" );
    retour = ARB_Initialiser( DEFINIR_VUE_3D, NULL, V3D_GER_ChangerVue, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf( stderr, "%s,%s l.%u : ERREUR dans l'initialisation\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      return;
    }
  }

  /* On adapte les types des objets */
  OBJ_ChangerType( DEFINIR_VUE_3D, V3D_LATITUDE , G_NOMBRE );
  OBJ_ChangerType( DEFINIR_VUE_3D, V3D_LONGITUDE, G_NOMBRE );

  /* On place les boutons plus joliment */
  OBJ_deplace( DEFINIR_VUE_3D, V3D_Y_VU       , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( DEFINIR_VUE_3D, V3D_Z_VU       , 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( DEFINIR_VUE_3D, V3D_LATITUDE   , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( DEFINIR_VUE_3D, V3D_LONGITUDE  , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( DEFINIR_VUE_3D, V3D_INCLINAISON, 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( DEFINIR_VUE_3D, V3D_OUVERTURE  , 0, +4, DEPLACE_PIXELS );

#ifndef _DEVELOPPE_
  OBJ_active( DEFINIR_VUE_3D, V3D_VISER_ATOME, FORCE_NON );
  OBJ_active( DEFINIR_VUE_3D, V3D_INCLINAISON, FORCE_NON );
  OBJ_active( DEFINIR_VUE_3D, V3D_OUVERTURE  , FORCE_NON );
#endif

  arbres[ DEFINIR_VUE_3D ].drapeaux.prepare = VRAI;
}
