/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2012

   Interface pour la gestion de base des graphes (copie, effacement,...)
   ==> Copie de graphes
*/

#include <stdio.h>
#include <stdlib.h>

#include "exafs.h"
#include "systeme.h"
#include "txt_utils.h"

#include "interface.h"
#include "serie.h"

#include "int_graphes_copier.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation concrète ~~~~~~~~ */

/* -------- Copier un graphe, éventuellement dans une nouvelle série -------- */

int I_GRF_Copier(struct graphe *graphe, boolean multiple)
{
  Graphe **graphes = NULL;
  Serie_Graphes *serie_arrivee;
  unsigned long nbr_graphes = 0;

  if ( VRAI == multiple ) {
    /* Choix des graphes à copier */
    if ( NULL == graphe ) {
      graphes = GRF_mchoisir( GR_TOUS, 1, 0, &nbr_graphes );
    } else {
      graphes = GRF_mchoisir( graphe->type, 1, 0, &nbr_graphes );
    }
    if ( NULL == graphes ) return -1; /* Aucun graphe sélectionné pour la copie... */
  } else if ( NULL == graphe ) {
    return -20;			/* Pas de graphe sélectionné pour la copie */
  }

  /* Choix de la série d'arrivée */
  serie_arrivee = I_SERIE_GRF_Choisir( -1, ROOT, 0 );
  if ( NULL == serie_arrivee ) {
    free( graphes ); graphes = NULL;
    return -2;			/* Aucune série choisie, copie impossible */
  }

  if ( VRAI == serie_arrivee->drapeaux.protegee ) {
    (void) BI_ERR_alerte( principale, &ERR_SerieProtegee, VRAI );
    free( graphes ); graphes = NULL;
    return -3;			/* La série sélectionnée étant protégée, on ne peut pas copier... */
  }

  if ( VRAI == multiple ) {
    unsigned long i;

    for ( i = 0 ; i < nbr_graphes ; i++ ) {
      GRF_Copier( graphes[ i ], serie_arrivee, FAUX );
    }
  } else {
    GRF_Copier( graphe, serie_arrivee, FAUX );
  }

  /* On a fini... */
  free( graphes ); graphes = NULL;
  return 0;
}
