/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface utilisateur : routines gnrales indpendantes du systme

   Fonctions avances pour l'historique des oprations
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <math.h>

#include "systeme.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Rsultats d'un test statistique -------- */

void HIST_ResultatTestStatistique(const char *nom_test, const char *H0, const char *H1,
				  double val_observee, double val_theorie, double val_H0,
				  double variance_H0, double proba_rejet_H0, boolean exact, const char *condition,...)
{
  if ( NULL == nom_test ) return;

  /* On sauvegarde au niveau 3 de l'historique : "rsultats dtaills" */

  HIST_Titre(3, nom_test, "TEST");
  HIST_Texte(3, 3, "Hypoth"e_GRAVE"se H0 : %s\n", H0);
  HIST_Texte(3, 3, "Hypoth"e_GRAVE"se H1 : %s\n", H1);
  HIST_Texte(3, 3, "Valeur attendue dans le cas id"e_AIGU"al (sous H0) : %g\n", val_theorie);
  if ( variance_H0 > 0.0 ) { /* On connat thoriquement la valeur moyenne... */
    HIST_Texte(3, 3, "Valeur moyenne attendue sous H0 : %g (avec un "e_AIGU"cart-type de %g)\n", val_H0, sqrt(variance_H0));
  }
  HIST_Texte(3, 3, "Valeur observ"e_AIGU"e : %g\n", val_observee);
  if (FAUX == exact) {
    HIST_Texte(3, 3, "ATTENTION, la loi sous H0 n'est qu'approximative !\n");
  }
  if ( (proba_rejet_H0 >= 0.0) && (proba_rejet_H0 <= 1.0) )
    HIST_Texte(3, 3, "Probabilit"e_AIGU" de rejeter "a_GRAVE" tort H0 avec la valeur observ"e_AIGU" : %.3g\n", proba_rejet_H0);
  if (condition != NULL) { /* Il y a des conditions d'application */
    va_list lst_cond;
    char *cond;

    HIST_Texte(3, 3, " -- Conditions d'application du test --\n");
    HIST_Texte(3, 3, "- %s ;\n", condition);

    va_start(lst_cond, condition);
    cond = va_arg(lst_cond, char *);
    while (cond != NULL) {
      HIST_Texte(3, 3, "- %s ;\n", cond);
      cond = va_arg(lst_cond, char *);
    }
    va_end(lst_cond);
  }
}

/* -------- Une option parmi plusieurs possibles -------- */

void HIST_OptionChoisie(unsigned int niveau, const char *preambule,
			unsigned int nbr_options, unsigned int option_choisie, const char *options[])
{
  if (NULL == preambule) return;
  if (option_choisie >= nbr_options) return;

  HIST_Texte(niveau, 0, "%s : %s", preambule, options[option_choisie]);
}

