/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2008 Emmanuel CURIS
   Création : le 17 mai 2008

   Interface graphique : Récupération des données expérimentales pour obtenir µ total

   Fichiers enregistrés à Bessy2 (Berlin) : format HDF
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef AVEC_HDF_
# include <mfhdf.h> /* Bibliothèque HDF v.4, NCSA */

#include "systeme.h"

#include "exafs.h"
#include "import_xas.h"
#include "interface.h"
#include "serie.h"

#include "int_import_Bessy2.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static const char *filtre_hdf[] = { "*.hdf" };

static BI_InfosSelecteur infos_selecteur = {
  (char *) txt_titre_selecteur_Bessy2_I0,
  BI_SELECTEUR_MULTIPLE|BI_SELECTEUR_FILTRE_UTILISATEUR|BI_SELECTEUR_AJOUTER_PREDEFINI|BI_SELECTEUR_AVEC_JOKERS|BI_SELECTEUR_MEMORISER_CHEMIN,
  NULL,
  1, filtre_hdf,
  0, NULL,
  (void (*)(unsigned int, unsigned int, const char *)) NULL
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* -------- Import binaire : HDF convention Bessy2 -------- */
int I_ES_ImportXAS_Bessy2(Infos_Extraction *infos)
{
  char **noms = NULL;      /* Les adresses des noms des fichiers à charger */
  int retour;
  unsigned long i, nombre_fichiers_i0;
  unsigned int que_garder_original __attribute__((__unused__)) = 0;

  EXT_ListeGraphes inter = { NULL, NULL, 1, NULL, NULL };
  Graphe **i0 = NULL;

  /* On mémorise ce que l'on voudra vraiment garder à la fin */
  que_garder_original = infos->que_garder;

  /* Première étape : récupération du I0 */
  retour = BI_SelecteurChargement( &infos_selecteur, &nombre_fichiers_i0, &noms, principale );
  if ( ( BI_SELECTEUR_ANNULER == retour ) || ( 0 == nombre_fichiers_i0 ) || ( NULL == noms ) ) {
    /* Chargement annulé */
#ifdef _DEVELOPPE_
    if ( ( ( NULL == noms ) && ( nombre_fichiers_i0 > 0 ) ) ||
	 ( ( NULL != noms ) && ( 0 == nombre_fichiers_i0 ) ) ) {
      (void) fprintf(stderr, "%s, %s l.%u : Incohérence dans le retour du sélecteur : %lu fichiers mais fichiers = %p\n",
		     __FILE__, __FUNCTION__, __LINE__, nombre_fichiers_i0, (void *) noms);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }
#endif
    return retour;
  }

  /* On réserve pour ces I0 */
  i0 = (Graphe **) malloc( sizeof( Graphe * ) * nombre_fichiers_i0 );
  if ( NULL == i0 ) {
    ERR_Memoire( "Liste des I0 lus", sizeof( Graphe * ) * nombre_fichiers_i0 );
    for ( i = 0 ; i < nombre_fichiers_i0 ; i++ ) {
      free( noms[ i ] ); noms[ i ] = NULL;
    }
    free( noms ); noms = NULL;

    return -25;
  }

  /* On lit ces I0, en utilisant une [sale] astuce */
  OPTION_A_FAIRE( "Mieux décomposer le chargement en généralisant à tous les HDF" );
  infos->que_garder = DETECT_GARDER_I0;
  for ( i = 0 ; i < nombre_fichiers_i0 ; i++ ) {
    inter.x = NULL;
    inter.i_0 = NULL;
    inter.nbr_spectres = 1;
    inter.spectres = NULL;
    inter.i_1 = NULL;

    retour = BESSY2_LireSpectre( noms[ i ], infos, &inter, NULL, 1 );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans BESSY2_LireSpectre() [code = %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
    }
  }

  /* Deuxième étape : récupération des I / signal */

  /* Tout s'est bien passé */
  return 0;
}
#else
struct infos_import_xas;

int I_ES_ImportXAS_Bessy2(struct infos_import_xas *infos __attribute__((__unused__)))
{
  (void) fprintf(stderr, "%s, %s l.%u : Support HDF absent, recompilez avec une bibliothèque HDF 4\n",
		 __FILE__, __FUNCTION__, __LINE__);
  return -1974;
}
#endif /* def AVEC_HDF_ */
