/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2007, Emmanuel CURIS
   Cration : 23 janvier 2007 [Graphes/liste.c dplac ici par souci de cohrence]

   Gestion gnrique des graphes

   Liste des graphes (fentre non-bloquante)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "exafs.h"
#include "fonctions.h"
#include "interface.h"
#include "extrait.h"	  /* Extraction EXAFS */
#include "fichiers.h"     /* Import/export */
#include "images.h"	  /* Pour IMG_CovarianceVersImage */
#include "lissage.h"      /* Appel de l'interpolation  */
#include "operations.h"   /* Appel de la dcomposition */
#include "statistiques.h" /* Appel du tri              */
#include "tracer.h"	  /* Superposition             */
#include "xanes.h"	  /* Extraction XANES */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_AFFICHAGE    20
#define LGMAX_NOM_GRAPHE   30
#define LGMAX_NOM_SERIE    45
#define LGMAX_GENRE_GRAPHE 21
#define LGMAX_TYPE_GRAPHE  40

#define NOMBRE_GRAPHES_VISIBLES ( LG_DERNIER_GRAF - LG_PREMIER_GRAF + 1 )

/* Les options valables pour la cration */

#define LGRF_CREER_SERIE    0
#define LGRF_CREER_VALEURS  2	/* 1 : sparateur */
#define LGRF_CREER_COUPLE   3
#define LGRF_CREER_FONCTION 4
#define LGRF_CREER_GRAPHE   5
#define LGRF_CREER_ALEATOIRE 6

#define LGRF_NOMBRE_OPT_CREATION 7

/* Les options valables pour les sries */

#define LGRF_SERIE_SAUVER    0
#define LGRF_SERIE_EXPORTER  1	/* 2 : sparateur */
#define LGRF_SERIE_EFFACER   3
#define LGRF_SERIE_RENOMMER  4	/* 5 : sparateur */
#define LGRF_SERIE_COPIER    6
#define LGRF_SERIE_DEPLACER  7
#define LGRF_SERIE_ISOLER    8	/* 9 : sparateur */
#define LGRF_SERIE_XCOMMUNS 10	/* 11: sparateur */
#define LGRF_SERIE_LOT      12

#define LGRF_NOMBRE_OPT_SERIE_GRF 13

/* Options de filtres d'affichage */

#define LGRGF_AFF_AUCUN        0
#define LGRGF_AFF_TOUS         1
#define LGRGF_AFF_MEME_TYPE    2
#define LGRGF_AFF_COMPATIBLES  3 /* 4 : sparateur */
#define LGRGF_AFF_TYPE_DONNE   5
#define LGRGF_AFF_PERSONNEL    6

#define LGRF_NOMBRE_OPT_AFFICHAGE 7

static unsigned long num_prem_grf_affiche = 0UL;
static Graphe       *prem_grf_affiche = NULL;

#include "liste_grf.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  LGRF_GererArbre(unsigned int num_objet, int x_souris, int y_souris);

static void LGRF_CouperAxeX(boolean couper);
static void LGRF_ToutCouper(boolean couper);

static void LGRF_GererMenuAffichage    (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOpt_Creation (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOpt_Generique(unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOpt_Serie    (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOptions_C    (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOptions_R    (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOptions_Serie(unsigned int num_arbre, unsigned int num_objet, unsigned long action);

typedef void (*FonctionDeSousMenu)(unsigned int, unsigned int, unsigned long);

static FonctionDeSousMenu LGRF_QuelleFonction_ParGenre(unsigned int genre_graphe);

static void LGRF_GererMenuOptions_Brut     (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LGRF_GererMenuOptions_Cinetique(unsigned int num_arbre, unsigned int num_objet, unsigned long action);

static FonctionDeSousMenu LGRF_QuelleFonction_ParType (unsigned int genre_graphe, unsigned int type_graphe);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Gestion de la bote de dialogue ------------ */

static int LGRF_GererArbre(unsigned int num_objet, int x, int y)
{
  int nouv_objet;
  int etat, boutons;

  if ( LG_NV_SERIE == num_objet ) {
    /* Crer une nouvelle srie : c'est le seul bouton qui marche toujours ! */
    /* Appel du sous-menu : automatique */
    return BI_EVENEMENT_NON_TRAITE;
  }

  if ( NULL == serie_courante ) {
    return BI_EVENEMENT_TRAITE; /* Aucune srie : aucun intrt ! */
  }

  /* Plus d'infos : bouton cliqu, touches spciales,... */
  BI_EtatSouris(NULL, NULL, &boutons, NULL, &etat);

  /* Bouton qui a un sens ds qu'il y a une srie ou plus, mme si la courante est vide */
  if ( LG_SERIE == num_objet ) { /* Choix de la srie courante */
    if ( 2 == boutons ) {
      /* On simule un clic sur "Renommer"... */
      LGRF_GererMenuOpt_Serie( 0, 0, LGRF_SERIE_RENOMMER );
    } else {
      Serie_Graphes *serie;

      serie = I_SERIE_GRF_Choisir( CHOIX_GRAPHE, LG_SERIE, LGMAX_NOM_SERIE );
      I_SERIE_GRF_Aller( serie );
    }

    return BI_EVENEMENT_TRAITE;
  }

  if ( serie_courante->premier_graphe == NULL ) { /* Aucun graphe dans la srie -> cette bote est sans intrt */
    if ( LG_OPTIONS_SERIE == num_objet ) return BI_EVENEMENT_NON_TRAITE;
    return BI_EVENEMENT_TRAITE;
  }

  if ( ( num_objet >= LG_PREMIER_GRAF ) && ( num_objet <= LG_DERNIER_GRAF ) ) {
    unsigned int i;
    Graphe *compteur;

    for ( i = LG_PREMIER_GRAF, compteur = prem_grf_affiche ; i != num_objet ; i++ ) {
      if ( compteur == NULL ) {
	/* Ne devrait jamais arriver, car normalement objet dsactiv ! */
	AVERTISSEMENT( "Clic sur une case sans graphe [objet %u] !", num_objet);
	return BI_EVENEMENT_TRAITE;
      }
      compteur = compteur->suivant;
    }

    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) { /* Shift-clic : change le drapeau de visibilit */
      compteur->drapeaux.superpose = !(compteur->drapeaux.superpose);
      OBJ_coche( CHOIX_GRAPHE, num_objet, compteur->drapeaux.superpose == VRAI );
      TRC_RedessinerSuperposition( compteur, FAUX, VRAI );

      return BI_EVENEMENT_TRAITE; /* Rien  changer : on ne fait que modifier visible/invisible */
    }

    /* On va au graphe indiqu, comme nouveau graphe courant */
    GRF_aller( compteur );

    return BI_EVENEMENT_TRAITE;
  }

  switch (num_objet)
  {
  case LG_AVANT:
    if ( prem_grf_affiche->precedent != NULL ) {
      prem_grf_affiche = prem_grf_affiche->precedent;
      num_prem_grf_affiche--;
    }
    ARB_ascenseur( CHOIX_GRAPHE, LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ASC_VERTICAL,
		   serie_courante->nbr_graphes, NOMBRE_GRAPHES_VISIBLES, num_prem_grf_affiche, ROOT, 0, 0, METTRE );
    GRF_PreparerListe( FAUX ); /* On raffiche */
    return BI_EVENEMENT_TRAITE;

  case LG_FOND_ASC:
  case LG_BOUTON_ASC: /* On fait glisser l'ascenseur */
    {
      unsigned long i;
      Graphe *compteur;

      num_prem_grf_affiche = ARB_ascenseur(CHOIX_GRAPHE,
					   LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ASC_VERTICAL,
					   serie_courante->nbr_graphes, NOMBRE_GRAPHES_VISIBLES, num_prem_grf_affiche,
					   num_objet, x, y, ECHANGE );
      for ( i = 0, compteur = serie_courante->premier_graphe ; i < num_prem_grf_affiche ; i++, compteur = compteur->suivant );
      prem_grf_affiche = compteur;
      GRF_PreparerListe( FAUX ); /* On raffiche */
    }
    return BI_EVENEMENT_TRAITE;

  case LG_APRES:
    if (prem_grf_affiche->suivant) {
      prem_grf_affiche = prem_grf_affiche->suivant;
      num_prem_grf_affiche++;
    }
    ARB_ascenseur( CHOIX_GRAPHE, LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ASC_VERTICAL,
		   serie_courante->nbr_graphes, NOMBRE_GRAPHES_VISIBLES, num_prem_grf_affiche, ROOT, 0, 0, METTRE);
    GRF_PreparerListe( FAUX ); /* On raffiche */
    return BI_EVENEMENT_TRAITE;

  case LG_CONVERTIR_X:
    OBJ_choisit( CHOIX_GRAPHE, num_objet, FORCE_OUI );
    I_GRF_ConvertirUnites( graphe_courant, AXE_X,
			  lit_chaine( CHOIX_GRAPHE, LG_UNITE_X, 0, NULL, TXT_REMPLACE ) );
    OBJ_choisit( CHOIX_GRAPHE, num_objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case LG_AFF_ERR_X: /* Doit-on afficher les erreurs en X ? */
    GRF_AfficherErreurs( graphe_courant, AXE_X, OBJ_choisit( CHOIX_GRAPHE, LG_AFF_ERR_X, ECHANGE ) );
    return BI_EVENEMENT_TRAITE;

  case LG_CONVERTIR_Y:
    OBJ_choisit( CHOIX_GRAPHE, num_objet, FORCE_OUI );
    I_GRF_ConvertirUnites( graphe_courant, AXE_Y,
			  lit_chaine( CHOIX_GRAPHE, LG_UNITE_Y, 0, NULL, TXT_REMPLACE ) );
    OBJ_choisit( CHOIX_GRAPHE, num_objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case LG_AFF_ERR_Y: /* Doit-on afficher les erreurs en Y ? */
    GRF_AfficherErreurs( graphe_courant, AXE_Y, OBJ_choisit( CHOIX_GRAPHE, LG_AFF_ERR_Y, ECHANGE ) );
    return BI_EVENEMENT_TRAITE;

  case LG_COULEUR_GRF: /* Choix de la couleur du trac */
    if ( graphe_courant->affichage != NULL ) {
      nouv_objet = GER_ssmenu( CHX_COULEUR, CHOIX_GRAPHE, LG_COULEUR_GRF, graphe_courant->affichage->couleur - 1 );
      if (nouv_objet > 0) { /* On a chang de couleur */
	GRF_couleur( graphe_courant, nouv_objet, -1, -1 );
      }
    }
    return BI_EVENEMENT_TRAITE;

  case LG_STYLE_LIGNE:   /* Choix du style de ligne     */
    if ( graphe_courant->affichage != NULL ) {
      short int nv_style;

      nv_style = BI_DemanderStyleLigne( CHOIX_GRAPHE, LG_STYLE_LIGNE );
      GRF_couleur( graphe_courant, -1, nv_style, -1 );
    }
    return BI_EVENEMENT_TRAITE;

  case LG_STYLE_MARQUE:		/* Choix des marques affiches */
    if ( graphe_courant->affichage != NULL ) {
      short int nv_marque;

      nv_marque = BI_DemanderMarque( CHOIX_GRAPHE, LG_STYLE_MARQUE );
      GRF_couleur( graphe_courant, -1, -1, nv_marque );
    }
    return BI_EVENEMENT_TRAITE;

  case LG_APPLIQUER:
    GRF_EnregistrerParametres();

    GRF_PreparerListe(FAUX); /* On refait l'ascenseur : titre modifi */
    GRF_MiseAJour( graphe_courant, VRAI );
    return BI_EVENEMENT_TRAITE;

  case LG_TYPE_GRAPHE:
    {
      int nouveau_type;

      nouveau_type = OBJ_ChoisirTypeGraphe( CHOIX_GRAPHE, LG_TYPE_GRAPHE, LGMAX_TYPE_GRAPHE, graphe_courant->type, QUESTION, VRAI );
      if ( nouveau_type != graphe_courant->type ) {
	int reponse;

	reponse = BI_ERR_alerte( principale, &MSG_ChangerType, VRAI );
	if ( 1 == reponse ) {
	  /* On mmorise le nom, la lgende,... */
	  GRF_EnregistrerParametres();

	  /* On change le type */
	  GRF_ChangerType( graphe_courant, nouveau_type );

	  /* On met  jour tout en fonction ! */
	  GRF_MiseAJour( graphe_courant, VRAI );
	}
      }
    }
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Grer le sous-menu proposant des filtre d'affichage -------- */

static void LGRF_GererMenuAffichage(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *graphe;

  if ( ( NULL == serie_courante ) || ( NULL == serie_courante->premier_graphe ) )
    return;

  switch ( action )
  {
  case LGRGF_AFF_AUCUN:		/* On force  ne pas tracer */
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant)
      graphe->drapeaux.superpose = FAUX;
    break;

  case LGRGF_AFF_TOUS:		/* On force  tracer */
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant)
      graphe->drapeaux.superpose = VRAI;
    break;

  case LGRGF_AFF_MEME_TYPE:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
      if ( graphe->type == graphe_courant->type )
	graphe->drapeaux.superpose = VRAI;
      else
	graphe->drapeaux.superpose = FAUX;
    }
    break;

  case LGRGF_AFF_COMPATIBLES:
    OPTION_A_FAIRE( "Slectionner les graphes compatibles [type, genre, units]" );
    break;

  case LGRGF_AFF_TYPE_DONNE:
    OPTION_A_FAIRE( "Slectionner les graphes d'un type indiqu" );
    break;

  case LGRGF_AFF_PERSONNEL:
    OPTION_A_FAIRE( "Slectionner les graphes suivant un filtre utilisateur" );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Mode de filtrage inconnu [%lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, action);
    return;
  }

  /* On mmorise les paramtres du graphe courant */
  GRF_EnregistrerParametres();

  /* On reconstruit la liste */
  GRF_PreparerListe(FAUX);

  /* On redessine la fentre de superposition [en forant, car tout a chang a priori !] */
  TRC_RedessinerSuperposition( NULL, VRAI, VRAI );
}

/* -------- Grer le sous-menu proposant des actions sur la srie courante -------- */

static void LGRF_GererMenuOpt_Creation(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *nouveau = NULL;

  switch ( action )
  {
  case LGRF_CREER_SERIE:
    I_SERIE_GRF_Creer( serie_courante, NULL );
    break;

  case LGRF_CREER_VALEURS:
    nouveau = I_GRF_CreerSerie( graphe_courant );
    break;

  case LGRF_CREER_COUPLE:
    nouveau = I_GRF_CreerCouples( graphe_courant );
    break;

  case LGRF_CREER_FONCTION:
    OPTION_A_FAIRE( "Crer une fonction simple Y=f(X)" );
    break;

  case LGRF_CREER_GRAPHE:
    nouveau = I_GRF_Generer( graphe_courant );
    break;

  case LGRF_CREER_ALEATOIRE:
    nouveau = I_GRF_CreerAleatoire( graphe_courant );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( nouveau != NULL ) {
    /* Ce nouveau graphe devient le graphe courant */
    GRF_aller( nouveau );
  }
}

/* -------- Grer le sous-menu proposant des actions sur la srie courante -------- */

static void LGRF_GererMenuOpt_Serie(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  char *nv_nom;

  switch ( action )
  {
  case LGRF_SERIE_SAUVER:
    (void) I_SERIE_GRF_Sauver( serie_courante );
    break;

  case LGRF_SERIE_EXPORTER:
    (void) EA_SerieComplete( serie_courante );
    break;

  case LGRF_SERIE_EFFACER:
    (void) I_SERIE_GRF_EffacerSerie( serie_courante, VRAI );
    break;

  case LGRF_SERIE_RENOMMER:
    nv_nom = BI_Demande_Texte( NULL, &Q_NouveauNomSerie, serie_courante->nom) ;
    if ( nv_nom != NULL ) {
      Serie_titre( serie_courante, nv_nom );
      free( nv_nom ); nv_nom = NULL;
      OBJ_CopierTexte( CHOIX_GRAPHE, LG_SERIE, serie_courante->nom, LGMAX_NOM_SERIE );
    }
    break;

  case LGRF_SERIE_COPIER:
    I_GRF_Copier( graphe_courant, VRAI );
    break;

  case LGRF_SERIE_DEPLACER:
    I_GRF_Deplacer( graphe_courant, VRAI );
    break;

  case LGRF_SERIE_ISOLER:
    I_GRF_Isoler( graphe_courant, VRAI );
    break;

  case LGRF_SERIE_XCOMMUNS:
    (void) Serie_AssignerX( serie_courante );
    break;

  case LGRF_SERIE_LOT:
    OPTION_A_FAIRE("Traitement par lot [arbre : TRAITE_LOT]" );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }
}

/* -------- Grer le sous-menu proposant des actions sur le graphe courant -------- */

static void LGRF_GererMenuOpt_Generique(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_SAUVER_GRAPHE:
    I_GRF_Sauver( graphe_courant );
    break;

  case FCT_GRF_EXPORTER_GRAPHE:
    GRF_ExportASCII( graphe_courant );
    break;

  case FCT_GRF_EXPORT_POSTSCRIPT:
    EXP_PS_GrapheUnique( graphe_courant );
    break;

  case FCT_GRF_EFFACER_GRAPHE:
    GRF_efface( graphe_courant, FAUX, FAUX );
    break;

  case FCT_GRF_COPIER_GRAPHE:
    I_GRF_Copier( graphe_courant, FAUX );
    break;

  case FCT_GRF_DEPLACER_GRAPHE:
    I_GRF_Deplacer( graphe_courant, FAUX );
    break;

  case FCT_GRF_ISOLER_GRAPHE:
    I_GRF_Isoler( graphe_courant, FAUX );
    break;

  case FCT_GRF_ERREUR_V_GRAPHE:
    if ( ( NULL == graphe_courant->erreur_x ) && ( NULL == graphe_courant->erreur_y ) ) {
      (void) BI_ERR_alerte( principale, &ERR_SansErreurs, VRAI );
    } else {
      unsigned int axe = 0;

      if ( NULL == graphe_courant->erreur_x ) {
	axe = AXE_Y;
      } else if ( NULL == graphe_courant->erreur_y ) {
	axe = AXE_X;
      } else {
	int reponse;

	reponse = BI_ERR_alerte( principale, &Q_QuellesErreurs, VRAI );
	if ( 3 == reponse ) return; /* On annule */
	if ( 2 == reponse ) axe = AXE_Y;
	if ( 1 == reponse ) axe = AXE_X;
      }

      /* On sort une copie du graphe, place dans la srie juste aprs le graphe trait */
      resultat = GRF_SortirErreurs( graphe_courant, axe );
    }
    break;

  case FCT_GRF_COVARIANCE_V_IMAGE:
    if ( graphe_courant->covariance != NULL ) {
      int reponse;

      reponse = BI_ERR_alerte( principale, &Q_Covariance, VRAI );
      if ( 3 == reponse ) return; /* On annule */

      IMG_CovarianceVersImage( graphe_courant, reponse == 2 );
    } else {
      (void) BI_ERR_alerte( principale, &ERR_SansCovariance, VRAI );
    }
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}

/* -------- Fonctions de gestion des sous-menus / par genre -------- */

static FonctionDeSousMenu LGRF_QuelleFonction_ParGenre(unsigned int genre_graphe)
{
  switch ( genre_graphe )
  {
  case GRF_FCT_REELLE:
    return LGRF_GererMenuOptions_R;

  case GRF_FCT_COMPLEXE:
    return LGRF_GererMenuOptions_C;

  case GRF_SERIE:
    return LGRF_GererMenuOptions_Serie;
  }

  return (FonctionDeSousMenu) NULL;
}

/* -------- Actions sur des graphes  valeurs complexes -------- */

static void LGRF_GererMenuOptions_C(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_PARTIE_REELLE:
    resultat = CMP_reelle( graphe_courant );
    break;

  case FCT_GRF_PARTIE_IMAGINAIRE:
    resultat = CMP_imaginaire( graphe_courant );
    break;

  case FCT_GRF_MODULE:
    resultat = CMP_module( graphe_courant );
    break;

  case FCT_GRF_ARGUMENT:
    resultat = CMP_argument( graphe_courant );
    break;

  case FCT_GRF_TRONQUER_C:
    OPTION_A_FAIRE( "Tronquer un graphe" );
    break;

  case FCT_GRF_NORMALISER_C:
    resultat = I_GRF_Normaliser( graphe_courant );
    break;

  case FCT_GRF_INTERPOLER_C:
    resultat = I_GRF_Interpolation( graphe_courant );
    break;

  case FCT_GRF_LISSER_C:
    OPTION_A_FAIRE( "Options de lissage [moyenne mobile,...]" );
    break;

  case FCT_GRF_TRANSFORMER_C:
    OPTION_A_FAIRE( "Transformation du graphe [(X,Y) -> (X,Y)]" );
    break;

  case FCT_GRF_DERIVER_C:
    OPTION_A_FAIRE( "Driver le graphe" );
    break;

  case FCT_GRF_INTEGRER_C:
    OPTION_A_FAIRE( "Intgrer le graphe" );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}

/* -------- Actions sur des graphes  valeurs relles -------- */

static void LGRF_GererMenuOptions_R(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_INVERSER_XY:
    (void) GRF_PermuterXY( graphe_courant );
    break;

  case FCT_GRF_TRIER_X:
    I_GRF_Trier( graphe_courant );
    resultat = graphe_courant;	/* Puisqu'a priori, il y a eu des changements... */
    break;

  case FCT_GRF_DECOMPOSER_R:
    OPTION_A_FAIRE( "Dcomposition sur une base de fonctions relles [reprise de l'interface]" );
    (void) I_GRF_Decomposer( graphe_courant );
    break;

  case FCT_GRF_OTER_ABERRANCE:
    OPTION_A_FAIRE( "ter une aberrance [reprise de l'interface]" );
    aberrance( graphe_courant );
    break;

  case FCT_GRF_TRONQUER_R:
    resultat = I_GRF_TronquerGraphe( graphe_courant );
    break;

  case FCT_GRF_NORMALISER_R:
    resultat = I_GRF_Normaliser( graphe_courant );
    break;

  case FCT_GRF_INTERPOLER_R:
    resultat = I_GRF_Interpolation( graphe_courant );
    break;

  case FCT_GRF_LISSER_R:    
    OPTION_A_FAIRE( "Options de lissage [moyenne mobile,...]" );
    break;

  case FCT_GRF_TRANSFORMER_R:
    resultat = I_GRF_Transformer( graphe_courant );
    break;

  case FCT_GRF_TRANSFORMER_LOG:
    resultat = I_GRF_SemiLog( graphe_courant );
    break;

  case FCT_GRF_DERIVER_R:
    resultat = I_GRF_Derivee( graphe_courant );
    break;

  case FCT_GRF_INTEGRER_R:
    resultat = I_GRF_Primitive( graphe_courant );
    break;

  case FCT_GRF_APPROX_DROITE:
    resultat = I_GRF_AjusterDroite( graphe_courant );
    break;

  case FCT_GRF_MODELISER:
    resultat = I_GRF_ModeliserGenerique( graphe_courant );
    break;

  case FCT_GRF_BRUITER:
    resultat = I_GRF_Bruiter( graphe_courant );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}

/* -------- Actions sur des sries de valeurs relles -------- */

static void LGRF_GererMenuOptions_Serie(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_CHOISIR_X:
    OPTION_A_FAIRE( "Choix d'une table de valeurs en X pour un graphe de type srie" );
    break;

  case FCT_GRF_STAT_DESCRIPTIVE:
    OPTION_A_FAIRE( "Description statistique d'une srie de donnes [reprise de l'interface]" );
    calc_param_stat( graphe_courant );
    break;

  case FCT_GRF_FONCTION_REPARTITION:
    resultat = I_STAT_FonctionRepartition( graphe_courant );
    break;

  case FCT_GRF_HISTOGRAMME:	/* Histogramme  partir des valeurs */
    OPTION_A_FAIRE( "Construction d'un histogramme [reprise de l'interface]" );
    resultat = estime_distribution( graphe_courant );
    break;

  case FCT_GRF_DROITE_HENRY:
    resultat = I_STAT_DroiteDeHenry( graphe_courant );
    break;

  case FCT_GRF_ANVA_1F:
    I_ANVA_1facteur( graphe_courant );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}

/* -------- Choix de la fonction de gestion du sous-menu en fonction du type -------- */

static FonctionDeSousMenu LGRF_QuelleFonction_ParType(unsigned int genre_graphe __attribute__((__unused__)), unsigned int type_graphe)
{
  switch ( type_graphe )
  {
  case GR_BRUT:
    return LGRF_GererMenuOptions_Brut;

  case GR_CINETIQUE:
    return LGRF_GererMenuOptions_Cinetique;
  }

  /* Par dfaut : aucune fonction particulire associe */
  return (FonctionDeSousMenu) NULL;
}

/* -------- Fonctions spcifiques d'un spectre d'absorption/fluorescence exprimental brut --------
 *  Les codes sont dfinis dans fonctions.h
 */

static void LGRF_GererMenuOptions_Brut(unsigned int num_arbre __attribute__((__unused__)), unsigned int num_objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_BRUT_CONVERSION:
    OPTION_A_FAIRE( "Conversion post-chargement fluo <-> transmission" );
    break;

  case FCT_GRF_BRUT_OTER_BLANC:
    resultat = I_XAN_OterBlanc( graphe_courant );
    break;

  case FCT_GRF_BRUT_MATRICE:
    resultat = I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_AUCUNE );
    break;

  case FCT_GRF_BRUT_TOUT_FAIRE:
    /* On veut faire tout le traitement du XANES */
    resultat = I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_XANES );
    break;

  case FCT_GRF_BRUT_REPRODUIRE:
    EXF_reproduit( graphe_courant );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}

/* -------- Fonctions spcifiques d'un spectre de cintique chimique --------
 *   (donc aussi pharmacocintique,... : concentration en fonction du temps)
 *  Les codes sont dfinis dans fonctions.h
 */

static void LGRF_GererMenuOptions_Cinetique(unsigned int num_arbre __attribute__((__unused__)), unsigned int num_objet __attribute__((__unused__)), unsigned long action)
{
  Graphe *resultat = NULL;

  switch ( action )
  {
  case FCT_GRF_CINETIQUE_BASIQUE:
    (void) I_PHARMACO_ParametresBasiques( graphe_courant );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer un nouveau graphe */
    if ( graphe_courant == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage du graphe courant" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* Le nouveau graphe devient le graphe courant */
      GRF_aller( resultat );
    }
  }
}
/* -------- Enregistrer les modifications ventuelles apportes  un graphe -------- */

void GRF_EnregistrerParametres( void )
{
  char *nv_nom, *nv_legende_X, *nv_unite_X, *nv_legende_Y, *nv_unite_Y;
  const char *ancien_texte;
  int retour = 0;

  ENTREE_FONCTION( __FUNCTION__, "", 0 );

  if ( NULL == graphe_courant ) {
    SORTIE_FONCTION( __FUNCTION__, "graphe_courant=NULL", 0 );
    return; /* Rien  faire ! */
  }
  if ( FAUX == arbres[ CHOIX_GRAPHE ].drapeaux.ouvert ) {
    /* Si la bote est ferme, l'utilisateur n'a pas pu modifier les valeurs => on se dispense de les relire
       D'autant plus que si elle n'a jamais t ouverte, les FTEXT ne sont pas convertis en textes infinis
       ==> on aurait une troncature des noms, lgendes, units...
     */
    SORTIE_FONCTION( __FUNCTION__, "arbre ferm", 0 );
    return; /* Rien  faire ! */
  }

  /* 1. On rcupre les valeurs entres */
  nv_nom       = lit_chaine( CHOIX_GRAPHE, LG_NOM      , 0, NULL, TXT_REMPLACE );
  nv_legende_X = lit_chaine( CHOIX_GRAPHE, LG_LEGENDE_X, 0, NULL, TXT_COPIER );
  nv_unite_X   = lit_chaine( CHOIX_GRAPHE, LG_UNITE_X  , 0, NULL, TXT_COPIER );
  nv_legende_Y = lit_chaine( CHOIX_GRAPHE, LG_LEGENDE_Y, 0, NULL, TXT_COPIER );
  nv_unite_Y   = lit_chaine( CHOIX_GRAPHE, LG_UNITE_Y  , 0, NULL, TXT_COPIER );

  /* 2. Maintenant seulement, on modifie
     En effet, si un texte a chang, en l'enregistrant
     on remet les anciennes valeurs dans l'arbre ==> on perd les suivants !
     C'est pour cela aussi que l'on travaille avec une COPIE de ces chanes, qu'il faudra librer ensuite !
  */

  /* Le titre */
  if ( NULL == nv_nom  ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans le nouveau nom du graphe !\n",
		   __FILE__, __FUNCTION__, __LINE__);
  } else if ( NULL == graphe_courant->nom ) {
    /* Le graphe n'avait pas de nom : on met celui propos, de toute faon */
    GRF_titre( graphe_courant,  nv_nom );
  } else {
    if ( strcmp( nv_nom, graphe_courant->nom ) != 0 ) { /* Il y a des diffrences */
      GRF_titre( graphe_courant,  nv_nom );
    }
  }

  /* La lgende en X */
  if ( NULL == nv_legende_X ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet lgende X\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    ancien_texte = GRF_QuelleLegende( graphe_courant, AXE_X );
    if ( NULL == ancien_texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme grave dans GRF_QuelleLegende [renvoie NULL !]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      if ( strcmp( nv_legende_X, ancien_texte ) != 0 ) { /* Lgende en X change */
	retour = GRF_ChangerLegende( graphe_courant, AXE_X, nv_legende_X, VRAI );
	if ( retour != 0 ) {
	  (void) BI_ERR_alerte( principale, &ERR_ChangementLegende, VRAI, 'X' );
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer la lgende en X [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
#endif
	}
      }
    }
  }
  free( nv_legende_X ); nv_legende_X = NULL;

  /* L'unit en X */
  if ( NULL == nv_unite_X ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet unit X\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    retour = 0;
    ancien_texte = GRF_QuelleUnite( graphe_courant, AXE_X );
    if ( NULL == ancien_texte ) {
      /* On attribue une unit, il n'y en avait pas avant : pas de message particulier ! */
      retour = GRF_ChangerUnite( graphe_courant, AXE_X, nv_unite_X );
    } else {
      if ( strcmp( nv_unite_X, ancien_texte ) != 0 ) { /* Unit en X change */
	int reponse;

	reponse = BI_ERR_alerte( principale, &MSG_ChangerUnite, VRAI, 'X', ancien_texte, nv_unite_X );
	if ( 1 == reponse ) {
	  retour = GRF_ChangerUnite( graphe_courant, AXE_X, nv_unite_X );
	}
      }
    }
    if ( retour != 0 ) {
      (void) BI_ERR_alerte( principale, &ERR_ChangementUnite, VRAI, 'X' );
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit en X [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    }
  }
  free( nv_unite_X ); nv_unite_X = NULL;

  /* La lgende en Y */
  if ( NULL == nv_legende_Y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet lgende Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    ancien_texte = GRF_QuelleLegende( graphe_courant, AXE_Y );
    if ( NULL == ancien_texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme grave dans GRF_QuelleLegende [renvoie NULL !]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      if ( strcmp( nv_legende_Y, ancien_texte ) != 0 ) { /* Lgende en Y change */
	retour = GRF_ChangerLegende( graphe_courant, AXE_Y, nv_legende_Y, VRAI );
	if ( retour != 0 ) {
	  (void) BI_ERR_alerte( principale, &ERR_ChangementLegende, VRAI, 'Y' );
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer la lgende en Y [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
#endif
	}
      }
    }
  }
  free( nv_legende_Y ); nv_legende_Y = NULL;

  /* L'unit en Y */
  if ( NULL == nv_unite_Y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet unit Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    retour = 0;
    ancien_texte = GRF_QuelleUnite( graphe_courant, AXE_Y );
    if ( NULL == ancien_texte ) {
      /* On attribue une unit, il n'y en avait pas avant : pas de message particulier ! */
      retour = GRF_ChangerUnite( graphe_courant, AXE_Y, nv_unite_Y );
    } else {
      if ( strcmp( nv_unite_Y, ancien_texte ) != 0 ) { /* Unit en Y change */
	int reponse;

	reponse = BI_ERR_alerte( principale, &MSG_ChangerUnite, VRAI, 'Y', ancien_texte, nv_unite_Y );
	if ( 1 == reponse ) {
	  retour = GRF_ChangerUnite( graphe_courant, AXE_Y, nv_unite_Y );
	}
      }
    }
    if ( retour != 0 ) {
      (void) BI_ERR_alerte( principale, &ERR_ChangementUnite, VRAI, 'Y' );
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit en Y [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    }
  }
  free( nv_unite_Y ); nv_unite_Y = NULL;

  SORTIE_FONCTION( __FUNCTION__, "RAS", 0 );
}

/* --------- Affichage de la liste des graphes --------------- */

void GRF_PreparerListe(boolean mode)
{
  Graphe *graphe;
  char   *chaine;
  unsigned int i;
  const char **table_options = NULL;
  unsigned long nombre_options = 0;

  ENTREE_FONCTION( __FUNCTION__, "mode=%d", mode );

  if (serie_courante == NULL) { /* Aucune srie en mmoire */
    prem_grf_affiche = NULL;
    num_prem_grf_affiche = 0UL;
  } else if (serie_courante->nbr_graphes < 15) { /* Rien de particulier  faire */
    prem_grf_affiche = serie_courante->premier_graphe;
    num_prem_grf_affiche = 0UL;
  } else if (mode == VRAI) { /* Il faut positionner l'ascenseur pour avoir le graphe courant visible
				==> on recherche donc le premier graphe affichable */
    Graphe *graphe;
    unsigned long num_graphe; /* Numro du graphe courant */

    for (num_graphe = 0, graphe = serie_courante->premier_graphe ;
	 graphe != graphe_courant ;
	 graphe = graphe->suivant, num_graphe++);

    if (num_graphe < 13) { /* On est prs du dbut */
      prem_grf_affiche = serie_courante->premier_graphe;
      num_prem_grf_affiche = 0UL;
    } else if ( num_graphe > serie_courante->nbr_graphes - 15 ) { /* On est prs de la fin */
      prem_grf_affiche = serie_courante->dernier_graphe;
      num_prem_grf_affiche = serie_courante->nbr_graphes - 1;
      for (i = 0, graphe = serie_courante->dernier_graphe ; i < 13 ; i++) {
	graphe = graphe->precedent;
	num_prem_grf_affiche--;
      }
      prem_grf_affiche = graphe;
    } else {
      graphe = graphe_courant;
      num_prem_grf_affiche = num_graphe;
      for ( i = 0 ; i <= 6 ; i++ ) {
	graphe = graphe->precedent;
	num_prem_grf_affiche--;
      }
      prem_grf_affiche = graphe;	
    }
  }

  /* L'ascenseur */
  if (serie_courante != NULL) {
    (void) ARB_ascenseur( CHOIX_GRAPHE, LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ASC_VERTICAL,
			  serie_courante->nbr_graphes, NOMBRE_GRAPHES_VISIBLES, num_prem_grf_affiche, ROOT, 0, 0, METTRE );
  } else {
    (void) ARB_ascenseur( CHOIX_GRAPHE, LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ASC_VERTICAL,
			  0, NOMBRE_GRAPHES_VISIBLES, 0, ROOT, 0, 0, METTRE );
  }
  
  /* On met  jour les chanes et les tats du dfileur */
  for ( i = LG_PREMIER_GRAF, graphe = prem_grf_affiche ; i <= LG_DERNIER_GRAF ; i++ ) {
    chaine = lit_chaine( CHOIX_GRAPHE, i, 0, NULL, TXT_REMPLACE );
    if ( NULL == chaine ) {
      (void) fprintf(stderr, "%s,%s l.%u : La chane %u du slecteur est nulle !\n",
		     __FILE__, __FUNCTION__, __LINE__, i);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }
    chaine += 2; /* Chane associe - 2 espaces pour le crochet */

    if ( graphe == NULL ) { /* Plus de graphe associ */
      chaine[ 0 ] = 0;

      OBJ_active (CHOIX_GRAPHE, i, FORCE_NON);
      OBJ_choisit(CHOIX_GRAPHE, i, FORCE_NON);
      OBJ_coche(CHOIX_GRAPHE, i, FORCE_NON);
    } else {
      if ( NULL == graphe->nom ) {
	strncpy( chaine, "<Sans nom>", LGMAX_NOM_GRAPHE );
      } else {
	strncpy( chaine, graphe->nom, LGMAX_NOM_GRAPHE );
      }

      OBJ_coche  (CHOIX_GRAPHE, i, graphe->drapeaux.superpose == VRAI);
      OBJ_active (CHOIX_GRAPHE, i, graphe->type != GR_INTERNE );
      OBJ_choisit(CHOIX_GRAPHE, i, graphe == graphe_courant);

      graphe = graphe->suivant;
    }
  }

  if ( NULL == serie_courante ) {
    OBJ_active( CHOIX_GRAPHE, LG_SERIE, FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_OPTIONS_SERIE, FORCE_NON );
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_SERIE, msg_aucune_serie, LGMAX_NOM_SERIE );
    LGRF_ToutCouper( VRAI );
    SORTIE_FONCTION( __FUNCTION__, "serie_courante=NULL", 0 );
    return;
  }

  OBJ_active( CHOIX_GRAPHE, LG_OPTIONS_SERIE, FORCE_OUI );
  OBJ_active( CHOIX_GRAPHE, LG_SERIE, FORCE_OUI );
  OBJ_CopierTexte( CHOIX_GRAPHE, LG_SERIE, serie_courante->nom, LGMAX_NOM_SERIE );

  if ( NULL == graphe_courant ) {
    LGRF_ToutCouper( VRAI );
    SORTIE_FONCTION( __FUNCTION__, "graphe_courant=NULL", 0 );
    return; /* Rien d'autre  faire ! */
  }

  /* Si un graphe est prsent, on peut voir le reste ! */
  LGRF_ToutCouper( FAUX );

  /* Les informations pour le graphe slectionn */
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_TYPE_GRAPHE , GRF_TexteTypeGraphe( graphe_courant->type )   , VRAI );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_GENRE_GRAPHE, GRF_TXT_Genre( graphe_courant->genre_donnees ), VRAI );

  OBJ_CopierTexte( CHOIX_GRAPHE, LG_NOM, graphe_courant->nom, 0 );

  if ( graphe_courant->affichage != NULL ) {
    OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_X, FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_UNITE_X  , FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_X, FORCE_OUI );

    OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_Y, FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_UNITE_Y  , FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_Y, FORCE_OUI );

    OBJ_active( CHOIX_GRAPHE, LG_COULEUR_GRF , FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_STYLE_LIGNE , FORCE_OUI );
    OBJ_active( CHOIX_GRAPHE, LG_STYLE_MARQUE, FORCE_OUI );

    OBJ_couleur( CHOIX_GRAPHE, LG_COULEUR_GRF, graphe_courant->affichage->couleur, COULEUR_FOND, FORCE_OUI);

    OBJ_MettreStyle( CHOIX_GRAPHE, LG_STYLE_LIGNE , graphe_courant->affichage->style );
    OBJ_MettreStyle( CHOIX_GRAPHE, LG_STYLE_MARQUE, graphe_courant->affichage->marque );
  } else {
    OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_X, FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_UNITE_X  , FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_X, FORCE_NON );

    OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_Y, FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_UNITE_Y  , FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_Y, FORCE_NON );

    OBJ_active( CHOIX_GRAPHE, LG_COULEUR_GRF , FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_STYLE_LIGNE , FORCE_NON );
    OBJ_active( CHOIX_GRAPHE, LG_STYLE_MARQUE, FORCE_NON );

    OBJ_MettreStyle( CHOIX_GRAPHE, LG_STYLE_LIGNE , 0 );
    OBJ_MettreStyle( CHOIX_GRAPHE, LG_STYLE_MARQUE, 0 );
  }

  OBJ_CopierTexte( CHOIX_GRAPHE, LG_LEGENDE_X, GRF_QuelleLegende( graphe_courant, AXE_X ), 0 );
  OBJ_CopierTexte( CHOIX_GRAPHE, LG_UNITE_X  , GRF_QuelleUnite  ( graphe_courant, AXE_X ), 0 );

  OBJ_CopierTexte( CHOIX_GRAPHE, LG_LEGENDE_Y, GRF_QuelleLegende( graphe_courant, AXE_Y ), 0 );
  OBJ_CopierTexte( CHOIX_GRAPHE, LG_UNITE_Y  , GRF_QuelleUnite  ( graphe_courant, AXE_Y ), 0 );

  if ( GR_INTERNE == graphe_courant->type ) {
    OBJ_active( CHOIX_GRAPHE, LG_OPT_GENERIQUE, FAUX );
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, FAUX );
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_TYPE , FAUX );
    OBJ_active( CHOIX_GRAPHE, LG_APPLIQUER, FAUX );
    OBJ_active( CHOIX_GRAPHE, LG_AFFICHAGE, FAUX );
  } else {
    OBJ_active( CHOIX_GRAPHE, LG_OPT_GENERIQUE, VRAI );
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, VRAI );
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_TYPE , VRAI );
    OBJ_active( CHOIX_GRAPHE, LG_APPLIQUER, VRAI );
    OBJ_active( CHOIX_GRAPHE, LG_AFFICHAGE, VRAI );
  }

  /* Erreurs en X ? */
  if ( VRAI == graphe_courant->drapeaux.X_serie ) {
    /* X externes, communs  la srie -> erreurs communes aussi ? */
    OBJ_active( CHOIX_GRAPHE, LG_AFF_ERR_X, serie_courante->erreurs_X != NULL );
  } else {
    OBJ_active( CHOIX_GRAPHE, LG_AFF_ERR_X, graphe_courant->erreur_x != NULL );
  }
  OBJ_choisit( CHOIX_GRAPHE, LG_AFF_ERR_X, graphe_courant->drapeaux.erreur_x );

  /* Erreurs en Y ? */
  OBJ_active ( CHOIX_GRAPHE, LG_AFF_ERR_Y, graphe_courant->erreur_y != NULL );
  OBJ_choisit( CHOIX_GRAPHE, LG_AFF_ERR_Y, graphe_courant->drapeaux.erreur_y );

  /* Options spcifiques en fonction du genre de donnes */
  table_options = FCT_ListeFonctions_GRF( FONCTION_EST_PAR_GENRE, graphe_courant->genre_donnees, 0, &nombre_options);
  if ( ( 0 == nombre_options ) || ( NULL == table_options ) ) {
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, FORCE_NON );
  } else {
    FonctionDeSousMenu fonction_de_gestion;

    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, FORCE_OUI );
    fonction_de_gestion = LGRF_QuelleFonction_ParGenre( graphe_courant->genre_donnees );
    OBJ_MettreMenu( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, nombre_options, table_options, titre_options, fonction_de_gestion );
  }

  /* Autres adaptations ventuelles en fonction du genre... */
  switch ( graphe_courant->genre_donnees )
  {
  case GRF_SERIE:
    if ( NULL == graphe_courant->x ) {
      /* On ne peut pas associer d'unit aux X dans ce cas ! */
      OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_X, FORCE_NON );
      OBJ_active( CHOIX_GRAPHE, LG_UNITE_X    , FORCE_NON );
    }
    break;

  case GRF_ERREUR_R:
  case GRF_ERREUR_C:
    AVERTISSEMENT( "Ce genre ne devrait jamais tre accessible  l'utilisateur [%u] !", graphe_courant->genre_donnees );
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, FORCE_NON );
    break;
  }

  /* Options spcifiques en fonction du type de donnes */
  table_options = FCT_ListeFonctions_GRF( FONCTION_EST_PAR_TYPE, graphe_courant->genre_donnees, graphe_courant->type, &nombre_options );
  if ( ( 0 == nombre_options ) || ( NULL == table_options ) ) {
    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_TYPE, FORCE_NON );
  } else {
    FonctionDeSousMenu fonction_de_gestion;

    OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_TYPE, FORCE_OUI );
    fonction_de_gestion = LGRF_QuelleFonction_ParType( graphe_courant->genre_donnees, graphe_courant->type );
    OBJ_MettreMenu( CHOIX_GRAPHE, LG_OPT_PAR_TYPE, nombre_options, table_options, titre_options, fonction_de_gestion );
  }

  SORTIE_FONCTION( __FUNCTION__, "RAS", 0 );
}

/* -------- Dsactiver les options associes  l'axe des X -------- */

static void LGRF_CouperAxeX(boolean couper)
{
  OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_X  , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_UNITE_X    , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_X, !couper );
  OBJ_active( CHOIX_GRAPHE, LG_AFF_ERR_X  , !couper );

  if ( VRAI == couper ) {
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_LEGENDE_X, "", 0 );
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_UNITE_X  , "", 0 );
  }
}

/* -------- Tout dsactiver, en l'absence de graphe -------- */

static void LGRF_ToutCouper(boolean couper)
{
  /* En l'absence de graphe courant : rien n'est utilisable pour les axes... */
  LGRF_CouperAxeX( couper );

  OBJ_active( CHOIX_GRAPHE, LG_LEGENDE_Y  , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_UNITE_Y    , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_CONVERTIR_Y, !couper );
  OBJ_active( CHOIX_GRAPHE, LG_AFF_ERR_Y  , !couper );

  /* Pas d'accs aux diverses options  */
  OBJ_active( CHOIX_GRAPHE, LG_AFFICHAGE    , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_APPLIQUER    , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_COULEUR_GRF  , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_STYLE_LIGNE  , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_STYLE_MARQUE , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_OPT_GENERIQUE, !couper );
  OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, !couper );
  OBJ_active( CHOIX_GRAPHE, LG_OPT_PAR_TYPE , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_NOM          , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_TYPE_GRAPHE  , !couper );
  OBJ_active( CHOIX_GRAPHE, LG_GENRE_GRAPHE , !couper );

  /* On supprime tout texte... */
  if ( VRAI == couper ) {
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_LEGENDE_Y, "", 0 );
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_UNITE_Y  , "", 0 );
    OBJ_CopierTexte( CHOIX_GRAPHE, LG_NOM      , "", 0 );

    OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_TYPE_GRAPHE , "", VRAI );
    OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_GENRE_GRAPHE, "", VRAI );
  }
}

/* ------ Liste des graphes : prparation de l'arbre -----------
   Pas static, car appel en tout dbut de programme pour initialiser les sous-menus
 */

void LGRF_PreparerArbre(void)
{
  const char **table_options = NULL;
  unsigned long nombre_options = 0;

  /* D'abord l'objet utilisateur, pour tre sr qu'il soit considr  l'initialisation */
  OBJ_ChangerType( CHOIX_GRAPHE, LG_STYLE_LIGNE , G_STYLELIGNE );
  OBJ_ChangerType( CHOIX_GRAPHE, LG_STYLE_MARQUE, G_STYLEMARQUE );

  /* Le slecteur de srie reste un bouton : facilite la gestion !
     En effet, le sous-menu appel doit tre gnr au vol pour rester  jour avec la table de sries...
  */

  /* Les slecteurs d'options peuvent tre convertis en sous-menus */
  OBJ_ChangerType( CHOIX_GRAPHE, LG_NV_SERIE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_GRAPHE, LG_NV_SERIE, LGRF_NOMBRE_OPT_CREATION, options_creation, titre_options, LGRF_GererMenuOpt_Creation );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_NV_SERIE, titre_options_creation, FAUX );

  OBJ_ChangerType( CHOIX_GRAPHE, LG_OPTIONS_SERIE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_GRAPHE, LG_OPTIONS_SERIE, LGRF_NOMBRE_OPT_SERIE_GRF, options_serie_graphes, titre_options, LGRF_GererMenuOpt_Serie );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_OPTIONS_SERIE, titre_options, FAUX );

  table_options = FCT_ListeFonctions_GRF( FONCTION_EST_GLOBALE, 0, 0, &nombre_options );
  OBJ_ChangerType( CHOIX_GRAPHE, LG_OPT_GENERIQUE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_GRAPHE, LG_OPT_GENERIQUE, nombre_options, table_options, titre_options, LGRF_GererMenuOpt_Generique );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_OPT_GENERIQUE, texte_opt_generique, FAUX );

  OBJ_ChangerType( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, G_SSMENU );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, texte_opt_par_genre, FAUX );
  /* Le menu, et la fonction de gestion, dpendent du genre de donnes stockes */

  OBJ_ChangerType( CHOIX_GRAPHE, LG_OPT_PAR_TYPE, G_SSMENU );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_OPT_PAR_TYPE, texte_opt_par_type, FAUX );
  /* Le menu, et la fonction de gestion, dpendent du genre de donnes stockes */

  /* Le slecteur de mode d'affichage aussi */
  OBJ_ChangerType( CHOIX_GRAPHE, LG_AFFICHAGE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_GRAPHE, LG_AFFICHAGE, LGRF_NOMBRE_OPT_AFFICHAGE, options_choix_affichage, titre_choix_affichage, LGRF_GererMenuAffichage );
  OBJ_RemplacerTexte( CHOIX_GRAPHE, LG_AFFICHAGE, texte_choisir_affichage, FAUX );

  /* Ensuite, on initialise */
  if ( FAUX == arbres[ CHOIX_GRAPHE ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( CHOIX_GRAPHE, "menu_graphes.html#liste", LGRF_GererArbre, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR  l'initialisation de l'arbre [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }
  }

  ARB_EnjoliverDefileur( CHOIX_GRAPHE, LG_FOND_LISTE, LG_PREMIER_GRAF, LG_DERNIER_GRAF,
			 LG_AVANT, LG_FOND_ASC, LG_BOUTON_ASC, LG_APRES, ARB_ASCENSEUR_DROITE );

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_AXE_X, 0, +500, DEPLACE_RELATIF );

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_UNITE_X, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace( CHOIX_GRAPHE, LG_UNITE_X    , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace( CHOIX_GRAPHE, LG_CONVERTIR_X, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_AXE_Y, 0, +500, DEPLACE_RELATIF );

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_UNITE_Y, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace( CHOIX_GRAPHE, LG_UNITE_Y    , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace( CHOIX_GRAPHE, LG_CONVERTIR_Y, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_COULEUR   , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_COULEUR_GRF   , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_STYL_MARK, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_STYLE_MARQUE  , 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_TYPE    , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_TYPE_GRAPHE , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_TXT_GENRE   , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_GRAPHE, LG_GENRE_GRAPHE, 0, +3, DEPLACE_PIXELS );

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_SERIE    , txt_lst_grf_serie );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_NV_SERIE     , txt_lst_grf_nouvelle );

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_AXE_X    , txt_lst_grf_axe_X );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_LEGENDE_X, txt_lst_grf_legende );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_UNITE_X  , txt_lst_grf_unite );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_CONVERTIR_X  , txt_lst_grf_convertir );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_AFF_ERR_X, txt_lst_grf_afficher_les_incertitudes );

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_AXE_Y    , txt_lst_grf_axe_Y );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_LEGENDE_Y, txt_lst_grf_legende );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_UNITE_Y  , txt_lst_grf_unite );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_CONVERTIR_Y  , txt_lst_grf_convertir );
  /* OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_AFF_ERR_Y, txt_lst_grf_afficher_les_incertitudes ); */

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_VOIR, txt_lst_grf_voir );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_APPLIQUER, txt_lst_grf_appliquer );

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_COULEUR, txt_lst_grf_couleur );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_STYLE_LIG, txt_lst_grf_style_ligne );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_STYL_MARK, txt_lst_grf_style_marque );

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_OPTIONS, txt_lst_grf_options );

  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_NOM, txt_lst_grf_nom );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_TYPE, txt_lst_grf_type );
  OBJ_TraduireTexte( CHOIX_GRAPHE, LG_TXT_GENRE, txt_lst_grf_genre );
#endif

  arbres[ CHOIX_GRAPHE ].drapeaux.prepare = VRAI;
  GRF_PreparerListe(FAUX); /* Premire init, brutale... */
}
