/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Gnralits du programme LASE (interface graphique)

   Gestion et prparation du menu des modles structuraux
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h"
#include "fichiers.h"
#include "graphes.h"
#include "interface.h"

#include "menu_structure.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int CLV_men_structure(unsigned int touche, int etat, int *renvoi);
int MEN_PreparerStructure( void );

/* -------- Prparation du menu -------- */

int MEN_PreparerStructure( void )
{
  arbres[MEN_STRUCTURE].clavier = CLV_men_structure;

# ifndef _DEVELOPPE_ /* Version de dveloppement uniquement */
  OBJ_active(MEN_STRUCTURE, MST_TIT_MOUV   , FORCE_NON);
  OBJ_active(MEN_STRUCTURE, MST_V_BANQUE   , FORCE_NON);
  OBJ_active(MEN_STRUCTURE, MST_FONDRE_MOL, FORCE_NON);
# endif

  return 0;
}

int MEN_structure(unsigned int option)
{
  Graphe *retour;

  switch (option)
  {
  case MST_INFOS:
    ARB_AppelerInformations();
    return BI_EVENEMENT_TRAITE;

  case MST_V_EXTRAIRE:
    GER_infos(TRAITE_SPECTRES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MST_V_AJUSTE:
    GER_infos(EXF_AJUSTEMENTS, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MST_V_THEORIE:
    GER_infos(CALC_THEORIQUES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MST_V_BANQUE:
    GER_infos(INFOS_BANQUE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MST_V_FLUOX:
    GER_infos(INFOS_FLUOX, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MST_CHARGER: /* Marche pour tout, normalement */
    retour = charger(NULL);
    if (retour != NULL) GRF_aller(retour);
    return BI_EVENEMENT_TRAITE;

  case MST_IMPORT_GRF: /* Importer un graphe */
    GRF_ImportASCII();
    return BI_EVENEMENT_TRAITE;

  case MST_IMPORT_3D: /* Importer une structure 3D */
    MOL_ImportASCII();
    return BI_EVENEMENT_TRAITE;

  case MST_QUITTER: /* Quitter le programme */
    return MEN_extrait(QUITTER);

  case MST_VOIR_3D: /* Appel de la vue 3d */
    I_MOL_Visualisation3D();
    return BI_EVENEMENT_TRAITE;

  case MST_EXPORT_POV:
    export_pov(mol_courante, mode_vue3d);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_TRAITE; /* Trait par dfaut */
}

/* -------- Raccourcis claviers -------- */

static int CLV_men_structure(unsigned int touche, int etat, int *renvoi)
{
  *renvoi = 0; /* On ne quitte pas un menu de toute faon... */

  if (etat & K_CTRL) /* On appuyait sur Control */
  {
    if ( touche <= 255 )
      touche = (unsigned int) toupper( (int) touche); /* Comportement indfini pour autres codes ! */
    switch ( touche )
    {
    case 'A':   /* ^A = appeler le menu d'Ajustements            */
      GER_infos(EXF_AJUSTEMENTS, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'C':   /* ^C = Charger - marche aussi pour les structures 3D, maintenant... */
      return MEN_extrait(CHARGER);

    case 'E':   /* ^E = appeler le menu d'Extraction d'un graphe */
      GER_infos(TRAITE_SPECTRES, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'L':   /* ^L = appeler la liste des molcules */
      FEN_premierplan( fenetre_onglets );
      BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_MOLECULES );
      return BI_EVENEMENT_TRAITE;

    case 'Q':		/* ^Q = Quitter */
      return MEN_extrait(QUITTER);

    case 'T':   /* ^T = appeler le menu de calculs thoriques    */
      GER_infos(CALC_THEORIQUES, 0, 0);
      return BI_EVENEMENT_TRAITE;

#ifdef _DEVELOPPE_
    default:
      printf("On appuyait sur la lettre no : %d (%c)\n", touche, (char) touche);
      break;
#endif
    }
  }

  return BI_EVENEMENT_NON_TRAITE; /* Par dfaut, message non-trait */
}
