/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Gnralits du programme LASE (interface graphique)

   Gestion et prparation du menu des calculs & modles thoriques
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "absorption.h"
#include "experience.h" /* Pour l'interface des expriences */
#include "feff.h"       /* Pour les paramtres du modle FEFF */
#include "graphes.h"    /* Pour GRF_aller() */
#include "interface.h"
#include "statistiques.h" /* Pour les modles statistiques */
#include "theorie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define VERSION_FEFF 820	/* Format du fichier de sortie : version FEFF  utiliser */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int CLV_men_theorie(unsigned int touche, int etat, int *renvoi);
int MEN_PreparerTheorie(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int MEN_PreparerTheorie( void )
{
  arbres[ MEN_THEORIE ].clavier = CLV_men_theorie;

# ifdef _TOS_ /* Pour que les accessoires soient accessibles */
  OBJ_active(MEN_THEORIE, MTH_V_THEORIE, FORCE_OUI);
# endif

# ifndef _DEVELOPPE_ /* Version de dveloppement uniquement */
  OBJ_active(MEN_THEORIE, TIT_EXTRAIT    , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_TIT_SEUIL, FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_TIT_GNXAS, FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_TIT_SYST , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_TIT_STAT , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_V_BANQUE   , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_EXF_V_MU0   , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_MU0_V_MTH   , FORCE_NON);
  OBJ_active(MEN_THEORIE, MEN_CALC_PHASE  , FORCE_NON);
  OBJ_active(MEN_THEORIE, MEN_CALC_AMP    , FORCE_NON);
  OBJ_active(MEN_THEORIE, MEN_LIBREPARCOUR, FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_CALC_RDF    , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_DEBYEWALLER , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_OPT_FEFF7   , FORCE_NON);
  OBJ_active(MEN_THEORIE, MENTH_MULTIFEFF , FORCE_NON);
  OBJ_active(MEN_THEORIE, MEN_XAN_GENERAL , FORCE_NON);
  OBJ_active(MEN_THEORIE, MEN_XAN_DEBOGUE , FORCE_NON);
  OBJ_active(MEN_THEORIE, MTH_LANCE_XANES , FORCE_NON);
# endif

  /* Initialisation russie */
  return 0;
}

int MEN_theorie(unsigned int option)
{
  Graphe *retour = NULL;

  switch (option)
  {
  case MENTH_INFOS:    /* Appeler les informations */
    ARB_AppelerInformations();
    return BI_EVENEMENT_TRAITE;

  case MTH_V_EXTRAIRE:
    GER_infos(TRAITE_SPECTRES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MTH_V_AJUSTE:
    GER_infos(EXF_AJUSTEMENTS, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MTH_V_STRUCTURE:
    GER_infos(STRUCTURE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MTH_V_BANQUE:
    GER_infos(INFOS_BANQUE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MTH_V_FLUOX:
    GER_infos(INFOS_FLUOX, 0, 0);
    return BI_EVENEMENT_TRAITE;

#ifdef _DEVELOPPE_
  case MENTH_MATRICE: /* Calculer le (matrice) */
    retour = TH_fond_continu(NULL);
    break;
#endif

  case MENTH_MU0:      /* Calculer un 0 thorique  */
    retour = CTH_mu0();
    break;

  case MTH_CALC_EXAFS: /* Calculer un signal  partir de ses composantes - Il va falloir le nommer */
    retour = TH_calc_exafs();
    break;

  case MTH_EXF_V_MU0: /* Calcule le mu(0) total pour un atome --- coefficient d'absorption atomique */
    retour = I_TH_XAS_ConstruireMu0();
    break;

  case MTH_MU0_V_MTH:  /* Recomposer un mu complet  partir de l'exafs, de mu(0) et de mu(m) */
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Calcul du coefficient d'absorption complet\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case MEN_CALC_PHASE: /* Calculer une phase thorique */
    retour = CTH_phase();
    break;

  case MTH_EXPERIENCE: /* Paramtrer une exprience */
    EXP_OuvrirFenetre();
    return BI_EVENEMENT_TRAITE;

  case FEFF_GENERAUX: /* Paramtres gnraux de FEFF */
    FEFF_ParametresGeneraux(NULL);
    return BI_EVENEMENT_TRAITE;

  case FEFF_POTENTIEL: /* Paramtres de calcul du potentiel de FEFF */
    FEFF_DefinirPotentiels(NULL);
    return BI_EVENEMENT_TRAITE;

  case FEFF_STRUCTURE: /* Paramtres structuraux pour FEFF */
    FEFF_DefinirStructure(NULL);
    return BI_EVENEMENT_TRAITE;

  case MENTH_VERIFIER: /* Vrification de la cohrence du modle FEFF */
    FEFF_VerifierModele(NULL);
    return BI_EVENEMENT_TRAITE;

  case FEFF_CHARGE:   /* On veut charger un fichier FEFF.INP */
    FEFF_charge();
    return BI_EVENEMENT_TRAITE;

  case FEFF_SAUVER:   /* On veut sauver le fichier FEFF.INP */
    I_FEFF_Sauver( NULL, OBJ_OptionMenu(  FEFF_PARAM_GENE, FPG_VERSION_FEFF, 0, QUESTION ) );
    return BI_EVENEMENT_TRAITE;

  case MEN_XAN_FEFF: /* Modification des paramtres FEFF modifis pour xanes */
    FEFF_ParametresXANES(NULL);
    return BI_EVENEMENT_TRAITE;

#ifdef _UNIX_ /* Version unix uniquement */
  case FEFF_LANCER: /* On lance un calcul avec Feff */
    lancer_feff();
    return BI_EVENEMENT_TRAITE;
#endif

  case MTH_DIST_CONTINU: /* Estimation d'une distribution */
    retour = modele_probas();
    break;
  }

  if (retour != NULL)
    GRF_aller(retour);
  else
    GRF_aller(graphe_courant);

  return BI_EVENEMENT_TRAITE;
}

/* ------------------- Raccourcis du menu de thorie ------------------ */

static int CLV_men_theorie(unsigned int touche, int etat, int *renvoi)
{
  *renvoi = 0; /* On ne quitte pas un menu de toute faon... */

  if (etat & K_CTRL) { /* On appuyait sur Control */
    if ( touche <= 255 )
      touche = (unsigned int) toupper( (int) touche); /* Comportement indfini pour autres codes ! */
    switch ( touche )
    {
    case 'A':   /* ^A = appeler le menu d'Ajustements            */
      GER_infos(EXF_AJUSTEMENTS, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'C':   /* ^C = Charger */
      return MEN_extrait(CHARGER);

    case 'E':   /* ^E = appeler le menu d'Extraction d'un graphe */
      GER_infos(TRAITE_SPECTRES, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'G':   /* ^G = appeler le menu de cration de structures */
      GER_infos(STRUCTURE, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'L':   /* ^L = appeler la liste des modles */
      FEN_premierplan( fenetre_onglets );
      BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_MODELES );
      return BI_EVENEMENT_TRAITE;

    case 'Q':		/* ^Q = Quitter */
      return MEN_extrait(QUITTER);

    case 'X':
      EXP_OuvrirFenetre();
      return BI_EVENEMENT_TRAITE;

#ifdef _DEVELOPPE_
    default:
      printf("On appuyait sur la lettre no : %d (%c)\n", touche, (char) touche);
      break;
#endif
    }
  }

  return BI_EVENEMENT_NON_TRAITE; /* Par dfaut, message non-trait */
}
