/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2005, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système

   Gestion fenêtre principale (qui s'occupe des graphes quasi-uniquement)
*/

#include <stdlib.h>
#include <stdio.h>

#include "systeme.h"
#include "exafs.h"

#include "couches.h"		/* Pour NC_AjusterSpectre */
#include "fichiers.h"
#include "filtrage.h"		/* Pour TF_reproduit */
#include "figures.h"		/* Inclus tracer.h : pour bloc_figure */
#include "fluorescence.h"	/* Pour FLUO_SimulerSpectre */
#include "interface.h"
#include "serie.h"
#include "statistiques.h"
#include "theorie.h"		/* Pour TH_evolution_temporelle */
#include "xanes.h"		/* Pour norm_xanes() */

#include "int_principale.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static InfosRegion2D zone_dessin = {
  0, 0, 0, 0,		/* Dimensions seront calculées à partir des % ci-dessous à l'ouverture */
  8, 8, 90, 90,		/* On occupe 90% de l'espace disponible, le reste pour les légendes */
  CADRE2D_AVEC_CADRE | CADRE2D_ECRAN,
  { /* Axe des X */
    NULL, NULL,
    AXE_AVEC_BORNES | AXE_AVEC_GRADUATIONS | AXE_AVEC_LEGENDE | AXE_AVEC_GRILLE | AXE_VISIBLE | AXE_ECRAN,
    GRADUATION_DECIMALE, 0, 0.0, 0.0, 0.0, ' ', 0, 10,
    NOIR, NOIR, VIOLET, GRIS_FONCE, 3
  },
  { /* Axe des Y */
    NULL, NULL,
    AXE_AVEC_BORNES | AXE_AVEC_GRADUATIONS | AXE_AVEC_LEGENDE | AXE_AVEC_GRILLE | AXE_VISIBLE | AXE_VERTICAL | AXE_ECRAN,
    GRADUATION_DECIMALE, 0, 0.0, 0.0, 0.0, ' ', 0, 10,
    NOIR, NOIR, VIOLET, GRIS_FONCE, 3
  },
  NULL, NULL,			/* Pas d'axe secondaire */
  NOIR, BLANC
};

#define MARGE_MINIMALE_GAUCHE  50 /* Marge pour la légende l'axe vertical --- en pixels */
#define MARGE_MINIMALE_BASSE   40 /* Marge pour la légende l'axe horizontal --- en pixels */
#define LARGEUR_COLONNE_STATS 150 /* Largeur d'une colonne pour les stats descriptives --- en pixels */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void I_RDS_Principale_BaM  (Fenetre *fenetre, int x_HG, int y_HG, unsigned int larg, unsigned int haut);
static void I_RDS_Principale_Henry(Fenetre *fenetre, int x_HG, int y_HG, unsigned int larg, unsigned int haut);
static void I_RDS_Principale_GRF  (Fenetre *fenetre, InfosRegion2D *zone_dessin);
static void I_RDS_Principale_Stat (Fenetre *fenetre, int x_HG, int y_HG, unsigned int larg, unsigned int haut);

static void RDS_principale(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);
static void ASC_principale(Fenetre *fenetre, int type, int ss_type, int position);
static int  CLC_principale(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y);
static int  CLV_principale(Fenetre *fenetre, unsigned int touche, int etat);
static int  MSG_principale(Fenetre *fenetre, Message *message);
static void RCT_principale(Fenetre *fenetre, int x, int y, int vx_x, int vx_y);

static void CLC_BoutonGauche(int n_clics, int etat, int x, int y);

static inline void P_CalculerCoordonnesZoneDessin(int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur);
static int         P_OptimiserBornes             (unsigned int, double b[6], void *);
InfosRegion2D     *P_QuelleRegion                (void); /* Utilisée par Export/PS.c pour récupérer zone_dessin ! */
static int         P_UtiliserZoneDessin          (void);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Ouverture de la fenêtre principale pour les graphes -------- */

int I_OuvrirPrincipale_GRF(void)
{
  /* Contrôle préliminaire */
  if ( principale != NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : Fenêtre principale pour les graphes déjà ouverte !\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return 0;			/* Ce n'est pas un vrai souci ! */
  }

  /* On crée la fenêtre : fenêtre utilisateur bête et méchante */
  principale = FEN_CreerFenetreSimple( (TOUS_SF_ASC & ~CLOSER) | ASC_VERTICAL | ASC_HORIZONTAL, /* Pas de fermeture */
				       "LASE : Logiciel d'analyse des spectres E.X.A.F.S.", txt_InfosInitiales,
				       RDS_principale, ASC_principale,
				       CLV_principale, CLC_principale, MSG_principale,
				       RCT_principale, (int (*)(Fenetre *)) NULL );
  if ( NULL == principale ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'ouvrir la fenêtre principale !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1; /* Erreur survenue... */
  }
  principale->drapeaux.redessin = VRAI; /* Message de redessin pour tout si chgmt de taille     */
  principale->msg_masque.plein_ecran  = VRAI;
  principale->msg_masque.taille       = VRAI;

  /* On l'ouvre avec une taille par défaut et les ascenseurs correctement positionnés */
  FEN_Ouvrir( principale ); /* Valeurs par défaut */

  /* Ouverture réussie */
  return 0;
}

/* -------- Fonction qui donne accès à zone_dessin à l'extérieur de ce fichier. ------- *
   ATTENTION, non déclarée dans les .h car devrait être peu utilisée !
 * ------------------------------------------------------------------------------------ */

InfosRegion2D *P_QuelleRegion(void)
{
  return &zone_dessin;
}

/* ~~~~~~~~~~~~~ Ascenseur de la fenêtre principale ~~~~~~~~~~~~~~~~~~~ */

static void ASC_principale(Fenetre *fenetre, int type, int ss_type, int position)
{
  char compteur = 0;
  Graphe *graphe;
  Serie_Graphes *serie;

  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec fenêtre = NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return; /* Ne devrait jamais arriver normalement... */
  }
  if ( NULL == serie_courante ) return; /* Rien en mémoire, a priori */

  graphe = serie_courante->premier_graphe;

  if ( WM_ARROWED == type ) { /* On a utilisé une des flèches de l'ascenseur */
    switch ( ss_type ) /* Action qui peut être envisagée */
    {
      /* ------ Ascenseur vertical : passage d'un graphe à l'autre */
    case WA_UPPAGE: /* On remonte de 10 graphes */
      if ( NULL == graphe_courant ) return; /* Série courante sans graphe */
      graphe = graphe_courant;
      while ( ( graphe->precedent != NULL ) && ( compteur < 10 ) ) {
	graphe = graphe->precedent;
	compteur++;
      }
      GRF_aller(graphe);
      break;
	
    case WA_DNPAGE: /* On redescend de 10 graphes */
      if ( NULL == graphe_courant ) return; /* Série courante sans graphe */
      graphe = graphe_courant;
      while ( ( graphe->suivant != NULL ) && ( compteur < 10 ) ) {
	graphe = graphe->suivant;
	compteur++;
      }
      GRF_aller(graphe);
      break;
	
    case WA_UPLINE: /* Graphe précédent */
      if ( NULL == graphe_courant ) return; /* Série courante sans graphe */
      GRF_aller( graphe_courant->precedent ); /* On va à ce graphe */
      break;
	
    case WA_DNLINE: /* Graphe suivant */
      if ( NULL == graphe_courant ) return; /* Série courante sans graphe */
      GRF_aller( graphe_courant->suivant ); /* On va à ce graphe */
      break;

      /* ------ Ascenseur horizontal : passage d'une série à l'autre */
    case WA_LFPAGE: /* 10 séries précédentes */
      serie = serie_courante;
      for ( compteur = 0 ; compteur < 10 ; compteur++ ) {
	if ( serie != premiere_serie ) {
	  Serie_Graphes *s;
	  s = premiere_serie;
	  while ( s->suivante != serie )
	    s = s->suivante;
	  serie = s;
	}
      }
      I_SERIE_GRF_Aller( serie );
      break;

    case WA_RTPAGE: /* 10 séries suivantes */
      serie = serie_courante;
      while ( ( serie->suivante != NULL ) && ( compteur < 10 ) ) {
	serie = serie->suivante;
	compteur++;
      }
      I_SERIE_GRF_Aller( serie );
      break;

    case WA_LFLINE: /* Série précédente */
      if ( serie_courante != premiere_serie ) {
	serie = premiere_serie;
	while ( serie->suivante != serie_courante )
	  serie = serie->suivante;
	I_SERIE_GRF_Aller( serie );
      }
      break;

    case WA_RTLINE: /* Série suivante */
      I_SERIE_GRF_Aller( serie_courante->suivante );
      break;
    }
  } else if ( WM_VSLID == type ) { /* Ascenseur vertical déplacé */
    unsigned long i;
    unsigned long num_graphe;

    if ( NULL == graphe ) return;

    num_graphe = (position * (serie_courante->nbr_graphes - 1)) / 1000;

    /* On va au graphe désiré */
    for ( i = 0 ; i < num_graphe ; i++ )
      graphe = graphe->suivant;

    if ( graphe != graphe_courant ) { /* Si c'est un autre graphe */
      GRF_aller( graphe );
    } else {
      position = (int) ( 1000.0 * (float) num_graphe ) / (float) (serie_courante->nbr_graphes - 1);
      FEN_ascenseur(fenetre, ASC_VERTICAL, position, -1);
    }
  } else if (type == WM_HSLID) { /* Ascenseur horizontal déplacé */
    unsigned long i, num_serie, nbr_series;

    for (nbr_series = 0, serie = premiere_serie ; serie != NULL ; serie = serie->suivante, nbr_series++);

    if ( NULL == premiere_serie ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas de première série, mais serie_courante != NULL !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }
    num_serie = (position * (nbr_series - 1)) / 1000;

    /* On va à la série souhaitée */
    for ( i = 0, serie = premiere_serie ; i < num_serie ; serie = serie->suivante, i++ );
    if ( serie != serie_courante ) {
      I_SERIE_GRF_Aller( serie );
    } else {
      if ( nbr_series > 1 ) {
	/* Plus d'une série : on place l'ascenseur au bon endroit ! */
	position = (int) (1000.0 * (float) num_serie) / (float) (nbr_series - 1);
	FEN_ascenseur( fenetre, ASC_HORIZONTAL, position, -1 );
      } else {
	/* Une seule série : l'ascenseur fait toute la largeur */
	FEN_ascenseur( fenetre, ASC_HORIZONTAL, 0, 1000 );
      }
    }
  }
}

/* -------- Mise à jour des ascenseurs (en taille et position) -------- */

void I_PrincipaleGRF_RefaireAscenseurs(boolean horizontal, boolean vertical)
{
  if ( VRAI == horizontal ) {
    /* Remise à jour de l'ascenseur horizontal : celui des séries */
    int position, taille;
    unsigned long numero_serie = 0, nombre_series;
    Serie_Graphes *s;

    /* On compte le nombre total de séries */
    for ( s = premiere_serie, nombre_series = 0 ; s != NULL ; s = s->suivante, nombre_series++ ) {
      if ( s == serie_courante ) {
	numero_serie = nombre_series; /* Numéro commence à 0 = première série */
      }
    }

    /* On en déduit la taille & la position de l'ascenseur horizontal */
    if ( nombre_series < 2 ) {
      taille = 1000;
      position = 0;
    } else {
      taille   = (int) (  1000.0 / (float) nombre_series );
      position = (int) ( (1000.0 * numero_serie) / (nombre_series - 1.0 ) );
    }

    /* On adapte l'ascenseur horizontal */
    FEN_ascenseur(principale, ASC_HORIZONTAL, position, taille);

    /* Si l'on a changé les séries, probablement l'ascenseur des graphes est faux aussi... */
    vertical = VRAI;
  }

  if ( VRAI == vertical ) {
    int position = 0, taille;

    /* Calcul de la position & de la taille de l'ascenseur vertical */
    if ( ( NULL == serie_courante ) || ( serie_courante->nbr_graphes < 2 ) ) {
      taille = 1000;
      position = 0;
    } else {
      taille = (int) ( 1000.0 / (float) (serie_courante->nbr_graphes) );
      OPTION_A_FAIRE( "Remise à jour de la position de l'ascenseur vertical [des graphes]" );
    }

    /* On adapte l'ascenseur */
    FEN_ascenseur( principale, ASC_VERTICAL, position, taille );
  }
}

/* ~~~~~~~~~~~~~ Gestion du clavier dans cette fenêtre ~~~~~~~~~~~~~~~~
   Les flèches sont gérées automatiquement, en simulation de l'ascenseur !

     +     = agrandissement de la vue
     -     = réduction de la vue
Ctrl-1 à 9 = choisir la couleur
Alt- 1 à 9 = choisir le style de tracé
     A     = ajuster le spectre
     b     = bruiter le spectre
     c     = condenser les erreurs
     E     = extraire l'EXAFS
Alt- E     = extraire l'EXAFS sans modéliser le préseuil
     F     = calcule la tf directe ou inverse
     I     = sauver une image à partir de l'écran (EXPÉRIMENTAL)
     m     = calcule la moyenne du spectre
     M     = appelle la soustraction de µ(matrice), sur un spectre brut
     N     = normaliser (brutal : max = 1, min = 0)
Alt- N     = normaliser (brut = on appelle norm_xanes ; exafs = on appelle norm_exafs)
     R     = reproduire l'extraction / le filtrage
     v     = spectre visible/invisible
     y     = barres d'erreur en Y visibles ou non
 */

static int CLV_principale(Fenetre *fenetre __attribute__((__unused__)), unsigned int touche, int etat)
{
  if ( etat & K_CTRL ) {
    /* Normalement, raccourci de menu --- sauf exception !
       On le traite donc en premier pour ne pas perdre le menu si aucune série n'est présente ! */
    if ( ( ( touche >= '1') && ( touche <= '9' ) ) ||
	 ( 'm' == touche ) || ( 127 == touche ) ) {
      /* Ctrl-touche avec un rôle particulier, cf. en-dessous */
    } else {
      return BI_EVENEMENT_NON_TRAITE;
    }
  }
  if ( touche > 255 ) {
    /* Les touches spéciales, touches de fonction,... ne sont pas gérées */
    return BI_EVENEMENT_NON_TRAITE;
  }

  if ( NULL == serie_courante ) return BI_EVENEMENT_TRAITE;

  /* Touches traitées en tout premier car concernent les séries, pas les graphes...
     Peut donc fonctionner même si graphe_courant = NULL (série active sans graphe, plusieurs séries) */

  if ( 127 == touche ) { /* Delete -> effacer le graphe, ou la série si elle est vide */
    if ( graphe_courant != NULL ) {
      if ( etat & K_CTRL ) { /* Ctrl-Del : on force l'effacement si graphe pas sauvé */
	I_GRF_Effacer( graphe_courant, FAUX, FAUX );
      } else {
	I_GRF_Effacer( graphe_courant, VRAI, FAUX );
      }
      GRF_aller( graphe_courant );
    } else if ( NULL == serie_courante->premier_graphe ) {
      /* Inutile de garder une série vide => pas de message de confirmation
	 Cependant, on n'efface pas une série dont on vient juste d'effacer le dernier graphe
       */
      SERIE_GRF_EffacerSerie( serie_courante, VRAI );
    }
    return BI_EVENEMENT_TRAITE;
  }

  if ( NULL == graphe_courant ) /* Pas de graphe -> rien à faire ! */
    return BI_EVENEMENT_NON_TRAITE;

  switch (touche) /* D'abord les codes ASCII */
  {
  case '+': /* Grossir  la partie visible du spectre : *2 */
    OPTION_A_FAIRE( "Zoom, *2" );
    return BI_EVENEMENT_TRAITE;
    
  case '-': /* Diminuer la partie visible du spectre : /2 */
    OPTION_A_FAIRE( "Zoom, /2" );
    return BI_EVENEMENT_TRAITE;

  case '1': case '2': case '3': case '4': case '5': /* Chiffres : + ctrl = couleur */
  case '6': case '7': case '8': case '9':           /*            + alt  = style   */
    if ( etat & K_CTRL ) {
      int nv_couleur;
      if ( etat & (K_LSHIFT | K_RSHIFT) )
	nv_couleur = touche - '1' + 10; /* shift = +10 */
      else
	nv_couleur = touche - '1' + 1;
      GRF_couleur( graphe_courant, nv_couleur, -1, -1 );
    } else if ( etat & K_ALT ) {
      int nv_style;
      if ( etat & (K_LSHIFT | K_RSHIFT) )
	nv_style = touche - '1' + 10; /* shift = +10 */
      else
	nv_style = touche - '1' + 1;
      GRF_couleur( graphe_courant, -1, nv_style, -1 );
    }
    return BI_EVENEMENT_TRAITE;

  case 'A': /* Ajuster le spectre */
    if (graphe_courant->drapeaux.exafs == VRAI) {
      NC_AjusterSpectre(graphe_courant);
    }
    return BI_EVENEMENT_TRAITE;

  case 'b': /* Bruiter le spectre */
    I_GRF_Bruiter( graphe_courant );
    return BI_EVENEMENT_TRAITE;

  case 'c':
    condenser_erreur(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case 'D': /* D : degradation sous faisceau... */
    GRF_aller(TH_evolution_temporelle());
    return BI_EVENEMENT_TRAITE;

  case 'E': /* E = Extraire (l'exafs) */
    if ( (graphe_courant->type == GR_MOINSMAT) ||
	 (graphe_courant->type == GR_BRUT) ) {
      if ( etat & K_ALT ) { /* On ne modélise pas le préseuil */
	Graphe *retour, *mu0;

	retour = I_XAS_OterMu0_Etape1( graphe_courant, I_XAS_CHAINE_EXAFS, &mu0 );
	retour = MU2_ouvre(retour, mu0, I_XAS_CHAINE_EXAFS, &mu0);
	retour = MU3_ouvre(retour, mu0, I_XAS_CHAINE_EXAFS);
	(void) TF_ouvre(retour, I_XAS_CHAINE_EXAFS);
	TT_exafs = VRAI;
      } else {
	(void) MEN_extrait(EXAFS_TOUT);
      }
    } else if ( graphe_courant->type == GR_EXAFS2 ) {
      Graphe *graphe = MU3_ouvre( graphe_courant, NULL, I_XAS_CHAINE_AUCUNE );
      (void) TF_ouvre( graphe, I_XAS_CHAINE_AUCUNE );
    }
    return BI_EVENEMENT_TRAITE;

#ifdef _DEVELOPPE_
  case 'f': /* Spectre de fluorescence d'un élément - pour déboguage seulement ! ; 1 keV -> 30 keV par pas de 2 eV avec excitation à 25 keV */
    {
      struct graphe *graphe;
      int Z;

      printf("Élément étudié : \n");
      scanf("%d", &Z);

      graphe = GRF_nv_GrilleXP( NULL, GR_FLUOX, GRF_FCT_REELLE, "Spectre de fluorescence complet", FAUX, 1000, 30000, 2 );
      FLUO_SimulerSpectre( graphe, 25000, Z, NULL, 1.0 );
      GRF_bornes( graphe, NE_PAS_FAIRE, DESORDONNE );
      GRF_aller( graphe );
    }
    return BI_EVENEMENT_TRAITE;
#endif

  case 'F': /* F = transformée de Fourier */
    if ( graphe_courant->drapeaux.exafs == VRAI ) {
      (void) TF_ouvre(graphe_courant, I_XAS_CHAINE_AUCUNE );
    } else if ( graphe_courant->type == GR_TF ) {
      TFI_ouvre(graphe_courant);
    }
    return BI_EVENEMENT_TRAITE;

#if defined(_DEVELOPPE_) && defined(AVEC_PNG_)
  case 'I':
    sv_image(principale);
    return BI_EVENEMENT_TRAITE;
#endif

  case 'l':			/* l : limites du graphe */
    I_GRF_AfficherLimites( graphe_courant );
    return BI_EVENEMENT_TRAITE;

  case 'm': /* m = moyenne (et autres paramètres statistiques)
	       ^M = ajuster une droite au graphe courant */
    if ( etat & K_CTRL ) {
      GRF_aller( I_GRF_AjusterDroite( graphe_courant ) );
    } else {
      if ( graphe_courant->genre_donnees == GRF_SERIE ) { /* Pas de sens sinon */
	calc_param_stat(graphe_courant);
      }
    }
    return BI_EVENEMENT_TRAITE;

  case 'M': /* M = soustraire le fond continu (la matrice) - sans chaînage */
    if ( (graphe_courant->type == GR_BRUT) || (graphe_courant->type == GR_SSTYPE) ) {
      GRF_aller( I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_AUCUNE ) );
    }
    return BI_EVENEMENT_TRAITE;

  case 'N': /* N = normaliser */
    if ( etat & K_ALT ) {
      if ( VRAI == graphe_courant->drapeaux.exafs )
	I_XAS_NormaliserEXAFS( graphe_courant );
      else if ( ( GR_BRUT     == graphe_courant->type ) ||
		( GR_MOINSMAT == graphe_courant->type ) )
	norm_xanes(graphe_courant);
      else {
	I_GRF_Normaliser( graphe_courant );
      }
    } else {
      I_GRF_Normaliser( graphe_courant );
    }
    GRF_aller( graphe_courant );
    return BI_EVENEMENT_TRAITE;

  case 'p': /* P = pharmacocinétique */
    if ( GR_CINETIQUE == graphe_courant->type ) {
      (void) I_PHARMACO_ParametresBasiques( graphe_courant );
    } else {
      AVERTISSEMENT( "Graphe de type %d, il faudrait %d", graphe_courant->type, GR_CINETIQUE );
    }

  case 'R': /* Reproduire l'extraction, si ça a un sens... */
    if ( (graphe_courant->drapeaux.exafs == VRAI) ||
	 (graphe_courant->type == GR_TF) )
      TF_reproduit(graphe_courant);
    else if ( ( graphe_courant->type == GR_BRUT ) ||
	      ( graphe_courant->type == GR_MOINSMAT ) )
      EXF_reproduit(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case 'v': /* Bascul : spectre visible/invisible */
    graphe_courant->drapeaux.superpose = !(graphe_courant->drapeaux.superpose);
    FEN_redessin( trace_fenetre, BI_RDS_UTIL );
    GRF_PreparerListe( FAUX ); /* On réaffiche la liste des graphes */
    return BI_EVENEMENT_TRAITE;

  case 'x': /* Bascule : barres d'erreur en X visibles / invisibles */
    if ( VRAI == graphe_courant->drapeaux.erreur_x ) {
      GRF_AfficherErreurs( graphe_courant, AXE_X, FAUX );
    } else if ( graphe_courant->erreur_x != NULL ) {
      GRF_AfficherErreurs( graphe_courant, AXE_X, VRAI );
    }
    return BI_EVENEMENT_TRAITE;

  case 'y': /* Bascule : barres d'erreur en Y visibles / invisibles */
    if ( VRAI == graphe_courant->drapeaux.erreur_y ) {
      GRF_AfficherErreurs( graphe_courant, AXE_Y, FAUX );
    } else if ( graphe_courant->erreur_y != NULL ) {
      GRF_AfficherErreurs( graphe_courant, AXE_Y, VRAI );
    }
    return BI_EVENEMENT_TRAITE;

  case Fleche_Haut: /* Flèche vers le haut : passer au graphe précédent */
    if ( etat & ( K_LSHIFT | K_RSHIFT ) ) { /* On va au graphe précédent _de même type_ */
      Graphe *nv_graphe = graphe_courant->precedent;
      int type = graphe_courant->type;

      while (nv_graphe != NULL) { /* Tant qu'il y a un graphe avant */
	if (nv_graphe->type == type) {
	  GRF_aller(nv_graphe);
	  return BI_EVENEMENT_TRAITE;
	}
	nv_graphe = nv_graphe->precedent;
      }

      /* Si l'on est là, c'est qu'aucun graphe ne convient */
      (void) BI_ERR_alerte(principale, &ERR_AucunGrapheAvant, FAUX);
      return BI_EVENEMENT_TRAITE;
    }
    /* Cas simple : géré automatiquement ! */
    break;

  case Fleche_Bas: /* Flèche vers le bas : passer au graphe suivant */
    if ( etat & ( K_LSHIFT | K_RSHIFT ) ) { /* On va au graphe suivant _de même type_ */
      Graphe *nv_graphe = graphe_courant->suivant;
      int type = graphe_courant->type;

      while (nv_graphe != NULL) { /* Tant qu'il y a un graphe avant */
	if (nv_graphe->type == type) {
	  GRF_aller(nv_graphe);
	  return BI_EVENEMENT_TRAITE;
	}
	nv_graphe = nv_graphe->suivant;
      }

      /* Si l'on est là, c'est qu'aucun graphe ne convient */
      (void) BI_ERR_alerte(principale, &ERR_AucunGrapheApres, FAUX);
      return BI_EVENEMENT_TRAITE;
    }
    /* Cas simple : géré automatiquement ! */
    break;

#ifdef _DEVELOPPE_
  case 1: /* Ctrl-Esc : sortie rapide sans confirmation */
    return BI_EVENEMENT_FIN_PRG;
#endif
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Doit-on tenir compte de la zone de dessin ? --------
    Renvoie -1 : complètement inutile
             0 : indique juste les marges
             1 : utilisation complète (axes, déplacement,...)
 */

static int P_UtiliserZoneDessin(void)
{
  if ( NULL == graphe_courant ) return -1;

  /* Genres de données pour lesquels la zone n'a guère de sens */
  if ( GRF_COURBE_ESPACE == graphe_courant->genre_donnees ) return 0;
  if ( GRF_SURFACE == graphe_courant->genre_donnees ) return 0;

  /* Modes de représentations pour lesquels la zone est superflue */
  if ( GRF_SERIE == graphe_courant->genre_donnees ) {
    if ( graphe_courant->affichage != NULL ) {
      /* On a demandé explicitement une représentation qui n'utilise pas de Y */
      if ( GRF_S_BOXPLOT == graphe_courant->affichage->methode ) return 0;
      if ( GRF_S_TUKEY   == graphe_courant->affichage->methode ) return 0;
    }

    /* Essai : combiner les différents modes de représentation d'une série de valeurs unidimensionnelle */
    if ( NULL == graphe_courant->x ) {
      return 0;
    }
  }

  /* Par défaut, on l'utilise */
  return 1;
}

/* ~-~-~-~-~-~-~-~-~ Gestion des clics de la souris ~-~-~-~-~-~-~-~-~ */

/* -------- Routine principale : redistribue aux routines spécialisées -------- */

static int CLC_principale(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_souris, int y_souris)
{
  if ( graphe_courant == NULL ) { /* Pas de graphe */
    if ( 1 == bouton ) {
      int reponse;

      reponse = BI_ERR_alerte( fenetre, &Q_SerieVide, VRAI );
      switch ( reponse )
      {
      case 1:
	charger( NULL );
	break;

      case 2:
	GRF_aller( I_GRF_Generer( NULL ) );
	break;

      case 3:
	SERIE_GRF_EffacerSerie( serie_courante, VRAI );
	break;
      }
    }
    if ( 2 == bouton ) {
      /* Menu déroulant : création rapide de graphes */
      (void) OPT_CreationGraphe( );
    }
    return BI_EVENEMENT_TRAITE; /* Rien à faire */
  }

  /* Sur quel bouton a-t-on cliqué ? */
  switch ( bouton )
  {
  case 1: /* Bouton gauche seul */
    CLC_BoutonGauche( n_clic, etat, x_souris, y_souris );
    return BI_EVENEMENT_TRAITE;

  case 2: /* Bouton du milieu seul [s'il existe] */
    /* Bouton = -1 : on fait apparaître le menu à la position de la souris */
    if ( etat & ( K_LSHIFT | K_RSHIFT ) ) {
      ARB_SimulerClic( CHOIX_GRAPHE, LG_OPT_PAR_GENRE, -1 );

      switch ( graphe_courant->genre_donnees )
      {
      case GRF_SERIE:
 	OPT_GraphesSerie( graphe_courant );
 	break;
      }
    } else {
      ARB_SimulerClic( CHOIX_GRAPHE, LG_OPT_GENERIQUE, -1 );
    }
    return BI_EVENEMENT_TRAITE;

  case 4: /* Bouton droit seul */
    /* Consulter/Modifier les infos du graphe */
    if ( GRF_Informations( graphe_courant ) == VRAI ) {
      I_MettreAJourPrincipale( VRAI );
    }
    return BI_EVENEMENT_TRAITE;

  default:
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : Bouton inconnu ou plusieurs boutons simultanément [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, bouton);
#endif
    return BI_EVENEMENT_NON_TRAITE;
  }

  /* On ne devrait jamais être là ! */
  return BI_EVENEMENT_ERREUR;
}

/* -------- Gestion d'un clic du bouton gauche -------- */

static void CLC_BoutonGauche(int n_clics, int etat, int x_clic, int y_clic)
{
  int avec_zone_dessin;

  avec_zone_dessin = P_UtiliserZoneDessin();

  /* Cas général : on utilise complètement la zone de dessin */
  if ( 1 == avec_zone_dessin ) {
    boolean traite = FAUX;
    int retour;
    unsigned int lieu_clic;

    lieu_clic = TRC_PositionDansRegion2D( &zone_dessin, x_clic, y_clic, 5 );
#ifdef _DEVELOPPE_
/*     printf( "%s : %d clic(s) en X=%d, Y=%d, position = %u, etat = %d\n", __FUNCTION__, n_clics, x_clic, y_clic, lieu_clic, etat ); */
/*     printf( "     région : BG X=%d, Y=%d, L=%u, H=%u\n", zone_dessin.x_BG, zone_dessin.y_BG, zone_dessin.largeur, zone_dessin.hauteur ); */
#endif

    if ( ( 2 == n_clics ) || ( etat & K_CTRL ) ) { /* Double clic = Ctrl-clic */
      switch (lieu_clic)
      {
      case TRC_EST_AU_CENTRE: /* On modifie les bornes */
	I_ParametrerBornes( &(zone_dessin.axe_X), &(zone_dessin.axe_Y), NULL, P_OptimiserBornes, NULL, principale );
	break;

      case TRC_EST_BORD_BAS:  /* On modifie l'axe des X [la légende est modifiable depuis la liste des graphes] */
      case TRC_EST_BORD_HAUT:
	I_ParametrerLegendes( &(zone_dessin.axe_X), AXE_X, FAUX, principale );
	break;

      case TRC_EST_BORD_GAUCHE: /* On modifie l'axe des Y [la légende est modifiable depuis la liste des graphes] */
      case TRC_EST_BORD_DROIT:
	I_ParametrerLegendes( &(zone_dessin.axe_Y), AXE_Y, FAUX, principale );
	break;
      }

      FEN_redessin( principale, BI_RDS_UTIL );
      return;
    }

    /* Clic simple : on essaye de changer la taille de la région 2D
       Tous les bords sont actifs : si on est sur un bord, on fait croire que l'on est au milieu !
    */
    if ( TRC_EST_BORD_GAUCHE == lieu_clic ) lieu_clic = TRC_EST_MILIEU_BORD_GAUCHE;
    if ( TRC_EST_BORD_DROIT  == lieu_clic ) lieu_clic = TRC_EST_MILIEU_BORD_DROIT;
    if ( TRC_EST_BORD_BAS    == lieu_clic ) lieu_clic = TRC_EST_MILIEU_BORD_BAS;
    if ( TRC_EST_BORD_HAUT   == lieu_clic ) lieu_clic = TRC_EST_MILIEU_BORD_HAUT;
    retour = I_BLOC_ChangerTaille_Souris( principale, &zone_dessin,
					  0, principale->w_haut, principale->w_larg, principale->w_haut,
					  lieu_clic );
    if ( retour < 0 ) return;
    if ( 0 == retour ) {
      FEN_redessin( principale, BI_RDS_UTIL );
      traite = VRAI;
    }
 
    /* 2e essai : le déplacement du bloc, si shift est enfoncée */
    if ( ( FAUX == traite ) && ( etat & ( K_LSHIFT | K_RSHIFT ) ) && ( lieu_clic != TRC_EST_HORS_REGION ) ) {
      retour = I_BLOC_Deplacer_Souris( principale, &(zone_dessin),
				       principale->w_x, principale->w_y + principale->w_haut, principale->w_larg, principale->w_haut,
				       x_clic - zone_dessin.x_BG, zone_dessin.y_BG - y_clic );
      if ( retour < 0 ) return; /* Erreur ou annulation */
      if ( 0 == retour ) {
	FEN_redessin( principale, BI_RDS_UTIL );
	traite = VRAI;
      }
    }

    /* 3e essai : zooms dans le bloc, si le bloc s'y prête */
    if ( ( FAUX == traite ) && ( lieu_clic != TRC_EST_HORS_REGION ) ) {
      retour = I_BLOC_ZoomerSouris( principale, &zone_dessin, 0, x_clic, y_clic );
      if ( retour < 0 ) return; /* Erreur ou annulation */
      if ( 0 == retour ) {
	FEN_redessin( principale, BI_RDS_UTIL );
	traite = VRAI;
      }
    }

    return; /* événement traité */
  }

  OPTION_A_FAIRE( "Clic, affichage sans zone de dessin" );
  return;
}

/* ~-~-~-~-~-~-~-~-~ Gestion des messages de la fenêtre ~-~-~-~-~-~-~-~-~ */

static int MSG_principale(Fenetre *fenetre __attribute__((__unused__)), Message *message)
{
  if ( WM_SIZED == message->type ) {	/* Taille de la fenêtre changée */
    return BI_EVENEMENT_TRAITE;
  }

  /* Message inconnu */
  return BI_EVENEMENT_NON_TRAITE;
}

/* ~-~-~-~-~-~-~-~-~ Redessin dans cette fenêtre ~-~-~-~-~-~-~-~-~-~-~-~-~ */

static void RDS_principale(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  int avec_zone_dessin;

  /* Préparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut);
  TRC_Couleur( NOIR );

  if ( NULL == graphe_courant ) { /* Aucun graphe en mémoire */
    if ( NULL == serie_courante ) {
      TRC_Message( fenetre, txt_ERR_AucuneSerie );
    } else {
      TRC_Message( fenetre, txt_ERR_AucunGraphe );
    }
    return;
  }

  /* Utilise-t-on une zone de dessin ? */
  avec_zone_dessin = P_UtiliserZoneDessin();

  /* Cas général : on utilise complètement la zone de dessin */
  if ( 1 == avec_zone_dessin ) {
    /* On adapte donc en conséquence la zone de dessin */
    P_CalculerCoordonnesZoneDessin( 0, 0, fenetre->w_larg, fenetre->w_haut );

    I_RDS_Principale_GRF( fenetre, &zone_dessin );
    return;
  }

  /* Si on est là, c'est que l'on dessine autre chose : quoi ? */
  switch ( graphe_courant->genre_donnees )
  {
  case GRF_SERIE:
    if ( NULL == graphe_courant->affichage ) {
      (void) fprintf(stderr, "%s, %s l.%u : Pas d'information d'affichage !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }

    if ( ( GRF_S_BOXPLOT == graphe_courant->affichage->methode ) || ( GRF_S_TUKEY == graphe_courant->affichage->methode ) ) {
      I_RDS_Principale_BaM( fenetre, 0, 0, fenetre->w_larg, fenetre->w_haut );
      return;
    } else {
      int x_BaM_HG, y_BaM_HG, x_Hist_HG, y_Hist_HG, x_Valeurs_HG, y_Valeurs_HG;
      unsigned int larg_BaM, haut_BaM, larg_Hist, haut_Hist, larg_Valeurs, haut_Valeurs;

      /* On coupe la région d'affichage en 3 : la boîte à moustaches en bas ; en haut, à droite les valeurs, à gauche l'histogramme */
      /* 1. Boîte à moustaches : moitié inférieure, toute la largeur */
      x_BaM_HG = 0; y_BaM_HG = fenetre->w_haut / 2;
      larg_BaM = fenetre->w_larg;
      haut_BaM = fenetre->w_haut - y_BaM_HG + 1;
      if ( haut_BaM > 100 ) {
	unsigned int haut_Henry, larg_Henry, x_valeurs, y_valeurs;

	haut_Henry = haut_BaM - 100; larg_Henry = haut_Henry;
	haut_BaM = 100;
	x_valeurs = larg_Henry;
	y_valeurs = y_BaM_HG + haut_BaM;

	/* Tracé de la droite de Henry */
	I_RDS_Principale_Henry( fenetre, 0, y_valeurs, larg_Henry, haut_Henry );

	/* Affichage des valeurs de stat. descriptive */
	I_RDS_Principale_Stat( fenetre, x_valeurs, y_valeurs, fenetre->w_larg - larg_Henry, haut_Henry );

	/* Séparateurs complémentaires */
	TRC_Separateur( fenetre, HORIZONTAL, x_BaM_HG, y_BaM_HG + haut_BaM, larg_BaM );
	TRC_Separateur( fenetre, VERTICAL  , x_valeurs, y_valeurs, haut_Henry );
      }

      I_RDS_Principale_BaM( fenetre, x_BaM_HG, y_BaM_HG, larg_BaM, haut_BaM );

      /* 2. Valeurs individuelles : coin supérieur, gauche */
      x_Valeurs_HG = 0; y_Valeurs_HG = 0;
      larg_Valeurs = fenetre->w_larg / 2;
      haut_Valeurs = fenetre->w_haut / 2;

      /* on optimise la taille de la zone de dessin */
      P_CalculerCoordonnesZoneDessin( x_Valeurs_HG, y_Valeurs_HG, larg_Valeurs, haut_Valeurs );

      /* on trace */
      I_RDS_Principale_GRF( fenetre, &zone_dessin );

      /* 3. Histogramme (ou autre estimation de la densité) : coin supérieur, droit */
      x_Hist_HG = fenetre->w_larg / 2; y_Hist_HG = 0;
      larg_Hist = fenetre->w_larg - x_Hist_HG + 1;
      haut_Hist = fenetre->w_haut / 2;

      /* on optimise la taille de la zone de dessin */
      P_CalculerCoordonnesZoneDessin( x_Hist_HG, y_Hist_HG, larg_Hist, haut_Hist );

      /* 4. Les séparateurs */
      TRC_Separateur( fenetre, VERTICAL  , x_Hist_HG, y_Hist_HG, haut_Hist );
      TRC_Separateur( fenetre, HORIZONTAL, x_BaM_HG, y_BaM_HG, larg_BaM );
    }
    return;

  case GRF_COURBE_ESPACE:
    OPTION_A_FAIRE( "Tracé d'un graphe en 3D" );
#ifdef avec_3_d
    /* Christophe : ici le cahier des charges pour le dessin en 3d

       Je te propose de mettre la fonction correspondante [nom proposé : TRC_graphe3d]
       dans le fichier Traces/trois_d.c, histoire que tu puisse bosser tranquillement.
       Tu serais intégralement chargé du maintien (mise à jour, débogage,...).

       Prototype de la fonction :
       TRC_graphe3d(Graphe *g, int x,y,l,h, Infos_3d *i, double xm,xM,ym,yM,zm,zM);
       avec
       g = graphe à dessiner en 3d
       x,y,l,h = rectangle écran où dessiner le résultat de la projection
       i = les infos nécessaires à la projection (voir graphes.h pour le contenu)
       xm,xM,ym,yM,zm,zM = la région de l'espace que l'on veut représenter.

       Il se peut que les infos/3d soient incompatibles avec la zone choisie -> priorité
       à la zone choisie, quitte à rechercher les paramètres "optimaux" de vue 3d.

       Note : pour un début, tu peux supposer que g est forcément de type GR_TROIS_D,
       mais il se peut éventuellement que des graphes 2D ou complexes soient passés.
       Dans le cas de graphes 2D, tu peux poser Z = 0; dans le cas de graphes complexes,
       ce serait plutôt une "surface" (X, Re(Y), Im(Y)).

       Si tu as le temps, je te propose de faire aussi
       TRC_cadre3d(x,y,l,h,i,xm,xM,ym,yM,zm,zM);
       qui dessine justement les axes choisis, éventuellement gradués.
    */

    TRC_graphe3d(graphe_courant, fenetre->w_x, fenetre->w_y, fenetre->w_larg, fenetre->w_haut,
		 graphe_courant->pointeur.infos_3d, graphe_courant->x_min, graphe_courant->x_max,
		 graphe_courant->y_min, graphe_courant->y_max,
		 graphe_courant->pointeur.infos_3d->z_min, graphe_courant->pointeur.infos_3d->z_max);
#endif
    return;
  }

  /* Si on est là, c'est que l'on ne sait pas comment redessiner
     ==> gros pavé rouge pour alerter !
  */
  TRC_Couleur( ROUGE );
  TRC_RectanglePlein( fenetre, x, y, larg, haut );
  OPTION_A_FAIRE( "Redessin : graphe de type non-géré !" );
}

/* -------- Tracer la boîte à moustaches et son axe, horizontaux, dans une région indiquée -------- */

static void I_RDS_Principale_BaM(Fenetre *fenetre, int x_HG, int y_HG, unsigned int larg, unsigned int haut )
{
  InfosRegion2D zd_boxplot;
  int y_BG, y_axe, y_boxplot;

  /* Tracer une boîte à moustache */
  /* a. Préparation des infos de dessin de la boîte à moustache */
  if ( NULL == graphe_courant->affichage->complement.infos_bmoustache ) {
    int retour;

    retour = TRC_GRF_PreparerBoiteAMoustache( graphe_courant, 0.25, 0.75, 0.0, 1.0 );
    if ( retour != 0 ) {
      TRC_Z_Message( fenetre, x_HG, y_HG, larg, haut, "Erreur dans la préparation du dessin" );
      return;
    }
  }

  /* Calcul des positions */
  y_BG = y_HG + haut - 1;
  if ( haut > 35 ) {
    y_axe = y_BG - 35;
    y_boxplot = y_HG + (haut - 35) / 2;
  } else {
    y_axe = y_BG - 10;
    y_boxplot = y_HG + (haut - 10) / 2;
  }

  /* b. Préparation de l'axe gradué en-dessous de la boîte */
  zd_boxplot = zone_dessin;
  TRC_RelatifVAbsolu( &zd_boxplot, x_HG, y_BG - 40, larg, haut );

  zd_boxplot.axe_X = zone_dessin.axe_Y;
  zd_boxplot.axe_X.legende = (char *) GRF_QuelleLegende( graphe_courant, AXE_Y );
  zd_boxplot.axe_X.proprietes &= ~AXE_VERTICAL;
  if ( graphe_courant->affichage->complement.infos_bmoustache->elements & BM_AVEC_IC ) {
    /* On s'assure que l'IC rentre ! */
    double borne_IC, m, s;

    m = graphe_courant->affichage->complement.infos_bmoustache->moyenne;
    s = graphe_courant->affichage->complement.infos_bmoustache->ecart_type;

    borne_IC = m - 2.0 * s;
    if ( borne_IC < zd_boxplot.axe_X.min ) {
      zd_boxplot.axe_X.min = borne_IC;
    }

    borne_IC = m + 2.0 * s;
    if ( borne_IC > zd_boxplot.axe_X.max ) {
      zd_boxplot.axe_X.max = borne_IC;
    }
  }

  TRC_LigneH   ( fenetre, zd_boxplot.x_BG, y_axe, zd_boxplot.largeur );
  TRC_AxeGradue( fenetre, zd_boxplot.x_BG, y_axe, zd_boxplot.x_BG + zd_boxplot.largeur - 1, y_axe, &(zd_boxplot.axe_X), 10, 8 );

  /* c. Tracé de la boîte à moustache */
  (void) TRC_BoiteAMoustache( fenetre, &zd_boxplot, graphe_courant, graphe_courant->affichage->complement.infos_bmoustache, y_boxplot );
}

/* -------- Tracer un cadre [région du plan avec axes gradués] + le graphe dedans -------- */

static void I_RDS_Principale_GRF(Fenetre *fenetre, InfosRegion2D *zone_dessin)
{
  unsigned int dess_erreurs;

  /* On trace un cadre délimitant la zone de dessin (+ axes, légendes,...)
     Comme les légendes ne peuvent pas être modifiées ici, on peut sans risque utiliser celle du graphe */
  zone_dessin->axe_X.legende = (char *) GRF_QuelleLegende( graphe_courant, AXE_X );
  zone_dessin->axe_Y.legende = (char *) GRF_QuelleLegende( graphe_courant, AXE_Y );

  TRC_DessineRegion2D( fenetre, zone_dessin );

  /* Affichage des barres d'erreur ? */
  dess_erreurs = 0; /* Aucune par défaut */
  if ( VRAI == graphe_courant->drapeaux.erreur_x ) dess_erreurs |= AXE_X;
  if ( VRAI == graphe_courant->drapeaux.erreur_y ) dess_erreurs |= AXE_Y;

  /* On trace le graphe (de sa couleur) */
  TRC_GrapheDansRegion2D( fenetre, zone_dessin, 0, graphe_courant,
 			  -1, -1, -1, dess_erreurs, NULL, NULL );

  zone_dessin->axe_X.legende = NULL; /* Plus besoin des légendes, normalement ! */
  zone_dessin->axe_Y.legende = NULL;
}

/* -------- Afficher la droite de Henry -------- */

static void I_RDS_Principale_Henry(Fenetre *fenetre, int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  InfosRegion2D zd_henry;
  int x_texte, y_texte;
  unsigned int larg_texte __attribute__((__unused__)), haut_texte;
  int attributs_texte[ 10 ];
  struct boite_moustache *infos;

  /* Dimensions d'un caractère */
  vqt_attributes( _PRG_station->handle, attributs_texte );
  larg_texte = attributs_texte[ 6 ];
  haut_texte = attributs_texte[ 7 ] + 2; /* Interligne de 2 pixels entre les textes */

  /* Le titre */
  TRC_Couleur( ROUGE );
  x_texte = x_HG + largeur / 2;
  y_texte = y_HG + 2;
  TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte_droite_Henry );

  /* La boîte à moustache comporte déjà pas mal d'infos */
  infos = graphe_courant->affichage->complement.infos_bmoustache;
  if ( NULL == infos ) {
    int retour;

    retour = TRC_GRF_PreparerBoiteAMoustache( graphe_courant, 0.25, 0.75, 0.0, 1.0 );
    if ( retour != 0 ) {
      TRC_Z_Message( fenetre, x_HG, y_HG, largeur, hauteur, "Erreur dans la préparation du dessin" );
      return;
    }
    infos = graphe_courant->affichage->complement.infos_bmoustache;
  }

  /* On calcule la droite de Henry, si nécessaire */
  if ( NULL == infos->henry ) {
    infos->henry = STAT_DroiteDeHenry( graphe_courant, STAT_FDR_EMPIRIQUE_NP1 );
  }

  /* Calcul de la zone de dessin */
  P_CalculerCoordonnesZoneDessin( x_HG, y_HG + haut_texte + 2, largeur, hauteur - ( haut_texte + 2 ) );
  zd_henry = zone_dessin;
  GRF_OptimiserAffichage( infos->henry, &(zd_henry.axe_X.min), &(zd_henry.axe_X.max), &(zd_henry.axe_Y.min), &(zd_henry.axe_Y.max) );

  zd_henry.axe_X.legende = (char *) legende_fractiles_empiriques;
  zd_henry.axe_Y.legende = (char *) legende_fractiles_gaussiens;

  /* Tracé */
  TRC_DessineRegion2D   ( fenetre, &zd_henry );
  TRC_GrapheDansRegion2D( fenetre, &zd_henry, 0, infos->henry, NOIR, BI_AUCUNE_LIGNE, BI_MARQUE_CROIX, 0, NULL, NULL );

  zd_henry.axe_X.legende = NULL;
  zd_henry.axe_Y.legende = NULL;
}

/* -------- Afficher les statistiques descriptives du graphe choisi -------- */

static void I_RDS_Principale_Stat(Fenetre *fenetre, int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  char *texte;
  const char *unite;
  int x_texte, y_texte;
  unsigned int larg_texte __attribute__((__unused__)), haut_texte;
  int attributs_texte[ 10 ];
  struct boite_moustache *infos;

  if ( ( largeur < 10 ) || ( hauteur < 10 ) ) return;	/* Taille insuffisante... */

  /* La boîte à moustache comporte déjà pas mal d'infos */
  infos = graphe_courant->affichage->complement.infos_bmoustache;
  if ( NULL == infos ) {
    int retour;

    retour = TRC_GRF_PreparerBoiteAMoustache( graphe_courant, 0.25, 0.75, 0.0, 1.0 );
    if ( retour != 0 ) {
      TRC_Z_Message( fenetre, x_HG, y_HG, largeur, hauteur, "Erreur dans la préparation du dessin" );
      return;
    }
    infos = graphe_courant->affichage->complement.infos_bmoustache;
  }
  unite = GRF_QuelleUnite( graphe_courant, AXE_Y );

  /* Dimensions d'un caractère */
  vqt_attributes( _PRG_station->handle, attributs_texte );
  larg_texte = attributs_texte[ 6 ];
  haut_texte = attributs_texte[ 7 ] + 2; /* Interligne de 2 pixels entre les textes */

  /* Le titre */
  TRC_Couleur( ROUGE );
  x_texte = x_HG + largeur / 2;
  y_texte = y_HG + 2;
  TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte_stats_descriptives );

  /* --- Colonne 1 : paramètres de position --- */
  TRC_Couleur( BLEU );
  x_texte = x_HG + LARGEUR_COLONNE_STATS / 2;
  y_texte = y_HG + 2 + haut_texte + 2;
  TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte_stats_position );

  TRC_Couleur( NOIR );
  x_texte = x_HG + 2;
  y_texte += haut_texte + 2;

  /* 1re ligne : valeur moyenne */
  texte = TXT_genere_chaine( "m   = %*f", infos->moyenne, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 2e ligne : valeur médiane */
  texte = TXT_genere_chaine( "me  = %*f", infos->mediane, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* --- Colonne 2 : paramètres de dispersion --- */
  TRC_Couleur( BLEU );
  x_texte = x_HG + LARGEUR_COLONNE_STATS + LARGEUR_COLONNE_STATS / 2;
  if ( x_texte >= (int) (x_HG + largeur) ) return;
  y_texte = y_HG + 2 + haut_texte + 2;
  TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte_stats_dispersion );

  TRC_Style( BI_POINTS );
  TRC_LigneV( fenetre, x_HG + LARGEUR_COLONNE_STATS, y_texte, hauteur - ( haut_texte + 2 ) );

  TRC_Couleur( NOIR );
  x_texte = x_HG + LARGEUR_COLONNE_STATS + 2;
  y_texte += haut_texte + 2;

  /* 1re ligne : variance */
  if ( graphe_courant->nombre_point > 1 ) {
    texte = TXT_genere_chaine( "s*"DEUX_EXPOSANT" = %*f"DEUX_EXPOSANT, infos->ecart_type * infos->ecart_type, unite );
  } else {
    texte = TXT_genere_chaine( "s*"DEUX_EXPOSANT"  = %*f"DEUX_EXPOSANT, 0.0, unite );
  }
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 2e ligne : écart-type */
  if ( graphe_courant->nombre_point > 1 ) {
    texte = TXT_genere_chaine( "s*  = %*f", infos->ecart_type, unite );
  } else {
    texte = TXT_genere_chaine( "s*  = %*f", 0.0, GRF_QuelleUnite( graphe_courant, AXE_Y ) );
  }
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 3e ligne : étendue */
  texte = TXT_genere_chaine( DELTA_MAJUSCULE"   = %*f", graphe_courant->y_max - graphe_courant->y_min, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 4e ligne : intervalle inter-quartiles */
  texte = TXT_genere_chaine( "IQR = %*f", infos->q3 - infos->q1, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* --- Colonne 3 : autres paramètres, divers --- */
  TRC_Couleur( BLEU );
  x_texte = x_HG + 2 * LARGEUR_COLONNE_STATS + ( largeur - 2 * LARGEUR_COLONNE_STATS ) / 2;
  y_texte = y_HG + 2 + haut_texte + 2;
  if ( x_texte >= (int) (x_HG + largeur) ) return;
  TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte_stats_autres );

  TRC_LigneV( fenetre, x_HG + 2 * LARGEUR_COLONNE_STATS, y_texte, hauteur - ( haut_texte + 4 ) );

  TRC_Couleur( NOIR );
  x_texte = x_HG + 2 * LARGEUR_COLONNE_STATS + 2;
  y_texte += haut_texte + 2;

  /* 1re ligne : nombre de valeurs */
  texte = TXT_genere_chaine( "%u valeurs", graphe_courant->nombre_point );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 2e ligne : valeurs extrêmes */
  texte = TXT_genere_chaine( "min = %*f ; max = %*f", graphe_courant->y_min, unite, graphe_courant->y_max, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;

  /* 3e ligne : quartiles */
  texte = TXT_genere_chaine( "Q1  = %*f ; Q3  = %*f", infos->q1, unite, infos->q3, unite );
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur,
	       x_texte, y_texte, BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_FER_EN_HAUT, 0, texte );
  free( texte ); texte = NULL;
  y_texte += haut_texte + 2;
}

/* -------- Convertir en coordonnées écran absolues la zone de dessin -------- */

static inline void P_CalculerCoordonnesZoneDessin(int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  TRC_RelatifVAbsolu( &zone_dessin, x_HG, y_HG + hauteur - 1, largeur, hauteur );

  /* On vérifie qu'il y a la place pour les légendes, sinon si l'on peut on corrige */
  if ( ( zone_dessin.x_BG < MARGE_MINIMALE_GAUCHE ) && ( zone_dessin.largeur > MARGE_MINIMALE_GAUCHE ) ) {
    unsigned int delta_largeur;

    delta_largeur = MARGE_MINIMALE_GAUCHE - zone_dessin.x_BG;
    if ( delta_largeur < zone_dessin.largeur ) {
      /* Il faut être sûr qu'il reste au moins quelques pixels à dessiner ! */
      zone_dessin.largeur -= delta_largeur;
      zone_dessin.x_BG = MARGE_MINIMALE_GAUCHE;
    }
  }
  if ( ( ( hauteur - zone_dessin.y_BG ) < MARGE_MINIMALE_BASSE ) && ( hauteur > MARGE_MINIMALE_BASSE ) ) {
    unsigned int delta_hauteur;

    delta_hauteur = zone_dessin.y_BG - ( hauteur - MARGE_MINIMALE_BASSE );
    if ( delta_hauteur < zone_dessin.hauteur ) {
      /* Il faut être sûr qu'il reste au moins quelques pixels à dessiner ! */
      zone_dessin.hauteur -= delta_hauteur;
      zone_dessin.y_BG = hauteur - MARGE_MINIMALE_BASSE;
    }
  }

#ifdef _DEVELOPPE_
  if ( zone_dessin.y_BG < zone_dessin.hauteur ) {
    (void) fprintf(stderr, "%s, %s l.%u : Bloc HORS-FENÊTRE [X=%d(%g%%), Y=%d(%g%%), L=%u(%g%%), H=%u(%g%%)]\n",
		   __FILE__, __FUNCTION__, __LINE__, zone_dessin.x_BG, zone_dessin.x_BG_pc, zone_dessin.y_BG, zone_dessin.y_BG_pc,
		   zone_dessin.largeur, zone_dessin.larg_pc, zone_dessin.hauteur, zone_dessin.haut_pc);
/*     FIN_ANTICIPEE( FIN_ERREUR_MINEURE ); */
  }
  if ( ( zone_dessin.largeur < 2 ) || ( zone_dessin.hauteur < 2 ) ) {
    AVERTISSEMENT( "Bloc réduit à un pixel [L=%u, H=%u, zone d'origine : %u × %u", zone_dessin.largeur, zone_dessin.hauteur, largeur, hauteur );
  }
#endif
}

/* -------- Optimiser les bornes en fonction du graphe courant et du mode choisi -------- */

static int P_OptimiserBornes(unsigned int methode, double b[6], void *infos __attribute__((__unused__)))
{
  double inutile;

  switch (methode)
  {
  case BORNES_XY:		/* Rectangle optimal en X et en Y */
    (void) GRF_OptimiserAffichage( graphe_courant, b, b + 1, b + 2, b + 3 );
    return 0;

  case BORNES_X:		/* On optimise les X uniquement, sans toucher aux Y */
    (void) GRF_OptimiserAffichage( graphe_courant, b, b + 1, &inutile, &inutile );
    return 0;

  case BORNES_Y:		/* On optimise les Y uniquement, sans toucher aux X */
    (void) GRF_OptimiserAffichage( graphe_courant, &inutile, &inutile, b + 2, b + 3 );
    return 0;

  case BORNES_XpourY:		/* On optimise les X pour contenir les Y indiqués */
    (void) GRF_QuelsXPourY( graphe_courant, b[ 2 ], b[ 3 ], b, b + 1 );
    return 0;

  case BORNES_YpourX:		/* On optimise les Y pour contenir les X indiqués */
    (void) GRF_QuelsYPourX( graphe_courant, b[ 0 ], b[ 1 ], b + 2, b + 3 );
    return 0;
  }

  /* Si on est là : méthode d'optimisation des bornes inconnue */
  AVERTISSEMENT( "Méthode d'optimisation des bornes inconnue [%u]", methode );
  return +1;
}

/* -------- Mettre à jour la zone de dessin en fonction du graphe courant -------- */

void I_MettreAJourPrincipale(boolean remettre_a_lechelle)
{
  if ( VRAI == remettre_a_lechelle ) {
    GRF_OptimiserAffichage( graphe_courant, &(zone_dessin.axe_X.min), &(zone_dessin.axe_X.max),
			    &(zone_dessin.axe_Y.min), &(zone_dessin.axe_Y.max) );
  }

  /* Mais ceci est toujours fait : si l'on change les légendes ou les unités, on n'a pas à refaire le dessin en général ! */
  zone_dessin.axe_X.unite = (char *) GRF_QuelleUnite( graphe_courant, AXE_X );
  zone_dessin.axe_Y.unite = (char *) GRF_QuelleUnite( graphe_courant, AXE_Y );

  /* Et on demande le redessin */
  FEN_redessin( principale, BI_RDS_TOUT );
}

/* ~~~~~~~~~~~~~ Gestion du "réticule" sur le graphe ~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static void RCT_principale(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  int avec_zone_dessin;
  char *infos;

  if ( NULL == graphe_courant ) {
    return; /* Pas de graphe -> pas de pointeur */
  }

  /* Utilise-t-on une zone de dessin complète ? */
  avec_zone_dessin = P_UtiliserZoneDessin();

  /* Cas simple : aucune zone de dessin -> pas de pointeur */
  if ( -1 == avec_zone_dessin ) {
    return;
  }

  TRC_Mode( BI_MODE_INVERSE );
  TRC_Style( BI_CONTINU );
  TRC_Couleur( NOIR );

  /* Cas général : on utilise complètement la zone de dessin */
  if ( 1 == avec_zone_dessin ) {
    boolean etait_dedans = FAUX;
    int retour;
    int x_HD, y_HD;
    Bloc_Figure pseudo_bloc;	/* Juste pour profiter des fonctions de dessin correspondantes ! */

    /* Contrôle de la validité des bornes */
    retour = AXE_ValiderBornes( &(zone_dessin.axe_X) );
    if ( retour != 0 ) {
      return;
    }
    retour = AXE_ValiderBornes( &(zone_dessin.axe_Y) );
    if ( retour != 0 ) {
      return;
    }
    pseudo_bloc.dessin = zone_dessin;

    /* Coordonnées de la zone de dessin */
    x_HD = zone_dessin.x_BG + zone_dessin.largeur;
    y_HD = zone_dessin.y_BG - zone_dessin.hauteur;

    /* Effacement du réticule, si on était dedans */
    if ( ( vx_x >= (int) zone_dessin.x_BG ) && ( vx_x <= x_HD ) &&
	 ( vx_y <= (int) zone_dessin.y_BG ) && ( vx_y >= y_HD ) ) {
      BLOC_TracerReticule( fenetre, &pseudo_bloc, AXE_X | AXE_Y, vx_x, vx_y );
      etait_dedans = VRAI;	/* On était donc déjà dedans */
    }

    /* Est-on maintenant dedans ou dehors ? */
    if ( ( x >= (int) zone_dessin.x_BG ) && ( x <= x_HD ) &&
	 ( y <= (int) zone_dessin.y_BG ) && ( y >= y_HD ) ) {
      double x_reel, y_reel;

      /* On est dedans */
      BLOC_TracerReticule( fenetre, &pseudo_bloc, AXE_X | AXE_Y, x, y );

      /* Recherche des coordonnées */
      retour = TRC_Z2D_EcranVXY( &zone_dessin, 0, x, y, &x_reel, &y_reel );
      if ( retour != 0 ) {
	infos = TXT_genere_chaine( "X = ??, Y = ?? - %s", GRF_TexteTypeGraphe( graphe_courant->type ) );
      } else {
	infos = TXT_genere_chaine( "X = %*f, Y = %*f - %s",
				   x_reel, GRF_QuelleUnite( graphe_courant, AXE_X ),
				   y_reel, GRF_QuelleUnite( graphe_courant, AXE_Y ),
				   GRF_TexteTypeGraphe( graphe_courant->type ) );
      }

      FEN_infos(fenetre, infos);
      free( infos ); infos = NULL;
    } else {
      /* On est dehors */
      if ( VRAI == etait_dedans ) {
	/* On vient de sortir ! */
	FEN_infos( fenetre, GRF_TexteTypeGraphe( graphe_courant->type ) );
      }
    }

    return;
  }

  OPTION_A_FAIRE( "Réticule, autre type de graphe" );
}
