/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2003, Emmanuel CURIS

   Paramtrage de l'affichage du graphe [fentre principale]
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "atomes.h"		/* Pour MOL_aller() */
#include "images.h"		/* Pour IMG_AllerImage() */
#include "interface.h"
#include "serie.h"

#include "int_series_grf.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_sauver_serie_graphes = NULL;

/* Chemins par dfaut : dfinies dans variables.c */
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Aller  une srie -------- */

void I_SERIE_GRF_Aller(Serie_Graphes *serie)
{
  if ( NULL == serie ) return;

  serie_courante = serie;

  /* On adapte les ascenseurs */
  I_PrincipaleGRF_RefaireAscenseurs( VRAI, FAUX );

  /* On change en consquence le graphe courant */
  GRF_aller( serie_courante->premier_graphe );

  /* On affichage au besoin ce qui va autour... */
  IMG_AllerImage( serie_courante->carto );
  MOL_aller( serie_courante->modele );
}

/* -------- Crer une nouvelle srie, en demandant le nom  l'utilisateur -------- */

struct serie *I_SERIE_GRF_Creer(struct serie *avant, const char *nom_propose)
{
  char *titre;
  struct serie *nouvelle_serie;

  /* On demande le titre de la srie  l'utilisateur */
  if ( NULL == nom_propose ) {
    if ( NULL == serie_courante ) {
      nom_propose = txt_premiere_serie;
    } else {
      nom_propose = serie_courante->nom;
    }
  }
  titre = BI_Demande_Texte( principale, &Q_NomNouvelleSerie, nom_propose );

  /* On cre cette srie */
  nouvelle_serie = SERIE_GRF_CreerSerie( avant, titre );

  /* On contrle que la cration a fonctionn */
  if ( NULL == nouvelle_serie ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la srie %s\n",
		   __FILE__, __FUNCTION__, __LINE__, titre);
#endif
    (void) BI_ERR_alerte( principale, &ERR_CreationImpossible, VRAI, titre );
  }

  I_PrincipaleGRF_RefaireAscenseurs( VRAI, FAUX ); /* On refait l'ascenseur horizontal */

  /* On a termin : on nettoie & on renvoie la srie cre */
  free( titre ); titre = NULL;
  return nouvelle_serie;
}

/* -------- Sauver une srie, en demandant le nom du fichier  l'utilisateur -------- */

int I_SERIE_GRF_Sauver(struct serie *serie)
{
  int retour;
  char *nom_fichier;
  const char *masque = "*.lase";
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  if ( NULL == serie ) return -1;

  /* On demande le nom du fichier */
  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) txt_sauver_serie;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_serie_graphes;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, principale );
  /* On mmorise les chemins */
  chemin_sauver_serie_graphes = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_serie_graphes );

  if ( NULL == nom_fichier ) return +1;	 /* Annulation ou erreur */

  /* On fait la sauvegarde demande */
  retour = Serie_SauverIsolee( serie, nom_fichier, 0 );
  free( nom_fichier ); nom_fichier = NULL;
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en sauvant la srie [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -2;
  }

  /* On mentionne la sauvegarde dans l'historique, puisqu'elle a russi */
  HIST_Titre( 1, "Sauvegarde", "E/S" );
  HIST_Texte( 1, 0, "Fichier : %s\n", nom_fichier );
  HIST_Texte( 1, 0, "S"e_AIGU"rie sauv"e_AIGU" : %s\n", serie->nom );

  /* La sauvegarde s'est bien pass */
  return 0;
}

/* -------- Insrer un graphe dans la srie indique, ou une nouvelle -------- */

struct serie *I_GRF_InsererDansSerie(struct serie *serie_cible, Graphe *graphe_avant, Graphe *graphe_a_inserer)
{
  int retour;

  if ( NULL == graphe_a_inserer ) return NULL;

  /* Dtermination de la srie cible */
  if ( NULL == serie_cible ) {
    serie_cible = GRF_QuelleSerie( graphe_avant );
    if ( NULL == serie_cible ) {
      serie_cible = serie_courante;
    }
  }
  if ( NULL == serie_cible ) {
    /* C'est qu'aucune srie n'a t cre ! */
    /* On cre donc une nouvelle srie, la premire */
    serie_cible = I_SERIE_GRF_Creer( serie_cible, txt_nom_serie_resultats );
  }

  /* Peut-on ajouter des graphes dans la srie retenue ? */
  while ( ( NULL == serie_cible ) || ( SERIE_EstProtegee( serie_cible ) ) ) {
    int reponse;

    /* La srie ne peut recevoir de graphe supplmentaire : que faire ? */
    reponse = BI_ERR_alerte( principale, &Q_SerieProtegee, VRAI, ( serie_cible != NULL ) ? ( serie_cible->nom ) : ("NULL") );
    if ( 1 == reponse ) {
      /* On cre une nouvelle srie */
      serie_cible = I_SERIE_GRF_Creer( serie_cible, txt_nom_serie_resultats );
    } else {
      /* On slectionne une srie parmi celles existantes */
      serie_cible = I_SERIE_GRF_Choisir( -1, ROOT, 0 );
    }
  }

  /* Est-ce que le graphe indiqu est bien dans la srie retenue ?
     Peut tre non s'il est dans une srie protge ==> aucun message particulier ici */
  if ( graphe_avant != NULL ) {
    struct serie *serie_grf_avant = NULL;

    serie_grf_avant = GRF_QuelleSerie( graphe_avant );
    if ( serie_grf_avant != serie_cible )
      graphe_avant = NULL;
  }

  /* On insre le graphe dans la srie indique */
  retour = GRF_InsererDansSerie( graphe_a_inserer, serie_cible, graphe_avant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'insrer le graphe %s dans la srie %s [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe_a_inserer->nom, serie_cible->nom, retour);
    return NULL;
  }

  /* On a russi : tout s'est bien pass
     On renvoie la srie [ventuellement cre] qui contient le graphe */
  return serie_cible;
}

/* -------- Choisir une srie parmi la liste des sries disponibles -------- */

struct serie *I_SERIE_GRF_Choisir(int numero_arbre, unsigned int numero_objet, unsigned long lgmax_texte)
{
  unsigned long i, nombre_series;
  int choix;
  char **textes = NULL;
  Serie_Graphes *serie;

  if ( NULL == premiere_serie ) {
    /* Aucune srie ralise : rien  choisir ! */
    if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
      OBJ_CopierTexte( numero_arbre, numero_objet, "Aucune s"e_AIGU"rie en m"e_AIGU"moire", lgmax_texte );
    }
    return NULL;
  }
  if ( NULL == premiere_serie->suivante ) {
    /* Une seule srie prsente : pas rellement de choix... */
    if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
      OBJ_CopierTexte( numero_arbre, numero_objet, premiere_serie->nom, lgmax_texte );
    }
    return premiere_serie; /* Une seule srie... */
  }

  /* On compte le nombre de sries en mmoire */
  for ( nombre_series = 0, serie = premiere_serie ; serie != NULL ; serie = serie->suivante, nombre_series++ );

  /* On prpare la table des noms des sries */
  textes = (char **) malloc( sizeof(char *) * nombre_series );
  if ( NULL == textes ) {
    ERR_Memoire( "Table des noms de sries", sizeof(char *) * nombre_series );
    return NULL;
  }
  for ( i = 0, serie = premiere_serie ; i < nombre_series ; i++, serie = serie->suivante ) {
    textes[ i ] = serie->nom;
  }

  choix = SS_menu_texte( (const char **) textes, nombre_series, "Liste des s"e_AIGU"ries de graphes", numero_arbre, numero_objet );
  free( textes ); textes = NULL;
  if ( choix < 0 ) return NULL;

  for ( i = 0, serie = premiere_serie ; i < (unsigned long) choix ; i++, serie = serie->suivante );
  if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
    OBJ_CopierTexte( numero_arbre, numero_objet, serie->nom, lgmax_texte );
  }

  return serie;
}

/* -------- Effacer une srie -------- */

void I_SERIE_GRF_EffacerSerie(Serie_Graphes *serie_a_effacer, boolean demander)
{
  int reponse;

  /* Tests prliminaires */
  if ( NULL == serie_a_effacer ) {
    /* On essaye d'effacer la srie courante... */
    serie_a_effacer = serie_courante;
  }
  if ( NULL == serie_a_effacer ) {
    /* Il n'existe donc aucune srie de graphes  effacer ! */
    return;
  }
  if ( VRAI == serie_a_effacer->drapeaux.protegee ) {
    /* Srie protge : effacement direct impossible... */
    (void) BI_ERR_alerte( principale, &ERR_SerieProtegee, VRAI, serie_a_effacer->nom );
    return;
  }

  /* S'il n'y a aucun graphe : on efface, sans demander */
  if ( NULL == serie_a_effacer->premier_graphe ) {
    (void) SERIE_GRF_EffacerSerie( serie_a_effacer, VRAI );
    return;
  }

  /* On demande si l'on est sr de l'effacement ? */
  if ( VRAI == demander ) {
    /* 1. On regarde s'il existe des graphes non-sauvs */
    Graphe *graphe;
    boolean non_sauve = FAUX;

    for ( graphe = serie_a_effacer->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( FAUX == graphe->drapeaux.sauve ) {
	non_sauve = VRAI;
	break;
      }
    }
    if ( VRAI == non_sauve ) {
      /* Certains graphes n'ont pas t sauvs : qu'en faire ?*/
      reponse = BI_ERR_alerte( principale, &Q_EffacerSerie_GraphesNonSauves, VRAI, serie_a_effacer->nom );
      if ( Q_EFFSERIE_ANNULER == reponse ) {
	/* Effacement annul */
	return;
      } else if ( Q_EFFSERIE_TOUTSAUVER == reponse ) {
	int code_retour;

	/* On sauve tout : on demande donc le nom du fichier ("sauver la srie") */
	code_retour = I_SERIE_GRF_Sauver( serie_a_effacer );
	if ( code_retour != 0 ) {
	  /* Problme dans la sauvegarde : annule ou erreur ==> effacement annul */
	  return;
	}
      } else if ( Q_EFFSERIE_SAUVERNONSAUVES == reponse ) {
	/* On sauve les graphes non-sauvs : un graphe par fichier */
	char *radical = NULL;

	/* 1. Demander le radical des fichiers */
	radical = BI_Demande_Texte( principale, &Q_RadicalFichier, "sv_grf_" );

	/* 2. Lancer la sauvegarde */
	SERIE_SauverGraphes( serie_a_effacer, radical, FAUX );
	free( radical ); radical = NULL;

	/* L, tous les graphes sont normalement sauvs */
      } else if ( Q_EFFSERIE_TOUTEFFACER == reponse ) {
	/* On marque artificiellement tous les graphes comme sauvs... */
	for ( graphe = serie_a_effacer->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
	  graphe->drapeaux.sauve = VRAI;
	}
      } else if ( Q_EFFSERIE_DEMANDER == reponse ) {
	/* Sera gr automatiquement en effaant les graphes un par un... */
      } else {
	AVERTISSEMENT( "Rponse inconnue : %d", reponse );
	return;
      }
    }

    /* 2. On demande si l'on veut effacer le modle ou l'image qui vont avec */

    /* Si on est l : effacement confirm (inutile de redemander) et graphes sauvs ou  effacer */
    demander = FAUX;
  }
  if ( VRAI == demander ) {
    reponse = BI_ERR_alerte( principale, &Q_EffacerSerie, VRAI, serie_a_effacer->nom );
    if ( Q_EFFSERIE_ANNULEREFFACEMENT == reponse ) {
      /* Effacement annul */
      return;
    }
  }

  /* On efface graphe par graphe
     Normalement, tous sont dj marqus comme sauvs sauf si demand de rpondre graphe par graphe
      ==> on ne demande que pour les non-sauvs !
   */
  while ( serie_a_effacer->premier_graphe != NULL ) {
    (void) I_GRF_Effacer( serie_a_effacer->premier_graphe, FAUX, VRAI );
  }

  /* On efface la srie */
  SERIE_GRF_EffacerSerie( serie_a_effacer, FAUX );

  /* On remet  jour les ascenseurs de la fentre principale */
  I_PrincipaleGRF_RefaireAscenseurs( VRAI, FAUX );
}
