/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Cration et gestion de modles en vue d'un ajustement

   Cration de modles (partie gnrique)
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "exafs.h" /* Pour l'arbre des paramtres sur les modles */

#include "feff.h" /* Pour FEFF_EffacerModele() */
#include "graphes.h"
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation concrte ~~~~~ */

/* -------- Effacer un modle dans la liste indique ----------\
 |   Renvoie 0 si tout s'est bien pass, <0 en cas de problme |
 |                                                             |
 | premier : premier lment de la liste des modles           |
 | modele  : le modle  effacer dans cette liste              |
 | efface_graphes : VRAI ==> on efface aussi les graphes       |
 \------------------------------------------------------------ */

int MODELE_EffacerModele(ModeleAjustement **premier, ModeleAjustement *modele, boolean efface_graphes)
{
  if ( NULL == premier ) return -1;
  if ( NULL == modele )  return -2;

  /* 1. On sort le modle du chanage */
  if ( *premier == modele ) { /* On efface le premier de la liste */
    *premier = modele->suivant;
  } else {
    struct modele_ajuste *avant = *premier;

    while ( avant->suivant != modele ) {
      avant = avant->suivant;
      if ( NULL == avant ) {
	(void) fprintf(stderr, "MOD_effacer : le modle indiqu n'est pas dans la liste de modles indique !\n");
	return -3;
      }
    }

    avant->suivant = modele->suivant;
  }

  /* 2. On efface ventuellement les graphes */
  if ( VRAI == efface_graphes ) {
    /* Modle, rsidu mais JAMAIS l'exprimental */
    GRF_efface( modele->modele, VRAI, VRAI );
    GRF_efface( modele->residu, VRAI, VRAI );
  }
  if ( modele->reference != modele->experience ) {
    /* Obtenu comme un graphe isol, dans ce cas */
    GRF_eff_isole( modele->reference );
  }

  /* 3. On efface les paramtres du modle */
  switch ( modele->type )
  {
  case MODELE_EXAFS_CHEMINS:
    FEFF_EffacerModele(modele->infos_modele.feff);
    break;

  case MODELE_EXAFS_GNXAS:
    OPTION_A_FAIRE( "infos GNXAS  effacer" );
    break;

  case MODELE_SEUIL_BASIQUE:
    OPTION_A_FAIRE( "infos seuil basique  effacer" );
    break;

  case MODELE_FLUOX:
    OPTION_A_FAIRE( "infos modle fluo X  effacer" );
    break;

  case MODELE_DECOMPOSITION:
    OPTION_A_FAIRE( "infos modle linaire  effacer" );
    break;

  default:
    AVERTISSEMENT( "type de modle inconnu [%d]", modele->type );
  }

  /* 4. On contrle qu'il n'apparat pas dans l'arbre d'dition */
  if ( modele == (ModeleAjustement *) ARB_LireComplement( PARAMS_MODELE ) ) {
    ARB_MettreComplement( PARAMS_MODELE, NULL );
    if ( VRAI == arbres[ PARAMS_MODELE ].drapeaux.ouvert ) {
      FEN_Fermer( ARB_QuelleFenetre( arbres + PARAMS_MODELE, FAUX ), VRAI );
    }
  }

  /* On controle qu'il n'est pas utilis dans la fenetre d'ajustement EXAFS */
  if ( VRAI == arbres[ RESULT_N_COUCHES ].drapeaux.ouvert ) {
    Fenetre *fenetre;
    Message message;

    /* On prpare un message de fermeture */
    message.type = WM_CLOSED;
    message.fenetre.application = 0;
    message.fenetre.en_plus = 0;
    message.fenetre.x    = -1; message.fenetre.y    = -1;
    message.fenetre.larg = -1; message.fenetre.haut = -1; /* Fermeture force... */

    /* On trouve la fenetre qui contient l'arbre */
    fenetre = ARB_QuelleFenetre( arbres + RESULT_N_COUCHES, FAUX );

    /* On poste le message */
    if ( ( fenetre != NULL ) && ( fenetre->message != ( int (*)(Fenetre *, Message *) ) NULL ) ) {
      (fenetre->message)(fenetre, &message);
    }
  }

  /* 5. On libre la mmoire */
  free( modele->nom_modele ); modele->nom_modele = NULL;
  free( modele ); modele = NULL;

  /* Effacement s'est fait correctement */
  return 0;
}
