/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Ce fichier : création le 6 juin 2002

   Module d'ajustement par un modèle à n couches

   Filtre des couches à utiliser en fonction de divers critères
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "atomes.h"
#include "couches.h"
#include "feff.h"
#include "interface.h"

#include "couche_filtre.txt"

int choisit_couche(Couche *prem_couche)
{
  unsigned int i, i_lim, Z __attribute__((__unused__));
  int retour;
  float r_max;
  boolean mode_et;
  Couche *couche;

  if (prem_couche == NULL) return FAUX;

  retour = ARB_Bloquant( CHOISIR_CHEMINS, tit_arb_choix_chemins, principale, 0 );
  if (retour == CCH_ANNULER) return FAUX; /* Rien à faire */

  i_lim = OBJ_LireEntier_U( CHOISIR_CHEMINS, CCH_IDX_LIMITE );
  Z = OBJ_NumeroAtomique( CHOISIR_CHEMINS, CCH_Z_ATOME );
  r_max = OBJ_LireNombre( CHOISIR_CHEMINS, CCH_R_LIMITE, NULL );

  mode_et = OBJ_choisit(CHOISIR_CHEMINS, CCH_ET, QUESTION);
  /* Vaut donc VRAI si on combine avec et, FAUX si on combine avec OU
     Ainsi, on peut faire ok = ok... | ok_... | ok_... ou ok = ok_... & sans se poser de question
  */

  for (couche = prem_couche, i = 0; couche != NULL; couche = couche->suivante, i++)
  {
    boolean ok_idx = mode_et, ok_R = mode_et, ok_Z = mode_et;
    boolean ok = FAUX;

    /* La couche convient-elle par rapport aux critères demandés ? */

    if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_INDEX, QUESTION) == VRAI )
    {
      /* Contrôle sur l'index de la couche */
      if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_IDX_SUP, QUESTION) == VRAI )
      {
	if (i > i_lim) ok_idx = VRAI;
	else ok_idx = FAUX;
      }
      else /* On s'intéresse aux couches d'index i < i_lim indiqué */
	if (i < i_lim) ok_idx = VRAI;
	else ok_idx = FAUX;
    }

#ifdef a_reprendre
    if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_TYPE_ATOME, QUESTION) == VRAI )
    { /* Contrôle sur les atomes par lesquels passe le chemin */
      ok_Z = FAUX;
      if (couche->chemin != NULL) /* On peut avoir ces informations */
      {
	Atome *atome;
	unsigned int j;

	for (j = 0; j < couche->chemin->nbr_atomes; j++)
	{
	  atome = couche->chemin->atomes[j];
	  if (atome->num_atomique == Z)
	  {
	    ok_Z = VRAI;
	    break;
	  }
	}
      }
#ifdef _DEVELOPPE_
      else
      {
	printf("La couche %u (%s) n'a pas de chemin associé !\n", i, couche->nom);
      }
#endif
    }
#endif

    if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_MAX_REFLEX, QUESTION) == VRAI )
    { /* Contrôle sur le nombre d'atomes par chemin : À FAIRE */
    }

    if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_RMAX, QUESTION) == VRAI )
    { /* Contrôle sur la longueur du chemin */
      ok_R = FAUX;
      if ( OBJ_choisit(CHOISIR_CHEMINS, CCH_RMAX_SUP, QUESTION) == VRAI ) {
	if ( VAR_LireValeur( &(couche->R), VARIABLE_LIRE_VALEUR_INITIALE ) > r_max) ok_R = VRAI;
      }
      else /* On s'intéresse aux couches de longueur R < Rmax indiqué */
	if ( VAR_LireValeur( &(couche->R), VARIABLE_LIRE_VALEUR_INITIALE ) < r_max) ok_R = VRAI;
    }

    /* Ici, on combine les résultats intermédiaires */
    if (mode_et == VRAI)
      ok = ok_idx && ok_R && ok_Z;
    else
      ok = ok_idx || ok_R || ok_Z;

    /* ------ Ici, si la couche est ok, on l'active/désactive ------ */
    if (ok == VRAI)
    {
#ifdef _DEVELOPPE_
      printf("La couche %u (%s) a été sélectionnée\n", i, couche->nom);
#endif
      if (retour == CCH_DESACTIVER)
      {
	couche->utilisee = FAUX;
	if (couche == couche_editee)
	  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, FORCE_NON);
      }
      else if (retour == CCH_ACTIVER)
      {
	couche->utilisee = VRAI;
	if (couche == couche_editee)
	  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, FORCE_OUI);
      } else if (retour == CCH_EFFACER) {
	COUCHE_Effacer( NULL, couche, VRAI, OBJ_choisit(CHOISIR_CHEMINS, CCH_EFF_AMP, QUESTION) );
	couche = prem_couche; /* Le chaînage a été modifié, couche n'est plus valable... On redémarre donc */
	i = 0;
      }
    } else if (OBJ_choisit(CHOISIR_CHEMINS, CCH_ETAT_INCHANG, QUESTION) == FAUX) {
      /* Couche non-valable : on fait le contraire */
      if (retour == CCH_DESACTIVER) { /* non valable pour la désactivation -> on active */
	couche->utilisee = VRAI;
	if (couche == couche_editee)
	  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, FORCE_OUI);
      } else if (retour == CCH_ACTIVER) { /* non valable pour l'activation -> on désactive */
	couche->utilisee = FAUX;
	if (couche == couche_editee)
	  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, FORCE_NON);
      }
    }
  }

  return VRAI; /* Tout s'est bien passé */
}
