/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Cration : 28 septembre 1998

   Module d'ajustement par un modle  n couches

   Ajustement par la mthode du pseudo-gradient (programmes EDA, modifie)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"

#include "couches.h"
#include "feff.h"
#include "graphes.h"
#include "modeles.h" /* Pour les variables,... */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* static boolean multi_abs = FAUX, multi_spectre = FAUX; */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* static int calc_increments(struct modele_ajuste *modele); */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Ralisation effective de la minimisation ----------\
 |   Renvoie <0 si sortie "anormale"                           |
 |           le code indiquant la cause de fin d'ajustement    |
 |                                                             |
 | Suppose que _toutes_ les variables soient de type VAR_LIBRE |
 | La routine appelante est charge d'initialiser les variables|
 | et de les librer aprs utilisation (routines init_rapide et|
 | termine_rapide)                                             |
 |                                                             |
 | infos = le _premier_ de la srie, si plusieurs              |
 \------------------------------------------------------------ */

#ifdef ancien
int NC_PseudoGradientEXAFS(struct modele_ajuste *modele)
{
  struct pg_variable *var, *variables;
  unsigned int i;
  double err_limite = 0.0, delta_err_limite = 0.0, increment_limite = 0.0;

  /* On prcise les conditions de fin */

  variables = modele->infos_variables.pseudogradient; /* Pour ne pas le faire  chaque fois */

  /* On dmarre l'ajustement */
  while (modele->iteration < modele->max_iteration) { /* On doit continuer les itrations */
    double pas, pas_max = 1.0;
    double ex_erreur, nv_erreur;
    int retour;

    calc_increments(modele); /* Calcul des incrments pour chacune des variables */

    /* On enregistre les anciennes valeurs */
    for (i = 0, var = variables ; i < modele->nbr_variables ; i++, var++)
      var->ex_valeur = var->var->contenu.valeur;

    /* On en dduit le pas pour chacune des variables */
    do {
      for (i = 0, var = variables ; i < modele->nbr_variables ; i++, var++) {
	double nv_valeur;

	pas = pas_max;
	nv_valeur = var->ex_valeur + pas * var->increment;
	/* On vrifie que l'on est bien dans l'intervalle de variations autoris */
	while ( (nv_valeur < var->var->min) || (nv_valeur > var->var->max) ) {
	  pas /= 2.0;
	  nv_valeur = var->ex_valeur + pas * var->increment;
	}

	var->var->contenu.valeur = nv_valeur;
      }

      /* Le calcul du spectre - suivant le type de modle utilis */
      retour = NC_CalculeEXAFS(modele);
      if (retour != 0) {
	fprintf(stderr, "NC_PseudoGradientEXAFS : problme survenu dans le calcul des contributions\n");
	return ARRET_D_URGENCE;
      }
      nv_erreur = MODELE_CalculerResidu(modele);


#ifdef _DEVELOPPE_
      /*      printf("Itration %d : erreur de dpart %g, erreur trouve %g\n",
	      infos->etape, infos->erreur, nv_erreur); */
#endif
      attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
		       0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
		       0,
		       NULL, NULL, NULL, NULL, NULL, NULL);
      if (modele->reference == NULL) { /* Au cours du traitement des vnements, graphe effac... */
	fprintf(stderr, "NC_PseudoGradientEXAFS : Graphe de rfrence effac - Arrt d'urgence de l'ajustement\n");
	return ARRET_D_URGENCE;
      }

      pas_max /= 2.0;
    } while ( (pas_max > 2.0e-5) && (nv_erreur > modele->erreur) );

    if ( nv_erreur > modele->erreur ) { /* On n'a pas russi  minimiser */
#ifdef _DEVELOPPE_
      /*      printf("Itration %d : impossible de diminuer l'erreur (%f) - ajustement arrt\n",
	      infos->etape, infos->erreur); */
#endif

      /* On remet les anciennes valeurs, qui donnaient un meilleur rsultat */
      for (i = 0, var = variables ; i < infos->nbr_var ; i++, var++)
	var->var->contenu.valeur = var->ex_valeur;

      /* On recalcule le spectre ajust */
      retour = NC_CalculeEXAFS(modele);
      if (retour != 0) {
	fprintf(stderr, "NC_PseudoGradientEXAFS : problme survenu dans le calcul des contributions\n");
	return ARRET_D_URGENCE;
      }
      modele->erreur = MODELE_CalculerResidu(modele);

      met_svaleur(RESULT_N_COUCHES, RNC_ERR_ABSOLUE , modele->erreur );
      met_svaleur(RESULT_N_COUCHES, RNC_ERR_PONDEREE, modele->erreur / (double) (modele->reference->nombre_point) );
/*       met_svaleur(RESULT_N_COUCHES, RNC_ERR_RELATIVE, infos->err_relative); */

      /* On termine l'ajustement */
      return IMPOSSIBLE_MINIMISER; /* Impossible de minimiser */
    }

    /* Si on est l, c'est que l'on a russi  minimiser l'erreur (joie !) */
    /* Le spectre ajust est dj calcul, le rsidu aussi, les valeurs ne sont pas mises  jour */

    /* On calcule ce qu'il faut */
    ex_erreur = modele->erreur;
    modele->erreur = MODELE_CalculerResidu(modele);

    met_svaleur(RESULT_N_COUCHES, RNC_ERR_ABSOLUE , modele->erreur );
    met_svaleur(RESULT_N_COUCHES, RNC_ERR_PONDEREE, modele->erreur / (double) (modele->reference->nombre_point) );
/*     met_svaleur(RESULT_N_COUCHES, RNC_ERR_RELATIVE, infos->err_relative); */

    HIST_Titre(5, "Erreur interm"e_AIGU"diaire", "ERR");
    HIST_Texte(5, 2, "It"e_AIGU"ration : %u\n", modele->iteration);
    HIST_Texte(5, 2, "absolue  : %.20g --- pond"e_AIGU"r"e_AIGU"e : %.20g\n",
	       modele->erreur, modele->erreur / (double) ( modele->reference->nombre_point ) );

    /* On affiche les valeurs courantes d'ajustement */
    FEN_redessin( ARB_fenetre( &(arbres[RESULT_N_COUCHES]) ), BI_RDS_UTIL ); /* On redessine tout */

    /* Vrifications de sortie */
    if (modele->erreur < err_limite) { /* Erreur plus faible que celle souhaite */
      return ERR_MIN_ATTEINTE;
    }

    if ( (ex_erreur - modele->erreur) < delta_err_limite ) /* Variation de l'erreur trop faible */
      return DELTAERR_TROP_FAIBLE;

    (modele->iteration)++; /* On passe  l'itration suivante */
    OBJ_MettreEntier_U( RESULT_N_COUCHES, RNC_ITERATION, modele->iteration );

    /* mettre une attente d'vnement */
    attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
		     0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
		     0,              /* Compteur  1, sinon plantage sur TOS 1.0 */
		     NULL, NULL, NULL, NULL, NULL, NULL);
  } /* Et c'est reparti pour un tour... */

  return 0; /* On a atteint le minimum suivant les conditions imposes */
}
#endif

/* -------- Calcul des incrments selon l'algorithme rapide type EDA -----------\
 |   Renvoie 0 si tout s'est bien pass, <0 sinon                               |
 |                                                                              |
 | incrment I = 0.5 * ( dS/dvariable ) / Somme_sur_k ( dmodle/dvariable )    |
 |                                                                              |
 |               Somme_sur_k [ residu(k) * (dmodle / dvariable)(k) / sigma_i] |
 |             = -------------------------------------------------------------- |
 |                  Somme_sur_k [ ( (dmodle / dvariable)(k) ) / sigma_i]     |
 |                                                                              |
 | Suppose implicitement que var->x_max > var->x_min et                         |
 |                       que toutes les variables soient  faire varier         |
 \----------------------------------------------------------------------------- */

int NC_IncrementsPG(struct modele_ajuste *modele)
{
  struct pg_variable *var, *variables;
  unsigned int num_var; /* Index de la variable utilise */
  unsigned long i; /* Index sur les k */
  register double *x, *y_residu, *y_modele, *y_erreur = NULL;
  double delta_e = 0.0;
  struct modele_ajuste *modele_variable = NULL;

  /* Prparation */
  variables = modele->infos_variables.pseudogradient;
  modele_variable = modele;

  if ( MODELE_EXAFS_CHEMINS == modele->type ) {
    if ( VRAI == modele->infos_modele.feff->drapeaux.DE0_general ) {
      delta_e = VAR_LireValeur( &(modele->infos_modele.feff->Delta_E), VARIABLE_LIRE_VALEUR_AJUSTEE );
      printf( "%s : DE0 = %g\n", __FUNCTION__, delta_e );
    }
  }

  if ( VRAI == modele->drapeaux.pondere ) {
#ifdef _DEVELOPPE_
    printf( " ==> %s <== Avec ponderation [ponderation : %d]\n",
	    __FUNCTION__, modele->drapeaux.pondere );
#endif
    y_erreur = modele->reference->erreur_y->y;
  }

  /* Rinitialisation */
  for ( num_var = 0, var = variables ; num_var < modele->nbr_variables ; num_var++, var++ ) {
    var->den = 0.0;
    var->num = 0.0;
  }

  /* Ici, boucle sur les points. On utilise une boucle sur les variables  l'intrieur
                                 pour profiter des calculs de khi_ajuste(k) communs 
				 plusieurs variables, le cas chant.                  */
  for ( i = 0, x = modele->modele->x, y_modele = modele->modele->y, y_residu = modele->residu->y ;
        i < modele->modele->nombre_point ;
        i++, x++, y_modele++, y_residu++ ) {
    Couche *couche = NULL;
    double khi_calc = 0.0, sigma2 = 1.0;

    if ( y_erreur != NULL ) {
      sigma2 = *y_erreur * *y_erreur;
      y_erreur++;
    }
    if ( 0.0 == sigma2 ) {
      (void) fprintf( stderr, "%s : ERREUR, sigma2 = 0 [point %lu, k = %g]\n\n",
		      __FUNCTION__, i, *x );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }

    for ( num_var = 0, var = variables ; num_var < modele->nbr_variables ; num_var++, var++ ) {
      double derivee = 0.0;

      if ( modele_variable != var->modele ) {
	modele_variable = var->modele; /* Modle sur lequel on travaille */
	if ( MODELE_EXAFS_CHEMINS == modele_variable->type ) {
	  if ( VRAI == modele_variable->infos_modele.feff->drapeaux.DE0_general ) {
	    delta_e = VAR_LireValeur( &(modele_variable->infos_modele.feff->Delta_E), VARIABLE_LIRE_VALEUR_AJUSTEE );
	    printf( "%s : DE0 dans boucle = %g\n", __FUNCTION__, delta_e );
	  }
	}
      }

      if ( (couche != var->origine.couche) ) { /* On change de couche : recalculer le signal EXAFS en ce point */
	couche = var->origine.couche;
	if ( FAUX == modele_variable->infos_modele.feff->drapeaux.DE0_general ) {
	  delta_e = VAR_LireValeur( &(couche->Delta_E), VARIABLE_LIRE_VALEUR_AJUSTEE );
	}
	khi_calc = calc_exafs_1point(*x, modele_variable->infos_modele.feff, couche, delta_e);
      }

      if ( couche != NULL ) { /* Variable de la contribution d'une couche */
	derivee = calc_derivee( *x, couche, modele_variable->infos_modele.feff, var->type, VAR_LireValeur(var->var, VARIABLE_LIRE_VALEUR_AJUSTEE ),  khi_calc, delta_e );
      } else { /* Variable globale, intra- ou inter-modle */
	derivee = calc_derivee( *x, NULL  , modele_variable->infos_modele.feff, var->type, VAR_LireValeur(var->var, VARIABLE_LIRE_VALEUR_AJUSTEE ), *y_modele, delta_e );
#ifdef _DEVELOPPE_
	/* if (var->type == VAR_GAMMA_GENE) {
	   printf("Calcul de la drive, variable globale de type Gamma_General\n");
	   printf("   k = %f, rsidu = %f, modle = %f, der = %f\n", *x, *y_residu, *y_modele, derivee);
	}*/
#endif
      }
      if ( isnan( derivee ) ) {
	printf("Calcul de la drive, variable %i\n", num_var );
	printf("   k = %f, rsidu = %f, modle = %f, der = %f\n", *x, *y_residu, *y_modele, derivee );
      }

      if ( y_erreur != NULL ) { /* On utilise les vraies erreurs ? */
	var->num += derivee * *y_residu / sigma2;
	var->den += derivee * derivee / sigma2;
      } else { /* Pas d'erreur ou erreur constante -> ne joue pas sur l'incrment car se simplifie
		  (mme en tenant compte de la pondration en k^n) */
	var->num += derivee * *y_residu;
	var->den += derivee * derivee;
      }
      if ( isnan( var->num ) || isnan( var->den ) ) {
	(void) fprintf( stderr, "%s : ERREUR, increment : composant = NaN [variable : %u, numerateur : %g, denominateur : %g, point k = %g [%lu], y_erreur = %g]\n\n",
			__FUNCTION__, num_var, var->num, var->den, *x, i, *y_erreur );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
    }
  }

  for ( num_var = 0, var = variables ; num_var < modele->nbr_variables ; num_var++, var++ ) {
    if (var->den == 0.0) {
#ifdef _DEVELOPPE_
      fprintf(stderr, "calc_increments (rapide) : variable constante, drive nulle...\n");
      fprintf(stderr, "                           type : %d - N = %f, D = %f\n", var->type, var->num, var->den);
#endif
      EXF_ERREUR(DIV_ZERO, "D"e_AIGU"nominateur nul", txt_err_calc_annule);
      return -1;
    } else {
      var->increment = var->num / var->den;
    }

#ifdef _DEVELOPPE_
    if ( isnan( var->increment ) ) {
      (void) fprintf( stderr, "%s : ERREUR, increment = NaN [variable : %u, numerateur : %g, denominateur : %g]\n\n",
		      __FUNCTION__, num_var, var->num, var->den );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      var->increment = 0;
    }
#endif
  }

  return 0; /* Tout s'est bien pass */
}

/* -------- Informations sur la j-e variable (pour Monte-Carlo essentiellement) --------\
     Renvoie un texte (constant) qui contient le nom du type de variable

 j        = numro de la variable pour laquelle on veut les infos
 couche   = la couche associe  la variable
 variable = la structure LM_variable complte, si non-NULL (attention, copie le CONTENU)
 */
#ifdef a_deplacer
char *infos_rapide(unsigned int j, char **couche, LM_variable *variable)
{
  LM_variable var;

  var = variables[j];

  if (couche != NULL)
  {
    if (var.couche == NULL)
      *couche = "Variable g"e_AIGU"n"e_AIGU"rale";
    else
      *couche = var.couche->nom;
  }
  if (variable != NULL)
  {
    *variable = var;
  }

  switch (var.type)
  {
  case VAR_SO2:
    return "S0"DEUX_EXPOSANT" (facteur de r"e_AIGU"duction)";

  case VAR_DELTAE_GENE:
  case VAR_DELTAE:
    return DELTA_MAJUSCULE"E0 (correction de l'"e_AIGU"nergie de seuil)";

  case VAR_GAMMA_GENE:
  case VAR_GAMMA:
    return GAMMA_MAJUSCULE" (libre parcours moyen)";

  case VAR_CONTRIBUTION:
    return "Contribution de l'absorbeur au signal total";

  case VAR_N:
    return "N (d"e_AIGU"g"e_AIGU"n"e_AIGU"rescence)";

  case VAR_R:
    return "R (longueur de chemin)";

  case VAR_DEBYE:
    return SIGMA""DEUX_EXPOSANT" (facteur de Debye-Waller)";

  case VAR_CUMUL_1:
    return "C1 (premier cumulant)";
  case VAR_CUMUL_2:
    return "C2 (deuxi"e_GRAVE"me cumulant)";
  case VAR_CUMUL_3:
    return "C3 (troisi"e_GRAVE"me cumulant)";
  case VAR_CUMUL_4:
    return "C4 (quatri"e_GRAVE"me cumulant)";
  }

  return "Variable inconnue";
}
#endif
