/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Cration : 27 juin 2002

   Ajustement d'un modle  N chemins (type FEFF)

   Initialisations des variables de l'ajustement (en fonction de l'algorithme)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "graphes.h"

#include "atomes.h" /* Pour les absorbeurs et les chemins */
#include "chemins.h" /* Pour les chemins -> infos sur une variable (unit) */
#include "couches.h"
#include "feff.h"
#include "modeles.h"
#include "theorie.h" /* Pour les types de lpm */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Initialisation pour un algorithme de type pseudo-gradient -------- */

int NC_PGInitVariables(struct modele_ajuste *modele, unsigned int prem_variable)
{
  Couche *couche;
  PG_variable *var;
  struct modele_feff *feff;

  if (modele == NULL) return -1; /* Normalement, ne devrait jamais arriver, mais bon... */
  if (prem_variable >= modele->nbr_variables) {
    (void) fprintf(stderr, "NC_PGInitVariables : incohrence pour prem_variable (%u, mais %u variables) !\n",
		   prem_variable, modele->nbr_variables);
    return -10;
  }

  var = modele->infos_variables.pseudogradient + prem_variable;
  if ( NULL == var ) {
    (void) fprintf(stderr, "NC_PGInitVariables : modle sans variables !\n");
    return -2;
  }

  feff = modele->infos_modele.feff;
  if (NULL == feff) {
    (void) fprintf(stderr, "NC_PGInitVariables : modle sans informations sur les chemins !\n");
    return -2;
  }

  /* On initialise les variables */
#ifdef _DEVELOPPE_
  printf("NC_PGInitVariables : il faudrait amliorer avec fonction + contrle de dbordement\n");
#endif

  /* 1) Variables globales */
  if ( feff->S0_2.type == VAR_LIBRE ) {
    var->var    = &(feff->S0_2);
    var->type   = VAR_SO2;
    var->origine.couche = NULL;
    var->modele = modele;
    var++;
    prem_variable++;
  }
  if ( (feff->drapeaux.DE0_general == VRAI) && (feff->Delta_E.type == VAR_LIBRE) ) {
    var->var  = &(feff->Delta_E);
    var->type = VAR_DELTAE_GENE;
    var->origine.couche = NULL;
    var->modele = modele;
    var++;
    prem_variable++;
  }
  if ( (feff->drapeaux.GAM_general == VRAI) &&
       ( (feff->type_lpm == MODELE_LPM_GAMMA) || (feff->type_lpm == MODELE_LPM_GAMMA_ETA) ) &&
       (feff->Gamma.type == VAR_LIBRE) ) {
    var->var  = &(feff->Gamma);
    var->type = VAR_GAMMA_GENE;
    var->origine.couche = NULL;
    var->modele = modele;
    var++;
    prem_variable++;
  }

  /* 2) Variables des couches */
  for (couche = feff->couches ; couche != NULL ; couche = couche->suivante) {
    if ( FAUX == couche->utilisee ) continue;

    if  ( couche->N.type == VAR_LIBRE ) {
      var->var  = &(couche->N);
      var->type = VAR_N;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->R.type == VAR_LIBRE ) {
      var->var  = &(couche->R);
      var->type = VAR_R;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->Debye_Waller.type == VAR_LIBRE ) {
      var->var  = &(couche->Debye_Waller);
      var->type = VAR_DEBYE;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->Cumul_1.type == VAR_LIBRE ) {
      var->var  = &(couche->Cumul_1);
      var->type = VAR_CUMUL_1;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->Cumul_2.type == VAR_LIBRE ) {
      var->var  = &(couche->Cumul_2);
      var->type = VAR_CUMUL_2;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->Cumul_3.type == VAR_LIBRE ) {
      var->var  = &(couche->Cumul_3);
      var->type = VAR_CUMUL_3;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( couche->Cumul_4.type == VAR_LIBRE ) {
      var->var  = &(couche->Cumul_4);
      var->type = VAR_CUMUL_4;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( (feff->drapeaux.DE0_general == VRAI) && (couche->Delta_E.type == VAR_LIBRE) ) {
      var->var  = &(couche->Delta_E);
      var->type = VAR_DELTAE;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
    if ( (feff->drapeaux.GAM_general == FAUX) && 
	 ( (feff->type_lpm == MODELE_LPM_GAMMA) || (feff->type_lpm == MODELE_LPM_GAMMA_ETA) ) &&
	 (couche->Gamma.type == VAR_LIBRE ) ) {
      var->var  = &(couche->Gamma);
      var->type = VAR_GAMMA;
      var->origine.couche = couche;
      var->modele = modele;
      var++;
    }
  }

  return 0; /* Tout s'est bien pass */
}

/* -------- Informations sur une variable (pour affichages) -------- */

char *NC_InfosVariable(int type, struct couche *couche)
{
  char *nom;
  const char *nom_variable = NULL;

  switch (type)
  {
  case VAR_SO2:
    nom_variable = "S0"DEUX_EXPOSANT" (facteur de r"e_AIGU"duction)";
    break;

  case VAR_DELTAE_GENE:
  case VAR_DELTAE:
    nom_variable = DELTA_MAJUSCULE"E0 (correction de l'"e_AIGU"nergie de seuil)";
    break;

  case VAR_GAMMA_GENE:
  case VAR_GAMMA:
    nom_variable = GAMMA_MAJUSCULE" (libre parcours moyen)";
    break;

  case VAR_CONTRIBUTION:
    nom_variable = "Contribution de l'absorbeur au signal total";
    break;

  case VAR_N:
    nom_variable = "N (d"e_AIGU"g"e_AIGU"n"e_AIGU"rescence)";
    break;

  case VAR_R:
    nom_variable = "R (longueur)";
    break;

  case VAR_DEBYE:
    nom_variable = SIGMA""DEUX_EXPOSANT" (Debye-Waller)";
    break;

  case VAR_CUMUL_1:
    nom_variable = "C1 (1er cumulant)";
    break;

  case VAR_CUMUL_2:
    nom_variable = "C2 (2e cumulant)";
    break;

  case VAR_CUMUL_3:
    nom_variable = "C3 (3e cumulant)";
    break;

  case VAR_CUMUL_4:
    nom_variable = "C4 (4e cumulant)";
    break;
  }

  if (NULL == couche) {
    return strdup(nom_variable);
  }

  nom = TXT_genere_chaine("%s (couche %s)", nom_variable, couche->nom);
  return nom;
}

/* -------- Quelle est l'unit de la variable considre ? -------- */

char *NC_UniteVariable(int type, struct couche *couche)
{
  const char *unite = NULL;

  switch (type)
  {
  case VAR_SO2:			/* Facteur de rduction S0 : sans unit */
    break;

  case VAR_DELTAE_GENE:
  case VAR_DELTAE:		/* Les nergies sont en eV par dfaut */
    unite = "eV";
    break;

  case VAR_GAMMA_GENE:
  case VAR_GAMMA:
    OPTION_A_FAIRE( "Unit pour G, lpm" );
    break;

  case VAR_CONTRIBUTION:	/* Contribution : en fraction du signal -> sans unit */
    break;

  case VAR_N:			/* Dgnrescence : voisins ou nombre de chemins suivant simple ou multiple */
    if ( couche != NULL ) {
      if ( couche->chemin != NULL ) {
	if ( 1 == couche->chemin->nbr_atomes ) {
	  unite = "voisin(s)";
	} else {
	  unite = "chemin(s) quivalent(s)";
	}
      }	/* Si pas de chemin dans la couche : impossible de savoir si simple ou multiple -> sans unit */
    } /* Si pas de couche : on ne distingue pas simple et multiple -> sans unit */
    break;

  case VAR_R:			/* Distances : en  par dfaut */
    unite = ANGSTROM;
    break;

  case VAR_DEBYE:		/* Facteur de Debye-Waller : en Ų par dfaut */
    unite = ANGSTROM""DEUX_EXPOSANT;
    break;

  case VAR_CUMUL_1:
    OPTION_A_FAIRE( "Unit du 1er cumulant" );
    break;

  case VAR_CUMUL_2:
    OPTION_A_FAIRE( "Unit du 2e cumulant" );
    break;

  case VAR_CUMUL_3:
    OPTION_A_FAIRE( "Unit du 3e cumulant" );
    break;

  case VAR_CUMUL_4:
    OPTION_A_FAIRE( "Unit du 4e cumulant" );
    break;
  }

  if ( NULL != unite ) {
    return strdup( unite );
  }
  return NULL;
}
