/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2001, Emmanuel CURIS
   Cration du fichier : 19 septembre 2001

   Oprations diverses sur les graphes, plus gnrales que l'EXAFS lui-mme
   (aussi bien mathmatiques que statistiques simples : tri en particulier)
   (les oprations statistiques avances sont dans ../Stats)

   Transformations quelconques sur les coordonnes du graphe de rfrence
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "c_formel.h"
#include "operations.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

boolean GRF_Xfonction(Graphe *graphe, const char *formule)
{
  unsigned long i;
  register double *x, *y;
  Expression *arbre_formule;

  if ( (graphe == NULL) || (formule == NULL) ) return FAUX;
  if ( strlen(formule) == 0 ) return FAUX;

  arbre_formule = construit_arbre(formule);
  if (arbre_formule == NULL) return FAUX;

  /* Idalement, il faudrait simplifier l'arbre */

  /* On fait le calcul */
  for (i = 0, x = graphe->x, y = graphe->y ; 
       i < graphe->nombre_point ; 
       i++, x++, y++) {
    double nv_x;

    /* On initialise les variables */
    Var_X.valeur_initiale = *x;
    Var_Y.valeur_initiale = *y;
    Var_I.valeur_initiale = (double) i;

    /* On calcule la valeur de la formule en ce point */
    if( calcule_arbre(arbre_formule, &nv_x) == VRAI ) {
      *x = nv_x;
    }
  }
  GRF_bornes(graphe, DESORDONNE, NE_PAS_FAIRE);

  effacer_arbre(arbre_formule); /* Plus besoin de l'arbre */

  return VRAI; /* La transformation a eu lieu */
}

boolean GRF_Yfonction(Graphe *graphe, const char *formule)
{
  unsigned long i;
  register double *x, *y;
  Expression *arbre_formule;

  if ( (graphe == NULL) || (formule == NULL) ) return FAUX;
  if ( strlen(formule) == 0 ) return FAUX;

  arbre_formule = construit_arbre(formule);
  if (arbre_formule == NULL) return FAUX;

  /* Idalement, il faudrait simplifier l'arbre */

  /* On fait le calcul */
  for (i = 0, x = graphe->x, y = graphe->y ; 
       i < graphe->nombre_point ; 
       i++, y++) {
    double nv_y;

    /* On initialise les variables */
    Var_Y.valeur_initiale = *y;
    Var_I.valeur_initiale = (double) i;

    if ( x != NULL ) {
      Var_X.valeur_initiale = *x;
      x++;
    } else {
      Var_X.valeur_initiale = Var_I.valeur_initiale;
    }

    /* On calcule la valeur de la formule en ce point */
    if( calcule_arbre(arbre_formule, &nv_y) == VRAI ) {
      *y = nv_y;
    }
  }
  GRF_bornes(graphe, NE_PAS_FAIRE, DESORDONNE);

  effacer_arbre(arbre_formule); /* Plus besoin de l'arbre */

  return VRAI; /* La transformation a eu lieu */
}
