/* Programme pour vérifier le calcul d'une gaussienne à N dimensions [probabilité, fractile,...] */

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "proba.h"

int main(void)
{
  double probabilite, *x = NULL, *facteurs = NULL;
  double x_commun;
  unsigned int dimension, i, n_controle, n_lu;
  unsigned long ddl;

  do {
    /* On demande la dimension */
    printf("Quelle est la dimension du vecteur gaussien ?\n");
    scanf( "%u", &dimension );
    if ( 0 == dimension ) {
      printf( "** Fin du programme **\n" );
      return 0;
    }

    /* On réserve les tableaux en conséquence */
    x = (double *) malloc( sizeof( double ) * dimension );
    if ( NULL == x ) {
      (void) fprintf(stderr, "%s, %s l.%u : Mémoire insuffisante, dimension = %u\n",
		     __FILE__, __FUNCTION__, __LINE__, dimension);
      return -1;
    }

    facteurs = (double *) malloc( sizeof( double ) * dimension );
    if ( NULL == facteurs ) {
      (void) fprintf(stderr, "%s, %s l.%u : Mémoire insuffisante, dimension = %u\n",
		     __FILE__, __FUNCTION__, __LINE__, dimension);
      free( x ); x = NULL;
      return -1;
    }

    /* On demande les ddl */
    printf("Nombre de degrés de liberté :\n");
    scanf("%lu", &ddl);

    /* On demande les valeurs en X */
    printf("Valeur de x :\n");
    scanf("%lf", &x_commun);
    for ( i = 0 ; i < dimension ; i++ ) {
      x[ i ] = x_commun;
    }

    /* On demande les tailles d'échantillons */
    printf("Taille du groupe témoin :\n");
    scanf("%u", &n_controle);
    if ( 0 == n_controle ) {
      (void) fprintf(stderr, "%s, %s l.%u : Pas de témoin !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      break;
    }
    for ( i = 0 ; i < dimension ; i++ ) {
      printf("Taille du groupe %u :\n", i + 1);
      scanf("%u", &n_lu );
      facteurs[ i ] = 1.0 / sqrt( 1.0 + (double) n_controle / (double) n_lu );
    }

    /* On appelle pour le calcul */
    probabilite = PROBA_FdR_MultiNormale_Multiplicatif( dimension, x, facteurs );

    /* On affiche le résultat */
    printf( "Probabilité trouvée [gaussienne] : %g\n", probabilite );

    /* Idem, mais pour la multi-Student */
    probabilite = PROBA_FdR_MultiStudent_Multiplicatif( dimension, x, facteurs, ddl );
    printf( "Probabilité trouvée [Student à %lu ddl] : %g\n", ddl, probabilite );

    /* On prépare pour le tour suivant */
    free( facteurs ); facteurs = NULL;
    free( x ); x = NULL;
  } while ( 1 );

  free( facteurs ); facteurs = NULL;
  free( x ); x = NULL;

  /* On ne devrait jamais être là ! */
  return -25;
}
