/* LASE : Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1998
   Cration     : 28 octobre 1998
   Modification : 28 octobre 1998

   Toutes les fonctions de traitement statistique de l'E.X.A.F.S.

   Messages d'erreur du module de calculs statistiques
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "systeme.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#ifdef _ENGLISH_
/*     v-----------------------------v
       v                             v */
static const char *textes[] = 
{
  "[1][-- Statistical computations --| "
     "|  Selected spectra do not have|"
     "|the same number of points !][ Stop |Troncate]",

  "[1][----- Standard deviation -----| "
     "|  Selected spectra are not of |"
     "| the same kind !][ Stop | Continue ]",

  "[1][-- Statistical computations --| "
     "| Inconsticency while computing"
     "|some statistical values.][ Continue ]",

  "[1][----- Standard deviation -----| "
     "|    There's no X common to all"
     "|the selected spectra !][ Cancel ]",

  "[1][----- Standard deviation -----| "
     "| Average standard deviation : "
     "|s(Y) = %.10g][ Continue ]",

  "[3][----- Spectra averaging -----"
     "|   You need to select at least"
     "|2 graphs to average !][ Cancel ]",

  "[2][----- Spectra averaging -----"
     "|   Selected graphs are not of"
     "|the same kind! Results are"
     "|impredictible.][Continue|Cancel]"
};
#else
/*     v-----------------------------v
       v                             v */
static const char *textes[] = 
{
  "[1][---- Calculs statistiques ----| "
     "|Les spectres choisis n'ont pas"
     "|tous le m"e_CIRC"me nombre de points !][Arr"e_CIRC"ter|Tronquer]",

  "[1][----- "E_AIGU"cart-type -----| "
     "|  Les spectres choisis ne sont"
     "| pas tous du m"e_CIRC"me type !][Arr"e_CIRC"ter|Continuer]",

  "[1][---- Calculs statistiques ----| "
     "|   Incoh"e_AIGU"rence lors du calcul"
     "| d'une grandeur statistique.][Continuer]",

  "[1][----- "E_AIGU"cart-type -----| "
     "|   Il n'y a aucun intervalle des "
     "| X commun "a_GRAVE" tous les spectres.][ Annuler ]",

  "[2][----- "E_AIGU"cart-type -----| "
     "| "E_AIGU"cart-type moyen : "
     "|s(Y) = %.10g][ Suite ]",

  "[3][----- Moyenne de graphes -----"
     "| Il faut au moins deux graphes"
     "|pour r"e_AIGU"aliser un calcul de"
     "|moyenne ! Calcul annul"e_AIGU"][ Bien ]",

  "[2][----- Moyenne de graphes -----"
     "|  Les spectres choisis ne sont"
     "|pas tous du mme type. Les"
     "|rsultats sont indtermins.][Suite|Abandon]"
};
#endif

#define NOMBRE_MESSAGES 7

static char vrai_texte[300];

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~~~~~~ */

int STAT_ERREUR(unsigned int code_erreur, ...)
{
  va_list params;
  int retour = 0;

  if ( code_erreur >= NOMBRE_MESSAGES )
  {
    return SYS_ERREUR(ERREUR_INCONNUE, code_erreur, "Statistique");
  }

  va_start(params, code_erreur);

  switch (code_erreur)
  {
  case NBR_PTS_DIFFERENTS: /* Messages sans paramtres */
  case TYPES_DIFFERENTS:
  case STAT_INCOHERENCE:
  case SANS_RECOUVREMENT:
  case MOY_TROP_PEU_GRAPHES:
  case MOY_TYPES_DIFFERENTS:
    retour = form_alert(1, textes[code_erreur]);
    break;

  case SIGMA_MOYEN:        /* Un paramtre, de type double */
    sprintf(vrai_texte, textes[code_erreur],
	    va_arg(params, double) );
    retour = form_alert(1, vrai_texte);
    break;
  }

  va_end(params);
  return retour;
}
