/* LASE - Logiciel d'Analyse des Spectres EXAFS
   LASE - (C) 1998-2003, Emmanuel CURIS

   Construction d'un spectre XAS par transmission
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "experience.h"
#include "serie.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Calcul d'un spectre de fluorescence complet [appelable extrieurement] -------- */

Graphe *EXP_SpectreXRF( Experience *experience, Serie_Graphes *serie_resultats )
{
  Faisceau *f_apres_i0 = NULL, *f_echantillon, *f_detecteur;
  Graphe *spectre;
  double i0;
  int retour;

  /* Vrifications [juste pour viter les plantages : la cohrence a d tre vrifie avant */
  if ( NULL == experience ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'exprience\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( NULL == experience->i1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de dtecteur de fluorescence !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( NULL == serie_resultats ) { /* Par dfaut, on essaye dans la srie courante */
    serie_resultats = serie_courante;
  }

  /* On calcule le spectre rsultat */
  /* 1. S'il y a un dtecteur I0 : calcul du signal aprs I0 */
  if ( experience->i0 != NULL ) {
    f_apres_i0 = FAISCEAU_Copier( experience->faisceau );
    i0 = DETECTEUR_FaisceauTraverse_I( experience->i0, experience->faisceau, experience->E_debut, f_apres_i0 );
  } else {
    i0 = 1.0;
    f_apres_i0 = experience->faisceau;
  }
  if ( NULL == f_apres_i0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de dterminer le faisceau aprs I0 --- Calcul annul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* 2. On projette le faisceau sur l'chantillon */
  f_echantillon = FAISCEAU_ProjeterSurEchantillon( f_apres_i0, experience->echantillon );
  if ( ( f_apres_i0 !=  experience->faisceau ) && ( f_apres_i0 != f_echantillon ) ) {
    FAISCEAU_Effacer( f_apres_i0 ); /* Plus besoin de lui */
    f_apres_i0 = NULL;
  }
  if ( NULL == f_echantillon ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans FAISCEAU_ProjeterSurEchantillon()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* 3. Squelette du faisceau peru par le dtecteur de fluo. */
  f_detecteur = FAISCEAU_GeometrieFluorescence( experience->i1, experience->table_X_fluo, experience->nbr_X_fluo );
  if ( NULL == f_detecteur ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans FAISCEAU_GeometrieFluorescence\n",
		   __FILE__, __FUNCTION__, __LINE__);
    if ( f_echantillon != experience->faisceau ) { /* Il devient donc inutile ! */
      FAISCEAU_Effacer( f_echantillon );
    }
    return NULL;
  }

  /* 4. Calcul du spectre de fluorescence complet : comme en un point du spectre de fluo / XAS */
  retour = FLUO_InitialiserEchantillon( experience->echantillon, f_echantillon, experience->E_debut );
  if ( f_echantillon != experience->faisceau ) { /* Il devient donc inutile ! */
    FAISCEAU_Effacer( f_echantillon );
    f_echantillon = NULL;
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans FLUO_InitialiserEchantillon [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return NULL;
  }

  spectre = FLUO_CalculerSpectre( experience, f_detecteur, experience->E_debut );
  FAISCEAU_Effacer( f_detecteur ); /* Plus besoin de lui */
  if ( NULL == spectre ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans FLUO_CalculerSpectre()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  retour = GRF_InsererDansSerie( spectre, serie_resultats, NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ATTENTION, ERREUR dans GRF_InsererDansSerie() ==> graphe perdu\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }
  GRF_ChangerGenre( spectre, GRF_FCT_REELLE );
  GRF_ChangerType ( spectre, GR_FLUOX );
  GRF_titre( spectre, "Spectre de fluorescence total" );

  /* On divise, si ncessaire, par I0 */
  if ( experience->i0 != NULL ) {
    if ( 0.0 == i0 ) {
    } else {
      GRF_FacteurMultiplicatif( spectre, 1.0 / i0, AXE_Y, NULL );
   }
  }

  /* Calcul termin */
  return spectre;
}
