/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS

   Module d'affichage graphique des résultats
   Graduation d'un axe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "math_utils.h" /* Pour Puissance */
#include "txt_utils.h" /* Pour TXT_CNV_D_vTexte */

#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int TRC_Graduer_Decimal(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe);
static int TRC_GRD_Decimal    (Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
			       int tic_DH, int tic_GB, int p10);
static int TRC_GraduerAxe_Decimal(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
				  int delta_texte, int taille_texte);

static int TRC_Graduer_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe);
static int TRC_GRD_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
		       int tic_DH, int tic_GB, int p10, int niveau, double pos_depart);
static int TRC_GraduerAxe_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
 			      int delta_texte, int taille_texte);
static int TRC_SousGRD_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe, int p10, double v_ref,
			   double place_legende, int x_txt, int y_txt);


static void GRD_hor_decimal(Fenetre *fenetre, int x_deb, int x_fin, int y,
			    double x_min, double x_max, int precision, int ht_tics);
static void GRD_ver_decimal(Fenetre *fenetre, int y_deb, int y_fin, int x,
			    double y_min, double y_max, int precision, int ht_tics);

static void GRD_txt_ver_decimal(Fenetre *fenetre, int y_deb, int y_fin, int x,
				double y_min, double y_max, int precision);

static void GRD_txt_ver_log(Fenetre *fenetre, int y_deb, int y_fin, int x,
			    double y_min, double y_max, int precision);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -~-~-~-~-~-~-~ Aides pour la graduation -~-~-~-~-~-~-~ */

/* -------- Puissance de 10 de la graduation de plus haut niveau -------- */

inline int AXE_GraduationMajeure(double min, double max)
{
  int p10_maximale;
  double max_absolu;

  if ( ( max > 0.0 ) && ( min < 0.0 ) ) {
    /* Valeur la plus grande, en valeur absolue */
    max_absolu = fabs( max );
    if ( fabs( min ) > max_absolu ) max_absolu = fabs( min );
  } else {
    max_absolu = (max - min);
  }

  /* La puissance de 10 correspondante donne le maximum */
  p10_maximale = (int) rint( log10( max_absolu ) );

  return p10_maximale;
}

/* -------- Pas optimal pour les textes -------- */

double AXE_GraduationOptimale(double min, double max, double etendue, double haut_texte, boolean vertical)
{
  double pas;
  double place_texte;
  int p10;

  if ( ( max == min ) || ( etendue == 0.0 ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Paramètres absurdes [m=%g, M=%g, d=%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, min, max, etendue);
    return 0.0; /* Pas de graduation possible */
  }
  if ( etendue < 0.0 ) etendue = -etendue; /* Axe Y écran, orienté vers le haut */
  if ( 0.0 >= haut_texte ) {
    (void) fprintf(stderr, "%s, %s l.%u : Hauteur de texte absurde [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, haut_texte);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -10.0;
  }

  /* Quelle place occupe le texte dans la direction de l'axe ? */
  if ( VRAI == vertical ) {
    place_texte = haut_texte * 1.5; /* 50% au moins de la taille du texte pour l'interligne */
  } else {
    unsigned int nbr_cars_max, nbr_cars_min;

    /* Nombre de caractères du maximum */
    if ( ( fabs( max ) > 1000.0 ) || ( fabs( max ) < 1.0e-3 ) ) {
      nbr_cars_max = 8; /* 8 caractères au plus, notation scientifique */
    } else {
      nbr_cars_max = 3 + TXT_nbr_car_I( (int) max ); /* On n'aura probablement pas plus de 2 chiffres après la virgule */
    }

    /* Nombre de caractères du minimum */
    if ( ( fabs( min ) > 1000.0 ) || ( fabs( min ) < 1.0e-3 ) ) {
      nbr_cars_min = 8; /* 8 caractères au plus, notation scientifique */
    } else {
      nbr_cars_min = 3 + TXT_nbr_car_I( (int) min ); /* On n'aura probablement pas plus de 2 chiffres après la virgule */
    }

    if ( nbr_cars_min > nbr_cars_max )
      place_texte = haut_texte * nbr_cars_min;
    else
      place_texte = haut_texte * nbr_cars_max;
    /* En fait, larg_chiffre < haut_texte en général -> assure aussi les espaces, normalement */
  }
  if ( place_texte > etendue ) {
    (void) fprintf(stderr, "%s, %s l.%u : L'axe est trop court pour pouvoir être gradué\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1.0;
  }

  /* On en déduit le pas pour passer d'un texte à l'autre */
  pas = place_texte * ( max - min ) / (etendue);
  p10 = (int) floor( log10( pas ) );
#ifdef _DEVELOPPE_
/*   printf("%s : pas exact = %g [%e, en 10^%d]", __FUNCTION__, pas, pas, p10 ); */
#endif
  pas /= Puissance( 10.0, p10 );

  /* On arrondit à 1, 2 ou 5 * 10^n */
  if ( pas > 5.0 ) pas = 10.0;
  else if ( pas > 2.0 ) pas = 5.0;
  else if ( pas > 1.0 ) pas = 2.0;
  else pas = 1.0;

  /* On revient dans la puissance de 10 d'origine */
  pas *= Puissance( 10.0, p10 );
#ifdef _DEVELOPPE_
/*   printf(" soit %g [%e]\n", pas, pas ); */
#endif

  /* Terminé */
  return pas;
}

/* -------- Préparer pour graduer ensuite -------- */

int AXE_PreparerGraduations(double x_d, double y_d, double x_f, double y_f, double delta, InfosAxe *axe,
			    struct infos_graduation *infos)
{
  int retour;
  int niveau_max;

  /* Contrôles */
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'informations indiquées\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  retour = AXE_ValiderBornes( axe );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la validation de l'axe\n",
		   __FILE__, __FUNCTION__, __LINE__);
    if ( retour < 0 ) return -2; /* Erreur non correctible */
  }

  /* Coordonnées de dessin limitant l'axe */
  infos->x_debut = x_d; infos->y_debut = y_d;
  infos->x_fin   = x_f; infos->y_fin   = y_f;

  /* Puissances de 10 à utiliser pour graduer */
  if ( axe->proprietes & AXE_VERTICAL ) {
    if ( y_f == y_d ) {
      (void) fprintf(stderr, "%s, %s l.%u : Axe vertical réduit à un point\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -15;
    }
    niveau_max     = AXE_PrecisionOptimale( axe->min, axe->max, y_f - y_d, delta );
    infos->p10_max = AXE_GraduationMajeure( axe->min, axe->max );
    infos->echelle = ( axe->max - axe->min ) / fabs( y_f - y_d ); /* abs car Y écran, orienté vers le haut -> y_f < y_d */
  } else {
    if ( x_f == x_d ) {
      (void) fprintf(stderr, "%s, %s l.%u : Axe horizontal réduit à un point\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -15;
    }
    niveau_max     = AXE_PrecisionOptimale( axe->min, axe->max, x_f - x_d, delta );
    infos->p10_max = AXE_GraduationMajeure( axe->min, axe->max );
    infos->echelle = ( axe->max - axe->min ) / ( x_f - x_d );
  }
  if ( niveau_max <= 0) {	/* Graduations impossibles à tracer ! */
    (void) fprintf(stderr, "%s, %s l.%u : Problème de détermination des graduations [code = %d ; axe : %g -> %g ; dessin %g × %g -> %g × %g ]\n",
		   __FILE__, __FUNCTION__, __LINE__, niveau_max, axe->min, axe->max, x_d, y_d, x_f, y_f );
    return +1;
  }

  /* Niveau de graduation */
  if ( ( axe->finesse == 0 ) || ( (signed) axe->finesse > niveau_max ) ) { /* Automatique ou trop élevé */
    infos->niveau = niveau_max;
  } else {
    infos->niveau = axe->finesse;
  }

  /* Prêt */
  return 0;
}

/* -------- Trouver jusqu'à quel niveau il est raisonnable de graduer --------
   L'axe représente les valeurs entre min et max
   Il occupe la longueur etendue ; les taquets doivent être séparés d'au moins espace_minimal (échelle affichage)
 */

inline int AXE_PrecisionOptimale(double min, double max, double etendue, double espace_minimal)
{
  int p10_minimale, p10_maximale;
  double echelle, delta;

  if ( ( min == max ) || ( espace_minimal <= 0.0 ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Paramètres absurdes [min = %g, max = %g, etendue = %g, delta = %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, min, max, etendue, espace_minimal);
    return -1;
  }
  if ( etendue <= 0.0 ) etendue = -etendue; /* Axe Y écran, y_fin < y_debut car orienté vers le haut ! */
  if ( espace_minimal >= etendue ) {
    (void) fprintf(stderr, "%s, %s l.%u : Axe trop petit, graduations invisibles [etendue = %g, delta=%g] !\n",
		   __FILE__, __FUNCTION__, __LINE__, etendue, espace_minimal);
    return 0; /* Aucune graduation possible ! */
  }

  /* Calcul de l'échelle de conversion */
  echelle = ( max - min ) / etendue;

  /* À quoi correspond l'écart minimal en valeurs vraies ? */
  delta = espace_minimal * echelle;

  /* On en déduit la puissance de 10 minimale à utiliser */
  p10_minimale = (int) ceil( log10( delta ) );

  /* Puissance de 10 maximale à utiliser */
  p10_maximale = AXE_GraduationMajeure( min, max );

  /* On gradue donc sur ce nombre de niveaux */
  return ( p10_maximale - p10_minimale );
}

/* -~-~-~-~-~-~-~ Dessin d'un axe gradué -~-~-~-~-~-~-~ */

/* -------- Routine d'appel générique --------
 |
 | station          : la station VDI à utiliser pour le dessin
 | x_debut, y_debut : position du point pour le min
 | x_fin  , y_fin   : position du point pour le max
 | axe              : l'axe à représenter
 | delta            : espace minimal entre 2 graduations [pixels]
 | haut_texte       : taille de texte à utiliser [pixels]
 * ------------------------------------------- */

int TRC_AxeGradue(Fenetre *fenetre, int x_debut, int y_debut, int x_fin, int y_fin, InfosAxe *axe,
		  int delta, int haut_texte)
{
  boolean vertical __attribute__((__unused__)) = FAUX;
  char *legende;
  int retour = -1;
  struct infos_graduation infos = { 0.0, 0.0, 0.0, 0.0, 0.0, 0, 0 };

  /* Contrôles */
  if ( NULL == axe ) return -1;
  if ( GRADUATION_AUCUNE == axe->type_graduation ) return 0;
  if ( delta <= 0 ) {
    delta = 2; /* Valeur par défaut : 2 pixels */
  }

  /* Dans quel sens graduer l'axe ? */
  if ( axe->proprietes & AXE_VERTICAL ) {
    vertical = VRAI;
    if ( y_debut == y_fin ) {
      (void) fprintf(stderr, "%s, %s l.%u : Axe réduit à un point verticalement\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  } else {
    if ( x_fin == x_debut ) {
      (void) fprintf(stderr, "%s, %s l.%u : Axe réduit à un point horizontalement\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  }

  /* On prépare les graduations */
  retour = AXE_PreparerGraduations( x_debut, y_debut, x_fin, y_fin, delta, axe, &infos );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de préparer les graduations [code %d] !\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }

  /* On met la couleur demandée pour les graduations, et le style en continu */
  TRC_Couleur( axe->couleur_graduations );
  TRC_Style( BI_CONTINU );

  /* En fonction du type de graduation demandé */
  switch ( axe->type_graduation )
  {
  case GRADUATION_DECIMALE:
    retour = TRC_Graduer_Decimal( fenetre, &infos, axe );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR en voulant graduer [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
    }
    if ( axe->proprietes & AXE_AVEC_GRADUATIONS ) {
      /* Texte des graduations : à 4 pixels du bord, avec un texte de la hauteur indiquée (en pixels) */
      retour = TRC_GraduerAxe_Decimal( fenetre, &infos, axe, 4, haut_texte );
    }
    break;

  case GRADUATION_LOGARITHMIQUE:
    (void) TRC_Graduer_Log( fenetre, &infos, axe );
    if ( axe->proprietes & AXE_AVEC_GRADUATIONS ) {
      /* Texte des graduations : à 4 pixels du bord, avec un texte de la hauteur indiquée (en pixels) */
      (void) TRC_GraduerAxe_Log( fenetre, &infos, axe, 4, haut_texte );
    }
    break;

  case GRADUATION_PARABOLIQUE:
    OPTION_A_FAIRE("Graduation en unités paraboliques");
    break;

  case GRADUATION_GAUSSIENNE:
    OPTION_A_FAIRE("Graduation en unités gaussiennes");
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Méthode de graduation inconnue [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, axe->type_graduation);
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR en voulant graduer l'axe\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  /* d. Légende de l'axe */
  legende = AXE_LireLegende( axe );
  if ( legende != NULL ) {
    TRC_Couleur( axe->couleur_legende );
    if ( axe->proprietes & AXE_VERTICAL ) {
      TRC_Texte( fenetre, x_debut - 50, (y_debut + y_fin) / 2, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_BAS, 90, legende );
    } else {
      TRC_Texte( fenetre, (x_debut + x_fin) / 2, y_debut + 20, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, legende );
    }

    /* Plus besoin de la légende */
    free( legende ); legende = NULL;
  }

  return retour;
}

/* -~-~-~-~-~-~-~ Graduations décimales "classiques" -~-~-~-~-~-~-~ */

/* -------- Graduation décimale d'un axe -------- */

static int TRC_Graduer_Decimal(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe)
{
  int niveau, retour;
  int tic_droite = 0, tic_gauche = 0;

  /* Définition de la taille de base des tics */
  tic_droite = (int) rint( axe->debord_droite ); /* Débord par défaut [pixels] */
  tic_gauche = (int) rint( axe->debord_gauche ); /* Débord par défaut [pixels] */
  if ( tic_droite < 0 ) tic_droite = 10; /* Tic de 10 pixels de haut au plus */
  if ( tic_gauche < 0 ) tic_gauche = 0;

  /* Pour chaque puissance de 10, on gradue, en commençant par la plus élevée */
  for ( niveau = 0 ; niveau <= infos->niveau ; niveau++ ) {
    retour = TRC_GRD_Decimal( fenetre, infos, axe, tic_droite, tic_gauche, infos->p10_max - niveau );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la graduation de niveau %d\n",
		     __FILE__, __FUNCTION__, __LINE__, niveau + 1);
      break;
    }

    /* Préparation pour le tour suivant */
    tic_droite -= 3; if ( tic_droite < 0 ) tic_droite = 0;
    tic_gauche -= 3; if ( tic_gauche < 0 ) tic_gauche = 0;
    if ( ( tic_droite <= 0 ) && ( tic_gauche <= 0 ) ) break; /* On a fini car les graduations sont invisibles */
  }

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Graduation décimale pour une puissance de 10 imposée -------- */

static int TRC_GRD_Decimal(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe, int tic_DH, int tic_GB, int p10)
{
  int points[ 4 ] = { 0, 0, 0, 0 };
  double depart, puis_10;
  double delta_tics, valeur;
  int chiffre;

  /* Initialisation */
  puis_10 = Puissance( 10.0, p10 ); /* La puissance de 10 avec laquelle on travaille */
  delta_tics = (int) rint( puis_10 / infos->echelle );
  if ( delta_tics < 2.0 ) {
    /* Normalement, avec la préparation, on ne devrait jamais arriver ici ! */
    AVERTISSEMENT( "Intervalle entre deux tics trop faible [%g < 2 pixels ]\n", delta_tics );
    return -5;
  }

  /* On récupère le chiffre des unités */
  if ( axe->min > 0.0 ) {
    double x_p;

    x_p = axe->min / puis_10;
    chiffre = (int) ( x_p - floor( x_p / 10.0 ) * 10.0);
    depart  = ( floor( x_p ) - ( double ) chiffre ) * puis_10;
  } else {
    double x_p;

    x_p = axe->min / puis_10;
    chiffre = (int) ( x_p - ceil( x_p / 10.0 ) * 10.0);
    depart  = ( ceil( x_p ) - ( double ) chiffre ) * puis_10;
  }

  /* On saute toutes les graduations avant le début de l'axe */
  valeur = depart + puis_10 * (double) chiffre;
  while ( valeur < axe->min ) {
    chiffre++;
    valeur = depart + puis_10 * (double) chiffre;
  }

  /* Préparation du tic : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    points[ 0 ] = infos->x_debut - tic_GB;
    points[ 2 ] = infos->x_debut + tic_DH;
  } else { /* Axe horizontal */
    points[ 1 ] = infos->y_debut + tic_GB;
    points[ 3 ] = infos->y_debut - tic_DH;
  }

  /* Tant que l'on est dans la partie visible, on fait les tics */
  while ( valeur <= axe->max ) {
    /* Position des points */
    if ( axe->proprietes & AXE_VERTICAL ) {
      points[ 1 ] = (int) rint( infos->y_debut - ( valeur - axe->min ) / infos->echelle );
      points[ 3 ] = points[ 1 ];
    } else {
      points[ 0 ] = (int) rint( infos->x_debut + ( valeur - axe->min ) / infos->echelle );
      points[ 2 ] = points[ 0 ];
    }

    if ( ( chiffre % 5) == 0 ) { /* Multiple de 5 : trait un peu plus long */
      if ( axe->proprietes & AXE_VERTICAL ) {
	if ( tic_GB > 0 ) points[ 0 ] -= 2;
	if ( tic_DH > 0 ) points[ 2 ] += 2;

	TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );

	if ( tic_GB > 0 ) points[ 0 ] += 2;
	if ( tic_DH > 0 ) points[ 2 ] -= 2;
      } else {
	if ( tic_GB > 0 ) points[ 1 ] += 2;
	if ( tic_DH > 0 ) points[ 3 ] -= 2;

	TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );

	if ( tic_GB > 0 ) points[ 1 ] -= 2;
	if ( tic_DH > 0 ) points[ 3 ] += 2;
      }
    } else {
      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
    }

    /* Passage au tic suivant */
    chiffre++;
    valeur = depart + puis_10 * (double) chiffre; /* Recalcul pour éviter de rester sur place si pas trop faible ! */
  }

  /* Terminé */
  return 0;
}

/* -------- Graduer en mettant le texte -------- */

static int TRC_GraduerAxe_Decimal(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
				  int delta_texte, int taille_texte )
{
  double depart, valeur;
  unsigned int i = 0;
  double pas_graduation;
  char texte[ 100 ];

  /* Calcul du pas de la graduation */
  pas_graduation = axe->pas_graduation;
  if ( pas_graduation <= 0.0 ) {
    if ( axe->proprietes & AXE_VERTICAL ) {
      pas_graduation = AXE_GraduationOptimale( axe->min, axe->max, infos->y_fin - infos->y_debut, taille_texte, VRAI );
    } else {
      pas_graduation = AXE_GraduationOptimale( axe->min, axe->max, infos->x_fin - infos->x_debut, taille_texte, FAUX );
    }
    if ( pas_graduation <= 0.0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de trouver le pas automatiquement ! [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, (int) pas_graduation);
      return -1;
    }
  }

  /* Où commence-t-on ? [on s'assure d'être sur un multiple du pas !] */
  depart = pas_graduation * floor( axe->min / pas_graduation );

  /* On se place au début des valeurs visibles */
  valeur = depart;
  while ( valeur < axe->min ) {
    i++;
    valeur = depart + pas_graduation * (double) i;
  }

  /* On affiche les valeurs dans la région choisie */
  while ( valeur <= axe->max ) {
    /* Correction des erreurs d'arrondi pour le 0 */
    if ( ( fabs( valeur ) < 1.0e-10 ) &&
	 ( ( fabs( axe->min ) > 1.0e-10 ) || ( fabs( axe->max ) > 1.0e-10 ) ) )
      valeur = 0.0;

    (void) TXT_CNV_D_vTexte( valeur, 0.0, 5, 3, 3, texte, NULL );
    if ( axe->proprietes & AXE_VERTICAL ) {
      if ( axe->proprietes & AXE_SECONDAIRE ) {
	TRC_Texte( fenetre, infos->x_debut + delta_texte, infos->y_debut - ( valeur - axe->min ) / infos->echelle,
		   BI_TEXTE_FER_A_GAUCHE, BI_TEXTE_CENTRER_V, 0, texte );
      } else {
	TRC_Texte( fenetre, infos->x_debut - delta_texte, infos->y_debut - ( valeur - axe->min ) / infos->echelle,
		   BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, texte );
      }
    } else {
      if ( axe->proprietes & AXE_SECONDAIRE ) {
	TRC_Texte( fenetre, infos->x_debut + ( valeur - axe->min ) / infos->echelle, infos->y_debut - delta_texte,
		   BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_BAS, 0, texte );
      } else {
	TRC_Texte( fenetre, infos->x_debut + ( valeur - axe->min ) / infos->echelle, infos->y_debut + delta_texte,
		   BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, texte );
      }
    }

    /* On passe à la suivante */
    i++;
    valeur = depart + pas_graduation * (double) i;
  }

  /* Graduations terminées */
  return 0;
}

/* -~-~-~-~-~-~-~ Graduations en échelle logarithmique -~-~-~-~-~-~-~ */

static int TRC_Graduer_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe)
{
  int tic_droite = 0, tic_gauche = 0;
  int p10_min, p10_max, p10;
  double pos_depart, pos_fin, place_puissance, hauteur __attribute__((__unused__));
  int points[ 4 ] = { 0, 0, 0, 0 };

  /* Définition de la taille de base des tics */
  tic_droite = (int) rint( axe->debord_droite ); /* Débord par défaut [pixels] */
  tic_gauche = (int) rint( axe->debord_gauche ); /* Débord par défaut [pixels] */
  if ( tic_droite < 0 ) tic_droite = 10; /* Tic de 10 pixels de haut au plus */
  if ( tic_gauche < 0 ) tic_gauche = 0;

  /* Préparation du tic : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    points[ 0 ] = infos->x_debut - tic_gauche;
    points[ 2 ] = infos->x_debut + tic_droite;
    hauteur = (double) ( infos->y_debut - infos->y_fin );
  } else { /* Axe horizontal */
    points[ 1 ] = infos->y_debut + tic_gauche;
    points[ 3 ] = infos->y_debut - tic_droite;
    hauteur = (double) ( infos->x_fin - infos->x_debut );
  }

  /* Quelle place occupe une puissance complète sur l'axe [= un intervalle de 1 en log] */
  place_puissance = 1.0 / infos->echelle;
  if ( place_puissance < 10.0 ) { /* Moins de 10 pixels : seules certaines puissances seront placées */
    p10 = 1 + (int) rint( log10( infos->echelle ) );
    (void) TRC_GRD_Decimal( fenetre, infos, axe, tic_droite, tic_gauche, p10 );
    return 0;
  }

  /* Quelles sont les puissances de 10 extrêmes impliquées ? */
  p10_min = floor( axe->min );
  p10_max = ceil ( axe->max );
/*   printf("%s : (%g -> %g) = (%g -> %g) : puissance %d à %d\n", __FUNCTION__, axe->min, axe->max, */
/* 	 pow(10.0, axe->min), pow(10.0, axe->max), p10_min, p10_max); */

  /* Pour chacune de ces puissances, on gradue */
  for ( p10 = p10_min ; p10 < p10_max ; p10++ ) {
    pos_depart  = p10 - axe->min; /* log10( 10^n ) = n : 1er tic */
    pos_depart /= infos->echelle;

    pos_fin = (p10 + 1) - axe->min; /* log10( 10^n+1 ) = n + 1 : dernier tic, 1er suivant */
    pos_fin /= infos->echelle;

    /* On place le tic de fin, s'il est dans la région */
    if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
      points[ 1 ] = (int) rint( infos->y_debut - pos_fin );
      if ( ( points[ 1 ] < infos->y_debut ) && ( points[ 1 ] > infos->y_fin ) ) {
	points[ 3 ] = points[ 1 ];
	pos_depart = infos->y_debut - pos_depart;
/* 	printf("%s : tic en %d [=>%g] = entre %d [%g] et %d [%g] (Écran : %g à %g)\n", */
/* 	       __FUNCTION__, p10 + 1, Puissance( 10.0, p10 + 1 ), p10, pos_depart, p10 + 1, infos->y_debut - pos_fin, */
/* 	       infos->y_debut, infos->y_fin); */
	TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
      }
    } else {
      points[ 0 ] = (int) rint( infos->x_debut + pos_fin );
      if ( points[ 0 ] < infos->x_fin ) {
	pos_depart += infos->x_debut;

/* 	printf("%s : tic en %d [=>%g] = entre %d [%g] et %d [%g] (Écran : %g à %g)\n", */
/* 	       __FUNCTION__, p10 + 1, Puissance( 10.0, p10 + 1 ), p10, pos_depart, p10 + 1, infos->x_debut + pos_fin, */
/* 	       infos->x_debut, infos->x_fin); */

	points[ 2 ] = points[ 0 ];
	TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
      }
    }

    /* On place le niveau en-dessous avec une taille inférieure */
    TRC_GRD_Log( fenetre, infos, axe,
		 ( tic_droite > 0 ) ? ( tic_droite - 3 ) : ( 0 ), ( tic_gauche > 0 ) ? ( tic_gauche - 3 ) : ( 0 ),
		 p10, 1, Puissance( 10.0, p10 ) );
  }

  /* Graduation effectuée */
  return 0;
}

/* -------- Placer les tics logarithmiques entre deux positions --------

   p10        : la puissance de 10 que l'on représente
   pos_depart : la position _écran_ du tic pour le tic "1"
 */

static int TRC_GRD_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
		       int tic_DH, int tic_GB, int p10, int niveau, double val_ref)
{
  double positions[ 10 ];
  int points[ 4 ] = { 0, 0, 0, 0 }, point_avant;
  unsigned int i;

  /* Les tics sont-ils assez grands ? */
  if ( ( tic_DH < 2 ) && ( tic_GB < 2 ) ) return +10;

  /* Position des tics à l'écran */
  for ( i = 0 ; i < 10 ; i++ ) {
    positions[ i ] = val_ref +  Puissance( 10.0, p10 ) * (double) i; /* Valeur décimale */
    positions[ i ] = log10( positions[ i ] ); /* Correspondance en log */
  }

  /* Préparation du tic : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    points[ 0 ] = infos->x_debut - tic_GB;
    points[ 2 ] = infos->x_debut + tic_DH;

    point_avant = (int) rint( infos->y_debut - ( positions[ 0 ] - axe->min ) / infos->echelle );
  } else { /* Axe horizontal */
    points[ 1 ] = infos->y_debut + tic_GB;
    points[ 3 ] = infos->y_debut - tic_DH;

    point_avant = (int) rint( infos->x_debut + ( positions[ 0 ] - axe->min ) / infos->echelle );
  }

  /* On essaye de placer les tics intermédiaires : puissance de 10 inférieure entre "1" & "2" */
  if ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) {
    TRC_GRD_Log( fenetre, infos, axe,
 		 ( tic_DH > 0 ) ? ( tic_DH - 2 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 2 ) : ( 0 ),
 		 p10 - 1, niveau + 1, val_ref );
  }

  /* On place les tics pour "2" à "9" [1 placé avant comme "x" d'un tour supérieur, "10" = "1" du suivant]  */
  for ( i = 1 ; i < 10 ; i++ ) {
    /* On trace les tics */
    if ( axe->proprietes & AXE_VERTICAL ) {
      points[ 1 ] = (int) rint( infos->y_debut - ( positions[ i ] - axe->min ) / infos->echelle );
      if ( points[ 1 ] >= infos->y_debut ) { /* On est en-dessous */
	int p_suivant;

	/* Position du point suivant */
	p_suivant = (int) rint( infos->y_debut - ( positions[ i + 1 ] - axe->min ) / infos->echelle );

	/* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
	if ( ( p_suivant < infos->y_debut ) && ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) ) {
	  TRC_GRD_Log( fenetre, infos, axe,
		       ( tic_DH > 0 ) ? ( tic_DH - 3 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 3 ) : ( 0 ),
		       p10 - 1, niveau + 1, pow( 10.0, positions[ i ] ) );
	}
	continue;
      }

      if ( points[ 1 ] <= infos->y_fin ) { /* On déborde par en haut */
	/* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
	if ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) {
	  TRC_GRD_Log( fenetre, infos, axe,
		       ( tic_DH > 0 ) ? ( tic_DH - 3 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 3 ) : ( 0 ),
		       p10 - 1, niveau + 1, pow( 10.0, positions[ i ] ) );
	}
	return +2; /* La suite sera hors-région */
      }

      if ( ( point_avant - points[ 1 ] ) < 4 ) return +5; /* Trop serrés : la suite aussi ! */
      point_avant = points[ 1 ];

      points[ 3 ] = points[ 1 ];
    } else {
      points[ 0 ] = (int) rint( infos->x_debut + ( positions[ i ] - axe->min ) / infos->echelle );
      if ( points[ 0 ] <= infos->x_debut ) { /* On est en-dessous */
	int p_suivant;

	/* Position du point suivant */
	p_suivant = (int) rint( infos->x_debut + ( positions[ i + 1 ] - axe->min ) / infos->echelle );

	/* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
	if ( ( p_suivant > infos->x_debut ) && ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) ) {
	  TRC_GRD_Log( fenetre, infos, axe,
		       ( tic_DH > 0 ) ? ( tic_DH - 3 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 3 ) : ( 0 ),
		       p10 - 1, niveau + 1, pow( 10.0, positions[ i ] ) );
	}
	continue;
      }
      if ( points[ 0 ] >= infos->x_fin ) {
	/* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
	if ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) {
	  TRC_GRD_Log( fenetre, infos, axe,
		       ( tic_DH > 0 ) ? ( tic_DH - 3 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 3 ) : ( 0 ),
		       p10 - 1, niveau + 1, pow( 10.0, positions[ i ] ) );
	}
	return +2; /* On déborde par en haut : la suite sera hors-région */
      }

      if ( ( points[ 0 ] - point_avant ) < 4 ) return +5; /* Trop serrés : la suite aussi ! */
      point_avant = points[ 0 ];

      points[ 2 ] = points[ 0 ];
    }
    TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );

    /* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
    if ( ( 0 == axe->finesse ) || ( niveau < (int) axe->finesse ) ) {
      TRC_GRD_Log( fenetre, infos, axe,
 		   ( tic_DH > 0 ) ? ( tic_DH - 3 ) : ( 0 ), ( tic_GB > 0 ) ? ( tic_GB - 3 ) : ( 0 ),
 		   p10 - 1, niveau + 1, pow( 10.0, positions[ i ] ) );
    }
  }

  return 0;
}

/* -------- Placer les textes / axe logarithmique -------- */

static int TRC_GraduerAxe_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe,
 			      int delta_texte, int taille_texte)
{
  int p10_min, p10_max, p10;
  double pos_depart, pos_fin, place_puissance, hauteur __attribute__((__unused__));
  double place_legende;
  int x_texte = 0, y_texte = 0;
  char texte[ 100 ];

  /* Préparation du texte : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    x_texte = infos->x_debut - delta_texte;
    hauteur = (double) ( infos->y_debut - infos->y_fin );
    place_legende = taille_texte * 1.5; /* 50% de marge = intertitre */
  } else { /* Axe horizontal */
    y_texte = infos->y_debut + delta_texte;
    hauteur = (double) ( infos->x_fin - infos->x_debut );
    place_legende = taille_texte * 8; /* À affiner plus tard */
  }

  /* Quelle place occupe une puissance complète sur l'axe [= un intervalle de 1 en log] */
  place_puissance = 1.0 / infos->echelle;
  if ( place_puissance < place_legende ) { /* Moins de 10 pixels : seules certaines puissances seront placées */
    place_legende = 3 + TXT_nbr_car_I( (int) axe->max ); /* 10^entier */
    p10 = (int) ceil( place_legende * infos->echelle );
    return 0;
  }

  /* Quelles sont les puissances de 10 extrêmes impliquées ? */
  p10_min = floor( axe->min );
  p10_max = ceil ( axe->max );

  /* Pour chacune de ces puissances, on place la légende */
  for ( p10 = p10_min ; p10 <= p10_max ; p10++ ) {
    pos_depart  = p10 - axe->min; /* log10( 10^n ) = n : 1er tic */
    pos_depart /= infos->echelle;

    pos_fin = (p10 + 1) - axe->min; /* log10( 10^n+1 ) = n + 1 : dernier tic, 1er suivant */
    pos_fin /= infos->echelle;

    (void) TXT_CNV_D_vTexte( Puissance( 10.0, p10 ), 0.0, 4, 3, 3, texte, NULL );

    /* On place la puissance du tic de début, s'il est dans la région */
    if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
      y_texte = (int) rint( infos->y_debut - pos_depart );
      if ( ( y_texte < infos->y_debut ) && ( y_texte > infos->y_fin ) ) {
	TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, texte );
      }
    } else {
      x_texte = (int) rint( infos->x_debut + pos_depart );
      if ( x_texte < infos->x_fin ) {
	TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, texte );
      }
    }

    /* On place le niveau en-dessous avec une taille inférieure */
    TRC_SousGRD_Log( fenetre, infos, axe, p10, Puissance( 10.0, p10 ), place_legende, x_texte, y_texte );
  }

  /* Graduation effectuée */
  return 0;
}

static int TRC_SousGRD_Log(Fenetre *fenetre, struct infos_graduation *infos, InfosAxe *axe, int p10, double val_ref,
			   double place_legende, int x_texte, int y_texte)
{
  double positions[ 11 ];
  int point_avant, point_fin;
  unsigned int i;
  char texte[ 100 ];

  /* Position des tics à l'écran */
  for ( i = 0 ; i <= 10 ; i++ ) {
    positions[ i ] = val_ref +  Puissance( 10.0, p10 ) * (double) i; /* Valeur décimale */
    positions[ i ] = log10( positions[ i ] ); /* Correspondance en log */
  }
  if ( ( ( positions[ 9 ] - positions[ 0 ] ) / infos->echelle ) <= place_legende ) return +5;

  /* Préparation du texte : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    point_avant = y_texte;
    point_fin = (int) rint( infos->y_debut - ( positions[ 10 ] - axe->min ) / infos->echelle );
  } else { /* Axe horizontal */
    point_avant = x_texte;
    point_fin = (int) rint( infos->x_debut + ( positions[ 10 ] - axe->min ) / infos->echelle );
  }

  /* Textes pour le niveau inférieur ? */
  TRC_SousGRD_Log( fenetre, infos, axe, p10 - 1, val_ref, place_legende, x_texte, y_texte );

  /* On place les textes pour "2" à "9" [1 placé avant comme "x" d'un tour supérieur, "10" = "1" du suivant]  */
  for ( i = 1 ; i < 10 ; i++ ) {

    /* On trace les tics */
    if ( axe->proprietes & AXE_VERTICAL ) {
      y_texte = (int) rint( infos->y_debut - ( positions[ i ] - axe->min ) / infos->echelle );
      if ( y_texte >= infos->y_debut ) { /* On est en-dessous */
	int p_suivant;

	/* Position du point suivant */
	p_suivant = (int) rint( infos->y_debut - ( positions[ i + 1 ] - axe->min ) / infos->echelle );

	/* On essaye de placer les tics intermédiaires : puissance de 10 inférieure */
	if ( p_suivant < infos->y_debut ) {
	  TRC_SousGRD_Log( fenetre, infos, axe, p10 - 1, pow( 10.0, positions[ i ] ), place_legende, x_texte, y_texte );
	}
	continue;
      }

      if ( y_texte <= infos->y_fin ) { /* On déborde par en haut */
	TRC_SousGRD_Log( fenetre, infos, axe, p10 - 1, pow( 10.0, positions[ i ] ), place_legende, x_texte, y_texte );
	return +2; /* La suite sera hors-région */
      }

      if ( ( ( point_avant - y_texte ) > place_legende ) && ( ( y_texte - point_fin ) > place_legende ) ) {
	(void) TXT_CNV_D_vTexte( val_ref + Puissance( 10.0, p10 ) * (double) i, 0.0, 4, 3, 3, texte, NULL );

	TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, texte );

	point_avant = y_texte;
      }
    } else {
      x_texte = (int) rint( infos->x_debut + ( positions[ i ] - axe->min ) / infos->echelle );
      if ( x_texte <= infos->x_debut ) { /* On est en-dessous */
	continue;
      }
      if ( x_texte >= infos->x_fin ) {
	return +2; /* On déborde par en haut : la suite sera hors-région */
      }

      if ( ( ( x_texte - point_avant ) > place_legende )  && ( ( point_fin - x_texte ) > place_legende ) )  {
	(void) TXT_CNV_D_vTexte( val_ref + Puissance( 10.0, p10 ) * (double) i, 0.0, 4, 3, 3, texte, NULL );

	TRC_Texte( fenetre, x_texte, y_texte, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_BAS, 0, texte );

	point_avant = x_texte;
      }
    }

    TRC_SousGRD_Log( fenetre, infos, axe, p10 - 1, pow( 10.0, positions[ i ] ), place_legende, x_texte, y_texte );
  }

  return 0;
}

/* -------- Positionner les axes dans la région -------- */

void TRC_PlacerAxes(Fenetre *fenetre, InfosRegion2D *region)
{
  double echelle;

  /* Contrôle */
  if ( NULL == region ) return;
  if ( ( 0 == region->largeur ) || ( 0 == region->hauteur ) ) return;

  /* 1. Placer l'axe des X si nécessaire */
  if ( region->axe_Y.proprietes & AXE_VISIBLE ) {
    TRC_Couleur( region->axe_X.couleur_axe );

    echelle = ( region->axe_Y.max - region->axe_Y.min ) / (double) region->hauteur;
    if ( ( region->axe_Y.min < 0.0 ) && ( region->axe_Y.max > 0.0 ) ) { /* L'axe est visible */
      TRC_LigneH( fenetre, region->x_BG, (int) rint( region->y_BG + region->axe_Y.min / echelle ), region->largeur );
    }
  }

  /* 2. Placer l'axe des Y si nécessaire */
  if ( region->axe_Y.proprietes & AXE_VISIBLE ) {
    TRC_Couleur( region->axe_Y.couleur_axe );

    echelle = ( region->axe_X.max - region->axe_X.min ) / (double) region->largeur;
    if ( ( region->axe_X.min < 0.0 ) && ( region->axe_X.max > 0.0 ) ) { /* L'axe est visible */
      TRC_LigneV( fenetre, region->x_BG - (int) rint( region->axe_X.min / echelle ), region->y_BG - region->hauteur + 1, region->hauteur );
    }
  }
}


/* -------- Axe horizontal, gradué en échelle décimale arithmétique -------- */

static void GRD_hor_decimal(Fenetre *fenetre, int x_deb, int x_fin, int y,
			    double x_min, double x_max, int precision, int ht_tics)
{
  int niveau, i, points[4];
  double echelle;
  float amplitude; /* Intervalle de variation - en log */

  if (x_deb == x_fin) return;
  if (x_max == x_min) return;
  if (x_max < x_min) {
    double s = x_max; x_max = x_min; x_min = s;
  }

/*   printf("Graduation horizontale : de %d [%f] à %d [%f]\n", x_deb, x_min, x_fin, x_max); */
/*   printf("       axe en y = %d, tics de hauteur %d\n", y, ht_tics); */

  amplitude = (float) rint( log10(x_max - x_min) ); /* 10^amplitude = pas de base */
/*  printf("Amplitude obtenue : %f\n", amplitude); */

  echelle = (x_max - x_min) / (double) (x_fin - x_deb);
  if (precision <= 0) { /* Quelle précision pour les graduations ? Critère : 4 pixels d'écart minimum */
    precision = (int) rint(amplitude - log10(4.0 * echelle));
    if (precision < 0) /* Intervalle trop petit pour mettre la moindre graduation */
      return;
    else
      precision++; /* 1 = on gradue au premier niveau,... */
  }
/*   printf("Précision maximale : %d\n", precision); */

  for (niveau = 0 ; niveau <= precision ; niveau++) {
    int dx, x_tic;
    double x_init, puis_10;

    puis_10 = pow(10.0, amplitude - niveau); /* La puissance de 10 avec laquelle on travaille */
    dx = (int) rint( puis_10 / echelle );
    if (dx < 3) break; /* Car intervalle entre tics trop faible */

    /* On récupère le chiffre des unités */
    if (x_min > 0.0) {
      double x_p;

      x_p = x_min / puis_10;
      i = (int) ( x_p - floor(x_p / 10.0) * 10.0);
      x_init = (floor(x_p) - (double) i) * puis_10;
    } else {
      double x_p;

      x_p = x_min / puis_10;
      i = (int) ( x_p - ceil(x_p / 10.0) * 10.0);
      x_init = (ceil(x_p) - (double) i) * puis_10;
    }
#ifdef _DEVELOPPE_
/*    printf("Graduer axe : tour %d - dx = %d, gradué en %f [début : %f, i = %d]\n", niveau, dx, puis_10, x_init, i); */
#endif

    x_tic = x_deb + (int) rint( (x_init  + puis_10 * (double) i - x_min)/echelle);
    while (x_tic < x_deb) { /* On se place au premier tic */
      i++;
      x_tic = x_deb + (int) rint( (x_init + puis_10 * (double) i - x_min) / echelle );
    }

    points[1] = y;
    if (ht_tics < 0) {
      points[1] += ht_tics;
    }

    while (x_tic <= x_fin) { /* On fait les tics visibles */
      points[0] = x_tic;
      points[2] = points[0];
      points[3] = y - ht_tics;

/*      printf("i = %d, x = %f", i, x_init + puis_10 * (double) i); */
      if ( niveau > 0 ) { /* On a déjà fait un tour */
	if ( (i % 10) == 0 ) { /* On est sur un multiple de 10 : déjà mis au tour d'avant, normalement */
/* 	  printf(" -- Multiple de 10, sauté\n"); */
	  i++;
	  x_tic = x_deb + (int) rint( (x_init + puis_10 * (double) i - x_min) / echelle );
	  continue;
	} else if ( (i % 5) == 0 ) { /* Multiple de 5 : trait un peu plus long */
/* 	  printf(" -- Multiple de 5, trait long\n"); */
	  points[3] -= 2;
	} else {
/* 	  printf("\n"); */
	}
      }
      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
      i++;
      x_tic = x_deb + (int) rint( (x_init + puis_10 * (double) i - x_min) / echelle );
    }

    /* Préparation pour le tour suivant */
    if (ht_tics < 0) {
      ht_tics += 3;
      if (ht_tics > -1) break;
    } else {
      ht_tics -= 3;
      if (ht_tics < 1) break;
    }
  }
}

/* -------- Axe vertical -------- */

static void GRD_ver_decimal(Fenetre *fenetre, int y_deb, int y_fin, int x,
			    double y_min, double y_max, int precision, int ht_tics)
{
  int niveau, points[4];
  double echelle;
  double amplitude; /* Intervalle de variation - en log */

  if ( (y_deb == y_fin) || (y_max == y_min) ) {
    (void) fprintf(stderr, "GRD_ver_decimal : bornes égales, graduation impossible !\n");
    return;
  }
  if (y_max < y_min) {
    double s = y_max; y_max = y_min; y_min = s;
  }

#ifdef _DEVELOPPE_
/*   printf("Graduation verticale : de %d [%f] à %d [%f]\n", y_deb, y_min, y_fin, y_max); */
/*   printf("       axe en x = %d, tics de hauteur %d\n", x, ht_tics); */
#endif

  amplitude = rint( log10(y_max - y_min) ); /* 10^amplitude = pas de base */
/*   printf("Amplitude obtenue : %f\n", amplitude); */

  echelle = (y_max - y_min) / (double) (y_fin - y_deb);
  if (precision <= 0) { /* Quelle précision pour les graduations ? Critère : 4 pixels d'écart minimum */
    precision = (int) rint(amplitude - log10(4.0 * echelle));
    if (precision < 0) /* Intervalle trop petit pour mettre la moindre graduation */
      return;
    else
      precision++; /* 1 = on gradue au premier niveau,... */
  }
/*   printf("Précision maximale : %d\n", precision); */

  for (niveau = 0 ; niveau <= precision ; niveau++) {
    int dy, y_tic;
    double y_init, puis_10;
    int i; /* Chiffre de départ, entre -9 et 9 a priori */

    puis_10 = pow(10.0, amplitude - niveau); /* La puissance de 10 avec laquelle on travaille */
    dy = (int) rint(puis_10 / echelle);
    if (dy < 3) break; /* Car intervalle entre tics trop faible */

    /* On récupère le chiffre des unités */
    if (y_min > 0.0) {
      double y_p;

      y_p = y_min / puis_10;
      i = (int) ( y_p - floor(y_p / 10.0) * 10.0);
      y_init = (floor(y_p) - (double) i) * puis_10;
    } else {
      double y_p;

      y_p = y_min / puis_10;
      i = (int) ( y_p - ceil(y_p / 10.0) * 10.0);
      y_init = (ceil(y_p) - (double) i) * puis_10;
    }
#ifdef _DEVELOPPE_
/*     printf("Graduer axe : tour %d - dy = %d, gradué en %f, départ à %f [%d] ; tics : %d\n", */
/* 	   niveau, dy, puis_10, y_init, i, ht_tics); */
#endif
    y_tic = y_fin - (int) rint((y_init + puis_10 * (double) i - y_min)/echelle);
    while (y_tic > y_fin) { /* On se place au premier tic */
      i++;
      y_tic = y_fin - (int) rint( (y_init + puis_10 * (double) i - y_min) / echelle );
    }

    points[0] = x;
    if (ht_tics < 0) {
      points[0] -= ht_tics;
    }

    while (y_tic >= y_deb) { /* On fait les tics visibles */
      points[1] = y_tic;
      points[3] = points[1];
      points[2] = x + ht_tics;

      if ( niveau > 0 ) { /* On a déjà fait un tour */
	if ( (i % 10) == 0 ) { /* On est sur un multiple de 10 : déjà mis au tour d'avant, normalement */
/* 	  printf("Tic sauté : i = %d, y_reel = %g\n", i, y_init + puis_10 * (double) i); */
	  i++;
	  y_tic = y_fin - (int) rint( (y_init + puis_10 * (double) i - y_min) / echelle );
	  continue;
	} else if ( (i % 5) == 0 ) { /* Multiple de 5 : trait un peu plus long */
	  points[2] += 2;
	}
      }
      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );

      i++;
      y_tic = y_fin - (int) rint( (y_init + puis_10 * (float) i - y_min) / echelle );
    }

    /* Préparation pour le tour suivant */
    if (ht_tics < 0) {
      ht_tics += 3;
      if (ht_tics > -1) break;
    } else {
      ht_tics -= 3;
      if (ht_tics < 1) break;
    }
  }
}

/* -------- Axe vertical : placement des textes -------- */

static void GRD_txt_ver_decimal(Fenetre *fenetre, int y_deb, int y_fin, int x,
				double y_min, double y_max, int precision)
{
  int haut_caractere, niveau, pas;
  double echelle, delta_y;
  int attributs_texte[10];
  int y_tic;
  double y_init, puis_10;
  int i, i_debut; /* Chiffre de départ, entre -9 et 9 a priori */

  if ( (y_deb == y_fin) || (y_max == y_min) ) {
    (void) fprintf(stderr, "%s,%s l.%u : bornes égales, graduation impossible !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }
  if (y_max < y_min) {
    double s = y_max; y_max = y_min; y_min = s;
  }
  
  /* On demande la hauteur et la largeur d'un caractère... */
  vqt_attributes( _PRG_station->handle, attributs_texte );
/*   larg_caractere = attributs_texte[6]; */
  haut_caractere = attributs_texte[7] + 10; /* Espace minimal entre deux textes : 10 pixels */

  echelle = (y_max - y_min) / (double) (y_fin - y_deb);

  delta_y = echelle * (double) haut_caractere; /* Delta(y) minimal pour séparer deux légendes */
  /* Puissance de 10 à faire varier correspondante */
  niveau = (int) floor(log10( delta_y ));
  /* Multiple de cette puissance qui sert à graduer */
  pas = (int) ceil( delta_y / pow(10.0, (double) niveau) );

  if (precision <= 0) { /* Quelle précision pour les graduations ? */
    precision = niveau;
  }

  /* On gradue */
  puis_10 = pow(10.0, (double) niveau); /* La puissance de 10 avec laquelle on travaille */

  /* On récupère le chiffre des unités */
  if (y_min > 0.0) {
    double y_p;

    y_p = y_min / puis_10;
    i = (int) ( y_p - floor(y_p / 10.0) * 10.0);
    y_init = floor(y_p - i) * puis_10;
  } else {
    double y_p;

    y_p = y_min / puis_10;
    i = (int) ( y_p - ceil(y_p / 10.0) * 10.0);
    y_init = ceil(y_p) * puis_10;
  }
  y_tic = y_fin - (int) rint((y_init + puis_10 * (double) i - y_min)/echelle);
#ifdef _DEVELOPPE_
/*   printf("Graduer/texte axe : placé en %g, départ à %f [%d] [delta minimal : %f = %d pixels], pas=%d, y_tic=%d [fin=%d]\n", */
/* 	 puis_10, y_init, i, delta_y, haut_caractere, pas, y_tic, y_fin); */
#endif
  while (y_tic > y_fin) { /* On se place au premier tic */
    i++;
    y_tic = y_fin - (int) ( rint( (y_init + puis_10 * (double) i - y_min) / echelle ) );
  }
  i_debut = i;
#ifdef _DEVELOPPE_
/*   printf("                  : placé en %f, départ à %f [%d] [delta minimal : %f = %d pixels], pas=%d, y_tic=%d [fin=%d]\n", */
/* 	 puis_10, y_init, i, delta_y, haut_caractere, pas, y_tic, y_fin); */
#endif

  while (y_tic >= y_deb) { /* On fait les tics visibles */
    if ( ( (i - i_debut) % pas) == 0 ) { /* On est sur un multiple du pas de graduation : on met la valeur */
      char valeur[50];
      double y_reel;
      
      y_reel = y_init + puis_10 * (double) i;
      (void) TXT_CNV_D_vTexte( y_reel, -1.0, 3, 2, 2, valeur, NULL);
      TRC_Texte( fenetre, x - 5, y_tic, BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, valeur );
#ifdef _DEVELOPPE_
/*       printf("%s : affiché, i=%d [%d][%g]\n", __FUNCTION__, i, (i - i_debut) % pas, y_reel ); */
/*     } else { */
/*       printf("%s : ignoré, i=%d [%d][%g]\n", __FUNCTION__, i, (i - i_debut) % pas, y_init + puis_10 * (double) i ); */
#endif
    }

    i++;
    y_tic = y_fin - (int) rint( (y_init + puis_10 * (double) i - y_min) / echelle );
  }

}

/* -~-~-~-~-~-~-~ Graduations logarithmiques -~-~-~-~-~-~-~ */

/* -------- Axe vertical : placement des textes -------- */

static void GRD_txt_ver_log(Fenetre *fenetre, int y_deb, int y_fin, int x,
			    double y_min, double y_max, int precision)
{
  int haut_caractere, attributs_texte[10];
  int y_precedent = -2000;
  double echelle;
  int p;
  double p_10;

  if ( (y_deb == y_fin) || (y_max == y_min) ) {
    (void) fprintf(stderr, "GRD_txt_ver_log : bornes égales, graduation impossible !\n");
    return;
  }
  if (y_max < y_min) {
    double s = y_max; y_max = y_min; y_min = s;
  }
  if (y_min < 0) {
    (void) fprintf(stderr, "GRD_txt_ver_log : borne négative [%f, %f], graduation impossible !\n", y_min, y_max);
    return;
  }
  
  /* On demande la hauteur d'un caractère... */
  vqt_attributes( _PRG_station->handle, attributs_texte );
  haut_caractere = attributs_texte[7]+ 2; /* Espace minimal entre deux textes : 2 pixels */

  echelle = ((double) (y_fin - y_deb)) / ( log(y_max) - log(y_min) );
  if ( precision <= 0 ) {
    precision = (int) ( (log(y_max) - log(y_min)) / log(10.0) );
  }

  /* Écriture des textes */
  p_10 = (int) (log( y_max ) / log(10.0)) - precision;
  p = 0;
  do {
    double facteur;
    int i;

    facteur = pow(10.0, p_10);

    /* Unités : tiret plus grand */
    for (i = 1 ; i <= 9 ; i++) {
      double y;

      y = facteur * (double) i;
      if ( (y >= y_min) && (y <= y_max) ) {
	int y_texte;

	y_texte = y_fin - (log(y) - log(y_min)) * echelle;
 	printf("... %f [en X=%d,Y=%d] -> ", y, 5, y_texte);
	if ( abs(y_texte - y_precedent) >= haut_caractere ) { /* Pas de chevauchement */
	  char valeur[50];

	  (void) TXT_CNV_D_vTexte( y, -1.0, 3, 2, 2, valeur, NULL);
/*  	  printf("%s\n", valeur); */
	  TRC_Texte( fenetre, x - 5, y_texte, BI_TEXTE_FER_A_DROITE, BI_TEXTE_CENTRER_V, 0, valeur );
	  y_precedent = y_texte;
	} else {
/* 	  printf("ignoré\n"); */
	}
      }
    }
    p++;
    p_10++;

  } while ( p <= precision );
}

/* -------- Routine principale (visible de l'extérieur) ---------------\
 | Ne renvoie rien                                                     |
 |                                                                     |
 | station : la station VDI dans laquelle dessiner                     |
 | x_deb, y_deb : coordonnées de début de l'axe (en coordonnées écran) |
 | x_fin, y_fin : coordonnées de fin de l'axe (en coordonnées écran)   |
 | min, max     : les valeurs extrêmes de cet axe                      |
 | type_axe     : comment graduer [cf Include/tracer.h]                |
 | precision    : la précision maximale de graduation (0 : automatique)|
 | ht_tics      : taille des tics pour les 1/10                        |
 |                     (<0 : déborde des deux côtés)                   |
 | avec_texte   : on ajoute la valeur numérique en-dessous du tic      |
 \-------------------------------------------------------------------- */

void TRC_gradue(Fenetre *fenetre, int x_deb, int y_deb, int x_fin, int y_fin,
		double min, double max, int type_axe, int precision, int ht_tics, boolean avec_texte)
{
  if ( y_deb == y_fin ) { /* Axe horizontal */
    switch ( type_axe ) {
    case GRADUE_DECIMAL:
      GRD_hor_decimal( fenetre, x_deb, x_fin, y_deb, min, max, precision, ht_tics);
      break;
    }
  } else if ( x_deb == x_fin ) { /* Axe vertical */
    switch ( type_axe ) {
    case GRADUE_DECIMAL:
      GRD_ver_decimal( fenetre, y_deb, y_fin, x_deb, min, max, precision, ht_tics);
      if ( VRAI == avec_texte ) {
	GRD_txt_ver_decimal( fenetre, y_deb, y_fin, x_deb, min, max, precision);
      }
      break;

    case GRADUE_LOG:
      if ( VRAI == avec_texte ) {
	GRD_txt_ver_log( fenetre, y_deb, y_fin, x_deb, min, max, precision);
      }
      break;
    }
  }
}
