/* LASE - Logiciel d'Analyse des spectres EXAFS
   (C) 1997 - 2002, Emmanuel CURIS

   Affichage graphique (rsultat,...)
   Trouver l'information sous la souris
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "atomes.h"
#include "feff.h"
#include "modeles.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

#ifdef AVEC_OPEN_GL_
static int TRC_GL_ClicVersSegment(Fenetre *fenetre, int x, int y,
				  double *x1, double *y1, double *z1, double *x2, double *y2, double *z2);
# define TRC_ClicVersSegment TRC_GL_ClicVersSegment
#endif

#ifndef TRC_ClicVersSegment
# define TRC_ClicVersSegment(f, x, y, x1, y1, z1, x2, y2, z2)  (FAUX * ( (int) f ) * (x) * (y) )
#endif

static double TRC_AtomeSurLaDroite(double alpha, double beta, double gamma, double A,
				   double x, double y, double z, double rayon);

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

/* -~-~-~-~-~-~-~-~ Routines utilitaires gnrales  -~-~-~-~-~-~-~-~ */

/* -------- Fentres Open-GL 3D : conversion (x, y) clic --------\
 |  Renvoie VRAI si tout s'est bien pass                        |
 |  Renvoie les deux points extrmes du segment de droite        |
 |   groupant tous les points potentiellement cliqus sur la vue |
 |   en 2D.  charge de la routine appelante de trouver lequel de|
 |   ces points a effectivement t cliqu...                    |
 |                                                               |
 | fenetre : la fentre clique                                  |
 | x, y    : position du clic dans la fentre                    |
 | x1, y1, z1 ; x2, y2, z2 : extrmits du segment               |
 \-------------------------------------------------------------- */

#ifdef AVEC_OPEN_GL_
static int TRC_GL_ClicVersSegment(Fenetre *fenetre, int x, int y,
				  double *x1, double *y1, double *z1, double *x2, double *y2, double *z2)
{
  GLint vp[4]; /* Pour le ViewPort */
  GLdouble x_gl, y_gl; /* Position du clic dans le ViewPort */
  GLdouble p_matrice[16], m_matrice[16]; /* Matrices utilises pour la projection */
  GLdouble x1_reel, y1_reel, z1_reel, /* Point le plus proche de l'observateur */
    x2_reel, y2_reel, z2_reel; /* Point le plus loin de l'observateur */

  if (BI_OpenGL_actif == FAUX) return FAUX; /* OpenGL inactif ==> pas de vue 3D */
  if (NULL == fenetre) return FAUX;

  /* Voir pour mettre le bon contexte OpenGL ? */

  /* On rcupre les informations OpenGL */
  glGetIntegerv(GL_VIEWPORT, vp); /* ViewPort */
  glGetDoublev(GL_MODELVIEW_MATRIX , m_matrice); /* Matrice de projection "modelview" */
  glGetDoublev(GL_PROJECTION_MATRIX, p_matrice); /* Matrice de projection */

#ifdef _DEVELOPPE_
  {
    unsigned int i, j;

    printf( "Matrice MODELVIEW :\n" );
    for (i = 0 ; i < 4 ; i++ ) {
      printf( " ( " );
      for (j = 0 ; j < 4 ; j++ ) {
	printf( "%g ", m_matrice[ i + 4 * j ] );
      }
      printf( ")\n" );
    }

    printf( "Matrice PROJECTION :\n" );
    for (i = 0 ; i < 4 ; i++ ) {
      printf( " ( " );
      for (j = 0 ; j < 4 ; j++ ) {
	printf( "%g ", p_matrice[ i + 4 * j ] );
      }
      printf( ")\n" );
    }

    printf( "Point cliqu dans la fentre : (%d %d) ; w_x = %d, w_y = %d\n", x, y, fenetre->w_x, fenetre->w_y );
  }
#endif

  /* Positions du clic dans le ViewPort */
  x_gl = (GLdouble) x;
  y_gl = (GLdouble) (vp[3] - 1 - y); /* ViewPort orient de bas en haut, crans de haut en bas */

  /* On rcupre les points extrmes */
  gluUnProject( x_gl, y_gl, 0.0, m_matrice, p_matrice, vp, &x1_reel, &y1_reel, &z1_reel);
  gluUnProject( x_gl, y_gl, 1.0, m_matrice, p_matrice, vp, &x2_reel, &y2_reel, &z2_reel);

  /* On renvoie le rsultat */
  *x1 = (double) x1_reel; *y1 = (double) y1_reel; *z1 = (double) z1_reel;
  *x2 = (double) x2_reel; *y2 = (double) y2_reel; *z2 = (double) z2_reel;

  return VRAI; /* Tout s'est bien pass */
}
#endif

/* -~-~-~-~-~-~-~-~ Modles structuraux : atome cliqu -~-~-~-~-~-~-~-~ */

/* -------- Position sur la droite du point d'intersection --------\
 | Cherche les solutions de l'quation (en t) ci-dessous :         |
 |    (x1 + alpha * t - x) + (y1 + beta * t - y) + (z1 + gamma * t - z) = R <=>
 |    A * t + 2 * B * t + C = 0                                   |
 | La solution doit tre entre 0 et 1, sinon renvoie -1.0          |
 |                                                                 |
 | alpha, beta, gamma : quation de la droite                      |
 | A                  : 1er coefficient du trinme                 |
 | x, y, z            : position du centre de la sphre            |
 | rayon              : rayon de la sphre                         |
 \---------------------------------------------------------------- */

static double TRC_AtomeSurLaDroite(double alpha, double beta, double gamma, double A,
				   double x, double y, double z, double rayon)
{
  double B, C, delta, q;
  double t1, t2;

  B = alpha * x + beta * y + gamma * z;
  C = x * x + y * y + z * z - rayon * rayon;
  if (C < 0.0) { /* C/A = t1 * t2, comme A > 0, si C < 0 alors l'oeil est dans la sphre ;
		    Comme j'utilise GL_CULL_FACE et que la sphre est dfinie seulement pour
		    l'extrieur, une telle sphre est donc invisible -> on n'en tient pas compte */
    printf( "C = %g <0, hors cran", C );
#ifdef _DEVELOPPE_
    /*	  printf("Atome sur la droite, mais  cheval sur le segment et autre chose\n"); */
#endif
    return -1.0;
  }

  delta = B * B - A * C; /* On travaille avec le discriminant RDUIT */
  if ( delta < 0.0 ) { /* Pas de racine... */
    printf( " delta = %g < 0 [A=%g, B=%g, C=%g]", delta, A, B, C ); 
    return -1.0;
  }

  /* Terme temporaire */
  q = -B;
  if ( B > 0.0 ) {
    q += -sqrt( delta );
  } else if ( B < 0.0 ) {
    q += +sqrt( delta );
  }

  /* Rq. t1 & t2 > 0.0 tous les deux ou < 0.0 tous les deux, car C/A positif */
  t1 = q / A;
  printf( " t1 = %g", t1 );
  if (t1 < 0.0) { /* Atome bien avant l'oeil... */
    return -1.0;
  }
  t2 = C / q;
  printf( " t2 = %g", t2 );

  /* On s'arrange pour toujours avoir t1 < t2 */
  if (t1 > t2) {
    double t = t2; t2 = t1; t1 = t;
  }
  if (t1 > 1.0) { /* Atome aprs la fin de la vue */
    printf( "Aprs la vue" );
    return -1.0;
  }

  return t1; /* Plus proche point d'intersection */
}

/* -------- Trouver l'atome le plus proche de l'oeil sur la droite indique ----------\
 |    Renvoie l'atome trouv, NULL en cas d'erreur ou si aucun n'a t trouv         |
 |                                                                                    |
 | Suppose que l'atome doit tre sur le segment de droite [(x1, y1, z1) (x2, y2, z2)] |
 |  renvoie par dfaut l'atome trouv le plus proche du point (x1, y1, z1)            |
 |                                                                                    |
 | fenetre  = la fentre dans laquelle on a cliqu                                    |
 | molecule = molcule dans laquelle chercher                                         |
 | x, y     = position du clic                                                        |
 | mode     = comment est affiche la molcule  l'cran...                           |
 \----------------------------------------------------------------------------------- */

Atome *ATM_trouver(Fenetre *fenetre, Molecule *molecule, double x, double y, int mode)
{
  Atome *trouve = NULL, *atome;
  double t_min = 2.0; /* Valeur absurde... */
  double A, alpha, beta, gamma;
  double x1 = 0.0, y1 = 0.0, z1 = 0.0, x2 = 0.0, y2 = 0.0, z2 = 0.0;

  if ( NULL == molecule ) return NULL;
  if ( FAUX == ( V3D_VOIR_ATOMES & mode) ) return NULL; /* On ne voit pas les atomes... */

  /* On rcupre les deux points extrmes */
  if ( FAUX == TRC_ClicVersSegment(fenetre, x, y, &x1, &y1, &z1, &x2, &y2, &z2) )
    return NULL;
  printf( "Le segment va de (%g, %g, %g)  (%g, %g, %g)\n", x1, y1, z1, x2, y2, z2 );

  alpha = (x2 - x1); /* Le segment a pour quation { X = x1 + alpha * t */
  beta  = (y2 - y1); /*    avec t dans [0, 1]      { Y = y1 + beta  * t */
  gamma = (z2 - z1); /*                            { Z = z1 + gamma * t */

  /* L'atome est une sphre de rayon R place en (x, y, z) -> quation (X-x) + (Y-y) + (Z-z) = R
     Le segment coupe cette sphre <=> il y a au moins un point d'intersection <=> l'quation :
      (x1 + alpha * t - x) + (y1 + beta * t - y) + (z1 + gamma * t - z) = R <=>
                        A * t + 2 * B * t + C = 0
     a au moins une solution avec 0 <= t <= 1
  */

  A = alpha * alpha + beta * beta + gamma * gamma;
  if (0.0 >= A) {
    (void) fprintf(stderr, "ATM_trouver : problme dans la vue, les points extrmes sont confondus !\n");
    return NULL;
  }

  /* Premier cas : on n'a affich que les atomes dans le modle */
  if ( mode & V3D_VOIR_MODELE ) {
    double x, y, z, r;
    double t;
    Absorbeur *absorbeur;

    if (NULL == molecule->absorbeurs) {
      (void) fprintf(stderr, "ATM_trouver : molcule sans absorbeur, pas de modle\n");
      return NULL;
    }
    absorbeur = molecule->absorbeurs + molecule->absorbeur_courant;

    if ( ((mode & V3D_VOIR_SELECTION) == FAUX) || ( VRAI == absorbeur->absorbeur->marques.visible ) ) {
      x = x1 - absorbeur->absorbeur->position.x; y = y1 - absorbeur->absorbeur->position.y; z = z1 - absorbeur->absorbeur->position.z;
      r = absorbeur->absorbeur->rayon / 100.0; /* Rayons en pm mais unit d'affichage = angstrom */
      if (mode & V3D_ATOME_REDUIT) /* Petites sphres uniquement */
	r /= 3.0;

      t = TRC_AtomeSurLaDroite(alpha, beta, gamma, A, x, y, z, r);
      if (t >= 0.0) {
	trouve = absorbeur->absorbeur;
	t_min = t;
      }
    }

    if (NULL == absorbeur->modeles) { /* Il n'y a que l'absorbeur, de toute faon... */
      return trouve;
    }

    /* Il faudrait ici s'occuper du modle FEFF */
    switch (absorbeur->modele_actif->type)
    {
    case MODELE_EXAFS_CHEMINS:
      {
	Modele_FEFF *modele;
	FEFF_atome *atm_feff;

	modele = absorbeur->modele_actif->infos_modele.feff;
	atm_feff = modele->parametres.prem_atome;
	while ( atm_feff != NULL ) {
	  if ( ((mode & V3D_VOIR_SELECTION) == FAUX) || ( VRAI == atm_feff->atome->marques.visible) ) {
	    x = x1 - atm_feff->atome->position.x; y = y1 - atm_feff->atome->position.y; z = z1 - atm_feff->atome->position.z;
	    r = atm_feff->atome->rayon / 100.0; /* Rayons en pm mais unit d'affichage = angstrom */
	    if (mode & V3D_ATOME_REDUIT) /* Petites sphres uniquement */
	      r /= 3.0;

	    t = TRC_AtomeSurLaDroite(alpha, beta, gamma, A, x, y, z, r);
	    if ( (t >= 0.0) && (t < t_min)) {
	      trouve = atm_feff->atome;
	      t_min = t;
	    }
	  }

	  atm_feff = atm_feff->suivant;
	}
      }
      break;
    }

    return trouve;
  }

  /* Si on es l : on cherche dans tous les atomes */
/*   printf("Recherche dans tous les atomes...\n"); */
  for ( atome = molecule->premier ; atome != NULL ; atome = atome->suivant ) {
    if ( ((mode & V3D_VOIR_SELECTION) == FAUX) || ( VRAI == atome->marques.visible) ) {
      double x, y, z, r;
      double t;

      x = x1 - atome->position.x; y = y1 - atome->position.y; z = z1 - atome->position.z;
      r = atome->rayon / 100.0; /* Rayons en pm mais unit d'affichage = angstrom */
      printf( "Atome %d : rayon = %g ==> %g ; en (%g, %g, %g) -", atome->num_atomique, atome->rayon, r,
	      atome->position.x, atome->position.y, atome->position.z ); 
      if (mode & V3D_ATOME_REDUIT) /* Petites sphres uniquement */
	r /= 3.0;

      t = TRC_AtomeSurLaDroite(alpha, beta, gamma, A, x, y, z, r);
      if ( (t >= 0.0) && (t < t_min) ) {
	trouve = atome;
	t_min = t;
      }
      printf("\n");
    }
  }

  return trouve;
}
