/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   Lecture "avance" des fichiers en binaire
*/

#define __USE_BSD /* Pour avoir, si possible, la dfinition de little/big endian */

#include <stdint.h> /* Pour avoir les types ISO avec taille indique */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <errno.h>

#ifdef _LINUX_
# include <endian.h>
#endif
#ifdef _SOLARIS_
# include <sys/isa_defs.h>
# ifdef _LITTLE_ENDIAN
#  define BYTE_ORDER 1234
# endif
# ifdef _BIG_ENDIAN
#  define BYTE_ORDER 4321
# endif
#endif

/* Les codages possibles des mots longs */
#ifndef LITTLE_ENDIAN
# define LITTLE_ENDIAN 1234
#endif
#ifndef BIG_ENDIAN
# define BIG_ENDIAN 4321
#endif
#ifndef PDP_ENDIAN
# define PDP_ENDIAN 3412
#endif

#if !defined( BYTE_ORDER )
# ifdef PETIT_BOUTIEN_
#  define BYTE_ORDER LITTLE_ENDIAN
# endif
# ifdef GRAND_BOUTIEN_
#  define BYTE_ORDER BIG_ENDIAN
# endif
#endif

/* On essaye de deviner,  partir du processeur... */
#if !defined( BYTE_ORDER )
# if defined( __ia64 ) || defined( __i386 )
#  define BYTE_ORDER LITTLE_ENDIAN
# endif
# if defined( __sparc )
#  define BYTE_ORDER BIG_ENDIAN
# endif
#endif

#include "bin_utils.h"
#include "utils_3d.h"
#include "txt_utils.h"

/* ~-~-~-~-~-~-~-~- Chargement des types C binaires de base ~-~-~-~-~-~-~-~- */

/* -------- Les types entiers signs -------- */

/* -------- 8 bits = char = short... -------- */

inline short BIN_Lire_8(FILE *fichier)
{
  signed char c;
  int n_lu;

  n_lu = fread( &c, sizeof( signed char ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

  return (short) c;
}

/* -------- 16 bits = 2 * char = mot suppos int... -------- */

inline int BIN_Lire_16(FILE *fichier, int mode)
{
  union {
    int16_t i;
    char c[2];
  } valeur;
  int n_lu;

  n_lu = fread( &(valeur.i), sizeof( int16_t ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

#if BYTE_ORDER == LITTLE_ENDIAN /* Machine en codage Intel = LSB */
  if ( BIN_MODE_SOURCE_MSB == mode ) {
    char c;

    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
  }
#else
  if ( BIN_MODE_SOURCE_LSB == mode ) {
    char c;

    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
  }
#endif

  return (int) valeur.i;
}

/* -------- 32 bits = 4 * char = long mot suppos long... -------- */

inline long BIN_Lire_32(FILE *fichier, int mode)
{
  char c;
  union {
    int32_t l;
    char c[2];
  } valeur;
  int n_lu;

  n_lu = fread( &(valeur.l), sizeof( int32_t ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

#if BYTE_ORDER == LITTLE_ENDIAN /* Machine en codage Intel = LSB */
  if ( BIN_MODE_SOURCE_MSB == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_IMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
    c = valeur.c[ 2 ]; valeur.c[ 2 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  } else if ( BIN_MODE_SOURCE_MMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  }
#else
  if ( BIN_MODE_SOURCE_LSB == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_IMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_MMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
    c = valeur.c[ 2 ]; valeur.c[ 2 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  }
#endif

  return (long) valeur.l;
}

/* -------- Les types entiers non-signs -------- */

/* -------- 8 bits = char = unsigned short... -------- */

inline unsigned short BIN_Lire_8U(FILE *fichier)
{
  unsigned char c;
  int n_lu;

  n_lu = fread( &c, sizeof( unsigned char ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

  return (unsigned short) c;
}

/* -------- 16 bits = 2 * char = mot suppos int... -------- */

inline unsigned int BIN_Lire_16U(FILE *fichier, int mode)
{
  union {
    uint16_t i;
    char c[2];
  } valeur;
  int n_lu;

  n_lu = fread( &(valeur.i), sizeof( uint16_t ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

#if BYTE_ORDER == LITTLE_ENDIAN /* Machine en codage Intel = LSB */
  if ( BIN_MODE_SOURCE_MSB == mode ) {
    char c;

    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
  }
#else
  if ( BIN_MODE_SOURCE_LSB == mode ) {
    char c;

    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
  }
#endif

  return (unsigned int) valeur.i;
}

/* -------- 32 bits = 4 * char = long mot suppos long... -------- */

inline unsigned long BIN_Lire_32U(FILE *fichier, int mode)
{
  char c;
  union {
    uint32_t l;
    char c[2];
  } valeur;
  int n_lu;

  n_lu = fread( &(valeur.l), sizeof( uint32_t ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

#if BYTE_ORDER == LITTLE_ENDIAN /* Machine en codage Intel = LSB */
  if ( BIN_MODE_SOURCE_MSB == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_IMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
    c = valeur.c[ 2 ]; valeur.c[ 2 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  } else if ( BIN_MODE_SOURCE_MMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  }
#else
  if ( BIN_MODE_SOURCE_LSB == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_IMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
    c = valeur.c[ 1 ]; valeur.c[ 1 ] = valeur.c[ 2 ]; valeur.c[ 2 ] = c;
  } else if ( BIN_MODE_SOURCE_MMIXTE == mode ) {
    c = valeur.c[ 0 ]; valeur.c[ 0 ] = valeur.c[ 1 ]; valeur.c[ 1 ] = c;
    c = valeur.c[ 2 ]; valeur.c[ 2 ] = valeur.c[ 3 ]; valeur.c[ 3 ] = c;
  }
#endif

  return (unsigned long) valeur.l;
}

/* -------- Chargement d'un flottant -------- */

inline float BIN_LireFlottant(FILE *fichier, int mode)
{
  int n_lu;
  union {
    float f;
    char c[ 4 ];
  } valeur;

  /* On lit le flottant */
  n_lu = fread( &(valeur.f), sizeof( float ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR de lecture du fichier.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0.0;
  }

  /* On fait l'change, si ncessaire */
#if BYTE_ORDER == LITTLE_ENDIAN /* Machine en codage Intel = LSB */
  if ( BIN_MODE_SOURCE_MSB == mode ) {
    char c;

    c = valeur.c[3]; valeur.c[3] = valeur.c[0]; valeur.c[0] = c;
    c = valeur.c[2]; valeur.c[2] = valeur.c[1]; valeur.c[1] = c;
  }
#else
  if ( BIN_MODE_SOURCE_LSB == mode ) {
    char c;

    c = valeur.c[3]; valeur.c[3] = valeur.c[0]; valeur.c[0] = c;
    c = valeur.c[2]; valeur.c[2] = valeur.c[1]; valeur.c[1] = c;
  }
#endif

  /* On renvoie la vraie valeur */
  return valeur.f;
}

/* ~-~-~-~-~-~-~-~- Chargement d'lments complexes ~-~-~-~-~-~-~-~- */

/* -------- Chargement d'une chane de caractres de longueur quelconque -------- */

inline char *BIN_CH_chaine(FILE *fichier)
{
  unsigned long longueur;
  char *texte;

  if ( NULL == fichier ) return NULL;

  fread( &longueur, sizeof( unsigned long ), 1, fichier );
  if ( 0 == longueur ) { /* Chane absente */
    return NULL;
  }

  printf("%s : position = %lu [%x] ; %lu cars  lire\n", __FUNCTION__, (unsigned long) ftell( fichier ), (unsigned) ftell( fichier ), longueur );

  texte = (char *) malloc( longueur + 1 ); /* N'oublions pas le 0 final... */
  if ( NULL == texte ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver pour %lu caractres\n",
		   __FILE__, __FUNCTION__, __LINE__, longueur);

    /* On saute la chane */
    fseek( fichier, longueur, SEEK_CUR );
    return NULL;
  }

  /* On lit la chane... */
  fread( texte, sizeof( char ), longueur, fichier );
  texte[ longueur ] = 0; /* ... sans oublier le 0 final ! */

  /* On a fini */
  printf("%s : Texte lu = %s\n", __FUNCTION__, texte );
  return texte;
}

/* -------- Chargement d'une orientation 3D -------- */

inline int BIN_CH_orientation(FILE *fichier, struct orientation_3d *orientation)
{
  int retour, n_lu;

  if ( NULL == fichier ) return -1;
  if ( NULL == orientation ) {
    (void) fprintf(stderr, "%s,%s l.%u : Orientation nulle !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* Position du centre du repre */
  retour = BIN_CH_vecteur_3d( fichier, &(orientation->position) );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans BIN_CH_vecteur_3d [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -4;
  }

  /* Angles dfinissant le repre */
  n_lu = fread( &(orientation->phi), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger phi [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->phi );
    return -5;
  }
  n_lu = fread( &(orientation->theta), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger theta [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->theta );
    return -5;
  }
  n_lu = fread( &(orientation->alpha), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger alpha [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->alpha );
    return -5;
  }

  /* Pas de problme */
  return 0;
}

/* -------- Chargement d'un vecteur  3 dimensions -------- */

inline int BIN_CH_vecteur_3d(FILE *fichier, struct vect_3d *vecteur)
{
  int n_lu;

  if ( NULL == fichier ) return -1;
  if ( NULL == vecteur ) {
    (void) fprintf(stderr, "%s,%s l.%u : Vecteur nul !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* Coordonnes X, Y et Z */
  n_lu = fread( &(vecteur->x), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger X [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->x );
    return -5;
  }
  n_lu = fread( &(vecteur->y), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger Y [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->y );
    return -5;
  }
  n_lu = fread( &(vecteur->z), sizeof( double ), 1, fichier );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de charger Z [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->z );
    return -5;
  }

  /* Pas de problme */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Analyse d'un dossier ~-~-~-~-~-~-~-~- */

/* -------- Liste des fichiers dans un dossier indiqu --------
    Renvoie une table des noms,  librer avec free()
    chaque nom de la liste doit aussi tre libr avec free()
 */

char **DIR_ListeFichiers(const char *dossier, const char *masque, unsigned long *nombre_fichiers)
{
  char **noms = NULL;
  DIR *d_dossier;
  unsigned long nombre = 0;
  struct dirent *fichier;

  /* Contrles */
  if ( NULL == dossier ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun dossier indiqu !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Ouverture du dossier en lecture */
  d_dossier = opendir( dossier );
  if ( NULL == d_dossier ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'ouvrir le dossier %s --- cause : %d\n",
		   __FILE__, __FUNCTION__, __LINE__, dossier, errno);
    return NULL;
  }

  /* On compte, dans ce dossier, combien de fichiers --- en mmorisant leur nom au vol ! */
  fichier = readdir( d_dossier );
  while ( fichier != NULL ) {
    char *nom;
    int retour;
    struct stat infos_fichier;

    /* Contrle du nom */
    if ( masque != NULL ) {
      (void) fprintf(stderr, "%s, %s l.%u :  FAIRE --- comparer le masque [%s] au nom de fichier [%s]\n",
		     __FILE__, __FUNCTION__, __LINE__, masque, fichier->d_name);
    }

    /* Chemin relatif du fichier */
    nom = TXT_genere_chaine( "%s/%s", dossier, fichier->d_name );
    if ( NULL == nom ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problmes de mmoire / recherche annule\n",
		     __FILE__, __FUNCTION__, __LINE__);
      break;
    }

    /* Infos sur le fichier */
    retour = stat( nom, &infos_fichier );
    if ( 0 == retour ) {
      if ( S_ISREG( infos_fichier.st_mode ) ) { /* Fichier classique */
	char **nv_noms;

	/* On rserve pour un nom de plus */
	nv_noms = (char **) realloc( noms, sizeof( char * ) * ( nombre + 1 ) );
	if ( NULL == nv_noms ) {
	  (void) fprintf(stderr, "%s, %s l.%u : Impossible de rajouter un nom dans la liste... On s'arrte donc l !\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  break;
	}
	noms = nv_noms;

	/* On a un fichier de plus dans la liste */
	nombre++;

	/* On le mmorise donc */
	noms[ nombre - 1 ] = nom;
      } else {
	free( nom ); nom = NULL;
      }
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR / fichier %s, errno = %d\n",
		     __FILE__, __FUNCTION__, __LINE__, nom, errno );
      free( nom ); nom = NULL;
    }

    fichier = readdir( d_dossier );
  }

  /* Plus besoin du dossier : on le ferme ! */
  (void) closedir( d_dossier ); d_dossier = NULL;

  /* On renvoie le nombre de fichiers */
  *nombre_fichiers = nombre;

  /* et la liste des noms [chaque nom  librer avec free()] */
  return noms;
}
