/* Bibliothque d'interface
   (C) 1998-2003, Emmanuel CURIS

   Manipulation de textes en relation avec l'affichage du temps
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#include "txt_utils.h"

/* -------- Variables locales -------- */

static const char *noms_mois[ 12 ] = {
  "janvier", "fvrier", "mars", "avril", "mai", "juin", "juillet", "aot", "septembre", "octobre", "novembre", "dcembre"
};

static const char *noms_jours[ 7 ] = {
  "lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "dimanche"
};

/* -------- Fonctions locales -------- */

static int TXT_Date_LeverAmbiguite   (int ordre, const char *format, int separateur);
static int TXT_Date_TrouverSeparateur( const char *texte );
static int TXT_Date_TrouverOrdre     ( unsigned int valeurs[ 3 ], int ordre );

/* -------- Ralisation pratique ------ */

/* ~-~-~-~-~-~-~-~-~ Crer un texte contenant la date ~-~-~-~-~-~-~-~-~ */

char *TXT_CreerDate(const char *format, struct tm *date)
{
  char *texte;
  size_t lg_texte, lg_ecrit;
  int sortie = FAUX;

  if ( NULL == date ) {		/* On rcupre la date actuelle */
    time_t heure;

    (void) time( &heure );	/* Heure courante, en nombre de secondes depuis le 1er janvier 1970 */
    date = localtime( &heure );	/* Conversion en structure tm */
    if ( NULL == date ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans localtime()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
    /* Remarque : d'aprs le man, la structure renvoye par localtime() est statique
       ==> on ne la libre pas... */
  }
  if ( NULL == format ) {
    format = "%x"; /* Format par dfaut pour le pays concern */
  }

  /* On rserve le texte. 50 caractres devraient suffire, mais on peut tendre aprs */
  lg_texte = 50;
  do {
    texte = (char *) calloc( lg_texte, sizeof( char ) );
    if ( NULL == texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver le texte [%lu caractres]\n",
		     __FILE__, __FUNCTION__, __LINE__, (unsigned long) lg_texte);
      return NULL;
    }

    /* Conversion */
    lg_ecrit = strftime( texte, lg_texte - 1 /* Place pour le 0 final */, format, date );
    if ( 0 == lg_ecrit ) { /* Trop court ? */
      free( texte );
      texte = NULL;
      lg_texte += 50;
    } else {
      sortie = VRAI;
    }

    if ( lg_texte > 1000 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Format trange [%s], abandon\n",
		     __FILE__, __FUNCTION__, __LINE__, format);
      return NULL;
    }
  } while ( FAUX == sortie );

  /* Conversion russie, on passe  la suite */
  return texte;
}

/* ~-~-~-~-~-~-~-~-~ Analyser un texte pour en extraire une date ~-~-~-~-~-~-~-~-~ */

/* -------- Routine gnrale, seule visible de l'extrieure -------- */

int TXT_AnalyserDate(const char *texte, const char *format, struct tm *date)
{
  char **elements;
  int separateur;
  unsigned int i_valeur = 0;
  unsigned int ordre = 0; /* 1 = jour, 2 = mois, 3 = anne */
  unsigned int valeurs[ 3 ]; /* Les trois valeurs lues, dans le dsordre */
  unsigned long i, nbr_elements;

  if ( texte == NULL ) return -1;
#ifdef _DEVELOPPE_
  if ( NULL == date ) {
    (void) fprintf(stderr, "%s,%s l.%u : Un des lments de la date n'est pas dfini.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }
#endif

  /* On met tout  zro */
  date->tm_mday = 0;
  date->tm_mon  = 0;
  date->tm_year = 0;
  date->tm_wday = 0;

  /* De toute faon, on ignore les espaces initiaux */
  while ( isspace( *texte ) ) {
    texte++;
  }
  if ( 0 == *texte ) {
    (void) fprintf(stderr, "%s,%s l.%u : Chane vide\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* On cherche un sparateur plausible */
  separateur = TXT_Date_TrouverSeparateur( format ); /* On commence par le format, si impos */
  if ( 0 == separateur ) {
    separateur = TXT_Date_TrouverSeparateur( texte ); /* On essaye de le deviner */
  }
  if ( 0 == separateur ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver un sparateur plausible !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -45;
  }
#ifdef _DEVELOPPE_
  (void) printf("%s : Sparateur trouv = %c [%d]\n", __FUNCTION__, separateur, separateur);
#endif

  /* On dcoupe en champs avec ce sparateur */
  elements = TXT_DecomposerChaine( texte, separateur, &nbr_elements );
  if ( NULL == elements ) {
    (void) fprintf(stderr, "%s,%s l.%u : La dcomposition a chou\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }

  /* On analyse ces lments pour y prendre les valeurs numriques */
  for ( i = 0, i_valeur = 0 ; i < nbr_elements ; i++ ) {
    unsigned long v_lue;

    v_lue = TXT_CNV_vNombre_UL( elements[ i ], NULL );
#ifdef _DEVELOPPE_
    (void) printf("%s : lment %lu / %lu = %s ==> %lu\n", __FUNCTION__, i + 1, nbr_elements, elements[ i ], v_lue);
#endif
    if ( 0 == v_lue ) { /* Champ numrique => on essaye de reconnatre un nom de mois */
      if ( 0 == ordre ) { /* On n'a pas encore dtect de mois */
	int mois;

	mois = TXT_QuelMois( elements[ i ] );
#ifdef _DEVELOPPE_
	(void) printf("%s : On essaye de deviner le mois, nom = %s => %ld\n", __FUNCTION__, elements[ i ], v_lue);
#endif
	
	if ( mois > 0 ) { /* On a reconnu un mois, donne aussi une indication sur l'ordre ! */
	  v_lue = mois;
	  if ( 0 == i_valeur )
	    ordre = 200;
	  else if ( 1 == i_valeur )
	    ordre = 20;
	  else
	    ordre = 2;
	}
      }
      if ( 0 == v_lue ) {
	(void) fprintf(stderr, "%s,%s l.%u :  FAIRE : reprer le jour de la semaine\n",
		       __FILE__, __FUNCTION__, __LINE__);
      }
    }
    if ( v_lue > 0 ) {
      valeurs[ i_valeur ] = v_lue; /* On mmorise */
      i_valeur++;
      if ( i_valeur >= 3 ) {
	if ( i < nbr_elements - 1 ) {
	  (void) fprintf(stderr, "%s,%s l.%u : Attention, + de trois champs reconnus -> on ignore les derniers\n",
			 __FILE__, __FUNCTION__, __LINE__);
	}
	break;
      }
    }
  }
#ifdef _DEVELOPPE_
  (void) printf("%s : lments retenus = %u / %u / %u [ordre = %d]\n", __FUNCTION__, valeurs[ 0 ], valeurs[ 1 ], valeurs[ 2 ], ordre );
#endif

  /* Plus besoin de la dcomposition en lignes */
  free( elements[ 0 ] );
  free( elements ); elements = NULL;

  /* On essaye de dduire l'ordre de ces lments */
  ordre = TXT_Date_TrouverOrdre( valeurs, ordre );

  /* On utilise le format pour lever les ambiguits ventuelles */
  ordre = TXT_Date_LeverAmbiguite( ordre, format, separateur );
#ifdef _DEVELOPPE_
  (void) printf("%s : ordre trouv =  %u\n", __FUNCTION__, ordre );
#endif

  /* On remet dans l'ordre */
  switch ( ordre )
  {
  case 123: /* J M A  = franais normal */
    date->tm_mday = valeurs[ 0 ];
    date->tm_mon  = valeurs[ 1 ] - 1;
    date->tm_year = valeurs[ 2 ] - 1900;
    break;

  case 132: /* J A M */
    date->tm_mday = valeurs[ 0 ];
    date->tm_mon  = valeurs[ 2 ] - 1;
    date->tm_year = valeurs[ 1 ] - 1900;
    break;

  case 231: /* M A J */ 
    date->tm_mday = valeurs[ 2 ];
    date->tm_mon  = valeurs[ 0 ] - 1;
    date->tm_year = valeurs[ 1 ] - 1900;
    break;

  case 213: /* M J A = anglo-saxon habituel */
    date->tm_mday = valeurs[ 1 ];
    date->tm_mon  = valeurs[ 0 ] - 1;
    date->tm_year = valeurs[ 2 ] - 1900;
    break;

  case 312: /* A J M */
    date->tm_mday = valeurs[ 1 ];
    date->tm_mon  = valeurs[ 2 ] - 1;
    date->tm_year = valeurs[ 0 ] - 1900;
    break;

  case 321: /* A M J = ISO ? */
    date->tm_mday = valeurs[ 2 ];
    date->tm_mon  = valeurs[ 1 ] - 1;
    date->tm_year = valeurs[ 0 ] - 1900;
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Format mal reconnu, ordre=%d\n",
		   __FILE__, __FUNCTION__, __LINE__, ordre);
    return -4;
  }

  /* Analyse russie */
  return 0;
}

/* -------- Lever l'ambiguit sur l'ordre des lments  l'aide du format -------- */

static int TXT_Date_LeverAmbiguite(int ordre, const char *format, int separateur)
{
  if ( ( 132 == ordre ) || ( 123 == ordre ) || ( 213 == ordre ) || ( 231 == ordre ) || ( 312 == ordre) || ( 321 == ordre ) )
    return ordre; /* Format non-ambigu, dtection automatique... */

  if ( NULL == format ) { /* Pas d'aide extrieure => on doit faire des paris ! */
    if ( 300 == ordre ) { /* Anne en premier */
      if ( '-' == separateur )
	return 321; /* Format ISO suppos : Anne-Mois-Jour */
    }

    if ( 3 == ordre ) {
      if ( '/' == separateur )
	return 123; /* Format europen postul... */
    }

    /* On ne peut pas faire mieux ! */
    return ordre;
  }

  (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- s'aider du format pour deviner la date\n",
		 __FILE__, __FUNCTION__, __LINE__);
  return ordre;
}

/* -------- Trouver l'ordre, dans le cas gnral -------- */

static int TXT_Date_TrouverOrdre( unsigned int valeurs[ 3 ], int ordre )
{
  if ( valeurs[ 0 ] > 31 ) { /* Anne en premier */
    if ( valeurs[ 1 ] > 12 ) { /* anne, jour, mois */
      return 312;
    } else if ( valeurs[ 2 ] > 12 ) { /* anne, mois, jour [ISO]*/
      return 321;
    } else { /* Format ambigu ! Y-??-??*/
      if ( 2 == ordre ) {
	return 312;
      } else if ( 20 == ordre ) {
	return 321;
      } else if ( 200 == ordre ) {
	(void) fprintf(stderr, "%s,%s l.%u : Incohrence : mois & anne dtects en position 1\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -1;
      } else {
	return 300;
      }
    }
  } else if ( valeurs[ 1 ] > 31 ) { /* Anne en second */
    if ( valeurs[ 0 ] > 12 ) { /* jour, anne, mois */
      return 132;
    } else if ( valeurs[ 2 ] > 12 ) { /* mois, anne, jour */
      return 231;
    } else { /* Format ambigu ! ??-Y-??*/
      if ( 2 == ordre ) {
	return 132;
      } else if ( 200 == ordre ) {
	return 231;
      } else if ( 20 == ordre ) {
	(void) fprintf(stderr, "%s,%s l.%u : Incohrence : mois & anne dtects en position 2\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -2;
      } else {
	return 30;
      }
    }
  } else if ( valeurs [ 2 ] > 31 ) { /* Anne en fin */
    if ( valeurs[ 0 ] > 12 ) { /* jour, mois, anne : France */
      return 123;
    } else if ( valeurs[ 1 ] > 12 ) { /* mois, jour, anne : tats-Unis */
      return 213;
    } else {
      if ( 2 == ordre ) {
	(void) fprintf(stderr, "%s,%s l.%u : Incohrence : mois & anne dtects en position 3\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -3;
      } else if ( 20 == ordre ) {
	return 123;
      } else if ( 200 == ordre ) {
	return 213;
      } else {
	return 3; /* Format ambigu */
      }
    }
  }

  /* Impossible de trouver la position de l'anne => format compltement ambigu */
  return ordre;
}

static int TXT_Date_TrouverSeparateur( const char *texte )
{
  const char *position;
  int av_espace = FAUX, av_tiret = FAUX;

  if ( NULL == texte ) return 0; /* Pas de texte ! */

  /* On parcourt caractre par caractre */
  position = texte;
  while ( *position != 0 ) {
    if ( isspace( *position ) ) { /* Il y a des espaces : sparateur plausible */
      av_espace = VRAI;
    } else if ( '/' == *position ) { /* Logiquement, c'est un sparateur sr */
      return '/';
    } else if ( '.' == *position ) { /* Logiquement, c'est un sparateur sr */
      return '.';
    } else if ( '-' == *position ) { /* Tiret : peut tre un sparateur */
      av_tiret = VRAI;
    } else if ( ( '[' == *position ) || (']' == *position ) || ( '%' == *position ) ) {
      /* % = description du format dans la chane classique -> sparateur interdit */
      /* [...] = explication du format -> pas sparateur plausible ! */
    } else if ( FAUX == isalnum( *position ) ) { /* Autre chose qu'une lettre ou un chiffre */
      (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- sparateur potentiel [%c = %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, *position, *position);
    }

    /* On passe au suivant */
    position++;
  }

  /* Si on est l : soit on n'a rien trouv, soit -, soit espace */
  if ( VRAI == av_espace ) { /* Alors - = signe moins d'une anne */
    return ' '; /* espace comme sparateur */
  }

  if ( VRAI == av_tiret ) { /* Alors - suppos un sparateur */
    return '-';
  }

  /* Aucun sparateur trouv */
  (void) fprintf(stderr, "%s,%s l.%u : Aucun sparateur plausible dtect dans %s\n",
		 __FILE__, __FUNCTION__, __LINE__, texte);
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Traitement des noms de mois & jous ~-~-~-~-~-~-~-~-~ */

/* -------- Donner les noms des jours, mois -------- */

const char *TXT_NomDeJour(unsigned int jour)
{
  if ( ( jour > 0 ) && ( jour <= 7 ) ) {
    return noms_jours[ jour ];
  }

  return "Jour inconnu";
}

const char *TXT_NomDeMois(unsigned int mois)
{
  if ( ( mois > 0 ) && ( mois <= 12 ) ) {
    return noms_mois[ mois ];
  }

  return "Mois inconnu";
}

/* -------- Trouver le numro du mois  partir de son nom -------- */

int TXT_QuelMois(const char *nom_mois)
{
  unsigned int i;

  /* On fait sauter des espaces initiaux ventuels */

  /* Premire chose : on compare aux mois connus, le plus fiable */
  for ( i = 0 ; i < 12 ; i++ ) {
    if ( 0 == strncmp( nom_mois, noms_mois[ i ], strlen( noms_mois[ i ] ) ) ) {
      return (i + 1); /* 1 = janvier, 12 = dcembre */
    }
  }

  /* Deuxime tape : avance progressive */
  switch ( *nom_mois )
  {
  case 'a': case 'A':
    nom_mois++;
    
    if ( ( 'b' == *nom_mois ) || ( 'B' == *nom_mois ) ) {
      return AVRIL; /* Espagnol, Occitan, Portugais : Abril ; Corse : Agostu */
    } else if ( ( 'g' == *nom_mois ) || ( 'G' == *nom_mois ) ) {
      return AOUT; /* Italien, Espagnol, Portugais : Agosto ; Occitan : Agost */
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      return AVRIL; /* Galique : Aibrean */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return AOUT;
    } else if ( ( 'p' == *nom_mois ) || ( 'P' == *nom_mois ) ) {
      return AVRIL; /* Italien, Corse : Aprile ; Anglais, Allemand, Nerlandais, Slovne : April ; Grec : Aprilios ; Estonien : Aprill */
    } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
      return MARS; /* Grec : Aritos */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return AOUT; /* Corse : Austu ; Roumain, Anglais, Allemand, Slovne, Estonien : August ; Nerlandais : Augustus ; Grec : Augustos ; Hongrois : Augusztus */
    } else if ( ( 'v' == *nom_mois ) || ( 'V' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
	return AOUT; /* Occitan : Avost */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return AVRIL; /* Occitan : Avriu */
      }
    } else if ( ( 'w' == *nom_mois ) || ( 'W' == *nom_mois ) ) {
      return AOUT; /* Gallois : Awst */
    }
    break;

  case '': case '':
    return AVRIL; /* Hongrois : prilis */
    break;

  case 'b': case 'B':
    nom_mois++;

    if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
      return MAI; /* Galique : Bhealtaine */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return AOUT; /* Lapon : Borgemnnu */
    }
    break;

  case 'c': case 'C':
    nom_mois++;
    if ( '\'' == *nom_mois ) nom_mois++; /* On ignore l'apostrophe */

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      return SEPTEMBRE; /* Lapon : Cakcamnnu */
    } else if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
      return FEVRIER; /* Breton : C'hwevrer ; Gallois : Chwefror */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return AVRIL; /* Lapon : Cuonomnnu */
    } else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
      return JUIN; /* Polonais : Czerwiec */
    }
    break;

  case 'd': case 'D':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'c' == *nom_mois ) || ( 'C' == *nom_mois ) ) {
	return DECEMBRE; /* Occitan : Decmbre ; Roumain : Decembrie ; Anglais, Hongrois, Nerlandais, Slovne : December ; Grec : decembrios */
      } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	return OCTOBRE; /* Galique : Deireadh Fomhair */
      } else if ( ( 's' == *nom_mois ) || ( 'S' == *nom_mois ) ) {
	return DECEMBRE; /* Scandinave : Desember */
      } else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
	return DECEMBRE; /* Estonien : Detsember */
      } else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
	return DECEMBRE; /* Portugais : Dezembro ; Allemand : Dezember */
      }
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return DECEMBRE;
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) { /* Di : confusion possible avec Dimanche */
      nom_mois++;

      if ( ( 'c' == *nom_mois ) || ( 'C' == *nom_mois ) ) {
	return DECEMBRE; /* Italien, Corse : Dicembre ; Espagnol : Diciembre */
      }
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return NOVEMBRE; /* Breton, Cornique : Du */
    } else {
      int car;

      car = (int) *nom_mois;
      if ( ( car > 127 ) || ( car < 0 ) ) { /* Grer les char signed ou unsigned... Soupir. */
	(void) fprintf(stderr, "%s,%s l.%u : D + accent, suppos e-accent aigu [%c = %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, *nom_mois, *nom_mois);
	return DECEMBRE;
      }
    }
    break;

  case 'e': case 'E':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      return  JANVIER; /* Galique : Eanair */
    } else if ( ( 'b' == *nom_mois ) || ( 'B' == *nom_mois ) ) {
      return AVRIL; /* Breton, Cornique : Ebrel ; Gallois : Ebrill */
    } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
      return AOUT; /* Finnois : Elokuu */
    } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
      return JANVIER; /* Espagnol : Enero */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return AOUT; /* Breton : Eost */ 
    } else if ( ( 's' == *nom_mois ) || ( 'S' == *nom_mois ) ) {
      return AOUT; /* Cornique : Est */ 
    }
    break;

  case 'f': case 'F':
    nom_mois++;

    /* F = fvrier suffisant, mais on continue pour viter la confusion avec Freitag & autres jours */
    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      return FEVRIER; /* Italien : Febbraio ; Espagnol : Febrero ; Occitan : Febrier ; Corse : Ferraghju ;
			 Roumain : Februarie ; Portugais : Feveireiro ; Gaelique : Feabrhra ; Anglais, Allemand, Slovne : Februar ;
			 Nerlandais : Februari ; Grec : Febrouarios ; Hongrois : Februr */
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return FEVRIER;
    } else {
      int car;

      car = (int) *nom_mois;
      if ( ( car > 127 ) || ( car < 0 ) ) { /* Grer les char signed ou unsigned... Soupir. */
	(void) fprintf(stderr, "%s,%s l.%u : F + accent, suppos e-accent aigu [%c = %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, *nom_mois, *nom_mois);
	return FEVRIER;
      }
    }
    break;

  case 'g': case 'G':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	return JUIN; /*  Lapon : Geassemnnu */
      } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	return JANVIER; /* Italien : Gennaio ; Occitan : Genier ou Genovier ; Breton, Cornique : Genver */
      }
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      return JUIN; /* Italien : Giugno */
    } else if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'j' == *nom_mois ) || ( 'J' == *nom_mois ) ) {
	nom_mois++;

	if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	  return JANVIER; /* Corse : Ghjennaghju */
	} else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
	  return JUIN; /* Corse : Ghjugnu */
	}
      }
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	return OCTOBRE; /* Lapon : Golggotmnnu */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return JUILLET; /* Gallois : Gorffennaf ; Cornique : Gortheren */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return JUILLET; /* Breton : Gouere */
      }
    } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
      return DECEMBRE; /* Polonais : Grudzien ; Vieux Slovne : Gruden */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return FEVRIER; /* Lapon : Guovvamnnu */
    } else if ( ( 'w' == *nom_mois ) || ( 'W' == *nom_mois ) ) {
      return SEPTEMBRE; /* Breton : Gwengolo ; Cornique : Gwyngala */
    }
    break;

  case 'h': case 'H':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) { /* He : confusion possible avec hetfo */
      nom_mois++;

      if ( ( 'd' == *nom_mois ) || ( 'D' == *nom_mois ) ) {
	return OCTOBRE; /* Cornique : Hedra */
      } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	nom_mois++;

	if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	  nom_mois++;

	  if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	    return JUIN; /* Estonien : Heinakuu */
	  } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
	    return JUILLET;  /* Finnois : Heinkuu */
	  }
	}
      } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	return FEVRIER; /* Finnois : Helmikuu */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return OCTOBRE; /* Breton : Here */
      }
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return AVRIL; /* Finnois : Huhtikuu */
    } else if ( ( 'y' == *nom_mois ) || ( 'Y' == *nom_mois ) ) {
      return OCTOBRE; /* Gallois : Hydref */
    }
    break;

  case 'i': case 'I':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      return JANVIER; /* Roumain : Ianuarie ; Grec : Ianouarios */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	return JANVIER; /* Gallois : Ionawr */
      } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
	nom_mois++;

	if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	  return JUILLET; /* Grec : Ioulios */
	} else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	  return JUIN; /* Grec : Iounios */
	}
      }
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	return JUILLET; /* Galique : Iuil */
      } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	return JUILLET; /* Roumain : Iulie */
      } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	return JUIN; /* Roumain : Iunie */
      }
    }
    break;

  case 'j': case 'J':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      return JANVIER; /* Portugais : Janeiro; Anglais, Slovne : Januar ; Allemand : Januar ; Nerlandais : Januari ; Estonien : Jaanuar ; Hongrois : Janur */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return DECEMBRE; /* Finnois : Joulukuu */
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return DECEMBRE; /* Estonien : Julukuu */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	nom_mois++;
      
	if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	  return JUILLET;
	} else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	  return JUIN; 
	}
      } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	  return JUILLET; /* Espagnol : Julio ; Occitan : Juhl, Juhlet ; Portugais : Julho ; Anglais : July ;
			     Allemand, Nerlandais : Juli ; Slovne : Julij */
      } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	return JUIN; /* Espagnol : Junio ; Occitan : Junh ; Portugais : Junho ; Anglais : June ; Allemand, Nerlandais : Juni ;
		      Slovne : Junij */
      } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
	return DECEMBRE; /* Lapon : Juovlamnnu */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return AVRIL; /* Estonien : Jrikuu */
      } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
	nom_mois++;
      
	if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	  return JUILLET; /* Estonien : Juuli */
	} else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	  return JUIN; /* Estonien : Juuni */
	}
      }
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      nom_mois++;
      
      if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	return JUILLET; /* Hongrois : Jlius */
      } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) {
	return JUIN; /* Hongrois : Jnius */
      }
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return AVRIL; /* Estonien : Jrikuu */
    }
    break;

  case 'k': case 'K':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return DECEMBRE; /* Breton : Kerzu */
      } else if ( ( 's' == *nom_mois ) || ( 'S' == *nom_mois ) ) {
	return JUIN; /* Finnois : Keskuu */
      } else if ( ( 'v' == *nom_mois ) || ( 'V' == *nom_mois ) ) {
	return DECEMBRE; /* Cornique : Kevardhu */
      }
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      return SEPTEMBRE; /* Vieux slovne : Kimavec */
    } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
      return AVRIL; /* Polonais : Kwiecien */
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return FEVRIER; /* Estonien : Knlaku */
    }
    break;

  case 'l': case 'L':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      return MAI; /* Estonien : Lehekuu */
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'p' == *nom_mois ) || ( 'P' == *nom_mois ) ) {
	return JUILLET; /* Polonais : Lipiec */
      } else if ( ( 's' == *nom_mois ) || ( 'S' == *nom_mois ) ) {
	return NOVEMBRE; /* Polonais, Vieux slovne : Listopad */
      }
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return OCTOBRE; /* Finnois : Lokakuu */
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return AOUT; /* Estonien : Likuskuu */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'g' == *nom_mois ) || ( 'G' == *nom_mois ) ) {
	return JUILLET; /* Italien : Luglio ; Corse : Lugliu */
      } else if ( ( 'n' == *nom_mois ) || ( 'N' == *nom_mois ) ) { /* Lun : confusion possible avec lundi */
	nom_mois++;

	if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	  return AOUT; /* Galique : Lunasa */
	}
      } else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
	return FEVRIER; /* Polonais : Luty */
      }
    }
    break;

  case 'm': case 'M':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	return MARS; /* Nerlandais : Maart ; Finoois : Maaliskuu */
      } else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	return MAI; /* Breton : Mae */
      } else if ( ( 'g' == *nom_mois ) || ( 'G' == *nom_mois ) ) {
	return MAI; /* Italien : Maggio ; Corse : Maghju */
      } else if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
	return MARS; /* Galique : Mharta */
      } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	return MAI; /* Occitan, Gallois, Allemand, Estonien : Mai ; Portugais : Maio ; Grec : Maios */
      } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
	return MAI; /* Roumain : Ma */
      } else if ( ( 'j' == *nom_mois ) || ( 'J' == *nom_mois ) ) {
	return MAI; /* Scandinave, Polonais, Slovne : Maj ; Majus */
      } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	return -10; /* Vieux Slovne : Mali Traven = Mai ; Mali Spran = Juillet ==> mali non identifiable ! */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) { /* Mar : confusion possible avec MARDI */
	nom_mois++;

	if ( ( 'c' ==  *nom_mois ) || ( 'C' == *nom_mois ) ) {
	  return MARS; /* Anglais : March */
	} else if ( ( '' ==  *nom_mois ) || ( '' == *nom_mois ) ) {
	  return MARS; /* Occitan : Mar ; Portugais : Maro */
	} else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	  return MARS; /* Slovne : Marec */
	} else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	  return NOVEMBRE; /* Finnois : Marraskuu */
	} else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
	  return MARS; /* Roumain : Martie ; Scandinave : Marts */
	} else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
	  return MARS;  /* Italien, Espagnol : Marzo ; Corse : Marzu ; Polonais : Marzec */
	}
      } else if ( ( 'w' == *nom_mois ) || ( 'W' == *nom_mois ) ) {
	return MARS; /* Gallois : Mawrth */
      } else if ( ( 'y' == *nom_mois ) || ( 'Y' == *nom_mois ) ) {
	return MAI; /* Espagnol : Mayo ; Anglais : May */
      }
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'j' == *nom_mois ) || ( 'J' == *nom_mois ) ) {
	return MAI; /* Hongrois : Mjus */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
	return MARS; /* Hongrois : Mrcius */
      }
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return MARS; /* Allemand : Mrz ; Estonien : Mrts */
    } else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	return  SEPTEMBRE; /* Galique : Mean Fomhair */
      } else if ( ( 'd' == *nom_mois ) || ( 'D' == *nom_mois ) ) {
	return SEPTEMBRE; /* Gallois : Medi */
      } else if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
	return JUIN; /* Gallois : Mehefin */
      } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	return MAI; /* Nerlandais : Mei */
      } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
	return JUIN; /* Galique : Meitheamh */
      } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) { /* Mer : ATTENTION, confusion avec mercredi */
	nom_mois++;

	if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
	  return MARS; /* Cornique : Merth */
	}
      } else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
	return JUIN; /* Cornique : Metheven */
      } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
	return MARS; /* Breton : Meurzh */
      } else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
	return JUIN; /* Breton : Mezheven */
      } else if ( ( 0 == *nom_mois ) || ( VRAI == isspace( *nom_mois ) ) ) {
	return MAI; /* Cornique : Me */
      }
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) { /* Mi : confusion possible avec mittwoch */
      nom_mois++;

      if ( ( 0 == *nom_mois ) || ( VRAI == isspace( *nom_mois ) ) ) {
	return DECEMBRE; /* Galique : Mi na Nollag */
      } else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	return MAI; /* Lapon : Miessemnnu */
      } else if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
	return SEPTEMBRE; /* Estonien : Mihklikuu */
      }
    }
    break;

  case 'n': case 'N':
    nom_mois++;

    if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return JANVIER; /* Estonien : Nrikuu */
    } else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      return NOVEMBRE; /* Estonien : Nevember */
    } else if ( ( 'j' == *nom_mois ) || ( 'J' == *nom_mois ) ) {
      return MARS; /* Lapon : Njukcamnnu */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return NOVEMBRE; /* Italien : Novembre ; Espagnol : Noviembre ; Occitan : Novmbre ; Roumain : Noiembrie ; Grec : Noembrios ;
			  Portugais : Novembro ; Anglais, Allemand, Hongrois, Nerlandais, Scandinave, Slovne : November */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return NOVEMBRE; /* Corse : Nuvembre */
    }
    break;

  case 'o': case 'O':
    nom_mois++;

    if ( ( 'c' == *nom_mois ) || ( 'C' == *nom_mois ) ) {
      return OCTOBRE; /* Espagnol : Octubre ; Occitan : Octobre ; Roumain : Octombrie ; Anglais : October ; Grec : Octobrios */
    } else if ( ( '' == *nom_mois ) || ( '' == *nom_mois ) ) {
      return JANVIER; /* Lapon : Oajagemnnu */
    } else if ( ( 'k' == *nom_mois ) || ( 'K' == *nom_mois ) ) {
      return OCTOBRE; /* Allemand, Nerlandais, Scandinave, Slovne, Estonien : Oktober ; Hongrois : Oktber */
    } else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
      return OCTOBRE; /* Italien : Ottobre ;  ; Corse : Ottobre */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      return OCTOBRE; /* Portugais : Outubro ; */
    }
    break;

  case 'p': case 'P':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
	return MARS; /* Estonien : Paastukuu */
      } else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
	return OCTOBRE; /* Polonais : Pazdziernik */
      }
    } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
	return AVRIL; /* Roumain : Prilie */
      } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
	return JANVIER; /* Vieux slovne : Prosinec */
      }
    }
    break;

  case 'r': case 'R':
    nom_mois++;

    if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
      return DECEMBRE; /* Gallois : Rhagfyr */
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return JUIN; /* Vieux slovne : Roznik */
    }
    break;

  case 's': case 'S':
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'm' == *nom_mois ) || ( 'M' == *nom_mois ) ) { /* Sam : confusion possible avec samedi */
	nom_mois++;

	if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
	  return NOVEMBRE; /* Galique : Samhain */
	}
      }
    } else if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
      return SEPTEMBRE; /* Italien, Corse : Settembre ; Espagnol, Roumain : Septembrie ; Occitan : Setmbre ; Portugais : Setembro ;
			   Anglais, Allemand, Nerlandais, Scandinave, Slovne, Estonien : September ; Grec : Septembrios */
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      return AOUT; /* Polonais : Sierpien */
    } else if ( ( 'k' == *nom_mois ) || ( 'K' == *nom_mois ) ) {
      return NOVEMBRE; /* Lapon : Skabmamnnu */
    } else if ( ( 't' == *nom_mois ) || ( 'T' == *nom_mois ) ) {
      return JANVIER; /* Polonais : Styczen */
    } else if ( ( 'u' == *nom_mois ) || ( 'U' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
	return JUILLET; /* Lapon : Suoidnemnnu */
      } else if ( ( 's' == *nom_mois ) || ( 'S' == *nom_mois ) ) {
	return MARS; /* Vieux slovne : Susec */
      } else if ( ( 'v' == *nom_mois ) || ( 'V' == *nom_mois ) ) {
	return JUILLET; /* Estonien : Suvekuu */
      }
    } else if ( ( 'v' == *nom_mois ) || ( 'V' == *nom_mois ) ) {
      return FEVRIER; /* Vieux slovne : Svecan */
    } else if ( ( 'y' == *nom_mois ) || ( 'Y' == *nom_mois ) ) {
      return SEPTEMBRE; /* Finnois : Syyskuu */
    } else if ( ( 'z' == *nom_mois ) || ( 'Z' == *nom_mois ) ) {
      nom_mois++;

      /* Sz = szeptember suffisant, on continue pour viter la confusion avec Szombat ! */
      if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	return SEPTEMBRE; /* Hongrois : Szeptember */
      }
    }
    break;

  case 't': case 'T': /* T : confusion possible avec The, utilis dans les dates anglaises */
    nom_mois++;

    if ( ( 'a' == *nom_mois ) || ( 'A' == *nom_mois ) ) {
      nom_mois++;

      if ( ( 'c' == *nom_mois ) || ( 'C' == *nom_mois ) ) {
	return NOVEMBRE; /* Gallois : Tachwedd */
      } else if ( ( 'l' == *nom_mois ) || ( 'L' == *nom_mois ) ) {
	(void) fprintf(stderr, "%s,%s l.%u : Vrifier si talvekuu = novembre ou octobre, en estonien !\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -258; /* Estonien : Talvekuu */
      } else if ( ( 'm' == *nom_mois ) || ( 'M' == *nom_mois ) ) {
	return JANVIER; /* Finnois : Tammikuu */
      }
    } else if ( ( 'o' == *nom_mois ) || ( 'O' == *nom_mois ) ) {
      return MAI; /* Finnois : Toukokuu */
    }
    break;

  case 'u': case 'U':
    return OCTOBRE; /* Corse : Uttobre */

  case 'v': case 'V':
    nom_mois++;

    if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) { /* Ve : confusion possible avec VENDREDI */
      nom_mois++;

      if ( ( 'e' == *nom_mois ) || ( 'E' == *nom_mois ) ) {
	return FEVRIER; /* Estonien : Veebruar */
      }
    } else if ( ( 'i' == *nom_mois ) || ( 'I' == *nom_mois ) ) {
      return OCTOBRE; /* Vieux slovne : Vinotk */
    }
    break;

  case 'w': case 'W':
    nom_mois++;

    if ( ( 'h' == *nom_mois ) || ( 'H' == *nom_mois ) ) {
      return FEVRIER; /* Cornique : Whevrer */
    } else if ( ( 'r' == *nom_mois ) || ( 'R' == *nom_mois ) ) {
      return SEPTEMBRE; /* Polonais : Wrzesien */
    }
    break;
  }

#ifdef _DEVELOPPE_
  (void) fprintf(stderr, "%s,%s l.%u : Nom de mois non-reconnu [%s]\n",
		 __FILE__, __FUNCTION__, __LINE__, nom_mois);
#endif
  return -1;
}
