/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998 - 2001, Emmanuel CURIS
   Cration de ce fichier: 5 mai 1999 [essentiellement rcupr de xanes.c]

   Module de traitement du XANES

   Calcul de l'aire sous une portion de courbe (Note : tout  fait utilisable pour autre chose que du XANES)
   Dans tous les cas, on suppose implicitement que le graphe trait est tri par X croissants
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "interface.h" /* Pour RCT_pointeur */
#include "operations.h" /* Pour le calcul des intgrales */
#include "tracer.h"
#include "xanes.h"

#include "xan_integre.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static int type_methode = INTG_TRAPEZE;
#define LG_MAX_NOM_METHODE 30

static const char *txt_algo_integre[] = {
  "Rectangles",
  "Point m"e_AIGU"dian",
  "Trap"e_GRAVE"zes",
  "Simpson",
  "Bode",
  "----",
  "Splines"
};

static double X_min, X_max;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static double intg_trapeze(Graphe *graphe, double min, double max);

static void RDS_intg(Fenetre *fen, int x, int y, unsigned int larg, unsigned int haut);
static int  CLC_intg(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

static int  GER_integre(unsigned int num_objet,int x, int y);
static int  prep_arbre_integre(unsigned int);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~- Les algorithmes de calcul, pour les graphes rels ~-~-~-~- */

/* -------- Mthode des trapzes --------

   Note : pas confiance en graphe->x_min / graphe->x_max parce que a ne sont pas forcment des valeurs relles
   Exemple : si c'est un histogramme, graphe->x_max = x_max rel + largeur de classe pour la voir !
   -> on raisonne seulement sur le nombre de points pour vrifier les dbordements !
 */

static double intg_trapeze(Graphe *graphe, double x_min, double x_max)
{
  double vx_x, vx_y;
  double somme = 0.0;
  register double *x, *y;
  unsigned long i = 0;

  if (x_min == x_max) /* Pas d'intgration  faire... */
    return 0;

  if (x_min > x_max) { /* On inverse les bornes */
    vx_x = x_max;
    x_max = x_min;
    x_min = vx_x;
  }

  if ( (x_min >= graphe->x_max) || (x_max <= graphe->x_min) ) {
    EXF_ERREUR(BORNES_FAUSSES);
    return 0;
  }

  /* On se place au dbut de l'intgration */
  x = graphe->x;
  y = graphe->y;
  while (*x < x_min) {
    x++; y++;
    i++;
    if (i >= graphe->nombre_point)
      return 0;
  }

  /* On commence l'intgration */
  vx_x = *x;
  vx_y = *y;
  while (*x < x_max) {
    x++; y++;
    somme += (*x - vx_x) * (*y + vx_y);
    vx_x = *x;
    vx_y = *y;
    i++;
    if (i >= graphe->nombre_point)
      return somme / 2.0;
  }

  return somme/2.0;
}

/* ~~~~~~~~~~~~~~~ Appel de l'arbre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean XAN_integre(Graphe *xanes)
{
  Fenetre *fen_integre;
  double resultat = 0.0;

  if (xanes == NULL) return FAUX; /* Calcul n'a pas de raison d'tre */

  fen_integre = FEN_CreerAvecArbre( TOUS_SF_ASC, "Aire sous une portion de la courbe", NULL,
				    INTG_XANES, BI_FEN_ARBRE_HAUT, prep_arbre_integre,
				    RDS_intg, (int (*)()) NULL, CLC_intg,
				    (int (*)()) NULL, RCT_pointeur, (int (*)()) NULL );
  if (fen_integre == NULL) return FAUX; /* Mme chose */

  fen_integre->drapeaux.redessin = VRAI; /* Redessin si changement de taille */
  X_min = xanes->x_min;
  X_max = xanes->x_max;
  OBJ_MettreIntervalle( INTG_XANES, INTG_XMIN, INTG_XMAX, 0, 0, xanes->x_min, xanes->x_max, 0.0, 0 );

  switch (type_methode)
  {
  case INTG_TRAPEZE:
    resultat = intg_trapeze(xanes, X_min, X_max);
  }

  OBJ_MettreNombre( INTG_XANES, INTG_RESULTAT, resultat, 0.0 );

  /* On lie les infos  la fentre */
  FEN_GRF_LierGraphe( fen_integre, xanes );

  return VRAI;
}

/* ~~~~~~~~~~~~~~~ Redessin de cette fentre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static void RDS_intg(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  int retour;
  Graphe *graphe;
  double echelle_X, echelle_Y;

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE ); /* Adresse du graphe  redessiner */
  if ( NULL == graphe ) {
    TRC_Message( fenetre, "Aucun graphe" );
    return;
  }

  retour = TRC_Echelles( fenetre->w_larg, fenetre->w_haut, graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max,
			 &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    TRC_Message( fenetre, "Graphe constant, en X ou en Y" );
    return;
  }

  /* On trace la courbe exprimentale - en noir */
  TRC_Graphe( fenetre, graphe, NOIR, BI_CONTINU, 0, 0, 0, fenetre->w_haut, 
	      graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, echelle_X, echelle_Y );

  /* On place les deux curseurs indiquant les limites */
  TRC_Style( BI_CONTINU );
  TRC_Epaisseur( 0 );

  if ( ( X_min >= graphe->x_min ) && ( X_min <= graphe->x_max ) ) {
    TRC_Couleur( ROUGE );
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_min, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );
  }

  if ( ( X_max >= graphe->x_min ) && ( X_max <= graphe->x_max ) ) {
    TRC_Couleur( VERT );
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_max, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );
  }
}

/* --------------- Gestion du clic souris dans cette fentre ----------------------------- *
 * Si clic : c'est qu'elle est en premier plan => on peut dessiner tranquille              */

static int CLC_intg(Fenetre *fenetre, int bouton, int n_clic, int etat __attribute__((__unused__)), int x_souris, int y_clic __attribute__((__unused__)))
{
  Graphe *graphe;
  double x_clic, echelle_X;
  double resultat;

  if ( n_clic > 1 ) {
    /* Clic multiple, sans fonction */
    return BI_EVENEMENT_NON_TRAITE;
  }

  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE ); /* Adresse du graphe  redessiner */
  if ( NULL == graphe ) {
    return BI_EVENEMENT_TRAITE;
  }
  if ( ( graphe->x_min == graphe->x_max ) || ( graphe->y_max == graphe->y_min ) ) {
    return BI_EVENEMENT_TRAITE;
  }

  echelle_X = (graphe->x_max - graphe->x_min) / (double) (fenetre->w_larg);
  x_clic = TRC_ConvertitX_EvP( x_souris, graphe->x_min, 0, echelle_X );

  TRC_Mode( BI_MODE_INVERSE );
  TRC_Style( BI_CONTINU );

  if ( bouton == 1 ) { /* Clic du bouton gauche */
    TRC_Couleur( ROUGE );
    /* On efface l'ancienne ligne */
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_min, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );

    /* On mmorise la nouvelle valeur */
    X_min = x_clic;
    OBJ_MettreNombre( INTG_XANES, INTG_XMIN, x_clic, 0.0 );

    /* On calcule */
    resultat = intg_trapeze(graphe, X_min, X_max);
    OBJ_MettreNombre( INTG_XANES, INTG_RESULTAT, resultat, 0.0 );

    /* On trace la nouvelle ligne */
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_min, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );
  } else {
    TRC_Couleur( VERT );
    /* On efface l'ancienne ligne */
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_max, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );
 
    /* On mmorise la nouvelle valeur */
    X_max = x_clic;
    OBJ_MettreNombre( INTG_XANES, INTG_XMAX, x_clic, 0.0 );

    /* On calcule */
    resultat = intg_trapeze(graphe, X_min, X_max);
    OBJ_MettreNombre( INTG_XANES, INTG_RESULTAT, resultat, 0.0 );

    /* On trace la nouvelle ligne */
    TRC_LigneV( fenetre, TRC_ConvertitX_PvE( X_max, graphe->x_min, 0, echelle_X ), 0, fenetre->w_haut );
  }

  return BI_EVENEMENT_TRAITE; /* vnement trait */
}

static int GER_integre(unsigned int num_objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  Fenetre *fenetre; /* Fentre courante               */
  Graphe *graphe;   /* Graphe associ  cette fentre */
  double resultat;
  int reponse;
  struct infos_demande valeur = { BI_CHAMP_DEMANDE_REEL, NULL, "eV", BI_EDITABLE_UNITE, 0.0, 0.0, 0.0, 0, NULL, { 0 }, { 0 }, { 0 } };

  fenetre = ARB_QuelleFenetre(arbres + INTG_XANES, FAUX); /* Fentre courante */
  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE ); /* Adresse du graphe  redessiner */
  if ( NULL == graphe ) return BI_EVENEMENT_TRAITE;

  switch (num_objet)
  {
  case INTG_XMIN: /* On demande X  l'utilisateur */
    valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    valeur.texte = "Xmin =";
    valeur.min = graphe->x_min;
    valeur.max = X_max;
    valeur.valeur_initiale.reel = X_min;

    reponse = BI_DemandeValeur( fenetre, &Q_BorneInferieure, &valeur );
    if ( reponse != (int) Q_BorneInferieure.bouton_ok ) {
      return BI_EVENEMENT_TRAITE;
    }

    X_min = valeur.valeur_donnee.reel;
    OBJ_MettreNombre( INTG_XANES, INTG_XMIN, X_min, 0.0 );

    resultat = intg_trapeze(graphe, X_min, X_max);
    OBJ_MettreNombre( INTG_XANES, INTG_RESULTAT, resultat, 0.0 );

    FEN_redessin(fenetre, BI_RDS_UTIL); /* On fait redessiner */
    return BI_EVENEMENT_TRAITE;

  case INTG_XMAX:	/* On demande X max  l'utilisateur */
    valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    valeur.texte = "Xmin =";
    valeur.min = X_min;
    valeur.max = graphe->x_max;
    valeur.valeur_initiale.reel = X_max;

    reponse = BI_DemandeValeur( fenetre, &Q_BorneSuperieure, &valeur );
    if ( reponse != (int) Q_BorneSuperieure.bouton_ok ) {
      return BI_EVENEMENT_TRAITE;
    }

    X_max = valeur.valeur_donnee.reel;
    OBJ_MettreNombre( INTG_XANES, INTG_XMAX, X_max, 0.0 );

    resultat = intg_trapeze(graphe, X_min, X_max);
    OBJ_MettreNombre( INTG_XANES, INTG_RESULTAT, resultat, 0.0 );

    FEN_redessin(fenetre, BI_RDS_UTIL); /* On fait redessiner */
    return BI_EVENEMENT_TRAITE; /* On a fini */
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre -------- */

static int prep_arbre_integre(unsigned int numero_arbre __attribute__((__unused__)))
{
  OBJ_agrandit(INTG_XANES, ROOT, 0, +1);
  OBJ_deplace(INTG_XANES, INTG_XMAX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INTG_XANES, IXAN_METHODE, 0, +1, DEPLACE_PIXELS);
  OBJ_cache(INTG_XANES, IXAN_ERREUR, FORCE_OUI); /* Visible seulement si erreur avec */

#ifndef _DEVELOPPE_
  OBJ_active(INTG_XANES, IXAN_AVEC_ERREUR, FORCE_NON);
  OBJ_active(INTG_XANES, IXAN_METHODE, FORCE_NON);
#endif

  met_chaine(INTG_XANES, IXAN_METHODE, LG_MAX_NOM_METHODE, txt_algo_integre[type_methode], TXT_COPIER);

  arbres[INTG_XANES].fonction.arbre = GER_integre;
  arbres[INTG_XANES].drapeaux.prepare = VRAI;

  return 0;
}
