/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2016

   Programme principal : lecture des paramètres par défaut
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "txt_utils.h"

/* ~~~~~~~~ Fonctions extérieures appelées dans ce module ~~~~~~~~ */

/* ~~~~~~~~ Définitions locales ~~~~~~~~ */

#define NOM_FICHIER_RC ".laserc"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~~~~~~~~~~~ */

static FILE *CONF_TrouverFichierRC( void );

/* ~~~~~~~~ Réalisation concrète ~~~~~~~~~~~~~~~~~~ */

/* ----------------- Recherche du fichier de configuration ------------------- */

static FILE *CONF_TrouverFichierRC( void )
{
  char *chemin, *chemin_complet;
  FILE *fichier = NULL;

  /* On essaye là où l'on a lancé le programme en premier */
  fichier = fopen( NOM_FICHIER_RC, "r" );
  if ( fichier != NULL ) {
    (void) printf( "Fichier de configuration trouvé : ./"NOM_FICHIER_RC"\n" );
    return( fichier );
  }

  /* Échec
     -> On essaye d'abord le chemin du répertoire local selon $PWD */
  chemin = getenv( "PWD" );	/* Ne DOIT PAS être libéré ni modifié... */
  if ( chemin != NULL ) {
    /* Existe-t-il un .laserc dans ce chemin ? */
    chemin_complet = TXT_genere_chaine( "%s/"NOM_FICHIER_RC, chemin );

    if ( chemin_complet != NULL ) {
#ifdef _DEVELOPPE_
      (void) fprintf( stderr, "%s,%s l.%u : On essaye le fichier %s\n",
		      __FILE__, __FUNCTION__, __LINE__, chemin_complet );

#endif
      fichier = fopen( chemin_complet, "r" );
      free( chemin_complet ); chemin_complet = NULL;

      if ( fichier != NULL ) {
	(void) printf( "Fichier de configuration trouvé : %s/"NOM_FICHIER_RC"\n", chemin );
	return( fichier );
      }
    }
  }

  /* -> On essaye ensuite le chemin du dossier personnel de l'utilisateur, selon $HOME */
  chemin = getenv( "HOME" );	/* Ne DOIT PAS être libéré ni modifié... */
  if ( chemin != NULL ) {
    /* Existe-t-il un .laserc dans ce chemin ? */
    chemin_complet = TXT_genere_chaine( "%s/"NOM_FICHIER_RC, chemin );

    if ( chemin_complet != NULL ) {
#ifdef _DEVELOPPE_
      (void) fprintf( stderr, "%s,%s l.%u : On essaye le fichier %s\n",
		      __FILE__, __FUNCTION__, __LINE__, chemin_complet );

#endif
      fichier = fopen( chemin_complet, "r" );
      free( chemin_complet ); chemin_complet = NULL;

      if ( fichier != NULL ) {
	(void) printf( "Fichier de configuration trouvé : %s/"NOM_FICHIER_RC"\n", chemin );
	return( fichier );
      }
    }
  }

  return fichier;
}

/* ----------------- Lecture du fichier de configuration ------------------- */

int CONF_LireFichierRC( void )
{
  FILE *fichier = NULL;

  /* On essaye d'ouvrir le fichier */
  fichier = CONF_TrouverFichierRC();
  if ( NULL == fichier ) {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de fichier de configuration trouvé\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return 0;
  }

  OPTION_A_FAIRE( "Lecture des chemins prédéfinis dans .laserc" );

  /* On ferme le fichier de configuration */
  (void) fclose( fichier ); fichier = NULL;

  /* Terminé sans problème */
  return 0;
}
