/* Traitement de l'E.X.A.F.S. sur ATARI
   Gestion gnrale des graphes (cration, effacement,...)
   (c) 1996-2002 - Emmanuel CURIS

  12, 19 janvier ; 10, 11, 13, 16, 25 mars 1998
    * Corrig le tri sur place : bouclait sans fin si non tri...
    * Fonction de tri sur place d'un graphe, par permutations : tri_en_place()
    * Inclus fichiers.h, pour le prototype de sauver()
    * Titres de menu "erreur" et "affichage" inactifs si aucun graphe en mmoire

   2 dcembre 1997
    * Affichage des menus quand ils changent plus correct
    * Cration de MEN_adapte() : nouvelle version de l'adaptation des menus
      suppression de adapte_menus()
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef VERSION_SIMPLIFIEE_
# define _LIMITE_ 1
#endif

#include "systeme.h"

#include "serie.h"

#include "exafs.h"
#include "extrait.h"  /* Pour contrles lors de l'effacement */
#include "fichiers.h" /* Pour sauver() */
#include "filtrage.h" /* TF_infos, pour contrles lors de l'effacement */
#include "operations.h" /* Pour contrles lors de l'effacement */
#include "science.h" /* Pour M_PI */
#include "tracer.h" /* Pour les infos de dessin */

#include "xanes.h" /* MAT_... pour l'effacement d'un m */

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

/* --------------- Gestion gnrale des graphes : cration, effacement,...    *
 ******************************************************************************/

/* ------ Propagation des modifications d'un graphe : on cherche toutes les ------\
 |  fentres qui utilisent le graphe indiqu et on leur envoie un message qui les |
 |  avertit d'utiliser le nouveau graphe  la place de l'ancien                   |
 |                                                                                |
 | ancien_graphe : le graphe qui a t modifi                                    |
 | nv_graphe     : le nouveau graphe  mettre  la place                          |
 | npt_change    : si nv_graphe == ancien_graphe, indique si le nombre de points  |
 |                 a chang.                                                      |
 \------------------------------------------------------------------------------- */

void GRF_modifier(Graphe *ancien_graphe, Graphe *nv_graphe, boolean npt_change)
{
  Graphe *grf_fenetre;
  Message message;
  Fenetre *fenetre;
  int numero_arbre, retour;
  boolean aucune_fermeture = VRAI;

  message.type = MSG_GRAPHE_MODIFIE; /* Modification de graphe */
  message.divers.pointeur = (void *) nv_graphe;

  if ( nv_graphe == ancien_graphe ) /* Obligatoirement par npt_change */
    message.divers.drapeau = npt_change;
  else if ( ( NULL == nv_graphe ) || ( NULL == ancien_graphe ) ||  /* Graphe apparat ou disparat */
	    ( nv_graphe->nombre_point != ancien_graphe->nombre_point ) )
    message.divers.drapeau = VRAI; /* Modification du nombre de points */
  else
    message.divers.drapeau = FAUX; /* Nombre de points identique */

  do {
    /* On n'a pas encore fait de fermeture pour ce tour */
    aucune_fermeture = VRAI;

    for ( fenetre = BI_premiere_fenetre ; fenetre ; fenetre = fenetre->suivante ) {
      if ( FAUX == fenetre->drapeaux.util_3 ) /* Pas de graphe associ -> elle ne peut pas tre concerne */
	continue;

      /* Le graphe est-il directement li  la fentre ? */
      grf_fenetre = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
      if ( grf_fenetre == ancien_graphe ) {
	if ( ( int (*)(Fenetre *, Message *) ) NULL == fenetre->message ) {
	  AVERTISSEMENT( "Fentre %s, graphe associ mais pas de gestion des messages !", fenetre->titre );
	  /* Par mesure de scurit, on ferme simplement la fentre... */
	  FEN_Detruire( fenetre );
	  retour = BI_EVENEMENT_FEN_FERMEE;
	} else {
	  retour = (fenetre->message)(fenetre, &message);
	}

	if ( BI_EVENEMENT_FEN_FERMEE == retour ) {
	  /* Des fentres ont t fermes : le chanage est chang => on doit redmarrer */
	  aucune_fermeture = FAUX;
	  break;
	}
      }

      /* Cas ventuel d'un graphe "cach" : en fonction de l'arbre dans la fentre ! */
      numero_arbre = FEN_QuelArbre( fenetre );
      if ( numero_arbre != -1 ) {
	MU0_infos *infos_mu0;

	switch ( fenetre->numero_arbre ) /* Quel est l'arbre associ ? */
	  {
	  case OTE_MU0_CALC: /* Soustraction d'un 0 dj calcul */
	    infos_mu0 = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
	    if ( infos_mu0 != NULL ) {
	      if ( ancien_graphe == infos_mu0->mu_0 ) { /* On modifie le 0 soustrait ! */
		(fenetre->message)(fenetre, &message);
	      }
	    }
	    break;
	  }
      }
    }
  } while ( FAUX == aucune_fermeture );

  if ( ancien_graphe == nv_graphe ) { /* Modification "en place" d'un graphe */
    if ( ancien_graphe == graphe_courant ) { /* Du graphe courant */
      FEN_redessin(principale, BI_RDS_UTIL);
    }

    TRC_RedessinerSuperposition( ancien_graphe, FAUX, VRAI );

    OPTION_A_FAIRE( "Mise  jour des figures !" );
  }
  if ( nv_graphe != NULL ) {
    nv_graphe->drapeaux.sauve = FAUX; /* Puisque l'on vient de le modifier... */
  }
}

/* ------ Nombre de fois qu'un graphe est utilis par une autre structure --------\
 |  Renvoie ce nombre d'utilisation                                               |
 |  La recherche dpend du type de graphe :                                       |
 |     - GR_DEVIATION : recherche les utilisations en tant que barre d'erreur     |
 |                                                                                |
 | graphe : celui pour lequel on effectue la recherche                            |
 \------------------------------------------------------------------------------- */

unsigned long GRF_utilisation(Graphe *graphe)
{
  Serie_Graphes *serie;
  unsigned long i = 0;
  Graphe *j;

  if (graphe == NULL) return 0;
  serie = GRF_QuelleSerie(graphe);
  if (serie == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_utilisation : graphe sans srie [%s]\n", graphe->nom);
    (void) fflush(stderr);
#endif
    return 0;
  }

  switch (graphe->type)
  {
  case GR_DEVIATION: /* Recherche d'utilisation comme barre d'erreur */
    for (j = serie->premier_graphe ; j ; j = j->suivant) {
      if (j->erreur_x == graphe) i++;
      if (j->erreur_y == graphe) i++;
    }
    break;

  default: /* Pas de ncessit spcifique de cherche le nombre d'utilisation */
    i = 0;
  }

  return i;
}
