/* LASE - logiciel d'analyse des spectres EXAFS
   (C) 1999 Emmanuel CURIS
   16 avril 1999

   Utilitaire de conversion : fichier LASE binaire (version 1.4) -> fichier ASCII
   Utilisation : lase_v_ascii fichier_1 fichier_2...
    ajoute le suffixe asc pour tre sr

    Sauve en 2 colonnes X, Y pour un graphe rel - 3 X, Y, Z pour un complexe
    S'il y a des barres d'erreur, ajoute une colonne (0 si absente pour les X mais Y)

    Option -c : permet d'ajouter des lignes de commentaire en dbut de fichier

   Utilise ch_interne (charger.c) pour charger le fichier
     -> par consquent, utilise aussi GRF_creer et GRF_effacer
     -> de plus, contient un succdan de SYS_ERREUR, et CH_selecteur, pour autoriser le linkage.
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "fichiers.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

FILE *sortie = NULL;

Graphe *lu = NULL;

unsigned long i, num_graphe;

boolean avec_commentaires = FAUX;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

int ecrit_fichier(char *nom, Graphe *lu);

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

int main(int n_args, char *args[])
{
  char *nom, *nom_sauve;
  int i;

  if (n_args < 2)
  {
    fprintf(stderr, "Utilisation : lase_v_ascii [-c] fichier_1 [fichier_2 ...]\n");
    return -1;
  }

  for (i = 1 ; i < n_args ; i++)
  {
    nom = args[i];

    if ( (nom[0] == '-') && (nom[1] == 'c') )
    {
      printf("Les fichiers de sortie suivants contiendront des commentaires, introduits par #\n");
      avec_commentaires = VRAI;
      continue;
    }

    nom_sauve = calloc(strlen(nom) + 5, 1);
    if (nom_sauve == NULL)
    {
      fprintf(stderr, "Mmoire insuffisante - conversion annule\n");
      return -2;
    }
    sprintf(nom_sauve, "%s.asc", nom);
    printf("Conversion de %s en %s\n", nom, nom_sauve);

    lu = ch_interne(nom);
    if (lu != NULL)
    {
      ecrit_fichier(nom_sauve, lu);
    }
    else
      fprintf(stderr, "Erreur de lecture de %s\n", nom);

    GRF_efface(lu, VRAI, VRAI);
    free(nom_sauve);
  }

  return 0;
}

/* -------- criture dans le fichier ASCII -------- */

int ecrit_fichier(char *nom_sauve, Graphe *lu)
{
  unsigned long i;
  FILE *ecrit;

  ecrit = fopen(nom_sauve, "wb");
  if (ecrit == NULL)
    return -1;

  if (avec_commentaires == VRAI)
  {
    fprintf(ecrit, "# Conversion automatique LASE -> ASCII, (C) 1999 Emmanuel CURIS\r\n");
    fprintf(ecrit, "# Nom, lgende X, lgende Y, signification des colonnes [rien = absente]\r\n");
    fprintf(ecrit, "# %s\r\n", lu->nom);
    fprintf(ecrit, "# %s\r\n", lu->axe_x);
    fprintf(ecrit, "# %s\r\n", lu->axe_y);
    fprintf(ecrit, "# Colonne : 1 \t 2 \t 3 \t 4 \t 5 \t 6\n");
    fprintf(ecrit, "# Contenu : X \t ");
    if (lu->drapeaux.genre == GR_NORMAL)
    {
      fprintf(ecrit, "Y \t ");
    }
    else /* Graphe  valeurs complexes */
    {
      fprintf(ecrit, "Re(Y) \t Im(Y) \t ");
    }
    if (lu->erreur_x)
    {
      fprintf(ecrit, "s(X) \t ");
    }
    if (lu->erreur_y)
    {
      if (lu->drapeaux.genre == GR_NORMAL)
      {
	fprintf(ecrit, "s(Y)");
      }
      else /* Graphe  valeurs complexes */
      {
	fprintf(ecrit, "s(Re(Y)) \t s(Im(Y))");
      }
    }
    fprintf(ecrit, "\r\n");
  }

  for (i = 0 ; i < lu->nombre_point ; i++)
  {
    fprintf(ecrit, "%f ", lu->x[i]);
    if (lu->drapeaux.genre == GR_NORMAL)
    {
      fprintf(ecrit, "%f ", lu->y[i]);
      if (lu->erreur_x)
      {
	fprintf(ecrit, "%f ", lu->erreur_x->y[i]);
      }
      if (lu->erreur_y)
	fprintf(ecrit, "%f ", lu->erreur_y->y[i]);
    }
    else /* Graphe  valeurs complexes */
    {
      Complexe *z = (Complexe *) (lu->y);
      fprintf(ecrit, "%f %f ", z[i].re, z[i].im);
      if (lu->erreur_x)
	fprintf(ecrit, "%f ", lu->erreur_x->y[i]);
      if (lu->erreur_y)
      {
	z = (Complexe *) (lu->erreur_y->y);
	fprintf(ecrit, "%f %f", z[i].re, z[i].im);
      }
    }

    fprintf(ecrit, "\r\n");
  }

  return 0;
}

/* -------- Fausses fonctions pour assurer l'dition de liens -------- */

int EXF_ERREUR(unsigned int code, ...)
{
  fprintf(stderr, "Erreur survenue - code %u\n", code);
  return 0;
}
